C$Procedure      DSPLAY (Display static descriptive information)
 
      SUBROUTINE LSTMID ( INPET, BODYID, LON, SCRATE, 
     .                    MNTYPE, SECOFF, MNET )
 
C$ Abstract
C
C     This routine find ET of the nearest, previous or next local
C     solar time specified as an integer number of seconds past
C     midnight of current local solar day..
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     None.
C
C$ Declarations
 
      IMPLICIT NONE
 
      DOUBLE PRECISION      INPET
      INTEGER               BODYID
      DOUBLE PRECISION      LON
      DOUBLE PRECISION      SCRATE
      CHARACTER*(*)         MNTYPE
      INTEGER               SECOFF
      DOUBLE PRECISION      MNET

 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     INPET      I   Input ET.
C     BODYID     I   NAIF ID of the body.
C     LON        I   Planetocentric longitude on the body.
C     SCRATE     I   Number of ET seconds in one local second.
C     MNTYPE     I   Time to look for: nearest, previous or next.
C     SECOFF     I   Offset from previous midnight in local seconds.
C     MNET       O   Midnight ET.
C
C$ Detailed_Input
C
C     ET             Input ephemeris time.
C
C     BODYID         NAIF ID of the body of interest.
C
C     LON            Planetocentric longitude on the body of interest.
C
C     SCRATE         Local time rate, i.e. number of ET seconds 
C                    in one local second.
C
C     MNTYPE         Time to look for -- 'NEAREST',
C                    'PREVIOUS' or 'NEXT'.
C
C     SECOFF         Offset from previous midnight in local seconds.
C                    Must be between 0 and SPD().
C
C$ Detailed_Output
C
C     MNET           Ephemeris time of the nearest, previous or next
C                    local time to a give input ET time.
C
C$ Parameters
C
C     MAXITR         Maximum number of iteration allowed in the loop.
C                    Currently set to 100. Usually there is no more 
C                    than 3-5 iterations are need to get to the 
C                    midnight time.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If number of iteration exceeds MAXITR, routine reports
C        SPICE(TOOMANYITERATIONS) error.
C
C     2) If value of MNTYPE is not one of the 'NEAREST', 'PREVIOUS' 
C        or 'NEXT', routine reports SPICE(BADMIDNIGHTTYPE) error.
C
C     3) If value of SECOFF is not between 0 and 86000, 
C        routine reports SPICE(BADTIMEOFFSET) error.
C
C$ Particulars
C
C     This routine is required to implement local solar days (SOLs) 
C     counting for the local solar time (LST). It's essential that 
C     caller provides correct SCRATE for a body. Also the routine 
C     assumes that there is enough SPICE kernel data loaded into 
C     the program to compute LST within +/- two local solar days 
C     of an input ET.
C
C$ Examples
C
C     Let our inputs be:
C
C        ETSTR  = '1997 AUG 12 12:00:12 TDB'
C        BODYID = 499
C        LON    = - 33.1D0
C        SCRATE = 1.0277116753731D0
C
C        CALL STR2ET( ETSTR, ET )
C        LON = LON * RPD()
C
C     then to compute nearest local midnight LSTMID must be called 
C     follows:
C
C        CALL LSTMID( ET, BODYID, LON, SCRATE, 'NEAREST', 0, MNET )
C     
C     to compute previous local noon:
C
C        CALL LSTMID( ET, BODYID, LON, SCRATE, 'PREVIOUS', 
C       .                                   INT( SPD()) / 2, MNET )
C     
C     to compute next local 6:00 a.m.:
C
C        CALL LSTMID( ET, BODYID, LON, SCRATE, 'NEXT', 
C       .                                   INT( SPD()) / 4, MNET )
C
C$ Restrictions
C
C     Sufficient SPICE kernel data loaded into the calling program to 
C     compute LST within +/- two local solar days of an input ET.
C
C$ Author_and_Institution
C
C     B.V.Semenov      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    CHRONOS Version 1.1.0, October 16, 2002 (BVS)
C
C        Bug fix: the search loop now forces termination is the
C        the delta value at MAXITR is 1 or -1. This has to be done
C        because of the integer output from ET2LST.
C
C-    CHRONOS Version 1.0.0, May 14, 1998 (BVS)
C
C
C-&

C
C     Local parameters.
C
      INTEGER               MAXITR
      PARAMETER           ( MAXITR = 100 )
      
C
C     Local variables.
C
      DOUBLE PRECISION      ET
               
      INTEGER               LSECS   
      INTEGER               HR
      INTEGER               MN
      INTEGER               SC
      INTEGER               I
      INTEGER               INTSPD
      
      CHARACTER*(24)        TIME
      CHARACTER*(6)         AMPM

C
C     SPICELIB functions.
C
      DOUBLE PRECISION      SPD
      LOGICAL               RETURN

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'LSTMID' )
      END IF

      ET = INPET
      INTSPD = INT ( SPD() )
      
C
C     Check if input offset within the right bounds.
C
      IF ( SECOFF .LT. 0 .OR. SECOFF .GT. INTSPD ) THEN
         CALL SETMSG ( 'Input offset expressed as count of ' //
     .                 'local seconds must be between 0 and 86000.')
         CALL SIGERR ( 'SPICE(BADTIMEOFFSET)'                      )
      END IF

C
C     Depending of the kind of midnight we need to compute, we
C     set initial ET for our iterations.
C
      IF       ( MNTYPE .EQ. 'NEAREST' ) THEN

C
C        We don't do anything but assign LSECS to start iterations
C        loop.
C
         LSECS = 1

      ELSE IF  ( MNTYPE .EQ. 'PREVIOUS' ) THEN

C
C        We step back a number of ET second between current local time
C        and previous desired local time.
C
         CALL ET2LST( ET, BODYID, LON, 'PLANETOCENTRIC', 
     .                HR, MN, SC, TIME, AMPM )
      
         LSECS = HR * 3600 + MN * 60 + SC - SECOFF
         IF ( LSECS .LT. 0 ) THEN
            LSECS = INTSPD + LSECS 
         END IF
         
         ET = ET - SCRATE * LSECS
         
      ELSE IF  ( MNTYPE .EQ. 'NEXT' ) THEN

C
C        We step forward a number of ET second between current
C        local time and following midnight.
C
         CALL ET2LST( ET, BODYID, LON, 'PLANETOCENTRIC', 
     .                HR, MN, SC, TIME, AMPM ) 
     
         LSECS = INTSPD - ( HR * 3600 + MN * 60 + SC - SECOFF )
         IF ( LSECS .GE. INTSPD ) THEN
            LSECS = LSECS - INTSPD 
         END IF
         
         ET = ET + SCRATE * LSECS
         
      ELSE

C
C        Unrecognizable MNTYPE. Complain and exit.
C
         CALL SETMSG ( 'Cannot recognize specification of the kind '//
     .                 'of local midnight to compute ''#''. ' //
     .                 'Recognizable values are ''NEAREST'', ' //
     .                 '''PREVIOUS'' and ''NEXT''.'               )
         CALL ERRCH  ( '#', MNTYPE                                )
         CALL SIGERR ( 'SPICE(BADMIDNIGHTTYPE)'                   )
         
      END IF
         
C
C     We stop iterations when LSECS is exact 0. We can do it because  
C     ET2LST returns integer number of local hours, minutes and 
C     seconds :). Before iterating we set counter to 0.

      I = 0 
               
      DO WHILE( LSECS .NE. 0 )
            
C
C        Get local time at current ET and convert it to number of ET
C        local seconds past previous time of interest.
C
         CALL ET2LST( ET, BODYID, LON, 'PLANETOCENTRIC', 
     .                      HR, MN, SC, TIME, AMPM )
           
         LSECS = HR * 3600 + MN * 60 + SC - SECOFF
         
C
C        If next time of interest is closer, adjust local seconds 
C        count correspondingly.
C
         IF ( LSECS .GT. INTSPD / 2 ) THEN
              LSECS = LSECS - INTSPD
         END IF
         
C
C        Increment current ET
C              
         ET = ET - LSECS  * SCRATE
               
C
C        Increase iterations counter and bail out if it's 
C        over the limit.
C
         I = I + 1
         
         IF ( I .GT. MAXITR ) THEN

C
C           Check if the last difference value is 1 or -1 local second.
C           If so, force the loop termination because it has probably
C           been this way for many iteration already. Otherwise
C           complain and exit.
C
            IF ( ( LSECS .EQ. 1 ) .OR. ( LSECS .EQ. -1 ) ) THEN
               LSECS = 0
            ELSE
               CALL SETMSG ( 'More than # iterations in the loop '   //
     .                       'determining local solar time midnight '//
     .                       'ET.'                                   )
               CALL ERRINT ( '#', MAXITR                             )
               CALL SIGERR ( 'SPICE(TOOMANYITERATIONS)'              )
            END IF

         END IF
               
      END DO
      
C
C     Assign output value and 
C
      MNET = ET
            
      CALL CHKOUT ( 'LSTMID' )
      RETURN      
      END

