C$Procedure      BODDEF ( Body name/ID code definition )
 
      SUBROUTINE BODDEF ( NAME, CODE )
 
C$ Abstract
C
C     Define a body name/ID code pair for later translation via
C     BODN2C or BODC2N.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     NAIF IDs
C
C$ Keywords
C
C     BODY
C     CONVERSION
C
C$ Declarations
 
      CHARACTER*(*)         NAME
      INTEGER               CODE
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     NAME       I   Common name of some body.
C     CODE       I   Integer code for that body.
C     MAXL       P   Maximum length of NAME string.
C
C$ Detailed_Input
C
C     NAME        is an arbitrary name of a body which could be
C                 a planet, satellite, barycenter, spacecraft,
C                 asteroid, comet, or other ephemeris object.
C
C                 The case and positions of blanks in a name are
C                 significant. BODC2N returns the same string
C                 (case and space) most recently mapped to a code.
C                 When NAME consists of more than one word, the
C                 words require separation by at least one blank.
C
C                 The kernel sub-system stores NAME as described in
C                 the BODDEF call, but creates an equivalence class
C                 based on NAME for comparisons in BODN2C. This class
C                 ignores leading/trailing whitespace, compresses
C                 interior whitespace to a single space, and ignores 
C                 character case.
C
C                 The following strings belong to the same equivalence 
C                 class:
C
C                         'JUPITER BARYCENTER'
C                         'Jupiter Barycenter'
C                         'JUPITER BARYCENTER   '
C                         'JUPITER    BARYCENTER'
C                         '   JUPITER BARYCENTER'
C
C                 However, 'JUPITERBARYCENTER' is distinct from
C                 the names above.
C
C                 When ignoring trailing blanks, NAME must be short
C                 enough to fit into the space defined by parameter
C                 MAXL.
C
C     CODE        is the integer ID code for assignment to body NAME.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     MAXL        is the maximum allowed length of a body NAME.
C                 Names exceeding this length will be truncated
C                 on assignment to a code with BODDEF.  The value
C                 of this parameter may be found in the include
C                 file 'zzbodtrn.inc'.
C
C$ Exceptions
C
C     1) Routines in the call tree of this routine may signal errors
C        if improper inputs are supplied, or if there is insufficient
C        room to store the requested addition.
C
C     2) If a name-code definition inserted into this routine seems to
C        have no effect, it is possible that the contents of the
C        definition are masked by the higher precedence kernel pool
C        assignments. See the "Particulars" section of this document
C        for more information.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     BODDEF is one of three related subroutines,
C
C        BODN2C      Body name to code
C
C        BODC2N      Body code to name
C
C        BODDEF      Body name/code definition
C
C     BODN2C and BODC2N perform translations between body names
C     and their corresponding integer ID codes which are used
C     in SPK and PCK files and routines.  A set of name/code
C     pairs are automatically defined during the first call to
C     one of these subroutines.  Additional name/code pairs may
C     be defined via BODDEF for two purposes:
C
C        1.  to associate another, perhaps more familiar or
C            abbreviated, name with a particular body integer
C            code that has already been defined, or
C
C        2.  to define a new body integer code and name,
C
C     Each body has a unique integer CODE, but may have several
C     names.  Thus you may associate more than one name with
C     a particular integer code.
C
C     CODE may already have a name as defined by a previous
C     call to BODDEF or as part of the set of default
C     definitions.  That previous definition will remain,
C     and a translation of that name will still give the
C     same CODE.  However, future translations of CODE will
C     give the new NAME instead of the previous one.  This
C     feature is useful for assigning a more familiar or
C     abbreviated name to a body. For example, in addition
C     to the default name for body 5, 'JUPITER BARYCENTER',
C     you could define the abbreviation 'JB' to mean 5.
C
C     Note: In the case where BODDEF performs a body/ID mapping
C     assignment for an unused body name and unused ID value,
C     any subsequent assignment to NAME destroys the pervious
C     mapping.
C
C        BODDEF( 'spud', 22)
C
C     then
C
C        BODDEF( 'spud', 23)
C
C     results in the state "spud" maps to 23, 23 maps to "spud",
C     and 22 maps to nothing (FOUND in BODC2N returns FALSE).
C
C     Please refer to the NAIF_IDS Required Reading document for
C     details about kernel pool name-code definitions, masking,
C     and assignment precedence.
C
C$ Examples
C
C     You may associate a new name for a previously defined code:
C
C            CALL BODDEF ( 'JB', 5 )
C
C     You may also define the name and integer code for a new body:
C
C            CALL BODDEF ( 'Asteroid Frank', 20103456 )
C
C     After these calls to BODDEF, BODN2C would return the following
C     translations:
C
C        Name                         Code    Found?
C        ------------------------   ------    ------
C        'JB'                            5    Yes
C        'Jupiter Barycenter'            5    Yes
C        'ASTEROID FRANK'         20103456    Yes
C        'ASTEROIDFRANK'                 -    No
C        'Frank'                         -    No
C
C     and BODC2N will return these translations:
C
C        Code        Name                     Found?
C        -------     -------------------      ------
C               5    'JB'                     Yes
C        20103456    'Asteroid Frank'         Yes
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     K.R. Gehringer  (JPL)
C     F.S. Turner     (JPL)
C     E.D. Wright     (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 23-JAN-2004 (EDW)
C
C        Rewrote header for clarity with regards to the
C        current capabilities of the kernel subsystem.
C
C-    SPICELIB Version 1.0.2, 26-AUG-2002 (FST)
C
C        Updated header to describe the parameter MAXL and
C        its effect on this module.  The exceptions section
C        was updated to include a more general discussion
C        of errors that routines in the call tree of this
C        routine may signal.
C
C-    SPICELIB Version 1.0.1, 12-AUG-2001 (EDW)
C
C        Updated header with information on new functionality.
C        The code-to-name retrieval routines now return the exact
C        string as defined in the last code/name mapping (case
C        and space).
C
C-    SPICELIB Version 1.0.0, 23-JAN-1996 (KRG)
C
C        This was the BODDEF entry point from the original BODTRN
C        subroutine that was in the NAIF toolkit SUPPORT library.
C        When the private subroutine ZZBODTRN was added to SPICELIB,
C        superceding the BODTRN from SUPPORT, the body ID code/name
C        translation interface from the original BODTRN was moved to
C        SPICELIB so that ID codes did not have to be hard coded by
C        users of the toolkit.
C
C        This subroutine simply calls the private subroutine ZZBODDEF
C        to perform its job.
C
C-&

C$ Index_Entries
C
C     body name/id code definition
C
C-&
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
C
C     Standard SPICELIB error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'BODDEF' )
      END IF
 
      CALL ZZBODDEF ( NAME, CODE )

C
C     No need for any error checking, since all we do is check out
C     and return anyway. We leave the error checking to the caller.
C
      CALL CHKOUT ( 'BODDEF' )
      RETURN
 
      END
