C$Procedure      CKMETA ( CK ID to associated SCLK )
 
      SUBROUTINE CKMETA ( CKID,  META, IDCODE )
 
C$ Abstract
C
C     This routine returns (depending upon the users' request)
C     the ID code of either the spacecraft or spacecraft clock
C     associated with a C-Kernel ID code.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     UTILITY
C
C$ Declarations
 
      IMPLICIT NONE
      INTEGER               CKID
      CHARACTER*(*)         META
      INTEGER               IDCODE
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     CKID       I   The ID code for some C kernel object.
C     META       I   The kind of meta data requested 'SPK' or 'SCLK'
C     IDCODE     O   The ID code for the clock of the C kernel.
C
C$ Detailed_Input
C
C     CKID        is the ID code for some object whose attitude
C                 and possibly angular velocity are stored in
C                 some C-kernel.
C
C     META        is a character string that indicates which piece
C                 of meta data to fetch.  Acceptable values are
C                 'SCLK' and 'SPK'. The routine is case insensitive.
C                 Leading and trailing blanks are insignificant.
C                 However, blanks between characters are regarded
C                 as being significant and will result in the error
C                 'SPICE(UNKNOWNCKMETA)' being signalled.
C
C$ Detailed_Output
C
C     IDCODE      if META is 'SCLK' then the value returned in IDCODE
C                 is the "ID code" of the spacecraft clock used for
C                 converting ET to TICKS and TICKS to ET for the
C                 C-kernel used to represent the attitude of the
C                 object with ID code CKID.
C
C                 if META is 'SPK' then the value returned in IDCODE
C                 is the "ID code" of the spacecraft on which the
C                 platform indicated by CKID is mounted.
C
C$ Parameters
C
C      None.
C
C$ Files
C
C      None.
C
C$ Exceptions
C
C     1) If the variable META is not recognized to be one of the
C        inputs 'SPK' or 'SCLK' then the error 'SPICE(UNKNOWNCKMETA)'
C        will be signalled.
C
C     2) If CKID is greater than -1000, the associated SCLK and SPK
C        ID's must be in the kernel pool.  If they are not present
C        a value of zero is returned for the requested item.  Zero
C        is never the valid ID of a spacecraft clock or ephemeris
C        object.
C
C$ Particulars
C
C     This is a utility routine for mapping C-kernels to associated
C     spacecraft clocks.  This is needed to facilitate the writing
C     of routines such as CKEZ and CKEZAV.
C
C$ Examples
C
C     Suppose you would like to look up the attitude of
C     an object in a C-kernel but have ET and seconds as your
C     input time and tolerance.
C
C     This routine can be used in conjunction with SCE2C and
C     CKGPAV to perform this task.
C
C     CALL CKMETA ( CKID,  'SCLK'      IDCODE )
C
C     CALL SCE2C  ( IDCODE, ET,        TICKS  )
C     CALL SCE2C  ( IDCODE, ET+SECTOL, TICK2  )
C
C     TOL = TICK2 - TICKS
C
C     CALL CKGPAV ( CKID, TICKS, TOL, REF, CMAT, AV, CLKOUT, FOUND )
C
C     IF ( FOUND ) THEN
C
C        CALL SCT2E ( IDCODE, CLKOUT, ETOUT )
C
C     END IF
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 09-MAR-1999 (NJB)
C
C        Comments referring to SCE2T have been updated to refer to 
C        SCE2C.  Occurrences of "id" replaced by "ID."
C
C-    SPICELIB Version 1.0.0, 4-OCT-1994 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Map C-kernel ID to SCLK and SPK ID.
C
C-&
 
 
C
C     SPICELIB Functions
C
      INTEGER               BSCHOI
      LOGICAL               RETURN
 
 
      INTEGER               WDSIZE
      PARAMETER           ( WDSIZE = 32 )
 
      INTEGER               NCK
      PARAMETER           ( NCK = 30 )
 
C
C     SCLK
C     SPK
C
 
      INTEGER               SCLK
      PARAMETER           ( SCLK   =          1 )
 
      INTEGER               SPK
      PARAMETER           ( SPK    = SCLK   + 1 )
 
 
      CHARACTER*(7)         BASE
      CHARACTER*(7)         MYMETA
      CHARACTER*(WDSIZE)    LOOKUP ( 2, NCK )
      CHARACTER*(WDSIZE)    AGENT  (    NCK )
 
 
 
      INTEGER               CKS    (    NCK )
      INTEGER               CKSORD (    NCK )
      INTEGER               CURRNT
      INTEGER               LAST
      INTEGER               N
      INTEGER               SCLKS  (    NCK )
      INTEGER               SPKS   (    NCK )
      INTEGER               THIS
 
 
      LOGICAL               FOUND  (    2   )
      LOGICAL               UPDATE
      SAVE
 
      DATA                  CURRNT / 0 /
      DATA                  LAST   / 0 /
      DATA                  BASE   / 'CKMETA.' /
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN ( 'CKMETA' )
 
C
C     See if we already have this CK ID in hand.
C
      THIS = BSCHOI ( CKID, CURRNT, CKS, CKSORD )
 
      IF ( THIS .GT. 0 ) THEN
 
C
C        We've got it.  Check to see if its value has been updated.
C        (Note that every CK has its own agent.)
C
         CALL CVPOOL ( AGENT(THIS), UPDATE )
 
         IF ( UPDATE ) THEN
 
            CALL GIPOOL( LOOKUP(SCLK,THIS), 1, 1, N,
     .                   SCLKS(THIS),       FOUND(1) )
 
            CALL GIPOOL( LOOKUP(SPK, THIS), 1, 1, N,
     .                   SPKS (THIS),       FOUND(2) )
         END IF
 
 
      ELSE
 
C
C        We don't have this on our handy list.  Find a place to put it.
C
         IF ( CURRNT .LT. NCK ) THEN
 
            CURRNT = CURRNT + 1
            LAST   = CURRNT
 
         ELSE
 
            LAST   = LAST + 1
 
            IF ( LAST .GT. NCK ) THEN
               LAST = 1
            END IF
 
         END IF
 
 
         THIS      = LAST
         CKS(THIS) = CKID
 
C
C        Recompute the order vector for the CKS; construct the
C        kernel pool variable names and the agent name.
C
         CALL ORDERI (  CKS,    CURRNT, CKSORD            )
         CALL INTSTR (  CKID,           LOOKUP(SCLK,THIS) )
         CALL PREFIX ( 'CK_',   0,      LOOKUP(SCLK,THIS) )
 
         AGENT (     THIS) =  BASE   // LOOKUP(SCLK,THIS)
         LOOKUP(SPK, THIS) =            LOOKUP(SCLK,THIS)
 
         CALL SUFFIX ( '_SCLK',  0,      LOOKUP(SCLK,THIS) )
         CALL SUFFIX ( '_SPK',   0,      LOOKUP(SPK, THIS) )
 
C
C        Set a watch for this item and fetch the current value
C        from the kernel pool (if there is a value there).
C
         CALL SWPOOL ( AGENT (THIS), 1,    LOOKUP(1,THIS)      )
         CALL CVPOOL ( AGENT (THIS),       UPDATE              )
 
         CALL GIPOOL(LOOKUP(SCLK,THIS), 1, 1, N,
     .               SCLKS(THIS),       FOUND(SCLK) )
 
         CALL GIPOOL(LOOKUP(SPK, THIS), 1, 1, N,
     .               SPKS (THIS),       FOUND(SPK ) )
C
C        If we didn't find it, we manufacture an ID code based upon
C        the "convention" used for all CKS so far.  However, the
C        convention assumes that the CK ID will be less than -1000
C        if it's not there is no sensible ID to return.  We return
C        zero in that case.
C
         IF ( CKS(THIS) .LE. -1000 ) THEN
 
            IF ( .NOT. FOUND(SCLK) ) THEN
               SCLKS(THIS) = CKS(THIS)/1000
            END IF
 
            IF ( .NOT. FOUND(SPK)  ) THEN
               SPKS(THIS) = CKS(THIS)/1000
            END IF
 
         ELSE
 
            IF ( .NOT. FOUND(SCLK) ) THEN
               SCLKS(THIS) = 0
            END IF
 
            IF ( .NOT. FOUND(SPK)  ) THEN
               SPKS(THIS) = 0
            END IF
 
         END IF
 
      END IF
 
      CALL CMPRSS ( ' ', 1, META, MYMETA )
      CALL LJUST  ( MYMETA,       MYMETA )
      CALL UCASE  ( MYMETA,       MYMETA )
 
      IF ( MYMETA .EQ. 'SPK' ) THEN
 
         IDCODE = SPKS (THIS)
 
      ELSE IF ( MYMETA .EQ. 'SCLK' ) THEN
 
         IDCODE = SCLKS(THIS)
 
      ELSE
 
         IDCODE = 0
 
         CALL SETMSG ( 'The CK meta data item "#" is not a '
     .   //            'recognized meta data item for the '
     .   //            'routine CKMETA.  The recognized value '
     .   //            'are "SPK" and "SCLK". ' )
 
         CALL ERRCH  ( '#', META )
         CALL SIGERR ( 'SPICE(UNKNOWNCKMETA)'  )
         CALL CHKOUT ( 'CKMETA' )
         RETURN
 
      END IF
 
      CALL CHKOUT ( 'CKMETA' )
 
      RETURN
      END
