C$Procedure      CKW04A ( CK type 04: Add data to a segment )
 
      SUBROUTINE CKW04A ( HANDLE, NPKTS, PKTSIZ, PKTDAT, SCLKDP )
      IMPLICIT NONE
 
C$ Abstract
C
C     Add data to a type 4 CK segment currently being written to
C     the file associated with HANDLE. See also CKW04B and CKW04E.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     CK.REQ
C     DAF.REQ
C     GS.REQ
C
C$ Keywords
C
C     POINTING
C
C$ Declarations
 
      INCLUDE               'ckparam.inc'
 
      INTEGER               HANDLE
      INTEGER               NPKTS
      INTEGER               PKTSIZ( * )
      DOUBLE PRECISION      PKTDAT( * )
      DOUBLE PRECISION      SCLKDP( * )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     HANDLE     I   The handle of an DAF file opened for writing.
C     NPKTS      I   Number of data packets to write to a segment.
C     PKTSIZ     I   The numbers of values in the data packets
C     PKTDAT     I   The data packets.
C     SCLKDP     I   The SCLK times associated with the data packets.
C
C$ Detailed_Input
C
C     HANDLE     is the file handle of a CK file in which a CK type 4
C                segment is currently being written.
C
C     NPKTS      is the number of data packets to write to a segment.
C
C     PKTSIZ     is the number of values in all data packets.
C
C     PKTDAT     is the data packets. The data packets in this array
C                must be organized as described in the $ Particulars
C                section of the header.
C
C     SCLKDP     contains the initial SCLK times corresponding to the
C                Chebyshev coefficients in PKTSIZ. The I'th time is
C                start time of the I'th packet coverage interval.
C                The times must form a strictly increasing sequence.
C
C$ Detailed_Output
C
C     None.      Data is stored in a segment in the DAF file
C                associated with HANDLE.
C
C$ Parameters
C
C     See 'ckparam.inc'.
C
C$ Files
C
C     See HANDLE in the $ Detailed_Input section.
C
C$ Exceptions
C
C     1) If the number of coefficient sets and epochs is not positive,
C        the error SPICE(INVALIDARGUMENT) will be signalled.
C
C     2) If size of any input packet is greater that maximum allowed
C        type 4 CK record size minus one, the error
C        SPICE(INVALIDARGUMENT) will be signalled.
C
C$ Particulars
C
C     This routine adds data to a type 4 CK segment that is currently
C     being written to the associated with HANDLE. The segment must
C     have been started by a call to the routine CKW04B, the routine
C     which begins a type 4 CK segment.
C
C     This routine is one of a set of three routines for creating and
C     adding data to type 4 CK segments. These routines are:
C
C        CKW04B: Begin a type 4 CK segment. This routine must be
C                called before any data may be added to a type 4
C                segment.
C
C        CKW04A: Add data to a type 4 CK segment. This routine may be
C                called any number of times after a call to CKW04B to
C                add type 4 records to the CK segment that was
C                started.
C
C        CKW04E: End a type 4 CK segment. This routine is called to
C                make the type 4 segment a permanent addition to the
C                DAF file. Once this routine is called, no further type
C                4 records may be added to the segment. A new segment
C                must be started.
C
C     A type 4 CK segment consists of coefficient sets for variable
C     order Chebyshev polynomials over consecutive time intervals of
C     a variable length. The gaps between intervals are allowed.
C     The Chebyshev polynomials represent individual quaternion
C     components q0, q1, q2 and q3 and individual angular velocities
C     AV1, AV2 and AV3 if they are included with the data.
C
C     The pointing data supplied to the type 4 CK writer (CKW04A)
C     is packed into an array as a sequence of records,
C
C        ----------------------------------------------------
C        | Record 1 | Record 2 | .. | Record N-1 | Record N |
C        ----------------------------------------------------
C
C     with each record in data packets has the following format.
C
C        ----------------------------------------------------
C        | The midpoint of the approximation interval       |
C        ----------------------------------------------------
C        | The radius of the approximation interval         |
C        ----------------------------------------------------
C        | Number of coefficients for q0                    |
C        ----------------------------------------------------
C        | Number of coefficients for q1                    |
C        ----------------------------------------------------
C        | Number of coefficients for q2                    |
C        ----------------------------------------------------
C        | Number of coefficients for q3                    |
C        ----------------------------------------------------
C        | Number of coefficients for AV1                   |
C        ----------------------------------------------------
C        | Number of coefficients for AV2                   |
C        ----------------------------------------------------
C        | Number of coefficients for AV3                   |
C        ----------------------------------------------------
C        | q0 Cheby coefficients                            |
C        ----------------------------------------------------
C        | q1 Cheby coefficients                            |
C        ----------------------------------------------------
C        | q2 Cheby coefficients                            |
C        ----------------------------------------------------
C        | q3 Cheby coefficients                            |
C        ----------------------------------------------------
C        | AV1 Cheby coefficients (optional)                |
C        ----------------------------------------------------
C        | AV2 Cheby coefficients (optional)                |
C        ----------------------------------------------------
C        | AV3 Cheby coefficients (optional)                |
C        ----------------------------------------------------
C
C$ Examples
C
C     Assume that we have:
C
C        HANDLE   is the handle of an CK file opened with write
C                 access.
C
C        SEGID    is a character string of no more than 40 characters
C                 which provides a pedigree for the data in the CK
C                 segment we will create.
C
C        INST     is the SPICE ID code for the instrument whose
C                 pointing data is to be placed into the file.
C
C        AVFLAG   angular rates flag.
C
C        REFFRM   is the name of the SPICE reference frame for the
C                 pointing data.
C
C        BEGTIM   is the starting encoded SCLK time for which the
C                 segment is valid.
C
C        ENDTIM   is the ending encoded SCLK time for which the segment
C                 is valid.
C
C        N        is the number of type 4 records that we want to
C                 put into a segment in an CK file.
C
C        NPKTS    is integer array which contains the lengths of
C                 variable size data packets
C
C        RECRDS   contains N type 4 records packaged for the CK
C                 file.
C
C        SCSTRT   contains the initial encoded SC time for each of
C                 the records contained in RECRDS, where
C
C                    SCSTRT(I) < SCSTRT(I+1), I = 1, N-1
C
C                    SCSTRT(1) <= FIRST, SCSTRT(N) < LAST
C
C     Then the following code fragment demonstrates how to create
C     a type 4 CK segment if all of the data for the segment is
C     available at one time.
C
C     C
C     C     Begin the segment.
C     C
C           CALL CKW04B ( HANDLE, BEGTIM, INST, REF, AVFLAG, SEGID )
C     C
C     C     Add the data to the segment all at once.
C     C
C           CALL CKW04A ( HANDLE, N, NPKTS, RECRDS, SCSTRT )
C     C
C     C     End the segment, making the segment a permanent
C     C     addition to the CK file.
C     C
C           CALL CKW04E ( HANDLE, ENDTIM )
C
C$ Restrictions
C
C     1) The type 4 CK segment to which the data is added must have
C        been started by the routine CKW04B, the routine which begins
C        a type 4 CK segment.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     Y.K. Zaiko     (JPL)
C     B.V. Semenov   (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 07-SEP-2001 (EDW)
C
C        Removed DAFHLU call; replaced ERRFNM call with ERRHAN.
C        Added IMPLICIT NONE.
C
C-    SPICELIB Version 1.0.0, 05-MAY-1999 (YKZ) (BVS)
C
C-&
 
C$ Index_Entries
C
C     add data to a type_4 ck segment
C
C-&
 
C
C     Spicelib functions.
C
      LOGICAL               RETURN
 
C
C     Local parameters.
C
 
C
C     The number of elements by which coefficients in each packet
C     have to be shifted to the left after numbers of coefficients
C     were packed into a single integer.
C
      INTEGER               SHFTAD
      PARAMETER           ( SHFTAD = 6 )
 
C
C     Local Variables.
C
      INTEGER               K
      INTEGER               KK
      INTEGER               NUMCFT ( QAVSIZ )
      INTEGER               DISPLM
      INTEGER               DISPM
 
C
C     Standard SPICELIB error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'CKW04A' )
      END IF
 
C
C     First, check if the number of coefficient sets and epochs
C     is positive and whether each packet is smaller than the
C     maximum size of a record that CKPFS can handle.
C
      DO K = 1, NPKTS
 
         IF ( PKTSIZ( K ) .LE. 0 ) THEN
 
            CALL SETMSG ( 'The number of coefficient sets and '      //
     .                    'epochs in the # data packet (record) '    //
     .                    'to be added to the DAF segment '          //
     .                    'in the file ''#'' was not positive. '     //
     .                    'Its value was: #.'                        )
            CALL ERRINT ( '#', K                                     )
            CALL ERRHAN ( '#', HANDLE                                )
            CALL ERRINT ( '#', PKTSIZ( K )                           )
            CALL SIGERR ( 'SPICE(INVALIDARGUMENT)'                   )
            CALL CHKOUT ( 'CKW04A'                                   )
            RETURN
 
         END IF
 
C
C        We do .GE. comparison because a type 4 CK record passed
C        inside CKPFS will have one more element -- time at which
C        the pointing will be evaluated.
C
         IF ( PKTSIZ( K ) .GE. CK4RSZ ) THEN
 
            CALL SETMSG ( 'The total size of the # data packet '     //
     .                    '(record) to be added to the DAF segment ' //
     .                    'in the file ''#'' is greater than the '   //
     .                    'maximum allowed type 4 record size #. '   //
     .                    'Its value was: #.'                        )
            CALL ERRINT ( '#', K                                     )
            CALL ERRHAN ( '#', HANDLE                                )
            CALL ERRINT ( '#', CK4RSZ - 1                            )
            CALL ERRINT ( '#', PKTSIZ( K )                           )
            CALL SIGERR ( 'SPICE(INVALIDARGUMENT)'                   )
            CALL CHKOUT ( 'CKW04A'                                   )
            RETURN
 
         END IF
 
 
      END DO
 
      DISPLM = 0
      DISPM = 0
C
C     The cycle below encodes groups of numbers of coefficients in
C     data packets to single double precision numbers and shift
C     data in packets to the left to decrease the data packet
C     lengths.
C
      DO K = 1, NPKTS
 
C
C        Encode integer numbers of coefficients for each component
C        to single double precision variable
C
         DO KK = 1, QAVSIZ
            NUMCFT( KK ) = INT ( PKTDAT( KK + 2 + DISPLM ) )
         END DO
 
         CALL ZZCK4I2D ( NUMCFT, QAVSIZ, CK4PCD, PKTDAT( 3 + DISPM ) )
 
C
C        Shift coefficients sets to the left to overwrite numbers of
C        packets
C
         DO KK = 4, PKTSIZ( K )
            PKTDAT( KK + DISPM ) = PKTDAT( KK + SHFTAD + DISPLM )
         END DO
 
C
C        Shift middle value and radii of interval
C
         PKTDAT( 1 + DISPM ) = PKTDAT( 1 + DISPLM )
         PKTDAT( 2 + DISPM ) = PKTDAT( 2 + DISPLM )
 
         DISPLM = DISPLM + PKTSIZ ( K )
 
C
C        Length of each data packet became less for 6 elements because
C        of encoding of 7 double precision numbers, which are the
C        numbers of polynomial coefficients, to one double precision
C        number
C
         PKTSIZ( K ) = PKTSIZ( K ) - SHFTAD
         DISPM = DISPM + PKTSIZ ( K )
 
      END DO
 
C
C     Add the data.
C
      CALL SGWVPK ( HANDLE, NPKTS, PKTSIZ, PKTDAT, NPKTS, SCLKDP )
 
C
C     No need to check FAILED() here, since all we do is check out.
C     Leave it up to the caller.
C
      CALL CHKOUT ( 'CKW04A' )
 
      RETURN
 
      END
