C$Procedure DAFGDA ( DAF, read data from address )
 
      SUBROUTINE DAFGDA ( HANDLE, BEGIN, END, DATA )
 
C$ Abstract
C
C     Read the double precision data bounded by two addresses within
C     a DAF.
C
C$ Copyright
C
C     Copyright (2001), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     DAF
C
C$ Keywords
C
C     FILES
C
C$ Declarations
 
      INTEGER               HANDLE
      INTEGER               BEGIN
      INTEGER               END
      DOUBLE PRECISION      DATA     ( * )
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     HANDLE     I   Handle of a DAF.
C     BEGIN,
C     END        I   Initial, final address within file.
C     DATA       O   Data contained between BEGIN and END.
C
C$ Detailed_Input
C
C     HANDLE      is the handle of a DAF.
C
C     BEGIN,
C     END         are the initial and final addresses of a contiguous
C                 set of double precision numbers within a DAF.
C                 Presumably, these make up all or part of a particular
C                 array.
C
C$ Detailed_Output
C
C     DATA        are the double precision data contained between
C                 the specified addresses within the specified file.
C
C$ Parameters
C
C      None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If BEGIN is zero or negative, the error SPICE(DAFNEGADDR)
C        is signalled.
C
C     2) If the BEGIN > END, the error SPICE(DAFBEGGTEND)
C        is signalled.
C
C     3) If HANDLE is invalid, routines in the call tree of DAFGDA
C        signal an appropriate error.
C
C     4) If the range of addresses covered between BEGIN and END
C        includes records that do not contain strictly double
C        precision data, then the values returned in DATA are
C        undefined.  See the Restrictions section below for details.
C
C$ Particulars
C
C     The principal reason that DAFs are so easy to use is that
C     the data in each DAF are considered to be one long contiguous
C     set of double precision numbers. You can grab data from anywhere
C     within a DAF without knowing (or caring) about the physical
C     records in which they are stored.
C
C     This routine replaces DAFRDA as the principle mechanism for
C     reading the contents of DAF arrays.
C
C$ Examples
C
C     The following code fragment illustrates the use of DAFGDA
C     to read data from an imaginary array. The array begins with a
C     directory containing 11 epochs. Each pair of epochs bounds
C     an interval, and each interval is covered by a set of eight
C     osculating elements.
C
C        CALL DAFUS ( SUM, ND, NI, DC, IC )
C        BEGIN = IC(5)
C        END   = IC(6)
C
C        CALL DAFGDA ( HANDLE, BEGIN, BEGIN+10, EPOCHS )
C
C        DO I = 1, 10
C           IF ( ET .GE. EPOCHS(I)  .AND.  ET .LE. EPOCHS(I+1) ) THEN
C              OFFSET = 11 + (I - 1) * 8
C
C              CALL DAFGDA ( HANDLE, OFFSET+1, OFFSET+8, ELEMENTS )
C              RETURN
C           END IF
C        END DO
C
C
C$ Restrictions
C
C     1) There are several types of records in a DAF.  This routine
C        is only to be used to read double precision data bounded
C        between two DAF addresses.  The range of addresses input
C        may not cross data and summary record boundaries.
C
C$ Literature_References
C
C     NAIF Document 167.0, "Double Precision Array Files (DAF)
C     Specification and User's Guide"
C
C$ Author_and_Institution
C
C     F.S. Turner     (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 16-NOV-2001 (FST)
C
C-&
 
C$ Index_Entries
C
C     read data from daf address
C
C-&
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
 
C
C     Local variables
C
      INTEGER               BSIZE
      PARAMETER           ( BSIZE = 128 )
 
      INTEGER               BEGR
      INTEGER               BEGW
      INTEGER               ENDR
      INTEGER               ENDW
 
      INTEGER               RECNO
      INTEGER               FIRST
      INTEGER               LAST
      INTEGER               NEXT
 
      LOGICAL               FOUND
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      END IF
 
C
C     Bad addresses?
C
      IF ( BEGIN .LE. 0 ) THEN
         CALL CHKIN  ( 'DAFGDA' )
         CALL SETMSG ( 'Negative value for BEGIN address: #' )
         CALL ERRINT ( '#', BEGIN )
         CALL SIGERR ( 'SPICE(DAFNEGADDR)' )
         CALL CHKOUT ( 'DAFGDA' )
         RETURN
 
      ELSE IF ( BEGIN .GT. END ) THEN
         CALL CHKIN  ( 'DAFGDA' )
         CALL SETMSG ( 'Beginning address (#) greater than ending '   //
     .                 'address (#).' )
         CALL ERRINT ( '#', BEGIN )
         CALL ERRINT ( '#', END   )
         CALL SIGERR ( 'SPICE(DAFBEGGTEND)' )
         CALL CHKOUT ( 'DAFGDA' )
         RETURN
      END IF
 
C
C     Convert raw addresses to record/word representations.
C
      CALL DAFARW ( BEGIN, BEGR, BEGW )
      CALL DAFARW ( END,   ENDR, ENDW )
 
C
C     Get as many records as needed. Return the last part of the
C     first record, the first part of the last record, and all of
C     every record in between. Any record not found is assumed to
C     be filled with zeros.
C
      NEXT = 1
 
      DO RECNO = BEGR, ENDR
 
         IF ( BEGR .EQ. ENDR ) THEN
            FIRST = BEGW
            LAST  = ENDW
 
         ELSE IF ( RECNO .EQ. BEGR ) THEN
            FIRST = BEGW
            LAST  = BSIZE
 
         ELSE IF ( RECNO .EQ. ENDR ) THEN
            FIRST = 1
            LAST  = ENDW
 
         ELSE
            FIRST = 1
            LAST  = BSIZE
         END IF
 
         CALL DAFGDR ( HANDLE, RECNO, FIRST, LAST, DATA( NEXT ), FOUND )
 
         IF ( .NOT. FOUND ) THEN
            CALL CLEARD ( LAST-FIRST+1, DATA( NEXT ) )
         END IF
 
         NEXT = NEXT + ( LAST-FIRST+1 )
 
      END DO
 
      RETURN
      END
