C$Procedure      DUCRSS ( Unit Normalized Cross Product and Derivative )
 
      SUBROUTINE DUCRSS ( S1, S2, SOUT )
 
C$ Abstract
C
C     Compute the unit vector parallel to the cross product of
C     two 3-dimensional vectors and the derivative of this unit vector.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     VECTOR
C     DERIVATIVE
C
C$ Declarations
 
      DOUBLE PRECISION    S1   ( 6 )
      DOUBLE PRECISION    S2   ( 6 )
      DOUBLE PRECISION    SOUT ( 6 )
 
C$ Brief_I/O
C
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     S1        I   Left hand state for cross product and derivative.
C     S2        I   Right hand state for cross product and derivative.
C     SOUT      O   Unit vector and derivative of the cross product.
C
C$ Detailed_Input
C
C     S1       This may be any state vector.  Typically, this
C              might represent the apparent state of a planet or the
C              Sun, which defines the orientation of axes of
C              some coordinate system.
C
C     S2       Any state vector.
C
C$ Detailed_Output
C
C     SOUT     This variable represents the unit vector parallel to the
C              cross product of the position components of S1 and S2
C              and the derivative of the unit vector.
C
C              If the cross product of the position components is
C              the zero vector, then the position component of the
C              output will be the zero vector.  The velocity component
C              of the output will simply be the derivative of the
C              cross product of the position components of S1 and S2.
C
C              SOUT may overwrite S1 or S2.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C     1) If the position components of S1 and S2 cross together to
C        give a zero vector, the position component of the output
C        will be the zero vector.  The velocity component of the
C        output will simply be the derivative of the cross product
C        of the position vectors.
C
C     2) If S1 and S2 are large in magnitude (taken together,
C        their magnitude surpasses the limit allowed by the
C        computer) then it may be possible to generate a
C        floating point overflow from an intermediate
C        computation even though the actual cross product and
C        derivative may be well within the range of double
C        precision numbers.
C
C$ Particulars
C
C     DUCRSS calculates the unit vector parallel to the cross product
C     of two vectors and the derivative of that unit vector.
C     The results of the computation may overwrite either of the
C     input vectors.
C
C$ Examples
C
C     One often constructs non-inertial coordinate frames from
C     apparent positions of objects.  However, if one wants to convert
C     states in this non-inertial frame to states in an inertial
C     reference frame, the derivatives of the axes of the non-inertial
C     frame are required.  For example consider an Earth meridian
C     frame defined as follows.
C
C        The z-axis of the frame is defined to be the vector
C        normal to the plane spanned by the position vectors to the
C        apparent Sun and to the apparent body as seen from an observer.
C
C        Let SUN be the apparent state of the Sun and let BODY be the
C        apparent state of the body with respect to the observer.  Then
C        the unit vector parallel to the z-axis of the Earth meridian
C        system and its derivative are given by the call:
C
C        CALL DUCRSS ( SUN, BODY, ZZDOT )
C
C$ Restrictions
C
C     No checking of S1 or S2 is done to prevent floating point
C     overflow. The user is required to determine that the magnitude
C     of each component of the states is within an appropriate range
C     so as not to cause floating point overflow. In almost every case
C     there will be no problem and no checking actually needs to be
C     done.
C
C$ Author_and_Institution
C
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 15-JUN-1995 (WLT)
C
C
C-&
 
 
C$ Index_Entries
C
C     Compute a unit cross product and its derivative
C
C-&
C
 
 
 
C
C     Not much to this.  Just get the cross product and its derivative.
C     Using that, get the associated unit vector and its derivative.
C
      CALL DVCRSS ( S1,   S2, SOUT )
      CALL DVHAT  ( SOUT,     SOUT )
 
 
      RETURN
      END
 
