C$Procedure      DVHAT ( Derivative and unit vector "V-hat" of a state)
 
      SUBROUTINE DVHAT ( S1, SOUT )
 
C$ Abstract
C
C     Find the unit vector corresponding to a state vector and the
C     derivative of the unit vector.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     VECTOR
C     DERIVATIVE
C
C$ Declarations
 
      DOUBLE PRECISION   S1   ( 6 )
      DOUBLE PRECISION   SOUT ( 6 )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     S1        I     State to be normalized.
C     SOUT      O     Unit vector S1 / |S1|, and its time derivative.
C
C$ Detailed_Input
C
C     S1       This is any double precision state. If the position
C              component of the state is the zero vector, this routine
C              will detect it and will not attempt to divide by zero.
C
C$ Detailed_Output
C
C     SOUT     SOUT is a state containing the unit vector pointing in
C              the direction of position component of S1 and the
C              derivative of the unit vector with respect to time.
C
C              SOUT may overwrite S1.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C     1) If S1 represents the zero vector, then the position
C        component of SOUT will also be the zero vector.  The
C        velocity component will be the velocity component
C        of S1.
C
C$ Particulars
C
C     Let S1 be a state vector with position and velocity components P
C     and V respectively.  From these components one can compute the
C     unit vector parallel to P, call it U and the derivative of U
C     with respect to time, DU.  This pair (U,DU) is the state returned
C     by this routine in SOUT.
C
C$ Examples
C
C     Suppose that STATE gives the apparent state of a body with
C     respect to an observer.  This routine can be used to compute the
C     instantaneous angular rate of the object across the sky as seen
C     from the observers vantage.
C
C        INTEGER               POS
C        PARAMETER           ( POS = 1 )
C
C        INTEGER               VEL
C        PARAMETER           ( VEL = 4 )
C
C        CALL DVHAT ( STATE, USTATE )
C
C        ANGULAR_RATE = VNORM ( USTATE(VEL) )
C
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C       W.L. Taber      (JPL)
C
C$ Literature_References
C
C       None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 15-JUN-1995 (WLT)
C
C-&
 
 
C$ Index_Entries
C
C     State of a unit vector parallel to a state vector
C
C-&
C
 
 
      INTEGER               POS
      PARAMETER           ( POS = 1 )
 
      INTEGER               VEL
      PARAMETER           ( VEL = 4 )
 
      DOUBLE PRECISION      LENGTH
 
C
C     Get the position portion of the output state and the length of
C     the input position.
C
      CALL UNORM ( S1(POS), SOUT(POS), LENGTH )
 
      IF ( LENGTH .EQ. 0.0D0 ) THEN
 
C
C        If the length of the input position is zero, just copy
C        the input velocity to the output velocity.
C
         CALL VEQU ( S1(VEL), SOUT(VEL) )
 
      ELSE
 
C
C        Otherwise the derivative of the unit vector is just the
C        component of the input velocity perpendicular to the input
C        position, scaled by the reciprocal of the length of the
C        input position.
C
 
         CALL VPERP ( S1(VEL),      SOUT(POS), SOUT(VEL) )
         CALL VSCL  ( 1.0D0/LENGTH, SOUT(VEL), SOUT(VEL) )
 
      END IF
 
      RETURN
      END
 
 
 
 
