C$Procedure      ET2LST ( ET to Local Solar Time )
 
      SUBROUTINE ET2LST ( ET, BODY, LONG, TYPE, HR, MN, SC, TIME, AMPM )
 
C$ Abstract
C
C    Given an ephemeris epoch ET, compute the local solar time for
C    an object on the surface of a body at a specified longitude.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     TIME.REQ.
C
C$ Keywords
C
C     TIME
C
C$ Declarations
 
      IMPLICIT NONE
      DOUBLE PRECISION      ET
      INTEGER               BODY
      DOUBLE PRECISION      LONG
      CHARACTER*(*)         TYPE
      INTEGER               HR
      INTEGER               MN
      INTEGER               SC
      CHARACTER*(*)         TIME
      CHARACTER*(*)         AMPM
 
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     ET         I   Epoch in seconds past J2000 epoch
C     BODY       I   ID-code of the body of interest
C     LONG       I   Longitude of surface point (RADIANS)
C     TYPE       I   Type of longitude 'PLANETOCENTRIC', etc.
C     HR         O   Local hour on a "24 hour" clock
C     MN         O   Minutes past the hour
C     SC         O   Seconds past the minute
C     TIME       O   String giving local time on 24 hour clock
C     AMPM       O   String giving time on A.M./ P.M. scale
C
C$ Detailed_Input
C
C     ET         is the epoch expressed in TDB seconds past
C                the J2000 epoch at which a local time is desired.
C
C     BODY       is the NAIF ID-code of a body on which local
C                time is to be measured.
C
C     LONG       is the longitude (either planetocentric or
C                planetographic) in radians of the site on the
C                surface of body for which local time should be
C                computed.
C
C     TYPE       is the form of longitude supplied by the variable
C                LONG.  Allowed values are 'PLANETOCENTRIC' and
C                'PLANETOGRAPHIC'.  Note the case of the letters
C                in TYPE is insignificant.  Both 'PLANETOCENTRIC'
C                and 'planetocentric' are recognized.
C
C$ Detailed_Output
C
C     HR         is the local "hour" of the site specified at the
C                epoch ET. Note that an "hour" of local time does not
C                have the same duration as an hour measured by
C                conventional clocks.  It is simply a representation
C                of an angle.  See C$ Particulars for a more
C                complete discussion of the meaning of local time.
C
C     MN         is the number of "minutes" past the hour of the
C                local time of the site at the epoch ET. Again note
C                that a "local minute" is not the same as a minute
C                you would measure with conventional clocks.
C
C     SC         is the number of "seconds" past the minute of the
C                local time of the site at the epoch ET.  Again note
C                that a "local second" is not the same as a second
C                you would measure with conventional clocks.
C
C     TIME       is a string expressing the local time
C                on a "24 hour" local clock.
C
C     AMPM       is a string expressing the local time on a "12 hour"
C                local clock together with the traditional AM/PM
C                label to indicate whether the sun has crossed
C                the local zenith meridian.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     Suitable SPK and PCK files must be loaded prior to calling
C     this routine so that the bodyfixed position of the sun
C     relative to BODY can be computed.
C
C$ Exceptions
C
C     1) This routine defines local solar time for any point on the
C        surface of the Sun to be 12:00:00 noon.
C
C     2) If the TYPE of the coordinates is not recognized, the
C        error 'SPICE(UNKNOWNSYSTEM)' will be signalled.
C
C     3) If the bodyfixed frame to associate with BODY cannot be
C        determined, the error 'SPICE(CANTFINDFRAME)' is signalled.
C
C     4) If insufficient data is available to compute the
C        location of the sun in bodyfixed coordinates, the
C        error will be diagnosed by a routine called by this one.
C
C$ Particulars
C
C     This routine returns the local solar time at a user
C     specified location on a user specified body.
C
C     Let SUNLNG be the planetocentric longitude (in degrees) of
C     the sun as viewed from the center of the body of interest.
C
C     Let SITLNG be the planetocentric longitude (in degrees) of
C     the site for which local time is desired.
C
C     We define local time to be 12 + (SITLNG - SUNLNG)/15
C
C     (where appropriate care is taken to map ( SITLNG - SUNLNG )
C     into the range from -180 to 180).
C
C     Using this definition, we see that from the point of view
C     of this routine, local solar time is simply a measure of angles
C     between meridians on the surface of a body.  Consequently,
C     this routine is not appropriate for computing "local times"
C     in the sense of Pacific Standard Time.   For computing times
C     relative to standard time zones on earth, see the routines
C     TIMOUT and STR2ET.
C
C$ Examples
C
C     The following code fragment illustrates how you
C     could print the local time at a site on Mars with
C     planetographic longitude 326.17 deg E at epoch ET.
C
C     (This example assumes all required SPK and PCK files have
C     been loaded).
C
C     Convert the longitude to radians, set the type of the longitude
C     and make up a mnemonic for MARS's ID-code.
C
C     LONG = 326.17 * RPD()
C     TYPE = 'PLANETOGRAPHIC'
C     MARS = 499
C
C     CALL ET2LST ( ET, MARS, LONG, TYPE, HR, MN, SC, TIME, AMPM )
C
C     WRITE (*,*) 'The local time at Mars 326.17 degrees E '
C     WRITE (*,*) 'planetographic longitude is: ', AMPM
C
C$ Restrictions
C
C     This routine relies on being able to determine the name
C     of the bodyfixed frame associated with BODY through the
C     frames subsystem.  If the BODY specified is NOT one of the
C     nine planets or their satellites, you will need to load
C     an appropriate frame definition kernel that contains
C     the relationship between the body id and the bodyfixed frame
C     name.  See the FRAMES required reading for more details
C     on specifying this relationship.
C
C$ Author_and_Institution
C
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 24-MAR-1998 (WLT)
C
C        The integer variable SUN was never initialized in the
C        previous version of the routine.  Now it is set to
C        the proper value of 10.
C
C-    SPICELIB Version 1.0.0, 9-JUL-1997 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Compute the local time for a point on a body.
C
C-&
C
C     SPICELIB Functions
C
      DOUBLE PRECISION      BRCKTD
      DOUBLE PRECISION      PI
      DOUBLE PRECISION      TWOPI
 
      INTEGER               PLNSNS
 
      LOGICAL               RETURN
 
C
C     Local Variables.
C
 
      INTEGER               WDSIZE
      PARAMETER           ( WDSIZE = 32 )
 
 
 
      CHARACTER*(2)         H
      CHARACTER*(2)         M
      CHARACTER*(2)         S
      CHARACTER*(4)         AMORPM
      CHARACTER*(WDSIZE)    FRAME
      CHARACTER*(WDSIZE)    MYTYPE
 
      DOUBLE PRECISION      ANGLE
      DOUBLE PRECISION      FACTOR
      DOUBLE PRECISION      HOURS
      DOUBLE PRECISION      LT
      DOUBLE PRECISION      MINS
      DOUBLE PRECISION      MYLONG
      DOUBLE PRECISION      RANGE
      DOUBLE PRECISION      Q
      DOUBLE PRECISION      SECNDS
      DOUBLE PRECISION      SLAT
      DOUBLE PRECISION      SLONG
      DOUBLE PRECISION      STATE ( 6 )
 
      INTEGER               FRCODE
      INTEGER               HRAMPM
      INTEGER               SUN
 
      LOGICAL               FOUND
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN ( 'ET2LST')
 
      SUN = 10
 
      CALL LJUST ( TYPE,   MYTYPE )
      CALL UCASE ( MYTYPE, MYTYPE )
 
      IF ( MYTYPE .EQ. 'PLANETOGRAPHIC' ) THEN
 
         FACTOR = DBLE( PLNSNS(BODY) )
         MYLONG = FACTOR*LONG
 
      ELSE IF ( MYTYPE .EQ. 'PLANETOCENTRIC' ) THEN
 
         MYLONG = LONG
 
      ELSE
 
         CALL SETMSG ( 'The coordinate system ''#'' is not a '
     .   //            'recognized system of longitude.  The '
     .   //            'recognized systems are '
     .   //            '''PLANETOCENTRIC'' and '
     .   //            '''PLANETOGRAPHIC''. ' )
 
         CALL ERRCH  ( '#', TYPE )
         CALL SIGERR ( 'SPICE(UNKNOWNSYSTEM)'  )
         CALL CHKOUT ( 'ET2LST' )
         RETURN
 
      END IF
C
C     It's always noon on the surface of the sun.
C
      IF ( BODY .EQ. 10 ) THEN
 
         HR = 12
         MN = 0
         SC = 0
 
         TIME = '12:00:00'
         AMPM = '12:00:00 P.M.'
 
         CALL CHKOUT ( 'ET2LST' )
         RETURN
 
      END IF
 
C
C     Get the bodyfixed position of the sun.
C
      CALL CIDFRM ( BODY, FRCODE, FRAME, FOUND )
 
      IF ( .NOT. FOUND ) THEN
 
         CALL SETMSG ( 'The bodyfixed frame associated with '
     .   //            'body # could not be determined.  This '
     .   //            'information needs to be "loaded" via a '
     .   //            'frames definition kernel.  See '
     .   //            'FRAMES.REQ for more details. ' )
         CALL ERRINT ( '#', BODY )
         CALL SIGERR ( 'SPICE(CANTFINDFRAME)'  )
         CALL CHKOUT ( 'ET2LST' )
         RETURN
 
      END IF
 
 
 
      CALL SPKEZ  (  SUN,   ET,    FRAME, 'LT+S', BODY, STATE, LT )
      CALL RECLAT (  STATE, RANGE, SLONG,  SLAT )
 
      ANGLE = MYLONG - SLONG
 
C
C     Force the angle into the region from -PI to PI
C
      CALL RMAIND ( ANGLE, TWOPI(), Q, ANGLE )
 
      IF ( ANGLE .GT. PI() ) THEN
         ANGLE = ANGLE - TWOPI()
      END IF
C
C     Convert the angle to "angle seconds" before or after local noon.
C
      SECNDS = 86400.0D0 * ANGLE / TWOPI()
      SECNDS = BRCKTD ( SECNDS, -43200.0D0, 43200.0D0 )
 
C
C     Get the hour, and minutes components of the local time.
C
      CALL RMAIND ( SECNDS, 3600.0D0, HOURS, SECNDS )
      CALL RMAIND ( SECNDS,   60.0D0, MINS,  SECNDS )
C
C     Construct the integer components of the local time.
C
      HR = 12 + INT(HOURS)
      MN =      INT(MINS)
      SC =      INT(SECNDS)
C
C     Set the A.M./P.M. components of local time.
C
      IF      ( HR .EQ. 24 ) THEN
 
         HR     = 0
         HRAMPM = 12
         AMORPM = 'A.M.'
 
      ELSE IF ( HR .GT. 12 ) THEN
 
         HRAMPM = HR - 12
         AMORPM = 'P.M.'
 
      ELSE IF ( HR .EQ. 12 ) THEN
 
         HRAMPM = 12
         AMORPM = 'P.M.'
 
      ELSE IF ( HR .EQ. 0 ) THEN
 
         HRAMPM = 12
         AMORPM = 'A.M.'
 
      ELSE
 
         HRAMPM = HR
         AMORPM = 'A.M.'
 
      END IF
 
C
C     Now construct the two strings we need.
C
      HOURS  = DBLE(HR)
      MINS   = DBLE(MN)
      SECNDS = DBLE(SC)
 
      CALL DPFMT ( HOURS,  '0x', H )
      CALL DPFMT ( MINS,   '0x', M )
      CALL DPFMT ( SECNDS, '0x', S )
 
      TIME = H // ':' // M // ':' // S
 
 
      HOURS = DBLE(HRAMPM)
 
      CALL DPFMT ( HOURS,  '0x', H )
 
      AMPM = H // ':' // M // ':' // S //  ' ' // AMORPM
 
      CALL CHKOUT ( 'ET2LST' )
      RETURN
      END
