C$Procedure      INELPL ( Intersection of ellipse and plane )
 
      SUBROUTINE INELPL ( ELLIPS, PLANE, NXPTS, XPT1, XPT2 )
 
C$ Abstract
C
C     Find the intersection of an ellipse and a plane.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     ELLIPSES
C     PLANES
C
C$ Keywords
C
C     ELLIPSE
C     GEOMETRY
C     MATH
C
C$ Declarations
 
      INTEGER               UBEL
      PARAMETER           ( UBEL = 9 )
 
      INTEGER               UBPL
      PARAMETER           ( UBPL = 4 )
 
      DOUBLE PRECISION      ELLIPS ( UBEL )
      DOUBLE PRECISION      PLANE  ( UBPL )
      INTEGER               NXPTS
      DOUBLE PRECISION      XPT1   ( 3 )
      DOUBLE PRECISION      XPT2   ( 3 )
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     ELLIPS     I   A SPICELIB ellipse.
C     PLANE      I   A SPICELIB plane.
C     NXPTS      O   Number of intersection points of plane and ellipse.
C     XPT1,
C     XPT2       O   Intersection points.
C
C$ Detailed_Input
C
C     ELLIPS         is a SPICELIB ellipse.
C
C     PLANE          is a SPICELIB plane.
C
C$ Detailed_Output
C
C     NXPTS          is the number of points of intersection of the
C                    geometric plane and ellipse represented by PLANE
C                    and ELLIPS. NXPTS may take the values 0, 1, 2 or
C                    -1.  The value -1 indicates that the ellipse lies
C                    in the plane, so the number of intersection points
C                    is infinite.
C
C     XPT1,
C     XPT2           are the points of intersection of the input plane
C                    and ellipse.  If there is only one intersection
C                    point, both XPT1 and XPT2 contain that point.  If
C                    the number of intersection points is zero or
C                    infinite, the contents of XPT1 and XPT2 are
C                    undefined.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1)  If the input ellipse is invalid, the error will be diagnosed
C         by routines called by this routine.
C
C     2)  If the input plane is invalid, the error will be diagnosed by
C         routines called by this routine.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     A plane may intersect an ellipse in 0, 1, 2, or infinitely many
C     points.  To get lots of intersection points, the ellipse must
C     lie in the plane.
C
C$ Examples
C
C     1)  If we want to find the angle of some ray above the limb of an
C         ellipsoid, where the angle is measured in a plane containing
C         the ray and a `down' vector, we can follow the procedure
C         given below.  We assume the ray does not intersect the
C         ellipsoid.  The result we seek is called ANGLE, imaginatively
C         enough.
C
C         We assume that all vectors are given in body-fixed
C         coordinates.
C
C            C
C            C     Find the limb of the ellipsoid as seen from the
C            C     point OBSERV.  Here A, B, and C are the lengths of
C            C     the semi-axes of the ellipsoid.
C            C
C                  CALL EDLIMB ( A, B, C, OBSERV, LIMB )
C
C            C
C            C     The ray direction vector is RAYDIR, so the ray is the
C            C     set of points
C            C
C            C        OBSERV  +  t * RAYDIR
C            C
C            C     where t is any non-negative real number.
C            C
C            C     The `down' vector is just -OBSERV.  The vectors
C            C     OBSERV and RAYDIR are spanning vectors for the plane
C            C     we're interested in.  We can use PSV2PL to represent
C            C     this plane by a SPICELIB plane.
C            C
C                  CALL PSV2PL ( OBSERV, OBSERV, RAYDIR, PLANE )
C
C            C
C            C     Find the intersection of the plane defined by OBSERV
C            C     and RAYDIR with the limb.
C            C
C                  CALL INELPL ( LIMB, PLANE, NXPTS, XPT1, XPT2 )
C
C            C
C            C     We always expect two intersection points, if DOWN
C            C     is valid.
C            C
C                  IF ( NXPTS .LT. 2 ) THEN
C
C                     [ do something about the error ]
C
C                  ENDIF
C
C            C
C            C     Form the vectors from OBSERV to the intersection
C            C     points.  Find the angular separation between the
C            C     boresight ray and each vector from OBSERV to the
C            C     intersection points.
C            C
C                  CALL VSUB   ( XPT1, OBSERV, VEC1 )
C                  CALL VSUB   ( XPT2, OBSERV, VEC2 )
C
C                  SEP1 = VSEP ( VEC1, RAYDIR )
C                  SEP2 = VSEP ( VEC2, RAYDIR )
C
C            C
C            C     The angular separation we're after is the minimum of
C            C     the two separations we've computed.
C            C
C                  ANGLE = MIN ( SEP1, SEP2 )
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman   (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 24-MAR-1992 (NJB) (WLT)
C
C        Output arguments XPT1, XPT2 are now correctly declared
C        with length 3.  Comment section for permuted index source
C        lines was added following the header.
C
C-    SPICELIB Version 1.0.0, 02-NOV-1990 (NJB)
C
C-&
 
C$ Index_Entries
C
C     intersection of ellipse and plane
C
C-&
 
C$ Revisions
C
C-    SPICELIB Version 1.1.0, 24-MAR-1992 (NJB) (WLT)
C
C        Output arguments XPT1, XPT2 are now correctly declared
C        with length 3.  They formerly were declared as scalars.
C        The correction will not affect the behavior of the routine
C        in programs that already declared the correponding arguments
C        correctly.
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-&
 
 
C
C     SPICELIB functions
C
      DOUBLE PRECISION      VDOT
      DOUBLE PRECISION      VNORMG
 
      LOGICAL               RETURN
      LOGICAL               VZEROG
 
 
C
C     Local variables
C
      DOUBLE PRECISION      ALPHA
      DOUBLE PRECISION      ANGLE1
      DOUBLE PRECISION      ANGLE2
      DOUBLE PRECISION      BETA
      DOUBLE PRECISION      CENTER (    3 )
      DOUBLE PRECISION      CONST
      DOUBLE PRECISION      NORMAL (    3 )
      DOUBLE PRECISION      POINT  (    3 )
      DOUBLE PRECISION      SMAJOR (    3 )
      DOUBLE PRECISION      SMINOR (    3 )
      DOUBLE PRECISION      TRANS  ( UBPL )
      DOUBLE PRECISION      V      (    2 )
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'INELPL' )
      END IF
 
C
C     The first thing we want to do is translate the plane and the
C     ellipse so as to center the ellipse at the origin.  To translate
C     the plane, just get a point and normal vector, and translate
C     the point.  Find the plane constant of the translated plane.
C
      CALL EL2CGV ( ELLIPS, CENTER, SMAJOR, SMINOR )
 
      CALL PL2NVP ( PLANE,  NORMAL, POINT )
      CALL VSUB   ( POINT,  CENTER, POINT )
      CALL NVP2PL ( NORMAL, POINT,  TRANS )
 
      CALL PL2NVC ( TRANS,  NORMAL, CONST )
 
C
C        Ok, we can get to work.  The locus of the ellipse is
C
C           cos(theta) SMAJOR  +  sin(theta) SMINOR,
C
C        and any point X of the ellipse that intersects the input plane
C        satisfies
C
C           < X, NORMAL >  =  CONST.
C
C        Substituting our expression for points on the ellipse into the
C        second equation, we arrive at
C
C              cos(theta) < SMAJOR, NORMAL >
C           +  sin(theta) < SMINOR, NORMAL >   =  CONST.        (1)
C
C        This equation merits a little analysis.  First, if NORMAL
C        is orthogonal to SMAJOR and SMINOR, the plane and ellipse must
C        be parallel.  Also, the left side of the equation is zero in
C        this case.   If CONST is non-zero, there are no solutions:
C        the ellipse and plane are parallel but do not intersect.  If
C        CONST is zero, the ellipse lies in the plane:  all values of
C        theta are solutions.  Let's get this case out of the way
C        right now, shall we?
C
      V(1) = VDOT ( SMAJOR, NORMAL )
      V(2) = VDOT ( SMINOR, NORMAL )
 
C
C     Test whether the plane and ellipse are parallel:
C
      IF (  VZEROG( V, 2 )  ) THEN
C
C        The ellipse lies in the plane if and only if CONST is zero.
C        In any case, we don't modify XPT1 or XPT2.
C
         IF ( CONST .EQ. 0.D0 ) THEN
            NXPTS = -1
         ELSE
            NXPTS = 0
         END IF
 
         CALL CHKOUT ( 'INELPL' )
         RETURN
 
      END IF
 
C
C        Now if NORMAL is not orthogonal to both SMAJOR and SMINOR,
C        the vector
C
C           V = (  < SMAJOR, NORMAL >,  < SMINOR, NORMAL >  )
C
C        is non-zero.  We can re-write (1) as
C
C           < U, V >  =  CONST,
C
C        where
C
C           U = ( cos(theta), sin(theta) ).
C
C        If alpha is the angle between U and V, we have
C
C           < U, V >  =  || U ||  *  || V ||  *  cos(alpha),
C
C        so
C
C           || V ||  *  cos(alpha)  =  CONST.                   (2)
C
C        This equation has solutions if and only if
C
C           || V ||  >    CONST.                                (3)
C                    -
C
C        CONST is positive, since PL2NVC returns a the distance
C        of between its input plane and the origin as the output
C        plane constant.
C
C        Let's return right now if there are no solutions.
C
 
      IF (  VNORMG ( V, 2 )  .LT.  CONST  )   THEN
 
         NXPTS = 0
         CALL CHKOUT ( 'INELPL' )
         RETURN
 
      END IF
 
C
C        Since (3) above is satisfied, the plane and ellipse intersect.
C        We can find alpha by the formula
C
C           alpha  =  +  arccos (  CONST  /  || V ||  )
C
C        Since alpha is the angular separation between U and V, we
C        can find U once we have the angular position of V; let's
C        call that beta.  The angular position of U (which we called
C        theta earlier) will be
C
C           theta   =   beta  +  alpha.
C                             -
C
C        The values of theta are the angles we seek.
C
 
      ALPHA   =  ACOS  (  CONST  /  VNORMG ( V, 2 )  )
 
      BETA    =  ATAN2 ( V(2), V(1) )
 
      ANGLE1  =  BETA - ALPHA
      ANGLE2  =  BETA + ALPHA
 
      IF ( ANGLE1 .EQ. ANGLE2 ) THEN
         NXPTS = 1
      ELSE
         NXPTS = 2
      END IF
 
C
C     Compute the intersection points.
C
      CALL VLCOM3 ( 1.0D0,         CENTER,
     .              DCOS(ANGLE1),  SMAJOR,
     .              DSIN(ANGLE1),  SMINOR,    XPT1 )
 
      CALL VLCOM3 ( 1.0D0,         CENTER,
     .              DCOS(ANGLE2),  SMAJOR,
     .              DSIN(ANGLE2),  SMINOR,    XPT2 )
 
 
      CALL CHKOUT ( 'INELPL' )
      RETURN
      END
