C$Procedure      LATCYL ( Latitudinal to cylindrical coordinates )
 
      SUBROUTINE LATCYL ( RADIUS, LONG, LAT, R, LONGC, Z )
 
C$ Abstract
C
C     Convert from latitudinal coordinates to cylindrical coordinates.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C      CONVERSION,  COORDINATES
C
C$ Declarations
 
      DOUBLE PRECISION   RADIUS
      DOUBLE PRECISION   LONG
      DOUBLE PRECISION   LAT
      DOUBLE PRECISION   R
      DOUBLE PRECISION   LONGC
      DOUBLE PRECISION   Z
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      RADIUS     I   Distance of a point from the origin.
C      LONG       I   Angle of the point from the XZ plane in radians.
C      LAT        I   Angle of the point from the XY plane in radians.
C      R          O   Distance of the point from the Z axis.
C      LONGC      O   Angle of the point from the XZ plane in radians.
C      Z          O   Height of the point above the XY plane.
C
C$ Detailed_Input
C
C      RADIUS     Distance of a point from the origin.
C
C      LONG       Angle of the point from the XZ plane in radians.
C
C      LAT        Angle of the point from the XY plane in radians.
C
C$ Detailed_Output
C
C      R          Distance of the point from the Z axis.
C
C      LONGC      Angle of the point from the XZ plane in radians.
C
C      Z          Height of the point above the XY plane.
C
C$ Parameters
C
C      None.
C
C$ Particulars
C
C      This routine returns the cylindrical coordinates of a point
C      whose position is input in latitudinal coordinates.
C
C      Latitudinal coordinates are defined by a distance from a central
C      reference point, an angle from a reference meridian, and an angle
C      above the equator of a sphere centered at the central reference
C      point.
C
C$ Examples
C
C     Other than the obvious conversion between coordinate systems
C     this routine could be used to obtain the axial projection
C     from a sphere to a cylinder about the z-axis that contains
C     the equator of the sphere.  The following code fragment
C     illustrates this idea.
C
C           CALL LATCYL ( RADIUS, LONG, LAT, R, LONG, Z )
C           R = RADIUS
C
C     R, LONG, and Z now contain the the coordinates of the projected
C     point. Such a projection is valuable because it preserves the
C     areas between regions on the sphere and their projections to the
C     cylinder.
C
C
C$ Restrictions
C
C      None.
C
C$ Exceptions
C
C     Error free.
C
C$ Files
C
C      None.
C
C$ Author_and_Institution
C
C      W.L. Taber      (JPL)
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (WLT)
C
C-&
 
C$ Index_Entries
C
C     latitudinal to cylindrical coordinates
C
C-&
 
 
C$ Revisions
C
C-     Beta Version 1.0.1, 1-Feb-1989 (WLT)
C
C      Example section of header upgraded.
C
C-&
 
 
C
C     Local variables
C
      DOUBLE PRECISION RH
      DOUBLE PRECISION ZZ
 
C
C     Convert to cylindrical, storing in temporary variables
C
      RH = RADIUS * DCOS(LAT)
      ZZ = RADIUS * DSIN(LAT)
 
C
C     Move the results to output variables.
C
      LONGC = LONG
      R      = RH
      Z      = ZZ
C
      RETURN
      END
