C$Procedure      LNKNFN ( LNK, number of free nodes )
 
      INTEGER FUNCTION LNKNFN ( POOL )
 
C$ Abstract
C
C     Return the number of free nodes in a doubly linked list pool.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     LNK
C
C$ Keywords
C
C     LIST
C
C$ Declarations
 
      IMPLICIT NONE
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      INTEGER               POOL ( 2,  LBPOOL : * )
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     POOL       I   A doubly linked list pool.
C     LBPOOL     P   Lower bound of pool column indices.
C
C     The function returns the number of free nodes in the pool.
C
C$ Detailed_Input
C
C     SIZE           is the number of nodes in the pool.
C
C     POOL           is a doubly linked list pool.
C
C$ Detailed_Output
C
C     The function returns the number of free nodes in the pool.
C
C$ Parameters
C
C     LBPOOL        is the lower bound of the column indices of the POOL
C                   array.  The columns indexed LBPOOL to 0 are reserved
C                   as a control area for the pool.
C
C$ Exceptions
C
C     Error free.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     This routine allows the caller to find the number of free nodes
C     available in a doubly linked list pool, without having to make
C     use of knowledge of the internal structure of the pool.
C
C     Routines that allocate nodes can use this routine to determine
C     how many nodes can be allocated safely---an attempt to allocate
C     a node when no free nodes are available causes a SPICELIB error
C     to be signalled.
C
C$ Examples
C
C     1)  Let POOL be a doubly linked list pool containing 5 nodes.
C         If POOL contains the list
C
C            4 <--> 5 <--> 1 <--> 2
C
C
C         and the node 3 is unallocated, then the function reference
C
C            NFREE  =  LNKNFN ( POOL )
C
C
C         will assign the value 1 to NFREE.
C
C
C$ Restrictions
C
C     Linked list pools must be initialized via the routine
C     LNKINI.  Failure to initialize a linked list pool
C     will almost certainly lead to confusing results.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman   (JPL)
C     W.L. Taber     (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 19-DEC-1995 (NJB) (WLT)
C
C-&
 
C$ Index_Entries
C
C     return number of nodes in linked list pool
C
C-&
 
 
C
C     Local parameters
C
 
C
C     The control area contains 3 elements.  They are:
C
C        The "size" of the pool, that is, the number
C        of nodes in the pool.
C
C        The number of free nodes in the pool.
C
C        The "free pointer," which is the column index of the first free
C        node.
C
C     Parameters defining the row and column indices of these control
C     elements are given below.
C
      INTEGER               SIZROW
      PARAMETER           ( SIZROW =   1 )
 
      INTEGER               SIZCOL
      PARAMETER           ( SIZCOL =   0 )
 
      INTEGER               NFRROW
      PARAMETER           ( NFRROW =   2 )
 
      INTEGER               NFRCOL
      PARAMETER           ( NFRCOL =   0 )
 
      INTEGER               FREROW
      PARAMETER           ( FREROW =   1 )
 
      INTEGER               FRECOL
      PARAMETER           ( FRECOL =  -1 )
 
C
C     Each assigned node consists of a backward pointer and a forward
C     pointer.
C
C        +-------------+       +-------------+       +-------------+
C        |  forward--> |       |  forward--> |       |  forward--> |
C        +-------------+  ...  +-------------+  ...  +-------------+
C        | <--backward |       | <--backward |       | <--backward |
C        +-------------+       +-------------+       +-------------+
C
C            node 1                 node I              node SIZE
C
C
C
      INTEGER               FORWRD
      PARAMETER           ( FORWRD  = 1 )
 
      INTEGER               BCKWRD
      PARAMETER           ( BCKWRD  = 2 )
 
C
C     Free nodes say that that's what they are.  The way they say it
C     is by containing the value FREE in their backward pointers.
C     Needless to say, FREE is a value that cannot be a valid pointer.
C
      INTEGER               FREE
      PARAMETER           ( FREE   = 0 )
 
 
 
 
C
C     Grab the number of free nodes from the control area.
C
      LNKNFN  =  POOL ( NFRROW, NFRCOL )
 
      RETURN
      END
