C$Procedure      LPARSM ( Parse a list of items )
 
      SUBROUTINE LPARSM ( LIST, DELIMS, NMAX, N, ITEMS )
 
C$ Abstract
C
C      Parse a list of items separated by multiple delimiters.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C      CHARACTER,  LIST,  PARSING,  STRING
C
C$ Declarations
 
      CHARACTER*(*)    LIST
      CHARACTER*(*)    DELIMS
      INTEGER          NMAX
      INTEGER          N
      CHARACTER*(*)    ITEMS ( * )
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      LIST       I    List of items delimited by DELIMS.
C      DELIMS     I    Single characters which delimit items.
C      NMAX       I    Maximum number of items to return.
C      N          O    Number of items in the list.
C      ITEMS      O    Items in the list, left justified.
C
C$ Detailed_Input
C
C      LIST        is a list of items delimited by any one of the
C                  characters in the string DELIMS. Consecutive
C                  delimiters, and delimiters at the beginning and
C                  end of the list, are considered to delimit blank
C                  items. A blank list is considered to contain
C                  a single (blank) item.
C
C      DELIMS      contains the individual characters which delimit
C                  the items in the list. These may be any ASCII
C                  characters, including blanks.
C
C                  However, by definition, consecutive blanks are NOT
C                  considered to be consecutive delimiters. Nor are
C                  a blank and any other delimiter considered to be
C                  consecutive delimiters. In addition, leading and
C                  trailing blanks are ignored.
C
C      NMAX        is the maximum number of items to be returned from
C                  the list. This allows the user to guard against
C                  overflow from a list containing more items than
C                  expected.
C
C$ Detailed_Output
C
C      N           is the number of items in the list. N may be
C                  any number between one and NMAX. N is always the
C                  number of delimiters plus one.
C
C      ITEMS       are the items in the list, left justified. Any item
C                  in the list to long to fit into an element of ITEMS
C                  is truncated on the right.
C
C$ Parameters
C
C     None.
C
C$ Particulars
C
C      None.
C
C$ Examples
C
C      The following examples illustrate the operation of LPARSM.
C
C      1) Let
C               LIST   = '  A number of words   separated   by spaces
C               DELIMS = ' '
C               NMAX   = 20
C
C         Then
C               ITEMS(1) = 'A'
C               ITEMS(2) = 'number'
C               ITEMS(3) = 'of'
C               ITEMS(4) = 'words'
C               ITEMS(5) = 'separated'
C               ITEMS(6) = 'by'
C               ITEMS(7) = 'spaces'
C
C      2) Let
C               LIST   = '  1986-187// 13:15:12.184 '
C               DELIMS = ' ,/-:'
C               NMAX   = 20
C
C         Then
C               ITEMS(1) = '1986'
C               ITEMS(2) = '187'
C               ITEMS(3) = ' '
C               ITEMS(4) = '13'
C               ITEMS(5) = '15'
C               ITEMS(6) = '12.184'
C
C$ Restrictions
C
C      None.
C
C$ Exceptions
C
C      Error free.
C
C$ Files
C
C      None.
C
C$ Author_and_Institution
C
C      I.M. Underwood  (JPL)
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     parse a list of items
C
C-&
 
 
 
 
C
C     Local variables
C
      CHARACTER*1      BLANK
      PARAMETER      ( BLANK = ' ' )
 
      INTEGER          B
      INTEGER          E
      INTEGER          EOL
 
 
 
C
C     Because speed is essential in many list parsing applications,
C     LPARSM parses the input list in a single pass. What follows
C     is nearly identical to LPARSE, except the Fortran INDEX function
C     is used to test for delimiters, instead of testing each character
C     for simple equality.
C
 
C
C     Nothing yet.
C
      N = 0
 
C
C     Blank list contains a blank item.
C
      IF ( LIST .EQ. BLANK ) THEN
 
         N        = 1
         ITEMS(1) = BLANK
 
      ELSE
 
C
C        Eliminate trailing blanks. EOL is the last non-blank
C        character in the list.
C
         EOL = LEN ( LIST )
 
         DO WHILE ( LIST(EOL:EOL) .EQ. BLANK )
            EOL = EOL - 1
         END DO
 
C
C        As the King said to Alice: 'Begin at the beginning.
C        Continue until you reach the end. Then stop.'
C
C        When searching for items, B is the beginning of the
C        current item; E is the end.
C
         B = 1
 
         DO WHILE ( B .LE. EOL )
 
C
C           Skip any blanks before the next item or delimiter.
C
            DO WHILE (      ( B         .LE. EOL   )
     .                .AND. ( LIST(B:B) .EQ. BLANK ) )
               B = B + 1
            END DO
 
C
C           The item ends at the next delimiter.
C
            E = B
            DO WHILE (      ( E                           .LE. EOL )
     .                .AND. ( INDEX ( DELIMS, LIST(E:E) ) .EQ. 0   ) )
               E = E + 1
            END DO
 
C
C           (This is different from LPARSE. If the delimiter was
C           a blank, find the next non-blank character. If it's not
C           a delimiter, back up. This prevents constructions
C           like 'a , b', where the delimiters are blank and comma,
C           from being interpreted as three items instead of two.
C           By definition, consecutive blanks, or a blank and any
C           other delimiter, do not count as consecutive delimiters.)
C
            IF ( E .LE. EOL .AND. LIST(E:E) .EQ. ' ' ) THEN
 
               DO WHILE (      ( E         .LE. EOL   )
     .                   .AND. ( LIST(E:E) .EQ. BLANK ) )
                  E = E + 1
               END DO
 
               IF ( INDEX ( DELIMS, LIST(E:E) ) .EQ. 0 ) THEN
                  E = E - 1
               END IF
 
            END IF
 
C
C           The item now lies between B and E. Unless, of course,
C           B and E are the same character, in which case we have
C           stumbled upon consecutive delimiters.
C
            N = N + 1
 
            IF ( E .GT. B ) THEN
               ITEMS(N) = LIST(B:E-1)
            ELSE
               ITEMS(N) = ' '
            END IF
 
C
C           If there are more items to be found, continue with
C           character following E (which is a delimiter).
C
            IF ( N .LT. NMAX ) THEN
               B = E + 1
            ELSE
               RETURN
            END IF
 
         END DO
 
C
C        If the list ended with a (non-blank) delimiter, add a
C        blank item to the end.
C
         IF (        INDEX ( DELIMS, LIST(EOL:EOL) ) .NE. 0
     .        .AND.  N                               .LT. NMAX ) THEN
 
            N        = N + 1
            ITEMS(N) = ' '
 
         END IF
 
      END IF
 
      RETURN
      END
