C$Procedure      M2Q ( Matrix to quaternion )

      SUBROUTINE M2Q ( R, Q )

C$ Abstract
C
C     Find a unit quaternion corresponding to a specified rotation
C     matrix.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     ROTATION
C
C$ Keywords
C
C     MATH
C     MATRIX
C     ROTATION
C
C$ Declarations
 
      DOUBLE PRECISION      R ( 3,  3 )
      DOUBLE PRECISION      Q ( 0 : 3 )
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     R          I   A rotation matrix.
C     Q          O   A unit quaternion representing R.
C
C$ Detailed_Input
C
C     R              is a rotation matrix.
C
C$ Detailed_Output
C
C     Q              is a unit quaternion representing R.  Q is a
C                    4-dimensional vector.  If R rotates vectors by an
C                    angle of r radians about a unit vector A, where
C                    r is in [0, pi], then if h = r/2,
C
C                       Q = ( cos(h), sin(h)A ,  sin(h)A ,  sin(h)A ).
C                                            1          2          3
C
C                    The restriction that r must be in the range [0, pi]
C                    determines the output quaternion Q uniquely
C                    except when r = pi; in this special case, both of
C                    the quaternions
C
C                       Q = ( 0,  A ,  A ,  A  )
C                                  1    2    3
C                    and
C
C                       Q = ( 0, -A , -A , -A  )
C                                  1    2    3
C
C                   are possible outputs, if A is a choice of rotation
C                   axis for R.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1)   If R is not a rotation matrix, the error SPICE(NOTAROTATION)
C          is signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     A unit quaternion is a 4-dimensional vector for which the sum of
C     the squares of the components is 1.  Unit quaternions can be used
C     to represent rotations in the following way:  Given a rotation
C     angle r in [0, pi] and a unit vector A that acts as a rotation
C     axis, we define the quaternion Q by
C
C        Q = ( cos(r/2), sin(r/2)A , sin(r/2)A , sin(r/2)A ).
C                                 1           2           3
C
C     As mentioned in $Detailed_Output, our restriction on the range of
C     r determines Q uniquely, except when r = pi.
C
C     The SPICELIB routine Q2M is an one-sided inverse of this routine:
C     given any rotation matrix R, the calls
C
C        CALL M2Q ( R, Q )
C        CALL Q2M ( Q, R )
C
C     leave R unchanged, except for round-off error.  However, the
C     calls
C
C        CALL Q2M ( Q, R )
C        CALL M2Q ( R, Q )
C
C     might preserve Q or convert Q to -Q.
C
C$ Examples
C
C     1)  A case amenable to checking by hand calculation:
C
C            To convert the rotation matrix
C
C                     +-              -+
C                     |  0     1    0  |
C                     |                |
C               R  =  | -1     0    0  |
C                     |                |
C                     |  0     0    1  |
C                     +-              -+
C
C            also represented as
C
C               [ pi/2 ]
C                       3
C
C            to a quaternion, we can use the code fragment
C
C               CALL ROTATE (  HALFPI(),  3,  R  )
C               CALL M2Q    (  R,             Q  )
C
C            M2Q will return Q as
C
C               ( sqrt(2)/2, 0, 0, -sqrt(2)/2 ).
C
C            Why?  Well, R is a coordinate transformation that
C            rotates vectors by -pi/2 radians about the axis vector
C
C               A  = ( 0, 0, 1 ),
C
C            so our definition of Q,
C
C               Q =
C
C               ( cos(r/2), sin(r/2)A , sin(r/2)A , sin(r/2)A  ),
C                                    1           2           3
C
C            implies that in this case,
C
C               Q =  ( cos(pi/4),  0,  0,  sin(pi/4)  )
C
C                 =  ( sqrt(2)/2,  0,  0, -sqrt(2)/2  ).
C
C
C     2)  Finding a quaternion that represents a rotation specified by
C         a set of Euler angles:
C
C            Suppose our original rotation R is the product
C
C               [ TAU ]  [ pi/2 - DELTA ]  [ ALPHA ] .
C                      3                 2          3
C
C            The code fragment
C
C               CALL EUL2M  ( TAU,   HALFPI() - DELTA,   ALPHA,
C              .              3,     2,                  3,      R )
C
C               CALL M2Q    ( R, Q )
C
C            yields a quaternion Q that represents R.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman   (JPL)
C
C$ Version
C
C-    SPICELIB Version 2.0.0, 17-SEP-1999 (WLT)
C
C        The routine was re-implemented to sharpen the numerical
C        stability of the routine and eliminate calls to SIN
C        and COS functions.
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 30-AUG-1990 (NJB)
C
C-&
 
C$ Index_Entries
C
C     matrix to quaternion
C
C-&
 
C
C     SPICELIB functions
C
      LOGICAL               ISROT
 
C
C     Local parameters
C
 
C
C     NTOL and DETOL are used to determine whether R is a rotation
C     matrix.
C
C     NTOL is the tolerance for the norms of the columns of R.
C
C     DTOL is the tolerance for the determinant of a matrix whose
C     columns are the unitized columns of R.
C
C
      DOUBLE PRECISION      NTOL
      PARAMETER           ( NTOL = 0.1D0 )
 
      DOUBLE PRECISION      DTOL
      PARAMETER           ( DTOL = 0.1D0 )
 
C
C     Local Variables
C
      DOUBLE PRECISION      C
      DOUBLE PRECISION      CC4
      DOUBLE PRECISION      FACTOR
      DOUBLE PRECISION      L2
      DOUBLE PRECISION      MTRACE
      DOUBLE PRECISION      POLISH
      DOUBLE PRECISION      S(3)
      DOUBLE PRECISION      S114
      DOUBLE PRECISION      S224
      DOUBLE PRECISION      S334
      DOUBLE PRECISION      TRACE
 
 
C
C     If R is not a rotation matrix, we can't proceed.
C
      IF (  .NOT. ISROT ( R, NTOL, DTOL )  ) THEN
         CALL CHKIN  ( 'M2Q' )
         CALL SETMSG ( 'Input matrix was not a rotation.' )
         CALL SIGERR ( 'SPICE(NOTAROTATION)'              )
         CALL CHKOUT ( 'M2Q'                              )
         RETURN
 
      END IF
C
C
C     If our quaternion is C, S1, S2, S3 (the S's being the imaginary
C     part) and we let
C
C        CSi = C  * Si
C        Sij = Si * Sj
C
C     then the rotation matrix corresponding to our quaternion is:
C
C        R(1,1)      = 1.0D0 - 2*S22 - 2*S33
C        R(2,1)      =         2*S12 + 2*CS3
C        R(3,1)      =         2*S13 - 2*CS2
C
C        R(1,2)      =         2*S12 - 2*CS3
C        R(2,2)      = 1.0D0 - 2*S11 - 2*S33
C        R(3,2)      =         2*S23 + 2*CS1
C
C        R(1,3)      =         2*S13 + 2*CS2
C        R(2,3)      =         2*S23 - 2*CS1
C        R(3,3)      = 1.0D0 - 2*S11 - 2*S22
C
C        From the above we can see that
C
C           TRACE = 3 - 4*(S11 + S22 + S33)
C
C        so that
C
C
C           1.0D0 + TRACE = 4 - 4*(S11 + S22 + S33)
C                         = 4*(CC + S11 + S22 + S33)
C                         - 4*(S11 + S22 + S33)
C                         = 4*CC
C
C        Thus up to sign
C
C          C = 0.5D0 * DSQRT( 1.0D0 + TRACE )
C
C        But we also have
C
C          1.0D0 + TRACE - 2.0D0*R(i,i) = 4.0D0 - 4.0D0(Sii + Sjj + Skk)
C                                       - 2.0D0 + 4.0D0(Sjj + Skk )
C
C                                       = 2.0D0 - 4.0D0*Sii
C
C        So that
C
C           1.0D0 - TRACE + 2.0D0*R(i,i) = 4.0D0*Sii
C
C        and so up to sign
C
C           Si = 0.5D0*DSQRT( 1.0D0 - TRACE + 2.0D0*R(i,i) )
C
C        in addition to this observation, we note that all of the
C        product pairs can easily be computed
C
C         CS1 = (R(3,2) - R(2,3))/4.0D0
C         CS2 = (R(1,3) - R(3,1))/4.0D0
C         CS3 = (R(2,1) - R(1,2))/4.0D0
C         S12 = (R(2,1) + R(1,2))/4.0D0
C         S13 = (R(3,1) + R(1,3))/4.0D0
C         S23 = (R(2,3) + R(3,2))/4.0D0
C
C     But taking sums or differences of numbers that are nearly equal
C     or nearly opposite results in a loss of precision. As a result
C     we should take some care in which terms to select when computing
C     C, S1, S2, S3.  However, by simply starting with one of the
C     large quantities cc, S11, S22, or S33 we can make sure that we
C     use the best of the 6 quantities above when computing the
C     remaining components of the quaternion.
C
 
      TRACE  = R(1,1) + R(2,2) + R(3,3)
      MTRACE = 1.0D0 - TRACE
 
      CC4    = 1.0D0  + TRACE
      S114   = MTRACE + 2.0D0*R(1,1)
      S224   = MTRACE + 2.0D0*R(2,2)
      S334   = MTRACE + 2.0D0*R(3,3)
 
C
C     Note that if you simply add CC4 + S114 + S224 + S334
C     you get four. Thus at least one of the 4 terms is greater than 1.
C
      IF ( 1.0D0 .LE. CC4 ) THEN
         C      =  DSQRT  ( CC4 * 0.25D0 )
         FACTOR =  1.0D0 /( C   * 4.0D0  )
 
         S(1)   = ( R(3,2) - R(2,3) )*FACTOR
         S(2)   = ( R(1,3) - R(3,1) )*FACTOR
         S(3)   = ( R(2,1) - R(1,2) )*FACTOR
 
      ELSE IF ( 1.0D0 .LE. S114 ) THEN
 
         S(1)   = DSQRT  ( S114 * 0.25D0 )
         FACTOR = 1.0D0 /( S(1) * 4.0D0  )
 
         C      = ( R(3,2) - R(2,3) ) * FACTOR
         S(2)   = ( R(1,2) + R(2,1) ) * FACTOR
         S(3)   = ( R(1,3) + R(3,1) ) * FACTOR
 
 
      ELSE IF ( 1.0D0 .LE. S224 ) THEN
 
         S(2)   = DSQRT  ( S224 * 0.25D0 )
         FACTOR = 1.0D0 /( S(2) * 4.0D0  )
 
         C      = ( R(1,3) - R(3,1) ) * FACTOR
         S(1)   = ( R(1,2) + R(2,1) ) * FACTOR
         S(3)   = ( R(2,3) + R(3,2) ) * FACTOR
 
      ELSE
 
         S(3)   = DSQRT  ( S334 * 0.25D0 )
         FACTOR = 1.0D0 /( S(3) * 4.0D0  )
 
         C      = ( R(2,1) - R(1,2) ) * FACTOR
         S(1)   = ( R(1,3) + R(3,1) ) * FACTOR
         S(2)   = ( R(2,3) + R(3,2) ) * FACTOR
 
      END IF
C
C     If the magnitude of this quaternion is not one, we polish it
C     up a bit.
C
      L2 = C*C + S(1)*S(1) + S(2)*S(2) + S(3)*S(3)
 
      IF ( L2 .NE. 1.0D0 ) THEN
         POLISH = 1.0D0/DSQRT(L2)
         C      =    C*POLISH
         S(1)   = S(1)*POLISH
         S(2)   = S(2)*POLISH
         S(3)   = S(3)*POLISH
      END IF
 
      IF ( C .GT. 0.0D0 ) THEN
         Q(0) = C
         Q(1) = S(1)
         Q(2) = S(2)
         Q(3) = S(3)
      ELSE
         Q(0) = -C
         Q(1) = -S(1)
         Q(2) = -S(2)
         Q(3) = -S(3)
      END IF
 
      RETURN
      END
