C$Procedure      NEARPT ( Nearest point on an ellipsoid )
 
 
      SUBROUTINE NEARPT ( POSITN, A, B, C, NPOINT, ALT )
 
C$ Abstract
C
C     This routine locates the point on the surface of an ellipsoid
C     that is nearest to a specified position. It also returns the
C     altitude of the position above the ellipsoid.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     ALTITUDE  
C     ELLIPSOID
C     GEOMETRY
C
C$ Declarations
 
      DOUBLE PRECISION      POSITN ( 3 )
      DOUBLE PRECISION      A
      DOUBLE PRECISION      B
      DOUBLE PRECISION      C
      DOUBLE PRECISION      NPOINT ( 3 )
      DOUBLE PRECISION      ALT
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      POSITN     I   Position of a point in bodyfixed frame.
C      A          I   Length of semi-axis parallel to x-axis.
C      B          I   Length of semi-axis parallel to y-axis.
C      C          I   Length on semi-axis parallel to z-axis.
C      NPOINT     O   Point on the ellipsoid closest to POSITN.
C      ALT        O   Altitude of POSITN above the ellipsoid.
C
C$ Detailed_Input
C
C     POSITN     3-vector giving the position of a point with respect
C                to the center of an ellipsoid. The vector is expressed
C                in a body-fixed reference frame. The semi-axes of the
C                ellipsoid are aligned with the x, y, and z-axes of the
C                body-fixed frame.
C
C     A          Length of the semi-axis of the ellipsoid that is
C                parallel to the x-axis of the body-fixed reference
C                frame.
C
C     B          Length of the semi-axis of the ellipsoid that is
C                parallel to the y-axis of the body-fixed reference
C                frame.
C
C     C          Length of the semi-axis of the ellipsoid that is
C                parallel to the z-axis of the body-fixed reference
C                frame.
C
C$ Detailed_Output
C
C     NPOINT     is the nearest point on the ellipsoid to POSITN.
C                NPOINT is a 3-vector expressed in the body-fixed
C                reference frame.
C
C     ALT        is the altitude of POSITN above the ellipsoid. If
C                POSITN is inside the ellipsoid, ALT will be negative
C                and have magnitude equal to the distance between
C                NPOINT and POSITN.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If any of the inputs A, B or C are non-positive the error
C        'SPICE(BADAXISLENGTH)' will be signaled.
C
C$ Files
C
C      None.
C
C$ Particulars
C
C     Many applications of this routine are more easily performed
C     using the higher-level SPICELIB routine SUBPT.  This routine
C     is the mathematical workhorse on which SUBPT relies.
C
C$ Examples
C
C     Example 1.
C
C     The code fragment below illustrates how you can use SPICELIB to
C     compute the apparent sub-earth point on the moon.
C
C     C
C     C     Load the ephemeris, leapseconds and physical constants
C     C     files first.  We assume the names of these files are
C     C     stored in the character variables SPK, LSK and
C     C     PCK.
C     C
C           CALL FURNSH ( SPK )
C           CALL FURNSH ( LSK )
C           CALL FURNSH ( PCK )
C
C     C
C     C     Get the apparent position of the moon as seen from the 
C     C     earth.  Look up this position vector in the moon 
C     C     body-fixed frame IAU_MOON. The orientation of the 
C     C     IAU_MOON frame will be computed at epoch ET-LT. 
C     C
C           CALL SPKPOS ( 'moon',  ET,    'IAU_MOON', 'LT+S', 
C          .              'earth', TRGPOS, LT                 )
C
C     C
C     C     Negate the moon's apparent position to obtain the 
C     C     position of the earth in the moon's body-fixed frame.
C     C 
C           CALL VMINUS ( TRGPOS, EVEC )
C           
C     C
C     C     Get the lengths of the principal axes of the moon.
C     C     Transfer the elements of the array RADII to the 
C     C     variables A, B, C to enhance readability.
C     C
C           CALL BODVAR (  301,    'RADII', DIM, RADII )
C           CALL VUPACK (  RADII,  A,       B,   C     )
C
C     C
C     C     Finally get the point SUBPNT on the surface of the
C     C     moon closest to the earth --- the sub-earth point.
C     C     SUBPNT is expressed in the IAU_MOON reference frame.
C     C
C           CALL NEARPT ( EVEC, A, B, C, SUBPNT, ALT )
C
C
C     Example 2.
C
C     One can use this routine to define a generalization of GEODETIC
C     coordinates called GAUSSIAN coordinates of a triaxial body.  (The
C     name is derived from the famous Gauss-map of classical
C     differential geometry).  The coordinates are longitude,
C     latitude, and altitude.
C
C     We let the x-axis of the body fixed coordinate system point
C     along the longest axis of the triaxial body.  The y-axis points
C     along the middle axis and the z-axis points along the shortest
C     axis.
C
C     Given a point P, there is a point on the ellipsoid that is
C     closest to P, call it Q.  The latitude and longitude of P
C     are determined by constructing the outward pointing unit normal
C     to the ellipsoid at Q.  Latitude of P is the latitude that the
C     normal points towards in the bodyfixed frame. Longitude is
C     the longitude the normal points to in the bodyfixed frame.
C     The altitude is the signed distance from P to Q, positive if P
C     is outside the ellipsoid, negative if P is inside.
C     (the mapping of the point Q to the unit normal at Q is the
C     Gauss-map of Q).
C
C     To obtain the Gaussian coordinates of a point whose position
C     in bodyfixed rectangular coordinates is given by a vector P,
C     the code fragment below will suffice.
C
C        CALL NEARPT ( P,    A, B, C, Q, ALT  )
C        CALL SURFNM (       A, B, C  Q, NRML )
C
C        CALL RECLAT ( NRML, R, LONG, LAT     )
C
C     The Gaussian coordinates are LONG, LAT, and ALT.
C
C
C$ Restrictions
C
C      None.
C
C$ Author_and_Institution
C
C      C.H. Acton      (JPL)
C      N.J. Bachman    (JPL)
C      W.L. Taber      (JPL)
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-     SPICELIB Version 1.1.1, 28-JUL-2003 (NJB) (CHA)
C
C        Various header corrections were made. 
C
C-     SPICELIB Version 1.1.0, 27-NOV-1990 (WLT)
C
C         The routine was substantially rewritten to achieve
C         more robust behavior and document the mathematics
C         of the routine.
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (WLT)
C
C-&
 
C$ Index_Entries
C
C     distance from point to ellipsoid
C     nearest point on an ellipsoid
C
C-&
 
 
C$ Revisions
C
C-     SPICELIB Version 1.1.0, 27-NOV-1990
C
C      The routine was nearly rewritten so that points
C      near the coordinate planes in the interior of the ellipsoid
C      could be handled without fear of floating point overflow
C      or divide by zero.
C
C      While the mathematical ideas involved in the original routine
C      are retained, the code is for the most part new.  In addition,
C      the new code has been documented far more extensively than was
C      NEARPT 1.0.0.
C
C
C-     Beta Version 2.0.0, 9-JAN-1989 (WLT)
C
C      Error handling added has been added for bad axes values.
C
C      The algorithm did not work correctly for some points inside
C      the ellipsoid lying on the plane orthogonal to the shortest
C      axis of the ellipsoid.  The problem was corrected.
C
C      Finally the algorithm was made slightly more robust and clearer
C      by use of SPICELIB routines and by normalizing the inputs.
C
C      Add an example to the header section.
C-&
 
C
C     SPICELIB functions
C
      DOUBLE PRECISION      VNORM
      DOUBLE PRECISION      BRCKTD
      DOUBLE PRECISION      VDIST
 
      LOGICAL               RETURN
      LOGICAL               OPSGND
 
 
 
C
C     Parameters
C
C
C     The parameter MAXSOL determines the maximum number of
C     iterations that will be performed in locating the
C     near point.  This must be at least 3.  To get strong
C     robustness in the routine, MAXSOL should be at least 4.
C
C
      INTEGER               MAXSOL
      PARAMETER           ( MAXSOL  = 6 )
 
C
C     Local Variables
C
      DOUBLE PRECISION      AXIS   ( 3 )
      DOUBLE PRECISION      AXISQR ( 3 )
      DOUBLE PRECISION      BESTHT
      DOUBLE PRECISION      BESTPT ( 3 )
      DOUBLE PRECISION      COPY   ( 3 )
      DOUBLE PRECISION      EPOINT ( 3 )
      DOUBLE PRECISION      ERR    ( 3 )
      DOUBLE PRECISION      ERRP   ( 3 )
      DOUBLE PRECISION      FACTOR
      DOUBLE PRECISION      HEIGHT
      DOUBLE PRECISION      LAMBDA
      DOUBLE PRECISION      LOWER
      DOUBLE PRECISION      NEWERR
      DOUBLE PRECISION      NORMAL ( 3 )
      DOUBLE PRECISION      OLDERR
      DOUBLE PRECISION      ORIGNL ( 3 )
      DOUBLE PRECISION      POINT  ( 3 )
      DOUBLE PRECISION      Q
      DOUBLE PRECISION      QLAST
      DOUBLE PRECISION      QLOWER
      DOUBLE PRECISION      QUPPER
      DOUBLE PRECISION      SCALE
      DOUBLE PRECISION      SIGN
      DOUBLE PRECISION      SPOINT ( 3 )
      DOUBLE PRECISION      TEMP
      DOUBLE PRECISION      TERM   ( 3 )
      DOUBLE PRECISION      TLAMBD ( 3 )
      DOUBLE PRECISION      UPPER
 
      INTEGER               BAD
      INTEGER               I
      INTEGER               IORDER ( 3 )
      INTEGER               SNGLPT
      INTEGER               SOLUTN
 
      LOGICAL               EXTRA
      LOGICAL               INSIDE
      LOGICAL               SOLVNG
      LOGICAL               TRIM
 
 
 
C
C     Save variables
C
      CHARACTER*36          MSSG ( 7 )
      SAVE                  MSSG
 
C
C     Initial values
C
      DATA                  MSSG / 'Axis A was nonpositive. ?',
     .                             'Axis B was nonpositive. ?',
     .                             'Axes A and B were nonpositive. ?',
     .                             'Axis C was nonpositive. ?',
     .                             'Axes A and C were nonpositive. ?',
     .                             'Axes B and C were nonpositive. ?',
     .                             'All three axes were nonpositive. ?'/
 
 
C
C     Here's what you can expect to find in the routine below.
C
C        Chapter 1.  Error and Exception Handling.
C
C        Chapter 2.  Mathematical background for the solution---the
C                    lambda equation.
C
C        Chapter 3.  Initializations for the main processing loop.
C
C        Chapter 4.  Mathematical Solution of the lambda equation.
C
C                    Section 4.1  Avoiding numerical difficulties.
C                    Section 4.2  Bracketing the root of the lambda
C                                 equation.
C                    Section 4.3  Refining the estimate of lambda.
C                    Section 4.4  Handling points on the central plane.
C
C        Chapter 5.  Decisions and initializations for sharpening
C                    the solution.
C
C        Chapter 6.  Clean up.
C
 
C
C     Error and Exception Handling.
C     ================================================================
C     ----------------------------------------------------------------
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'NEARPT' )
      END IF
 
 
C
C     Check the axes to make sure that none of them is less than or
C     equal to zero. If one is, signal an error and return.
C
      BAD = 0
 
      IF ( A .LE. 0.0D0 ) THEN
         BAD = BAD + 1
      END IF
 
      IF ( B .LE. 0.0D0 ) THEN
         BAD = BAD + 2
      END IF
 
      IF ( C .LE. 0.0D0 ) THEN
         BAD = BAD + 4
      END IF
 
 
      IF ( BAD .GT. 0 ) THEN
         CALL SETMSG ( MSSG (BAD)                                      )
         CALL ERRCH  ( '?', 'The A,B, and C axes were '               //
     .                      '#, #, and # respectively.'                )
 
         CALL ERRDP  ( '#', A )
         CALL ERRDP  ( '#', B )
         CALL ERRDP  ( '#', C )
 
         CALL SIGERR ( 'SPICE(BADAXISLENGTH)' )
         CALL CHKOUT ( 'NEARPT' )
         RETURN
      END IF
 
 
 
C
C     Mathematical background for the solution---the lambda equation.
C     ================================================================
C     ----------------------------------------------------------------
C
C--
C--   Here is the background and general outline of how this problem is
C--   going to be solved.
C--
C--   We want to find, a point on the ellipsoid
C--
C--
C--         X**2       Y**2       Z**2
C--        ------  +  ------  +  ------  =  1
C--         A**2       B**2       C**2
C--
C--   that is closest to the input point POSITN.
C--
C--       If one cares about the gory details, we know that
C--       such a point must exist because the
C--       ellipsoid is a compact subset of Euclidean 3-space
C--       and the distance function between the input point
C--       and the ellipsoid is a continuous functions.
C--       Since continuous functions on compact sets
C--       actually achieve their minimums at some point of
C--       the compact set, we are guaranteed that a closest
C--       point exists.
C--
C--   If we let NPOINT be a closest point to POSITN, then the
C--   line segment joining POSITN to NPOINT is parallel to
C--   the normal to the ellipsoid at NPOINT.  Moreover,
C--   suppose we let SEGMENT(P) be the line segment that
C--   conects an arbitrary point P with POSITN.  It can
C--   be shown that there is only one point P on the
C--   ellipsoid in the same octant at POSITN such that the
C--   normal at P is parallel to SEGMENT(P)
C--
C--
C--       More gory details: A normal to a point (X,Y,Z)
C--       on the ellipsoid is given by
C--
C--         (X/A**2, Y/B**2, Z/C**2)
C--
C--       Given a fixed LAMBDA, and allowing (X,Y,Z) to
C--       range over all points on the ellipsoid, the set
C--       of points
C--
C--
C--               LAMBDA*X      LAMBDA*Y      LAMBDA*Z
C--         ( X + --------, Y + --------, Z + -------- )
C--                 A**2          B**2          C**2
C--
C--       describes another ellipsoid with axes having lengths
C--
C--                LAMBDA         LAMBDA        LAMBDA
C--            A + ------ ,   B + ------ ,  C + ------  .
C--                  A              B             C
C--
C--
C--       Moreover, as long as LAMBDA > - MIN( A**2, B**2, C**2 )
C--       none of these ellipsoids intersect.  Thus, as long as
C--       the normal lines are not allowed to cross the coordinate plane
C--       orthogonal to the smallest axis (called the central plane)
C--       they do not intersect.
C--
C--
C--       Finally every point that does not lie on the central plane
C--       lies on one of the "lambda" ellipsoids described above.
C--
C--       Consequently, for each point, P, not on the central plane
C--       there is a unique point, NPOINT, on the ellipsoid, such that
C--       the normal line at NPOINT also contains P and does not cross
C--       the central plane.
C--
C--
C--   From the above discussion we see that we can mathematically
C--   solve the near point problem by finding a point NPOINT
C--   on the ellipsoid given by the equation:
C--
C--         X**2       Y**2       Z**2
C--        ------  +  ------  +  ------  =  1
C--         A**2       B**2       C**2
C--
C--
C--   such that for some value of LAMBDA
C--
C--        POSITN = NPOINT + LAMBDA*NORMAL(NPOINT).
C--
C--   Moreover, if POSITN = (X_o,Y_o,Z_o) then LAMBDA must satisfy
C--   the equation:
C--
C--            2                   2                   2
C--         X_o                 Y_o                 Z_o
C--   -----------------  + ------------------  + ------------------ = 1
C--                   2                     2                     2
C--   ( A + LAMBDA/A )      ( B + LAMBDA/B )      ( C + LAMBDA/C )
C--
C--
C--   and LAMBDA must be greater than -MIN(A**2,B**2,C**2)
C--
C--
C--   Once LAMBDA is known, NPOINT can be computed from the equation:
C--
C--        POSITN = NPOINT + LAMBDA*NORMAL(NPOINT).
C--
C--
C--   The process of solving for LAMBDA can be viewed as selecting
C--   that ellipsoid
C--
C--              2                     2               2
C--             x                     y               z
C--        --------------- + ---------------- + ---------------  = 1
C--                      2                  2                 2
C--        (a + lambda/a)    ( b + lambda/b)    (c + lambda/c)
C--
C--   that contains the input point POSITN.  For lambda = 0, this
C--   ellipsoid is just the input ellipsoid.  When we increase
C--   lambda we get a larger "inflated" ellipsoid.  When we
C--   decrease lambda we get a smaller "deflated" ellipsoid.  Thus,
C--   the search for lambda can be viewed as inflating or deflating
C--   the input ellipsoid (in a specially prescribed manner) until
C--   the resulting ellipsoid contains the input point POSITN.
C--
C--   The mathematical solution laid out above, has some numeric
C--   flaws.  However, it is robust enough so that if it is applied
C--   repeatedly, we can converge to a good solution of the near point
C--   problem.
C--
C--   In the code that follows, we will first solve the lambda equation
C--   using the original input point.  However, for points near the
C--   central plane the solution we obtain may not lie on the
C--   ellipsoid.  But, it should lie along the correct normal line.
C--
C--   Using this first candidate solution, we find the closest point
C--   to it on the ellipsoid.  This second iteration always produces
C--   a point that is as close as you can get to the ellipsoid.
C--   However, the normal at this second solution may not come as close
C--   as desired to pointing towards the input position.  To overcome
C--   this deficiency we sharpen the second solution.
C--
C--   To sharpen a solution we use the computed near point, the computed
C--   altitude of POSITN and the normal at the near point to approximate
C--   POSITN.  The difference between the the approximated position of
C--   POSITN and the input value of POSITN is called the error vector.
C--   To get a sharpened solution we translate the computed near point
C--   by the component of the error vector orthogonal to the normal
C--   and then find the mathematical near point to our translated
C--   solution.
C--
C--   The sharpening process is repeated until it no longer produces
C--   an "improved" near point.
C--
C--   At each step of this procedure, we must compute a solution to
C--   the "lambda" equation in order to produce our next estimate of
C--   the near point.  If it were possible to create a "private"
C--   routine in FORTRAN that only this routine could access, we
C--   would do it.  However, things being what they are, we have to
C--   compute the lambda solution in a loop.  We keep track of which
C--   refinement we are working on by counting the number of
C--   lambda solutions that are computed.
C--
 
 
 
 
 
 
 
 
 
 
 
 
 
C
C     Initializations for the main processing loop
C     ================================================================
C     ----------------------------------------------------------------
C
C
C     Let the game begin!
C
C     First order the axes of the ellipsoid and corresponding
C     component of POSITN by the size of lengths of axes.  Knowing
C     which axes are smallest will simplify our task of computing
C     lambda when the time comes.
C
      AXIS(1)  = A
      AXIS(2)  = B
      AXIS(3)  = C
 
      CALL VEQU   ( POSITN,    POINT  )
 
      CALL ORDERD ( AXIS,   3, IORDER )
      CALL REORDD ( IORDER, 3, AXIS   )
      CALL REORDD ( IORDER, 3, POINT  )
 
C
C     Rescale everything so as to avoid underflows when squaring
C     quantities and copy the original starting point.
C
C     Be sure that this is UNDONE at the end of the routine.
C
      SCALE     = 1.0D0/AXIS(1)
 
      CALL VSCL   ( SCALE,  AXIS,   AXIS  )
      CALL VSCL   ( SCALE,  POINT,  POINT )
      CALL VEQU   ( POINT,          ORIGNL)
 
 
      AXISQR(1) = AXIS(1)*AXIS(1)
      AXISQR(2) = AXIS(2)*AXIS(2)
      AXISQR(3) = AXIS(3)*AXIS(3)
 
C
C     We will need to "solve" for the NEARPT at least 3 times.
C     SOLUTN is the counter that keeps track of how many times
C     we have actually solved for a near point.  SOLVNG indicates
C     whether we should continue solving for NEARPT.
C
      SOLUTN =  1
      SOLVNG = .TRUE.
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
      DO WHILE ( SOLVNG )
 
 
C
C       Mathematical solution of the lambda equation.
C       ================================================================
C       ----------------------------------------------------------------
C--
C--
C--      Make a stab at solving the mathematical problem of finding the
C--      near point.  In other words, solve the lambda equation.
C--
C
C        Avoiding Numerical difficulties
C        -------------------------------
C
C        First make a copy of POINT, then to avoid numerical
C        difficulties later on, we will assume that any component of
C        POINT that is not sufficiently different from zero to
C        contribute to an addition to the corresponding component
C        of AXIS, is in fact zero.
C
         CALL VEQU ( POINT, COPY )
 
         DO I = 1, 3
 
            IF (      ( 0.5D0*POINT(I) + AXIS(I) .EQ.  AXIS(I) )
     .           .OR. ( 0.5D0*POINT(I) - AXIS(I) .EQ. -AXIS(I) ) )
     .      THEN
               POINT(I) = 0.0D0
            END IF
 
         END DO
 
C
C        OK. Next we set up the logical that indicates whether
C        the current point is inside the ellipsoid.
C
         INSIDE = .FALSE.
 
 
 
 
 
 
 
 
C
C        Bracketing the root of the lambda equation.
C        -------------------------------------------
C
C        Let (x,y,z) stand for (POINT(1), POINT(2), POINT(3)) and
C        let (a,b,c) stand for (AXIS(1),  AXIS(2),   AXIS(3)).
C
C        The main step in finding the near point is to find the
C        root of the lambda equation:
C
C                 2                     2               2
C                x                     y               z
C       0 = --------------- + ---------------- + ---------------  - 1
C                         2                  2                 2
C           (a + lambda/a)    ( b + lambda/b)    (c + lambda/c)
C
C
C        We let Q(lambda) be the right hand side of this equation.
C        To find the roots of the equation we determine
C        values of lambda that make Q greater than 0 and less than 0.
C        An obvious value to check is lambda = 0.
C
         Q = (POINT(1)/AXIS(1))**2
     .     + (POINT(2)/AXIS(2))**2
     .     + (POINT(3)/AXIS(3))**2
     .     - 1.0D0
 
 
C
C        On the first solution pass, we will determine the sign of
C        the altitude of the input POSITN
C
         IF ( SOLUTN .EQ. 1 ) THEN
 
            IF       ( Q .GE. 0 ) THEN
               SIGN = 1.0D0
            ELSE  IF ( Q .LT. 0 ) THEN
               SIGN = -1.0D0
            END IF
 
         END IF
 
C
C        OK. Now for the stuff we will have to do on every solution
C        pass.
C
         IF ( Q .EQ. 0.0D0 ) THEN
 
C
C           In this case the point is already on the ellipsoid
C           (pretty lucky eh?)  We simply set our bracketing values,
C           QLOWER and QUPPER, to zero so that that bisection
C           loop won't ever get executed.
C
 
            QLOWER = 0.0D0
            QUPPER = 0.0D0
            LOWER  = 0.0D0
            UPPER  = 0.0D0
            LAMBDA = 0.0D0
 
         ELSE IF ( Q .GT. 0.0D0 ) THEN
 
C
C           The input point is outside the ellipsoid (we expect that
C           this is the usual case).  We want to choose our lower
C           bracketing value so that the bracketing values for lambda
C           aren't too far apart.  So we just make sure that the largest
C           term of the expression for Q isn't bigger than 4.
C
            DO I = 1, 3
               TLAMBD(I) = ( 0.5D0*DABS(POINT(I)) - AXIS(I) ) * AXIS(I)
            END DO
 
            LOWER = MAX ( 0.0D0, TLAMBD(1), TLAMBD(2), TLAMBD(3) )
 
 
C
C           Choose the next value of lambda so that the largest term
C           of Q will be no more than 1/4.
C
            UPPER  = 2.0D0 * MAX ( DABS( AXIS(1)*POINT(1) ),
     .                             DABS( AXIS(2)*POINT(2) ),
     .                             DABS( AXIS(3)*POINT(3) ) )
            LAMBDA = UPPER
 
         ELSE
 
C
C           In this case the point POSITN is inside the ellipsoid.
C
            INSIDE = .TRUE.
 
C
C           This case is a bit of a nuisance. To solve the lambda
C           equation we have to find upper and lower bounds on
C           lambda such that one makes Q greater than 0, the other
C           makes Q less than 0.  Once the root has been bracketed
C           in this way it is a straight forward problem to find
C           the value of LAMBDA that is closest to the root we
C           seek.  We already know that for LAMBDA = 0, Q is negative.
C           So we only need to find a value of LAMBDA that makes
C           Q positive.  But... the expression for Q has singularities
C           at LAMBDA = -a**2, -b**2, and -c**2.
C
C           These singularities are not necessarily to be avoided.
C           If the numerator of one of the terms for Q is zero, we
C           can simply compute Q ignoring that particular term.  We
C           say that a singularity corresponding to a term whose
C           numerator is zero is a viable singularity.  By being
C           careful in our computation of Q, we can assign LAMBDA to
C           the value of the singularity. A singularity that is not
C           viable is called a true singularity.
C
C           By choosing LAMBDA, just slightly greater than the largest
C           true singularity, we can bracket the root we seek.
C
C           First we must decide which singularity is the first true
C           one.
C
            SNGLPT = 4
 
            DO I = 3, 1, -1
 
               IF ( POINT(I) .NE. 0 ) THEN
                  SNGLPT = I
               END IF
 
            END DO
 
C
C           If there is a singular point, compute LAMBDA so that the
C           largest term of Q is equal to 4.
C
            IF ( SNGLPT .LE. 3 ) THEN
 
               DO I = 1, 3
                  IF ( POINT(I) .EQ. 0 ) THEN
                     TLAMBD(I) = -AXISQR(3)
                  ELSE
                     TLAMBD(I) = AXIS(I) * (   0.5D0*DABS(POINT(I))
     .                                       - AXIS(I)              )
                  END IF
               END DO
 
               LAMBDA = MAX ( TLAMBD(1), TLAMBD(2), TLAMBD(3) )
               LOWER  = LAMBDA
               UPPER  = 0.0D0
 
            ELSE
 
C
C              The point must be at the origin.  In this case
C              we know where the closest point is. WE DON'T have
C              to compute anything. It's just at the end of the
C              shortest semi-major axis.  However, since we
C              may have done some rounding off, we will make
C              sure that we pick the side of the shortest axis
C              that has the same sign as COPY(1).
C
C              We are going to be a bit sneaky here.  We know
C              where the closest point is so we are going to
C              simply make POINT and COPY equal to that point
C              and set the upper and lower bracketing bounds
C              to zero so that we won't have to deal with any
C              special cases later on.
C
               IF ( COPY(1) .LT. 0 ) THEN
                  POINT(1) = -AXIS(1)
                  COPY(1)  = -AXIS(1)
               ELSE
                  POINT(1) =  AXIS(1)
                  COPY(1)  =  AXIS(1)
               END IF
 
               COPY(2) = 0.0D0
               COPY(3) = 0.0D0
 
               UPPER   = 0.0D0
               LOWER   = 0.0D0
               LAMBDA  = 0.0D0
               Q       = 0.0D0
 
               INSIDE  = .FALSE.
 
            END IF
 
         END IF
 
 
C
C        OK. Now compute the value of Q at the two bracketing
C        values of LAMBDA.
C
         DO I = 1, 3
 
            IF ( POINT(I) .EQ. 0 ) THEN
 
               TERM(I) = 0.0D0
 
            ELSE
 
C
C              We have to be a bit careful for points inside the
C              ellipsoid. The denominator of the factor we are
C              going to compute is ( AXIS + LAMBDA/AXIS ).
C              Numerically this may be too close to zero for us
C              to actually divide POINT by it.  However, since
C              our solution algorithm for lambda does not depend
C              upon the differentiability of Q, we can simply truncate
C              its individual terms when we are in danger of
C              division overflows.
C
               TRIM =       0.5D0   * DABS( POINT(I) )
     .               .GT. ( AXIS(I) + LAMBDA/AXIS(I) )
 
               IF ( INSIDE .AND. TRIM ) THEN
                  FACTOR = 2.0D0
               ELSE
                  FACTOR =  POINT(I) / ( AXIS(I) + LAMBDA/AXIS(I) )
               END IF
 
               TERM(I) = FACTOR*FACTOR
 
            END IF
         END DO
 
         IF ( .NOT. INSIDE ) THEN
            QLOWER = Q
            QUPPER = TERM(1) + TERM(2) + TERM(3) - 1.0D0
         ELSE
            QUPPER = Q
            QLOWER = TERM(1) + TERM(2) + TERM(3) - 1.0D0
         END IF
 
         LAMBDA = UPPER
         Q      = QUPPER
         QLAST  = QLOWER
 
 
 
 
 
 
 
 
C
C        Refining the estimate of lambda.
C        --------------------------------
C
C        Now perform the Taber root extraction algorithm.
C
         DO WHILE ( UPPER .GT. LOWER )
 
C
C           Depending upon how Q compares with Q at the
C           bracketing endpoints we adjust the endpoints
C           of the bracketing interval
C
            IF ( Q .EQ. 0.0D0 ) THEN
 
               LOWER = LAMBDA
               UPPER = LAMBDA
 
            ELSE IF ( OPSGND (Q, QLOWER) ) THEN
 
               UPPER  = LAMBDA
               QUPPER = Q
 
            ELSE IF ( OPSGND (Q, QUPPER) ) THEN
 
               LOWER  = LAMBDA
               QLOWER = Q
 
            END IF
 
C
C           If the current value of Q has the opposite sign
C           from the last computation of Q, we find our next
C           approximation of LAMBDA by the secant method.  Otherwise,
C           we just get the next value of LAMBDA by bisection.
C           (Note bisection will be the dominant choice since
C           most functions are concave or convex and not
C           flopping back and forth.
C
            IF ( OPSGND (Q, QLAST) ) THEN
               LAMBDA = (QUPPER*LOWER - UPPER*QLOWER)/(QUPPER - QLOWER)
            ELSE
               LAMBDA = 0.5D0*LOWER + 0.5D0*UPPER
            END IF
 
C
C           It's quite possible as we get close to the root
C           for Q that round off errors in the computation of the
C           next value of LAMBDA will push it outside of the
C           current bracketing interval.  Force it back in to
C           the current interval.
C
            LAMBDA = BRCKTD ( LAMBDA, LOWER, UPPER )
 
 
C
C           If LAMBDA hasn't moved away from one of the bracketing
C           endpoints, push it away by bisection
C
C
            IF (      ( LAMBDA .EQ. LOWER )
     .           .OR. ( LAMBDA .EQ. UPPER ) ) THEN
 
               LAMBDA = 0.5D0*LOWER + 0.5D0*UPPER
 
            END IF
 
 
C
C           If we are still at an endpoint, we might as well
C           cash in our chips.  We aren't going to be able
C           to get away from the endpoints.  Set LOWER
C           and UPPER equal so that the loop will finally
C           terminate.
C
            IF (      ( LAMBDA .EQ. LOWER )
     .           .OR. ( LAMBDA .EQ. UPPER ) ) THEN
 
C
C              Make the decision as to which way to push
C              the boundaries, by selecting that endpoint
C              at which Q is closest to zero.
 
               IF ( DABS(QLOWER) .LT. DABS(QUPPER) ) THEN
                  UPPER = LOWER
               ELSE
                  LOWER = UPPER
               END IF
 
            END IF
 
 
C
C           If lower and upper aren't the same, we compute the
C           value of Q at our new guess for LAMBDA.
C
            IF ( UPPER .NE. LOWER ) THEN
 
               DO I = 1, 3
 
                  IF ( POINT(I) .EQ. 0 ) THEN
 
                     TERM(I) = 0.0D0
 
                  ELSE
 
                     TRIM =       0.5D0   * DABS( POINT(I) )
     .                     .GT. ( AXIS(I) + LAMBDA/AXIS(I) )
 
                     IF ( INSIDE .AND. TRIM ) THEN
                        FACTOR = 2.0D0
                     ELSE
                        FACTOR = POINT(I) / ( AXIS(I) + LAMBDA/AXIS(I) )
                     END IF
 
                     TERM(I) = FACTOR*FACTOR
 
                  END IF
               END DO
 
               QLAST = Q
               Q     = TERM(1) + TERM(2) + TERM(3) - 1.0D0
 
            END IF
 
C
C           Loop back through the bracketing refinement code.
C
 
        END DO
 
C
C       Now we have LAMBDA, compute the nearest point based upon
C       this value.
C
        LAMBDA = LOWER
 
 
        DO I = 1, 3
 
           IF ( POINT(I) .EQ. 0.0D0 ) THEN
              SPOINT(I) = 0.0D0
           ELSE
              SPOINT(I) = COPY(I) / ( 1.0D0 +  LAMBDA/AXISQR(I) )
           END IF
        END DO
 
C
C       Handling points on the central plane.
C       -------------------------------------
C
C       I suppose you thought you were done at this point.
C       Not necessarily.  If POINT is INSIDE the ellipsoid and
C       happens to lie in the y-z plane, there is a possibility
C       (perhaps even likelihood) that the nearest point on the
C       ellipsoid is NOT in the y-z plane. we must consider
C       this possibility next.
C
        IF ( INSIDE  .AND. (      SNGLPT .EQ. 2
     .                       .OR. SNGLPT .EQ. 3 ) ) THEN
 
C
C          There are two ways to get here.  SNGLPT = 2 or SNGLPT = 3.
C          Fortunately these two cases can be handled simultaneously
C          by code.  However, they are most easily understood if treated
C          separately.
C
C          Case 1.  SNGLPT = 2
C
C          The input to the lambda solution POINT lies in the YZ plane.
C          We have already detected one critical point of the
C          distance function to POINT restricted to the ellipsoid.
C          This point also lies in the YZ-plane.  However, when
C          POINT lies on the YZ-plane close to the center of the
C          ellipsoid, there may be a point that is nearest that does
C          not lie in the YZ-plane. Assuming the existence of such a
C          point, (x,y,z) it must satisfy the equations
C
C                   lambda*x
C             x  +  --------  = POINT(1)  = 0
C                      a*a
C
C
C                    lambda*y
C             y  +  --------  = POINT(2)
C                      b*b
C
C
C                   lambda*z
C             z  +  --------  = POINT(3)
C                      c*c
C
C
C          Since we are assuming that this undetected solution (x,y,z)
C          does not have x equal to 0, it must be the case that
C
C          lambda = -a*a.
C
C          Because of this, we must have
C
C             y    =  POINT(2) / ( 1 - (a**2/b**2) )
C             z    =  POINT(3) / ( 1 - (a**2/c**2) )
C
C          The value of x is obtained by forcing
C
C             (x/a)**2 + (y/b)**2 + (z/c)**2 = 1.
C
C          This assumes of course that a and b are not equal.  If
C          a and b are the same, then the solution we found above
C          by deflating the original ellipsoid, will find the
C          near point.
C
C             (If a and b are equal, the ellipsoid deflates to a
C              segment on the z-axis when lambda = -a**2.  Since
C              y is not zero, the deflating ellipsoid must pass
C              through (x,y,z) before it collapses to a segment.)
C
C
C          Case 2.  SNGLPT = 3
C
C          The input to the lambda solution POINT lies on the Z-axis.
C          The solution obtained in the generic case above will
C          locate the critical point of the distance function
C          that lies on the Z.  However, there will also be
C          critical points in the XZ-plane and YZ plane.  The point
C          in the XZ-plane is the one to examine.  Why?  We are looking
C          for the point on the ellipsoid closest to POINT.  It must
C          lie in one of these two planes. But the ellipse of
C          intersection with the XZ-plane fits inside the ellipse
C          of intersection with the YZ-plane.  Therefore the closest
C          point on the YZ-ellipse must be at a greater distance than
C          the closest point on the XZ-ellipse.  Thus, in solving
C          the equations
C
C
C                   lambda*x
C             x  +  --------  = POINT(1)  = 0
C                      a*a
C
C
C                    lambda*y
C             y  +  --------  = POINT(2)  = 0
C                      b*b
C
C
C                   lambda*z
C             z  +  --------  = POINT(3)
C                      c*c
C
C
C          We have
C
C             lambda = -a*a,
C
C             y      =  0,
C
C             z      =  POINT(3) / ( 1 - (a**2/c**2) )
C
C          The value of x is obtained by forcing
C
C             (x/a)**2 + (y/b)**2 + (z/c)**2 = 1.
C
C          This assumes that a and c are not equal.  If
C          a and c are the same, then the solution we found above
C          by deflating the original ellipsoid, will find the
C          near point.
C
C             ( If a = c, then the input ellipsoid is a sphere.
C               The ellipsoid will deflate to the center of the
C               sphere.  Since our point is NOT at the center,
C               the deflating sphere will cross through
C               (x,y,z) before it collapses to a point )
C
C          We begin by assuming this extra point doesn't exist.
C
           EXTRA = .FALSE.
 
C
C          Next let's note a few simple tests we can apply to
C          eliminate searching for an extra point.
C
C          First of all the smallest axis must be different from
C          the axis associated with the first true singularity.
C
C
C          Next, whatever point we find, it must be true that
C
C               |y| < b, |z| < c
C
C          because of the condition on the absolute values, we must
C          have:
C
C               | POINT(2) | <= b - a*(a/b)
C
C               | POINT(3) | <= c - a*(a/c)
C
 
 
           IF (     (     AXIS (1)  .NE. AXIS(SNGLPT)                )
     .         .AND.(DABS(POINT(2)) .LE. AXIS(2) - AXISQR(1)/AXIS(2) )
     .         .AND.(DABS(POINT(3)) .LE. AXIS(3) - AXISQR(1)/AXIS(3) ))
     .     THEN
 
C
C             Compute the y, and z components (2 and 3) of the extra
C             point.
C
              EPOINT(2) = POINT(2) / ( 1.0D0 - AXISQR(1)/AXISQR(2) )
              EPOINT(3) = POINT(3) / ( 1.0D0 - AXISQR(1)/AXISQR(3) )
 
C
C             See if these components can even be on the ellipsoid...
C
              TEMP      = 1.0D0  - ( EPOINT(2) / AXIS(2) )**2
     .                           - ( EPOINT(3) / AXIS(3) )**2
 
              IF ( TEMP .GT. 0 ) THEN
 
C
C                ...and compute the x component of the point.
C
                 EPOINT(1) = AXIS(1) * DSQRT(TEMP)
                 EXTRA     = .TRUE.
 
              END IF
 
           END IF
 
 
C
C          Ok.  If an extra point is possible, check and see if it
C          is the one we are searching for.
C
           IF ( EXTRA ) THEN
 
              IF (      VDIST(EPOINT, POINT)
     .             .LT. VDIST(SPOINT, POINT) ) THEN
 
                 CALL VEQU ( EPOINT, SPOINT )
 
              END IF
 
           END IF
 
        END IF
 
 
 
 
 
 
 
 
 
 
 
 
C
C       Decisions and initializations for sharpening the solution.
C       ================================================================
C       ----------------------------------------------------------------
C
        IF ( SOLUTN .EQ. 1 ) THEN
 
C--
C--         The first solution for the nearest point may not be
C--         very close to being on the ellipsoid.  To
C--         take care of this case, we next find the point on the
C--         ellipsoid, closest to our first solution point.
C--         (Ideally the normal line at this second point should
C--         contain both the current solution point and the
C--         original point).
C--
            CALL   VEQU ( SPOINT, POINT  )
            CALL   VEQU ( SPOINT, BESTPT )
 
            BESTHT = VDIST ( BESTPT, ORIGNL )
 
         ELSE IF ( SOLUTN .EQ. 2 ) THEN
 
C--
C--         The current solution point will be very close to lying
C--         on the ellipsoid.  However, the normal at this solution
C--         may not actually point towards the input point.
C--
C--         With the current solution we can predict
C--         the location of the input point.  The difference between
C--         this predicted point and the actual point can be used
C--         to sharpen our estimate of the solution.
C--
C--         The sharpening is performed by
C--
C--            1.) Compute the vector ERR that gives the difference
C--                between the input point (POSITN) and the point
C--                computed using the solution point, normal and
C--                altitude.
C--
C--            2.) Find the component of ERR that is orthogonal to the
C--                normal at the current solution point.  Call this
C--                component ERRP
C--
C--            3.) Translate the solution point by ERRP to get POINT.
C--
C--            4.) Find the point on the ellipsoid closest to POINT.
C--                (step 4 is handled by the solution loop above).
C--
 
C
C           First we need to compute the altitude
C
            HEIGHT  = SIGN * VDIST(SPOINT, ORIGNL)
 
C
C           Next compute the difference between the input point and
C           the one we get by moving out along the normal at our
C           solution point by the computed altitude.
C
            CALL SURFNM ( AXIS(1), AXIS(2), AXIS(3), SPOINT, NORMAL )
            CALL VLCOM3 ( 1.0D0,   ORIGNL,
     .                   -1.0D0,   SPOINT,
     .                   -HEIGHT,  NORMAL,  ERR    )
 
C
C           Find the component of the error vector that is
C           perpendicular to the normal, and shift our solution
C           point by this component.
C
            CALL VPERP  ( ERR,     NORMAL, ERRP   )
            CALL VADD   ( SPOINT,  ERRP,   POINT  )
 
            OLDERR = VNORM ( ERR )
            BESTHT = HEIGHT
 
C
C           Finally store the current solution point, so that if
C           this sharpening doesn't improve our estimate of the
C           near point, we can just return our current best guess.
C
            CALL VEQU   ( SPOINT,          BESTPT )
 
 
         ELSE IF ( SOLUTN .GT. 2 ) THEN
 
C--
C--         This branch exists for the purpose of testing our
C--         "sharpened" solution and setting up for another sharpening
C--         pass.
C--
C--         We have already stored our best guess so far in BESTPT and
C--         the vector ERR is the difference
C--
C--            ORIGNL - ( BESTPT + BESTHT*NORMAL )
C--
C--         We have just computed a new candidate "best" near point.
C--         SPOINT.
C--
C--         If the error vector
C--
C--            ORIGNL - ( SPOINT + HEIGHT*NORMAL)
C--
C--         is shorter than our previous error, we will make SPOINT
C--         our best guess and try to sharpen our estimate again.
C--
C--         If our sharpening method hasn't improved things, we just
C--         call it quits and go with our current best guess.
C--
 
C
C           First compute the altitude,
C
            HEIGHT  = SIGN * VDIST( SPOINT, ORIGNL )
 
C
C           ... compute the difference
C
C              ORIGNL - SPOINT - HEIGHT*NORMAL,
C
            CALL SURFNM ( AXIS(1), AXIS(2), AXIS(3), SPOINT, NORMAL )
            CALL VLCOM3 ( 1.0D0,   ORIGNL,
     .                   -1.0D0,   SPOINT,
     .                   -HEIGHT,  NORMAL,  ERR    )
 
C
C           ...and determine the magnitude of the error due to our
C           sharpened estimate.
C
            NEWERR = VNORM ( ERR )
 
C
C           If the sharpened estimate yields a smaller error ...
C
            IF ( NEWERR .LT. OLDERR ) THEN
 
C
C              ...our current value of SPOINT becomes our new
C              best point and the current altitude becomes our
C              new altitude point.
C
               OLDERR  =  NEWERR
               BESTHT  =  HEIGHT
 
               CALL  VEQU ( SPOINT,  BESTPT      )
 
C
C              Next, if we haven't passed the limit on the number of
C              iterations allowed we prepare the initial point for our
C              "sharpening" estimate.
C
               IF ( SOLUTN .LE. MAXSOL ) THEN
 
                  CALL VPERP ( ERR,     NORMAL, ERRP   )
                  CALL VADD  ( SPOINT,  ERRP,   POINT  )
 
               END IF
 
C
C           If things didn't get better, there is no point in
C           going on.  Just set the SOLVNG flag to .FALSE.
C
            ELSE
 
               SOLVNG = .FALSE.
 
            END IF
 
         END IF
 
C
C        Increment the solution counter so that eventually this
C        loop will terminate.
C
         SOLUTN = SOLUTN + 1
 
         SOLVNG = SOLVNG .AND. ( SOLUTN .LE. MAXSOL )
 
      END DO
 
 
 
C
C     Clean up.
C     ==================================================================
C     ------------------------------------------------------------------
C
C     Re-scale and re-order the components of our solution point. Scale
C     and copy the value of BESTHT into the output argument.
C
      CALL VSCL ( 1.0D0/SCALE, BESTPT, BESTPT )
 
      DO I = 1,3
         NPOINT( IORDER(I) ) = BESTPT(I)
      END DO
 
      ALT = BESTHT/SCALE
 
      CALL CHKOUT ( 'NEARPT' )
      RETURN
 
      END
