C$Procedure      PACKAD ( Pack a double precision array )
 
      SUBROUTINE PACKAD ( IN,
     .                    PACK,   NPACK,
     .                    MAXOUT, NOUT,   OUT )
 
C$ Abstract
C
C      Pack the contents of a double precision array. That is,
C      take a set of arbitrarily spaced elements from an input
C      array, and make them adjacent elements in an output array.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C      ARRAY,  ASSIGNMENT,  UTILITY
C
C$ Declarations
 
      DOUBLE PRECISION   IN      ( * )
      INTEGER            PACK    ( * )
      INTEGER            NPACK
      INTEGER            MAXOUT
 
      INTEGER            NOUT
      DOUBLE PRECISION   OUT     ( * )
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      IN         I   Input array.
C      PACK       I   Indices of elements to be packed.
C      NPACK      I   Number of indices.
C      MAXOUT     I   Maximum number of elements in the output array.
C      NOUT       O   Number of elements in the output array.
C      OUT        O   Output array.
C
C$ Detailed_Input
C
C      IN          is the input array.
C
C      PACK        is the set of elements to be packed into the output
C                  array. PACK(i) is the index of the element in the
C                  input array that is to become the i'th element of
C                  the output array. If the output array is to over-
C                  write the input array, the elements of PACK must
C                  be ordered: PACK(1) < PACK(2) < ... < PACK(NPACK).
C
C      NPACK       is the number of elements to be packed into the
C                  output array.
C
C      MAXOUT      is the maximum number of elements to be packed
C                  into the output array. If NPACK is larger than
C                  MAXOUT, the extra items are ignored.
C
C$ Detailed_Output
C
C      NOUT        is the number of elements in the output array.
C                  NOUT may overwrite NPACK or MAXOUT.
C
C      OUT         is the output array. This array contains up to
C                  MAXOUT elements from the input array, located
C                  in the first NOUT elements of the array.
C
C$ Parameters
C
C     None.
C
C$ Particulars
C
C      The indicated elements are moved from their current locations
C      in the input array to consecutive positions in the output array.
C
C            OUT(   1) = IN(PACK(   1))
C            OUT(   2) = IN(PACK(   2))
C             .
C             .
C            OUT(NOUT) = IN(PACK(NOUT))
C
C      NOUT is either NPACK or MAXOUT, whichever is smaller.
C
C$ Examples
C
C      The most common use for this routine is to remove unwanted
C      items from an array or set of arrays. For example, suppose
C      that the arrays NAME, CODE, RADIUS and MASS contain the
C      names, NAIF integer ID codes, radii, and masses of a set of
C      NSAT satellites. Suppose further that the user selects a subset
C      of the original set of satellites from a menu of some sort.
C      Let the indices of these satellites be the NSEL elements of the
C      array SEL. The following sequence would remove the names,
C      codes, etc., of the unselected satellites from the arrays.
C
C            CALL PACKAC ( NAME,   SEL, NSEL, NSAT, NSAT, NAME   )
C            CALL PACKAI ( CODE,   SEL, NSEL, NSAT, NSAT, CODE   )
C            CALL PACKAD ( RADIUS, SEL, NSEL, NSAT, NSAT, RADIUS )
C            CALL PACKAD ( MASS,   SEL, NSEL, NSAT, NSAT, MASS   )
C
C      Note that in this example, the output array overwrites the
C      input array. Note also that on input NSAT is the maximum
C      number of elements to be transferred to the output array,
C      while on output it is the number of elements in the array.
C
C      In the example above, suppose that NAME and PACK contain
C      the following:
C
C            NAME = 'MIMAS'          PACK = 2, 4, 6, 7
C                   'ENCELADUS'
C                   'TETHYS'
C                   'DIONE'
C                   'RHEA'
C                   'TITAN'
C                   'HYPERION'
C                   'IAPETUS'
C                   'PHOEBE'
C
C      Then, following the call to PACKAC, NOUT and NAME contain
C      the following:
C
C            NOUT = 4                NAME = 'ENCELADUS'
C                                           'DIONE'
C                                           'TITAN'
C                                           'HYPERION'
C
C
C$ Restrictions
C
C      The elements in the PACK array MUST be ordered if the output
C      array is to overwrite the input array.
C
C$ Exceptions
C
C     If an element in the PACK array is less than 1, the error
C     SPICE(INVALIDINDEX) is signalled.
C
C$ Files
C
C      None.
C
C$ Author_and_Institution
C
C      H.A. Neilan     (JPL)
C      W.L. Taber      (JPL)
C      I.M. Underwood  (JPL)
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (WLT) (IMU)
C
C-&
 
C$ Index_Entries
C
C     pack a d.p. array
C
C-&
 
 
 
C$ Revisions
C
C-     Beta Version 2.0.0, 4-JAN-1989 (HAN)
C
C         Error handling was added to detect array indices that are
C         out of bound. If any element contained in the PACK array is
C         less than one, an error is signalled, and the output array is
C         not packed.
C
C-&
 
 
 
 
 
C
C     Spicelib functions
C
      LOGICAL               RETURN
 
 
C
C     Local variables
C
      INTEGER          I
 
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'PACKAD' )
      END IF
 
C
C     First, determine how many items to transfer.
C
      NOUT = MIN ( NPACK, MAXOUT )
 
C
C     Check to see if PACK contains valid array indices.
C
      DO I = 1, NOUT
 
         IF ( PACK (I) .LT. 1 ) THEN
            CALL SETMSG ( 'Element number * contains index *.' )
            CALL ERRINT ( '*', I                )
            CALL ERRINT ( '*', PACK(I)          )
            CALL SIGERR ( 'SPICE(INVALIDINDEX)' )
            CALL CHKOUT ( 'PACKAD' )
            RETURN
         END IF
 
      END DO
 
C
C     Transfer them. Just like it says in the header.
C
      DO I = 1, NOUT
         OUT(I) = IN(PACK(I))
      END DO
 
 
      CALL CHKOUT ( 'PACKAD' )
      RETURN
      END
