 
C$Procedure      PARTOF ( Parabolic time of flight )
 
      SUBROUTINE PARTOF ( MA, D )
 
C$ Abstract
C
C     Solve the time of flight equation MA = D + (D**3) / 3
C     for the parabolic eccentric anomaly D, given mean anomaly.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     CONIC
C
C$ Declarations
 
      DOUBLE PRECISION      MA
      DOUBLE PRECISION      D
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     MA         I   Mean anomaly at epoch.
C     D          O   Parabolic eccentric anomaly.
C
C$ Detailed_Input
C
C     MA          is the parabolic mean anomaly of an orbiting body at
C                 some epoch t,
C
C                                         3  1/2
C                       MA = (t-T) (mu/(2q ))
C
C                 where T is the time of periapsis passage, mu is
C                 the gravitational parameter of the primary body,
C                 and q is the perifocal distance.
C
C$ Detailed_Output
C
C     D           is the corresponding parabolic anomaly. This is the
C                 solution to the time of flight equation
C
C                                 3
C                       MA = D + D / 3
C
C$ Parameters
C
C      None.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     Iterate to solve
C
C                            3
C           f(D,MA,p) = D + D / 3 - MA = 0
C
C$ Examples
C
C     ELLTOF, HYPTOF, and PARTOF are used by CONICS.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     [1] Roger Bate, Fundamentals of Astrodynamics, Dover, 1971.
C
C$ Author_and_Institution
C
C     I.M. Underwood  (JPL)
C     W.L. Taber      (JPL)
C
C$ Version
C
C-    SPICELIB Version 2.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 2.0.0, 19-APR-1991 (WLT)
C
C        A write statement left over from debugging days was removed.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     parabolic time of flight
C
C-&
 
 
 
 
C$ Revisions
C
C-    SPICELIB Version 2.0.0, 19-APR-1991 (WLT)
C
C        A write statement left over from debugging days was removed.
C
C-    Beta Version 1.0.1, 27-JAN-1989 (IMU)
C
C        Examples section completed.
C
C-&
 
 
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
      DOUBLE PRECISION      DCBRT
 
C
C     Local parameters
C
      DOUBLE PRECISION      TOL
      PARAMETER           ( TOL = 1.D-13 )
 
C
C     Local variables
C
      DOUBLE PRECISION      M
      DOUBLE PRECISION      FN
      DOUBLE PRECISION      DERIV
      DOUBLE PRECISION      DERIV2
      DOUBLE PRECISION      CHANGE
 
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'PARTOF' )
      END IF
 
 
 
C
C     If the mean anomaly is zero, the eccentric anomaly is also zero
C     (by inspection). If the mean anomaly is negative, we can pretend
C     that it's positive (by symmetry).
C
 
      IF ( MA .EQ. 0.D0 ) THEN
         D = 0.D0
         CALL CHKOUT ( 'PARTOF' )
         RETURN
      ELSE
         M = ABS ( MA )
      END IF
 
 
C
C     We need an initial guess for the eccentric anomaly D. The function
C     is well behaved, so just about any guess will do.
C
      D = DCBRT ( 3.D0 * M )
 
 
C
C     Use the Newton second-order method,
C
C                                           2
C          F    = F  - (f/f')*(1 + f*f''/2f' )
C           i+1    i
C
C     where
C
C                     3
C          f   = D + D / 3 - M
C
C                     2
C          f'  = 1 + D
C
C
C          f'' = 2 D
C
 
      CHANGE = 1.D0
 
      DO WHILE ( ABS ( CHANGE ) .GT. TOL )
 
         FN     = D    + D**3/3.D0 - M
         DERIV  = 1.D0 + D**2
         DERIV2 = 2.D0 * D
 
         CHANGE = (FN/DERIV) * ( 1.D0 + (FN*DERIV2) / (2.D0*DERIV**2) )
         D      = D - CHANGE
 
      END DO
 
      IF ( MA .LT. 0.D0 ) THEN
         D = -D
      END IF
 
 
      CALL CHKOUT ( 'PARTOF' )
      RETURN
      END
