C$Procedure      PXFORM ( Position Transformation Matrix )
 
      SUBROUTINE PXFORM ( FROM, TO, ET, ROTATE )
 
C$ Abstract
C
C     Return the matrix that transforms position vectors from one
C     specified frame to another at a specified epoch.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      FRAMES
C
C$ Declarations
 
      IMPLICIT NONE
      CHARACTER*(*)         FROM
      CHARACTER*(*)         TO
      DOUBLE PRECISION      ET
      DOUBLE PRECISION      ROTATE  ( 3, 3 )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     FROM       I   Name of the frame to transform from.
C     TO         I   Name of the frame to transform to.
C     ET         I   Epoch of the rotation matrix.
C     ROTATE     O   A rotation matrix
C
C$ Detailed_Input
C
C     FROM        is the name of some reference frame in which
C                 a position vector is known.
C
C     TO          is the name of a reference frame in which it
C                 is desired to represent a position vector.
C
C     ET          is the epoch in ephemeris seconds past the epoch
C                 of J2000 (TDB) at which the position transformation
C                 matrix ROTATE should be evaluated.
C
C$ Detailed_Output
C
C     ROTATE      is the matrix that transforms position vectors from
C                 the reference frame FROM to the frame TO at epoch ET.
C                 If (x, y, z) is a position relative to the frame FROM
C                 then the vector ( x', y', z') is the same position
C                 relative to the frame TO at epoch ET.  Here the
C                 vector ( x', y', z' ) is defined by the equation:
C
C                   -   -       -        -     -  -
C                  | x'  |     |          |   | x  |
C                  | y'  |     |  ROTATE  |   | y  |
C                  | z'  |  =  |          |   | z  |
C                   -   -       -        -     -  -
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If sufficient information has not been supplied via loaded
C        SPICE kernels to compute the transformation between the
C        two frames, the error will be diagnosed by a routine
C        in the call tree to this routine.
C
C     2) If either frame FROM or TO is not recognized the error
C        'SPICE(UNKNOWNFRAME)' will be signaled.
C
C$ Particulars
C
C     This routine provides the user level interface to computing
C     position transformations from one reference frame to another.
C
C     Note that the reference frames may be inertial or non-inertial.
C     However, the user must take care that sufficient SPICE kernel
C     information is loaded to provide a complete position
C     transformation path from the FROM frame to the TO frame.
C
C$ Examples
C
C     Suppose that you have geodetic coordinates of a station on the
C     surface of the earth and that you need the inertial (J2000)
C     position of this station.  The following code fragment
C     illustrates how to transform the position of the station to a
C     J2000 position.
C
C        CALL BODVAR ( 399, RADII, N, ABC  )
C
C        EQUATR   =  ABC(1)
C        POLAR    =  ABC(3)
C        F        = (POLAR - EQUATR) / EQUATR
C
C        CALL GEOREC ( LONG, LAT, 0.0D0,  EQUATR, F, EPOS )
C
C        CALL PXFORM ( 'IAU_EARTH', 'J2000', ET,  ROTATE  )
C        CALL MXV    (  ROTATE,      EPOS,   JPOS         )
C
C     The state JPOS is the desired J2000 position of the station.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     C.H. Acton      (JPL)
C     N.J. Bachman    (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 29-JUL-2003 (NJB) (CHA)
C
C        Various header corrections were made.
C
C-    SPICELIB Version 1.0.0, 05-APR-1999 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Find a position transformation matrix
C
C-&
 
C
C     Spicelib Functions
C
      LOGICAL               RETURN
 
C
C     Local Variables.
C
      INTEGER               FCODE
      INTEGER               TCODE
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN ( 'PXFORM' )
 
 
      CALL NAMFRM ( FROM, FCODE )
      CALL NAMFRM ( TO,   TCODE )
C
C     Only non-zero id-codes are legitimate frame id-codes.  Zero
C     indicates that the frame wasn't recognized.
C
      IF      ( FCODE .NE. 0 .AND. TCODE .NE. 0 ) THEN
 
         CALL REFCHG ( FCODE, TCODE, ET, ROTATE )
 
      ELSE IF ( FCODE .EQ. 0 .AND. TCODE .EQ. 0 ) THEN
 
         CALL SETMSG ( 'Neither of the frames # or # was '
     .   //            'recognized as a known reference frame. ' )
         CALL ERRCH  ( '#', FROM )
         CALL ERRCH  ( '#', TO   )
         CALL SIGERR ( 'SPICE(UNKNOWNFRAME)'  )
 
      ELSE IF ( FCODE .EQ. 0 ) THEN
 
         CALL SETMSG ( 'The frame # was not recognized as a '
     .   //            'known reference frame. ' )
         CALL ERRCH  ( '#', FROM )
         CALL SIGERR ( 'SPICE(UNKNOWNFRAME)'  )
 
      ELSE IF ( TCODE .EQ. 0 ) THEN
 
         CALL SETMSG ( 'The frame # was not recognized as a '
     .   //            'known reference frame. ' )
         CALL ERRCH  ( '#', TO   )
         CALL SIGERR ( 'SPICE(UNKNOWNFRAME)'  )
 
      END IF
 
      CALL CHKOUT ( 'PXFORM' )
      RETURN
      END
