C$Procedure QXQ (Quaternion times quaternion)

      SUBROUTINE QXQ ( Q1, Q2, QOUT )

C$ Abstract
C
C     Multiply two quaternions.
C     
C$ Copyright
C
C     Copyright (2002), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     ROTATION
C 
C$ Keywords
C
C     MATH
C     POINTING
C     ROTATION
C
C$ Declarations

      IMPLICIT NONE

      DOUBLE PRECISION      Q1   ( 0 : 3 )
      DOUBLE PRECISION      Q2   ( 0 : 3 )
      DOUBLE PRECISION      QOUT ( 0 : 3 )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     Q1         I   First SPICE quaternion factor.
C     Q2         I   Second SPICE quaternion factor.
C     QOUT       O   Product of Q1 and Q2.
C
C$ Detailed_Input
C
C     Q1             is a 4-vector representing a SPICE-style
C                    quaternion.
C
C                    Note that multiple styles of quaternions
C                    are in use.  This routine will not work properly
C                    if the input quaternions do not conform to
C                    the SPICE convention.  See the Particulars
C                    section for details.
C
C     Q2             is a second SPICE quaternion.
C
C$ Detailed_Output
C
C     QOUT           is 4-vector representing the quaternion product 
C
C                       Q1 * Q2
C
C                    Representing Q(i) as the sums of scalar (real)
C                    part s(i) and vector (imaginary) part v(i)
C                    respectively,
C
C                       Q1 = s1 + v1
C                       Q2 = s2 + v2
C
C                    QOUT has scalar part s3 defined by
C
C                       s3 = s1 * s2 - <v1, v2>
C
C                    and vector part v3 defined by
C
C                       v3 = s1 * v2  +  s2 * v1  +  v1 x v2
C
C                    where the notation < , > denotes the inner
C                    product operator and x indicates the cross
C                    product operator.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Particulars
C
C     There are (at least) two popular "styles" of quaternions; these
C     differ in the layout of the quaternion elements, the definition
C     of the multiplication operation, and the mapping between the set
C     of unit quaternions and corresponding rotation matrices.
C
C     SPICE-style quaternions have the scalar part in the first
C     component and the vector part in the subsequent components. The
C     SPICE convention, along with the multiplication rules for SPICE
C     quaternions, are those used by William Rowan Hamilton, the
C     inventor of quaternions.
C
C     Another common quaternion style places the scalar component
C     last.  This style is often used in engineering applications.
C
C     The correspondence between SPICE quaternions and rotation
C     matrices is defined as follows:  Let R be a rotation matrix that
C     transforms vectors from a right-handed, orthogonal reference
C     frame F1 to a second right-handed, orthogonal reference frame F2.
C     If a vector V has components x, y, z in the frame F1, then V has
C     components x', y', z' in the frame F2, and R satisfies the
C     relation:
C
C        [ x' ]     [       ] [ x ]
C        | y' |  =  |   R   | | y |
C        [ z' ]     [       ] [ z ]
C
C
C     Letting Q = (q0, q1, q2, q3) be the SPICE unit quaternion
C     representing R, we have the relation
C
C             R  =
C
C        +-                                                          -+
C        |           2    2                                           |
C        | 1 - 2 ( q2 + q3 )    2 (q1 q2 - q0 q3)   2 (q1 q3 + q0 q2) |
C        |                                                            |
C        |                                                            |
C        |                                2    2                      |
C        | 2 (q1 q2 + q0 q3)    1 - 2 ( q1 + q3 )   2 (q2 q3 - q0 q1) |
C        |                                                            |
C        |                                                            |
C        |                                                    2    2  |
C        | 2 (q1 q3 - q0 q2)    2 (q2 q3 + q0 q1)   1 - 2 ( q1 + q2 ) |
C        |                                                            |
C        +-                                                          -+
C
C
C     To map the rotation matrix R to a unit quaternion, we start by
C     decomposing the rotation matrix as a sum of symmetric
C     and skew-symmetric parts:
C
C                                        2
C        R = [ I  +  (1-cos(theta)) OMEGA  ] + [ sin(theta) OMEGA ]
C
C                     symmetric                   skew-symmetric
C
C
C     OMEGA is a skew-symmetric matrix of the form
C
C                   +-             -+
C                   |  0   -n3   n2 |
C                   |               |
C         OMEGA  =  |  n3   0   -n1 |
C                   |               |
C                   | -n2   n1   0  |
C                   +-             -+
C
C     The vector N of matrix entries (n1, n2, n3) is the rotation axis
C     of R and theta is R's rotation angle.  Note that N and theta
C     are not unique.
C
C     Let
C
C        C = cos(theta/2)
C        S = sin(theta/2)
C
C     Then the unit quaternions Q corresponding to R are
C
C        Q = +/- ( C, S*n1, S*n2, S*n3 )
C
C     The mappings between quaternions and the corresponding rotations
C     are carried out by the SPICELIB routines
C
C        Q2M {quaternion to matrix}
C        M2Q {matrix to quaternion}
C
C     M2Q always returns a quaternion with scalar part greater than
C     or equal to zero.
C
C$ Examples
C
C     1)  Let QID, QI, QJ, QK be the "basis" quaternions
C
C            QID  =  ( 1, 0, 0, 0 )
C            QI   =  ( 0, 1, 0, 0 )
C            QJ   =  ( 0, 0, 1, 0 )
C            QK   =  ( 0, 0, 0, 1 )
C
C         respectively.  Then the calls
C
C            CALL QXQ ( QI, QJ, IXJ )
C            CALL QXQ ( QJ, QK, JXK )
C            CALL QXQ ( QK, QI, KXI )
C
C         produce the results
C
C            IXJ = QK
C            JXK = QI
C            KXI = QJ
C
C         All of the calls
C
C            CALL QXQ ( QI, QI, QOUT )
C            CALL QXQ ( QJ, QJ, QOUT )
C            CALL QXQ ( QK, QK, QOUT )
C
C         produce the result
C
C            QOUT  =  -QID
C
C         For any quaternion Q, the calls
C
C            CALL QXQ ( QID, Q,   QOUT )
C            CALL QXQ ( Q,   QID, QOUT )
C
C         produce the result
C
C            QOUT  =  Q
C
C
C
C     2)  Composition of rotations:  let CMAT1 and CMAT2 be two
C         C-matrices (which are rotation matrices).  Then the
C         following code fragment computes the product CMAT1 * CMAT2:
C
C
C            C
C            C     Convert the C-matrices to quaternions.
C            C
C                  CALL M2Q ( CMAT1, Q1 )
C                  CALL M2Q ( CMAT2, Q2 )
C
C            C
C            C     Find the product.
C            C
C                  CALL QXQ ( Q1, Q2, QOUT )
C
C            C
C            C     Convert the result to a C-matrix.
C            C
C                  CALL Q2M ( QOUT, CMAT3 )
C
C            C
C            C     Multiply CMAT1 and CMAT2 directly.
C            C
C                  CALL MXM ( CMAT1, CMAT2, CMAT4 )
C
C            C
C            C     Compare the results.  The difference DIFF of
C            C     CMAT3 and CMAT4 should be close to the zero
C            C     matrix.
C            C
C                  CALL VSUBG ( 9, CMAT3, CMAT4, DIFF )
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 18-AUG-2002 (NJB)
C
C
C-&
 
C$ Index_Entries
C
C     quaternion times quaternion
C     multiply quaternion by quaternion
C-&


C
C     SPICELIB functions
C
      DOUBLE PRECISION      VDOT

C
C     Local variables
C
      DOUBLE PRECISION      CROSS ( 3 )


C
C     Compute the scalar part of the product.
C
      QOUT(0)  =  Q1(0) * Q2(0)  -  VDOT( Q1(1), Q2(1) )

C
C     And now the vector part.  The SPICELIB routine VLCOM3 computes
C     a linear combination of three 3-vectors.
C
      CALL VCRSS  ( Q1(1),  Q2(1),  CROSS )

      CALL VLCOM3 ( Q1(0),  Q2(1), 
     .              Q2(0),  Q1(1),
     .              1.D0,   CROSS,  QOUT(1) )

      RETURN
      END


      
