C$Procedure REPMI  ( Replace marker with integer )
 
      SUBROUTINE REPMI ( IN, MARKER, VALUE, OUT )
 
C$ Abstract
C
C     Replace a marker with an integer.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     CHARACTER
C     CONVERSION
C     STRING
C
C$ Declarations
 
      CHARACTER*(*)         IN
      CHARACTER*(*)         MARKER
      INTEGER               VALUE
      CHARACTER*(*)         OUT
 
      INTEGER               MAXLI
      PARAMETER           ( MAXLI = 11 )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     IN         I   Input string.
C     MARKER     I   Marker to be replaced.
C     VALUE      I   Replacement value.
C     OUT        O   Output string.
C     MAXLI      P   Maximum length of an integer.
C
C$ Detailed_Input
C
C     IN             is an arbitrary character string.
C
C     MARKER         is an arbitrary character string. The first
C                    occurrence of MARKER in the input string is
C                    to be replaced by VALUE.
C
C                    Leading and trailing blanks in MARKER are NOT
C                    significant. In particular, no substitution is
C                    performed if MARKER is blank.
C
C     VALUE          is an arbitrary integer.
C
C$ Detailed_Output
C
C     OUT            is the string obtained by substituting the text
C                    representation of VALUE for the first occurrence
C                    of MARKER in the input string.
C
C                    OUT and IN must be identical or disjoint.
C
C$ Parameters
C
C     MAXLI          is the maximum expected length of the text
C                    representation of an integer. 11 characters are
C                    sufficient to hold any integer whose absolute
C                    value is less than 10 billion.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error Free.
C
C     1) If OUT does not have sufficient length to accommodate the
C        result of the substitution, the result will be truncated on
C        the right.
C
C     2) If MARKER is blank, or if MARKER is not a substring of IN,
C        no substitution is performed. (OUT and IN are identical.)
C
C$ Particulars
C
C     This is one of a family of related routines for inserting values
C     into strings. They are typically to construct messages that
C     are partly fixed, and partly determined at run time. For example,
C     a message like
C
C        'Fifty-one pictures were found in directory [USER.DATA].'
C
C     might be constructed from the fixed string
C
C        '#1 pictures were found in directory #2.'
C
C     by the calls
C
C        CALL REPMCT ( STRING, '#1', N_PICS,  'C', STRING )
C        CALL REPMC  ( STRING, '#2', DIR_NAME,     STRING )
C
C     which substitute the cardinal text 'Fifty-one' and the character
C     string '[USER.DATA]' for the markers '#1' and '#2' respectively.
C
C     The complete list of routines is shown below.
C
C        REPMC    ( Replace marker with character string value )
C        REPMD    ( Replace marker with double precision value )
C        REPMF    ( Replace marker with formatted d.p. value )
C        REPMI    ( Replace marker with integer value )
C        REPMCT   ( Replace marker with cardinal text)
C        REPMOT   ( Replace marker with ordinal text )
C
C$ Examples
C
C     1. Let
C
C           IN = 'Invalid operation value.  The value was #.'
C
C        Then following the call,
C
C           CALL REPMI ( IN, '#', 5, IN  )
C
C        IN is
C
C           'Invalid operation value.  The value was 5.'
C
C
C     2. Let
C
C           IN = 'Left endpoint exceeded right endpoint.
C                 The left endpoint was:  XX.  The right
C                 endpoint was:  XX.'
C
C        Then following the call,
C
C           CALL REPMI ( IN, '  XX  ', 5, OUT )
C
C        OUT is
C
C           'Left endpoint exceeded right endpoint.  The left
C            endpoint was:  5.  The right endpoint was:  XX.
C
C
C     3. Let
C
C           NUM    = 23
C           CHANCE = 'fair'
C           SCORE  = 4.665D0
C
C        Then following the sequence of calls,
C
C           CALL REPMI ( 'There are & routines that have a '  //
C          .             '& chance of meeting your needs.'    //
C          .             'The maximum score was &.',
C          .             '&',
C          .             NUM,
C          .             MSG  )
C
C           CALL REPMC ( MSG, '&', CHANCE, MSG )
C
C           CALL REPMF ( MSG, '&', SCORE, 4, 'F', MSG )
C
C        MSG is
C
C           'There are 23 routines that have a fair chance of
C            meeting your needs.  The maximum score was 4.665.'
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman   (JPL)
C     I.M. Underwood (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 15-AUG-2002 (WLT)
C
C        The routine is now error free.
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 30-AUG-1990 (NJB) (IMU)
C
C-&
 
C$ Index_Entries
C
C     replace marker with integer
C
C-&
 
 
 
 
 
C
C     SPICELIB functions
C
      INTEGER               FRSTNB
      INTEGER               LASTNB
 
C
C     Local variables
C
      CHARACTER*(MAXLI)     SUBSTR
 
      INTEGER               MRKPOS
 
 
 

C
C     If MARKER is blank, no substitution is possible.
C
      IF  ( MARKER .EQ. ' ' ) THEN
 
         OUT = IN
         RETURN
 
      END IF
 
C
C     Locate the leftmost occurrence of MARKER, if there is one
C     (ignoring leading and trailing blanks). If MARKER is not
C     a substring of IN, no substitution can be performed.
C
      MRKPOS = INDEX (  IN,
     .                  MARKER ( FRSTNB(MARKER):LASTNB(MARKER) )    )
 
      IF  ( MRKPOS .EQ. 0 ) THEN
 
         OUT = IN
 
         RETURN
 
      END IF
 
C
C     Okay, MARKER is non-blank and has been found. Convert the
C     integer to text, and substitute the text for the marker.
C
      CALL INTSTR ( VALUE, SUBSTR )
 
      CALL ZZREPSUB ( IN,
     .                MRKPOS,
     .                MRKPOS + LASTNB(MARKER) - FRSTNB(MARKER),
     .                SUBSTR( :LASTNB(SUBSTR) ),
     .                OUT  )
 
      RETURN
      END
