C$Procedure      RETURN ( Immediate Return Indicator )
 
      LOGICAL FUNCTION RETURN ()
 
C$ Abstract
C
C     True if SPICELIB routines should return immediately upon entry.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      ERROR
C
C$ Keywords
C
C      ERROR
C
C$ Declarations
C
C      None.
C
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C
C      The function returns the value, .TRUE., if and only if SPICELIB
C      routines should return immediately upon entry.
C
C$ Detailed_Input
C
C      None.
C
C$ Detailed_Output
C
C      The function returns the value, .TRUE., if and only if SPICELIB
C      routines should return immediately upon entry.  The criterion
C      for this is that the error response action is set to
C      'RETURN', and an error condition exists.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C      This routine does not detect any errors.
C
C      However, this routine is part of the SPICELIB error
C      handling mechanism.
C
C$ Files
C
C      None.
C
C$ Particulars
C
C      Please read the "required reading" first!
C
C      This routine can be referenced in non-toolkit code; in
C      fact, its use is encouraged.  Its purpose is to signal
C      to the routine calling it that the caller should
C      return immediately.  The reference to RETURN should
C      be the first executable line of the calling program.
C
C      In 'RETURN' mode, SPICELIB routines
C      that have external references, or that can
C      detect errors, return immediately upon entry when an
C      error condition exists.  They use RETURN to determine
C      when these conditions are met.  Non--toolkit routines
C      can do the same.
C
C      Additionally, when an error is signalled in 'RETURN' mode,
C      no further errors can be signalled until the error condition
C      is reset by a call to RESET.  Calls to SIGERR simply have
C      no effect.  Therefore, the error messages set in response
C      to the FIRST error that was detected will be saved until
C      RESET is called.  These messages can be retrieved by
C      calls to GETMSG.
C
C      There are a number of advantages to using this mechanism.
C      First, the likelihood of an error resulting in crash
C      in a different routine is greatly reduced.  Second,
C      a program does not have to test the error status
C      (using a reference to FAILED) after each call to a toolkit
C      routine, but rather can make one test of status at the end
C      of a series of calls.  See "Examples" below.
C
C      See the subroutine ERRACT for definitions of the error action 
C      codes.
C
C$ Examples
C
C      1.  In this example, we show how to place a reference
C          to RETURN in your code:
C
C          C
C          C     No executable lines precede this one.
C          C
C          C     Test whether to return before doing
C          C     anything else.
C          C
C
C                IF ( RETURN() )  RETURN
C
C
C                [ rest of code goes here]
C
C                          .
C                          .
C                          .
C
C
C      2.  Here's how one might code a sequence of calls
C          to routines with code that follows the pattern
C          given in example #1 above:
C
C                         .
C                         .
C                         .
C
C                [ code may go here ]
C
C          C
C          C     We call routines A, B, and C;  then we
C          C     test for errors, using the SPICELIB error
C          C     status indicator, FAILED:
C          C
C
C                CALL  A
C                CALL  B
C                CALL  C
C
C                IF ( FAILED() ) THEN
C
C          C
C          C        If we're here, an error occurred.  The
C          C        error might have been detected by A, B, C,
C          C        or by a routine called by one of them.
C          C        Get the explanation of the short error message
C          C        and output it using the routine, USER_OUT
C          C        [USER_OUT is a fictitious routine]:
C          C
C
C                   CALL GETMSG ( 'EXPLAIN', MSG )
C
C                   CALL USER_OUT ( MSG )
C
C                END IF
C
C                [ rest of code goes here ]
C
C                           .
C                           .
C                           .
C
C
C
C$ Restrictions
C
C      This routine has no effect unless the error action is 'RETURN'!
C
C$ Literature_References
C
C      None.
C
C$ Author_and_Institution
C
C      N.J. Bachman    (JPL)
C      K.R. Gehringer  (JPL)
C
C$ Version
C
C-     SPICELIB Version 2.0.0, 22-APR-1996 (KRG)
C
C         This subroutine has been modified in an attempt to improve 
C         the general performance of the SPICELIB error handling 
C         mechanism. The specific modification has been to change the 
C         type of error action from a short character string to an 
C         integer. This change is backwardly incompatible because the 
C         type has changed.
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (NJB)
C
C-&
 
C$ Index_Entries
C
C     immediate return indicator
C
C-&

C$ Revisions
C
C-     SPICELIB Version 2.0.0, 22-APR-1996 (KRG)
C
C         This subroutine has been modified in an attempt to improve 
C         the general performance of the SPICELIB error handling 
C         mechanism. The specific modification has been to change the 
C         type of error action from a short character string to an 
C         integer. This change is backwardly incompatible because the 
C         type has changed.
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     Beta Version 1.1.0, 17-FEB-1989 (NJB)
C
C         Added parentheses to the declaration of RETURN.
C
C-&

C
C     SPICELIB functions
C
      LOGICAL               FAILED

C
C     Local Parameters
C
C     Define the mnemonic for the return action.
C
      INTEGER               IRETRN
      PARAMETER           ( IRETRN = 3 )
C
C     Local Variables
C
      INTEGER               ACTION
C
C     Immediate return is indicated only in 'RETURN' mode,
C     when an error condition is in effect:
C
      CALL GETACT ( ACTION )
 
      RETURN  =  ( ACTION .EQ. IRETRN ) .AND. FAILED()
 
      RETURN
      END
