C$Procedure      SCPART ( Spacecraft Clock Partition Information )
 
      SUBROUTINE SCPART ( SC, NPARTS, PSTART, PSTOP )
 
C$ Abstract
C
C     Get spacecraft clock partition information from a spacecraft
C     clock kernel file.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SCLK
C
C$ Keywords
C
C     TIME
C
C$ Declarations
 
      INTEGER               SC
      INTEGER               NPARTS
      DOUBLE PRECISION      PSTART  ( * )
      DOUBLE PRECISION      PSTOP   ( * )
 
      INTEGER               MXPART
      PARAMETER           ( MXPART = 50 )
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     SC         I   NAIF spacecraft identification code.
C     NPARTS     O   The number of spacecraft clock partitions.
C     PSTART     O   Array of partition start times.
C     PSTOP      O   Array of partition stop times.
C     MXPART     P   Maximum number of partitions.
C
C$ Detailed_Input
C
C     SC         is the NAIF ID for the spacecraft whose clock partition
C                information is being requested.
C
C$ Detailed_Output
C
C     NPARTS     is the number of spacecraft clock time partitions
C                described in the kernel file for spacecraft SC.
C
C     PSTART     is an array containing NPARTS partition start times
C                represented as encoded ticks.  The values contained
C                in PSTART are whole numbers.
C
C     PSTOP      is an array containing NPARTS partition end times
C                represented as encoded ticks. The values contained
C                in PSTOP are whole numbers.
C
C$ Parameters
C
C     MXPART     is the maximum number of partitions for any
C                spacecraft clock.  SCLK kernels contain start
C                and stop times for each partition. The value
C                of MXPART may be increased as required.
C
C$ Exceptions
C
C     1)  If the kernel variables containing the spacecraft clock
C         partition start and stop times have not been loaded in the
C         kernel pool, the error will be diagnosed by routines called
C         by this routine.
C
C     2)  If the number of start and stop times are different then
C         the error SPICE(NUMPARTSUNEQUAL) is signalled.
C
C$ Files
C
C     An SCLK kernel containing spacecraft clock partition start
C     and stop times for the spacecraft clock indicated by SC must
C     be loaded into the kernel pool.
C
C$ Particulars
C
C     SCPART looks for two variables in the kernel pool for each
C     spacecraft's partition information. If SC = -nn, then the names of
C     the variables are
C
C         'SCLK_PARTITION_START_nn' and
C         'SCLK_PARTITION_END_nn'.
C
C     The start and stop times returned are in encoded units of "ticks".
C
C$ Examples
C
C     1)  The following program fragment finds and prints out partition
C         start and stop times in clock format for the Galileo mission.
C         In this example, Galileo partition times are assumed to be
C         in the kernel file SCLK.KER.
C
C            CHARACTER*(30)        START
C            CHARACTER*(30)        STOP
C
C            CALL LDPOOL ( 'SCLK.KER' )
C
C            SC = -77
C
C            CALL SCPART ( SC, NPARTS, PSTART, PSTOP )
C
C            DO I = 1, NPARTS
C
C               CALL SCFMT ( SC, PSTART( I ), START )
C               CALL SCFMT ( SC, PSTOP ( I ), STOP  )
C
C               WRITE (*,*)
C               WRITE (*,*) 'Partition ', I, ':'
C               WRITE (*,*) 'Start = ', START
C               WRITE (*,*) 'Stop  = ', STOP
C
C            END DO
C
C$ Restrictions
C
C     1) This routine assumes that an SCLK kernel appropriate to the
C        spacecraft identified by SC has been loaded into the kernel
C        pool.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman   (JPL)
C     J.M. Lynch     (JPL)
C     R.E. Thurman   (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 22-MAR-1993 (JML)
C
C        The routine now uses the kernel pool watch capability.
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 03-SEP-1990 (NJB) (JML) (RET)
C
C-&
 
C$ Index_Entries
C
C     spacecraft_clock partition information
C
C-&
 
C$ Revisions
C
C-    SPICELIB Version 1.1.0, 22-MAR-1993 (JML)
C
C        The routine now uses the kernel pool watch capability.
C
C-&
 
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
 
C
C     Local parameters
C
      CHARACTER*(*)         NMPSTR
      PARAMETER           ( NMPSTR = 'SCLK_PARTITION_START' )
 
      CHARACTER*(*)         NMPSTP
      PARAMETER           ( NMPSTP = 'SCLK_PARTITION_END'   )
 
C
C     Local variables
C
      CHARACTER*(60)        KVNAME  ( 2 )
 
      DOUBLE PRECISION      PRTSA   ( MXPART )
      DOUBLE PRECISION      PRTSO   ( MXPART )
 
      INTEGER               I
      INTEGER               LSTPRT
      INTEGER               NPRTSA
      INTEGER               NPRTSO
      INTEGER               OLDSC
 
      LOGICAL               FIRST
      LOGICAL               UPDATE
 
      SAVE                  FIRST
      SAVE                  OLDSC
      SAVE                  LSTPRT
      SAVE                  PRTSA
      SAVE                  PRTSO
 
C
C     Initial values.
C
      DATA                  FIRST   / .TRUE. /
      DATA                  OLDSC   / 0      /
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SCPART' )
      END IF
 
C
C     On the first pass through the subroutine, or if the
C     spacecraft code changes, set watches on the SCLK kernel
C     variables for the current clock.
C
      IF (  FIRST  .OR.  ( SC .NE. OLDSC )  ) THEN
C
C        Make up a list of names of kernel variables that we'll use.
C
         KVNAME(1) = NMPSTR
         KVNAME(2) = NMPSTP
 
         DO I = 1, 2
            CALL SUFFIX ( '_#',        0,         KVNAME(I) )
            CALL REPMI  ( KVNAME(I),  '#',  -SC,  KVNAME(I) )
         END DO
C
C        Set a watch on all of the kernel variables used.
C
         CALL SWPOOL ( 'SCPART',  2, KVNAME )
C
C        Keep track of the last spacecraft ID encountered.
C
         OLDSC =  SC
 
         FIRST = .FALSE.
 
      END IF
 
C
C     If any of the kernel pool variables that this routine uses
C     have been updated, or if the spacecraft ID changes, look up
C     the new values from the kernel pool.
C
      CALL CVPOOL ( 'SCPART', UPDATE )
 
      IF ( UPDATE ) THEN
C
C        Read the values from the kernel pool.
C
         CALL SCLD01 ( NMPSTR, SC, MXPART, NPRTSA, PRTSA )
         CALL SCLD01 ( NMPSTP, SC, MXPART, NPRTSO, PRTSO )
C
C        Error checking.
C
         IF ( NPRTSA .NE. NPRTSO  ) THEN
            CALL SETMSG ( 'The number of partition start and stop '  //
     .                    'times are unequal for spacecraft #.    '    )
            CALL ERRINT ( '#', SC                                      )
            CALL SIGERR ( 'SPICE(NUMPARTSUNEQUAL)'                     )
            CALL CHKOUT ( 'SCPART'                                     )
            RETURN
         END IF
 
C
C        Buffer the number of partitions and the partition start
C        and stop times.
C
         LSTPRT = NPRTSA
C
C        The partition start and stop times must be whole numbers.
C
         DO I = 1, LSTPRT
            PRTSA(I) = ANINT ( PRTSA(I)  )
            PRTSO(I) = ANINT ( PRTSO(I)  )
         END DO
 
      END IF
 
C
C     Copy the values in local buffers to the output arguments.
C
      NPARTS = LSTPRT
 
      DO I = 1, NPARTS
         PSTART(I) = PRTSA(I)
         PSTOP(I)  = PRTSO(I)
      END DO
 
      CALL CHKOUT ( 'SCPART' )
      RETURN
      END
