C$Procedure      SCTYPE ( SCLK type )
 
      INTEGER FUNCTION SCTYPE ( SC )
 
C$ Abstract
C
C     Return the spacecraft clock type for a specified spacecraft.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SCLK
C
C$ Keywords
C
C     TIME
C
C$ Declarations
 
      INTEGER               SC
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     SC         I   NAIF spacecraft ID code.
C
C     The function returns the spacecraft clock type associated with the
C     spacecraft specified by SC.
C
C$ Detailed_Input
C
C     SC             is a NAIF ID code for a spacecraft, whose
C                    spacecraft clock `type' is desired.
C
C$ Detailed_Output
C
C     The function returns the spacecraft clock type associated with the
C     spacecraft specified by SC.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1)  If the kernel variable that assigns a SCLK type to the
C         spacecraft specified by SC is not found in the kernel pool,
C         the error is diagnosed by routines called by this routine.
C         SCTYPE returns the value 0 in this case.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     The raison d'etre of this routine is that it consolidates the code
C     that maps spacecraft ID's to clock types.  While any routine may
C     call SCTYPE, it is unlikely that there will be a need for
C     non-SPICELIB routines to call this routine directly.
C
C$ Examples
C
C     1)  Find the SCLK type for Galileo.
C
C            During program initialization, we load a SCLK kernel file
C            into the kernel pool.  We will pretend that the name of
C            this file is GLLSCLK.KER.  You must use the actual name of
C            an SCLK kernel that is accessible by your program to try
C            this example.
C
C                C
C                C     Load the SCLK kernel.
C                C
C                      CALL LDPOOL ( 'GLLSCLK.KER' )
C                                 .
C                                 .
C                                 .
C                C
C                C     Print out the clock type for Galileo.
C                C
C                      TYPE = SCTYPE ( -77 )
C
C                      PRINT *, 'Galileo clock type is ', TYPE
C
C
C     2)  Find the SCLK type for Mars Observer.
C
C
C                C
C                C     Load the SCLK kernel.
C                C
C                      CALL LDPOOL ( 'MOSCLK.KER' )
C                                 .
C                                 .
C                                 .
C                C
C                C     Print out the clock type for Mars Observer.
C                C
C                      TYPE = SCTYPE ( -94 )
C
C                      PRINT *, 'Mars Observer clock type is ', TYPE
C
C$ Restrictions
C
C     This routine assumes that an SCLK kernel appropriate to the
C     spacecraft specified by SC has been loaded into the kernel pool.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman   (JPL)
C     J.M. Lynch     (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 22-MAR-1993 (JML)
C
C        1) The routine now uses the kernel pool watch capability.
C
C        2) The routine now returns a value of zero if RETURN is
C           true on entry.
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 04-SEP-1990 (NJB)
C
C-&
 
C$ Index_Entries
C
C     spacecraft_clock type
C
C-&
 
C$ Revisions
C
C-    SPICELIB Version 1.1.0, 22-MAR-1993 (JML)
C
C        1) The routine now uses the kernel pool watch capability.
C
C        2) The routine now returns a value of zero if RETURN is
C           true on entry.
C
C-&
 
 
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
 
C
C     Local parameters
C
      CHARACTER*(*)         TYPLBL
      PARAMETER           ( TYPLBL = 'SCLK_DATA_TYPE' )
 
C
C     Local variables
C
      CHARACTER*(60)        KVNAME
 
      INTEGER               N
      INTEGER               TYPE
      INTEGER               OLDSC
 
      LOGICAL               FIRST
      LOGICAL               UPDATE
 
C
C     Save some variables.
C
      SAVE     TYPE
      SAVE     OLDSC
      SAVE     FIRST
 
 
C
C     Initial values.
C
      DATA                  FIRST   / .TRUE. /
      DATA                  OLDSC   / 0      /
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
 
         SCTYPE = 0
 
         RETURN
 
      ELSE
 
         CALL CHKIN ( 'SCTYPE' )
 
      END IF
 
C
C     On the first pass through the subroutine, or if the spacecraft
C     ID code changes, set a watch on the SCLK kernel variable for
C     the current clock type.
C
      IF (  FIRST  .OR.  ( SC .NE. OLDSC )  ) THEN
C
C        Construct the name of the kernel variable that is needed.
C
         KVNAME = TYPLBL
 
         CALL SUFFIX ( '_#',     0,         KVNAME )
         CALL REPMI  ( KVNAME,  '#',  -SC,  KVNAME )
 
C
C        Set a watch on the kernel variable needed.
C
         CALL SWPOOL ( 'SCTYPE', 1, KVNAME )
C
C        Keep track of the last spacecraft ID encountered.
C
         OLDSC =  SC
 
         FIRST = .FALSE.
 
      END IF
 
C
C     If the kernel pool variable that this routine uses has
C     been updated, or if the spacecraft id code changes, look
C     up the new value from the kernel pool.
C
      CALL CVPOOL ( 'SCTYPE',  UPDATE )
 
      IF ( UPDATE ) THEN
C
C        Find the clock type for the specified mission.
C
         TYPE   = 0
 
         CALL SCLI01 ( TYPLBL, SC, 1, N, TYPE )
 
      END IF
 
      SCTYPE = TYPE
 
      CALL CHKOUT ( 'SCTYPE' )
      RETURN
      END
