C$Procedure      SPHCYL ( Spherical to cylindrical coordinates )
 
      SUBROUTINE SPHCYL ( RADIUS, COLAT, SLONG, R, LONG, Z )
 
C$ Abstract
C
C      This routine converts from spherical coordinates to cylindrical
C      coordinates.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C      CONVERSION,  COORDINATES
C
C$ Declarations
 
      DOUBLE PRECISION   RADIUS
      DOUBLE PRECISION   COLAT
      DOUBLE PRECISION   SLONG
      DOUBLE PRECISION   R
      DOUBLE PRECISION   LONG
      DOUBLE PRECISION   Z
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  -------------------------------------------------
C      RADIUS     I   Distance of point from origin.
C      COLAT      I   Polar angle (co-latitude in radians) of point.
C      SLONG      I   Azimuthal angle (longitude) of point (radians).
C      R          O   Distance of point from Z axis.
C      LONG       O   angle (radians) of point from XZ plane.
C      Z          O   Height of point above XY plane.
C
C$ Detailed_Input
C
C      RADIUS     Distance of the point from origin.
C
C      COLAT      Polar angle (co-latitude in radians) of the point.
C
C      SLONG      Azimuthal angle (longitude) of the point (radians).
C
C$ Detailed_Output
C
C      R          Distance of the point of interest from Z axis.
C
C      LONG       cylindrical angle (radians) of the point from the
C                 XZ plane.
C
C      Z          Height of the point above XY plane.
C
C$ Parameters
C
C      None.
C
C$ Particulars
C
C      This returns the cylindrical coordinates of a point whose
C      position is input through spherical coordinates.
C
C$ Examples
C
C
C     Other than the obvious conversion between coordinate systems
C     this routine could be used to obtain the axial projection
C     from a sphere to a cylinder about the z-axis that contains
C     the equator of the sphere.  The following code fragment
C     illustrates this idea.
C
C           CALL SPHCYL ( RADIUS, COLAT, LONG, R, LONG, Z )
C           R = RADIUS
C
C     R, LONG, and Z now contain the the coordinates of the projected
C     point. Such a projection is valuable because it preserves the
C     areas between regions on the sphere and their projections to the
C     cylinder.
C
C$ Restrictions
C
C      None.
C
C$ Exceptions
C
C     Error free.
C
C$ Files
C
C      None.
C
C$ Author_and_Institution
C
C      W.L. Taber      (JPL)
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (WLT)
C
C-&
 
C$ Index_Entries
C
C     spherical to cylindrical coordinates
C
C-&
 
 
 
C$ Revisions
C
C-     Beta Version 1.0.1, 1-Feb-1989 (WLT)
C
C      Example section of header upgraded.
C
C-&
 
C
C     Local Variables
C
      DOUBLE PRECISION RR
      DOUBLE PRECISION ZZ
 
C
C     Convert to cylindrical coordinates, storing the results in
C     temporary variables.
C
      RR = RADIUS*DSIN(COLAT)
      ZZ = RADIUS*DCOS(COLAT)
 
C
C     Move the results to the output variables.
C
      LONG   = SLONG
      R      = RR
      Z      = ZZ
 
      RETURN
      END
