C$Procedure      SPKE02 ( Evaluate Chebyshev polynomials, type 2 )
C
      SUBROUTINE SPKE02 ( ET, RECORD, XYZDOT )
C
C$ Abstract
C
C     Evaluate a single data record from an PCK or SPK segment of type 2
C     (Chebyshev Polynomials, 3 components).
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPK
C     PCK
C
C$ Keywords
C
C     EPHEMERIS
C
C$ Declarations
C
      DOUBLE PRECISION      ET
      DOUBLE PRECISION      RECORD   ( * )
      DOUBLE PRECISION      XYZDOT   ( 6 )
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     ET         I   Target epoch.
C     RECORD     I   Data record.
C     XYZDOT     O   3 components and their derivatives.
C
C$ Detailed_Input
C
C     ET          is a target epoch, at which a state vector is to
C                 be computed.
C
C     RECORD      is a data record which, when evaluated at epoch ET,
C                 will give the 3 component and their derivatives.
C
C$ Detailed_Output
C
C     XYZDOT      is a 6-vector. In order, X, Y, Z, X', Y', and Z'.
C                 Units for state evaluations will be km and km/sec.
C                 Units for angles will be radians and radians/sec.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     The exact format and structure of type 2 (Chebyshev polynomials,
C     position only) segments are described in the SPK and PCK Required
C     Reading file.
C
C     A type 2 segment contains three sets of Chebyshev coefficients,
C     one set each for components X, Y, and Z.  SPKE02
C     calls the routine CHBINT for each set to evalute the polynomial
C     AND its first derivative (which it computes internally) at the
C     input epoch, thereby arriving at the complete state.
C
C$ Examples
C
C     The data returned by the routine is in its rawest form,
C     taken directly from the segment.  As such, it will be meaningless
C     to a user unless he/she understands the structure of the data type
C     completely.
C
C
C     C
C     C     Get a segment applicable to a specified body and epoch.
C     C
C           CALL SPKSFS ( BODY, ET, HANDLE, DESCR, IDENT, FOUND )
C
C     C
C     C     Look at parts of the descriptor.
C     C
C           CALL DAFUS ( DESCR, 2, 6, DCD, ICD )
C           CENTER = ICD( 2 )
C           REF    = ICD( 3 )
C           TYPE   = ICD( 4 )
C
C           IF ( TYPE .EQ. 2 ) THEN
C
C              CALL SPKR02 ( HANDLE, DESCR, ET, RECORD )
C                  .
C                  .  Look at the RECORD data.
C                  .
C              CALL SPKE02 ( ET, RECORD, XYZDOT )
C                  .
C                  .  Check out the evaluated state.
C                  .
C           END IF
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 168.0, "S- and P- Kernel (SPK) Specification and
C     User's Guide"
C
C$ Author_and_Institution
C
C     R.E. Thurman    (JPL)
C     K.S. Zukor      (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.4, 22-MAR-1994 (KSZ)
C
C     Comments changed so this can be used as
C     a generic Chebyshev evaluator, rather than just for
C     SPK type 2 files.  (KSZ)
C
C-    SPICELIB Version 1.0.3, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.2, 23-AUG-1991 (HAN)
C
C        SPK02 was removed from the Required_Reading section of the
C        header. The information in the SPK02 Required Reading file
C        is now part of the SPK Required Reading file.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (RET)
C
C-&
C
C$ Index_Entries
C
C     evaluate type_2 spk segment
C
C-&
 
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
 
C
C     Local variables
C
      INTEGER               COFLOC
      INTEGER               DEGP
      INTEGER               I
      INTEGER               NCOF
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SPKE02' )
      END IF
 
 
C
C     The first number in the record is the record size.  Following it
C     are two numbers that will be used later, then the three sets of
C     coefficients.  The number of coefficients for each variable can
C     be determined from the record size, since there are the same
C     number of coefficients for each variable.
C
      NCOF = ( INT( RECORD( 1 ) ) - 2 ) / 3
 
C
C     The degree of each polynomial is one less than the number of
C     coefficients.
C
      DEGP = NCOF - 1
 
C
C     Call CHBINT once for each variable to evaluate the position
C     and velocity values.
C
       DO   I = 1, 3
 
C
C        The coefficients for each variable are located contiguously,
C        following the first three words in the record.
C
         COFLOC = NCOF*( I - 1 ) + 4
 
C
C        CHBINT needs as input the coefficients, the degree of the
C        polynomial, the epoch, and also two variable transformation
C        parameters, which are located, in our case, in the second and
C        third slots of the record.
C
         CALL CHBINT ( RECORD( COFLOC ), DEGP, RECORD( 2 ), ET,
     .                 XYZDOT( I ), XYZDOT( I + 3 )                )
 
      END DO
 
      CALL CHKOUT ( 'SPKE02' )
      RETURN
      END
 
 
 
 
