C$Procedure      SPKE03 ( S/P Kernel, evaluate, type 3 )
 
      SUBROUTINE SPKE03 ( ET, RECORD, STATE )
 
C$ Abstract
C
C     Evaluate a single SPK data record from a segment of type 3
C     (Chebyshev Polynomials, position and velocity).
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPK
C
C$ Keywords
C
C     EPHEMERIS
C
C$ Declarations
 
      DOUBLE PRECISION      ET
      DOUBLE PRECISION      RECORD   ( * )
      DOUBLE PRECISION      STATE    ( 6 )
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     ET         I   Target epoch.
C     RECORD     I   Data record.
C     STATE      O   State (position and velocity).
C
C$ Detailed_Input
C
C     ET          is a target epoch, at which a state vector is to
C                 be computed.
C
C     RECORD      is a data record which, when evaluated at epoch ET,
C                 will give the state (position and velocity) of some
C                 body, relative to some center, in some inertial
C                 reference frame.
C
C$ Detailed_Output
C
C     STATE       is the state. In order, X, Y, Z, X', Y', and Z'.
C                 Units are km and km/sec.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     The exact format and structure of type 3 (Chebyshev polynomials,
C     position and velocity) segments are described in the SPK
C     Required Reading file.
C
C     A type 3 segment contains six sets of Chebyshev coefficients,
C     one set each for the position coordinates X, Y, and Z, and one
C     set each for the velocity coordinates X', Y', and Z'.  SPKE03
C     calls the routine CHBVAL to evalute each polynomial, and arrive
C     at the complete state.
C
C$ Examples
C
C     The SPKEnn routines are almost always used in conjunction with
C     the corresponding SPKRnn routines, which read the records from
C     SPK files.
C
C     The data returned by the SPKRnn routine is in its rawest form,
C     taken directly from the segment.  As such, it will be meaningless
C     to a user unless he/she understands the structure of the data type
C     completely.  Given that understanding, however, the SPKRnn
C     routines might be used to examine raw segment data before
C     evaluating it with the SPKEnn routines.
C
C
C     C
C     C     Get a segment applicable to a specified body and epoch.
C     C
C           CALL SPKSFS ( BODY, ET, HANDLE, DESCR, IDENT, FOUND )
C
C     C
C     C     Look at parts of the descriptor.
C     C
C           CALL DAFUS ( DESCR, 2, 6, DCD, ICD )
C           CENTER = ICD( 2 )
C           REF    = ICD( 3 )
C           TYPE   = ICD( 4 )
C
C           IF ( TYPE .EQ. 3 ) THEN
C
C              CALL SPKR03 ( HANDLE, DESCR, ET, RECORD )
C                  .
C                  .  Look at the RECORD data.
C                  .
C              CALL SPKE03 ( ET, RECORD, STATE )
C                  .
C                  .  Check out the evaluated state.
C                  .
C           END IF
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 168.0, "S- and P- Kernel (SPK) Specification and
C     User's Guide"
C
C$ Author_and_Institution
C
C     R.E. Thurman    (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.3, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.2, 23-AUG-1991 (HAN)
C
C        SPK03 was removed from the Required_Reading section of the
C        header. The information in the SPK03 Required Reading file
C        is now part of the SPK Required Reading file.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (RET)
C
C-&
 
C$ Index_Entries
C
C     evaluate type_3 spk segment
C
C-&
 
 
 
 
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
 
C
C     Local variables
C
      INTEGER               NCOF
      INTEGER               DEGP
      INTEGER               COFLOC
      INTEGER               I
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SPKE03' )
      END IF
 
 
C
C     The first number in the record is the record size.  Following it
C     are two numbers that will be used later, then the six sets of
C     coefficients.  The number of coefficients for each quantity can
C     be determined from the record size, since there are the same
C     number of coefficients for each quantity.
C
      NCOF = ( INT( RECORD( 1 ) ) - 2 ) / 6
 
C
C     The degree of each polynomial is one less than the number of
C     coefficients.
C
      DEGP = NCOF - 1
 
C
C     Call CHBVAL once for each quantity to evaluate the position
C     and velocity values.
C
      DO I = 1, 6
 
C
C        The coefficients for each variable are located contiguously,
C        following the first three words in the record.
C
         COFLOC = NCOF*( I - 1 ) + 4
 
C
C        CHBVAL needs as input the coefficients, the degree of the
C        polynomial, the epoch, and also two variable transformation
C        parameters, which are located, in our case, in the second and
C        third slots of the record.
C
         CALL CHBVAL ( RECORD( COFLOC ), DEGP, RECORD( 2 ), ET,
     .                 STATE( I )                               )
 
      END DO
 
      CALL CHKOUT ( 'SPKE03' )
      RETURN
      END
