C$Procedure SPKE10 ( Evaluate SPK record, type 10 )
 
      SUBROUTINE SPKE10 ( ET, RECORD, STATE )
 
C$ Abstract
C
C     Evaluate a single SPK data record from a segment of type 10
C     (NORAD two-line element sets.).
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPK
C
C$ Keywords
C
C     EPHEMERIS
C
C$ Declarations
 
      IMPLICIT NONE
      DOUBLE PRECISION      ET
      DOUBLE PRECISION      RECORD   ( * )
      DOUBLE PRECISION      STATE    ( 6 )
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     ET         I   Target epoch.
C     RECORD     I   Data record.
C     STATE      O   State (position and velocity).
C
C$ Detailed_Input
C
C     ET          is a target epoch, specified as ephemeris seconds past
C                 J2000, at which a state vector is to be computed.
C
C     RECORD      is a data record which, when evaluated at epoch ET,
C                 will give the state (position and velocity) of some
C                 body, relative to some center, in some inertial
C                 reference frame.
C
C                 The structure of RECORD is:
C
C                     RECORD(1)
C                        .            Geophysical Constants such as
C                        .            GM, J2, J3, J4, etc.
C                        .
C                     RECORD(NGEOCN)
C
C                     RECORD(NGEOCN + 1)
C                        .
C                        .            elements and epoch for the body
C                        .            at epoch 1.
C                        .
C                     RECORD(NGEOCN + NELEMN )
C
C                     RECORD(NGEOCN + NELEMN + 1)
C                        .
C                        .            elements and epoch for the body
C                        .            at epoch 2.
C                        .
C                     RECORD(NGEOCN + 2*NELEMN )
C
C                 Epoch 1 and epoch 2 are the times in the segment that
C                 bracket ET.  If ET is less than the first time in the
C                 segment then both epochs 1 and 2 are equal to the
C                 first time.  And if ET is greater than the last time
C                 then, epochs 1 and 2 are set equal to this last time.
C
C$ Detailed_Output
C
C     STATE       is the state produced by evaluating RECORD at ET.
C                 Units are km and km/sec.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If there is a problem evaluating the two-line elements,
C     the error will be diagnosed by EV2LIN.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     This routine interpolates a state from the two reference sets
C     of two-line element sets contained in RECORD.
C
C     It is assumed that this routine is used in conjunction with
C     the routine SPKR10 as shown here:
C
C        CALL SPKR10 ( HANDLE, DESCR, ET, RECORD         )
C        CALL SPKE10 (                ET, RECORD, STATE  )
C
C     Where it is known in advance that the HANDLE, DESCR pair points
C     to a type 10 data segment.
C
C$ Examples
C
C     The SPKEnn routines are almost always used in conjunction with
C     the corresponding SPKRnn routines, which read the records from
C     SPK files.
C
C     The data returned by the SPKRnn routine is in its rawest form,
C     taken directly from the segment.  As such, it will be meaningless
C     to a user unless he/she understands the structure of the data type
C     completely.  Given that understanding, however, the SPKRnn
C     routines might be used to examine raw segment data before
C     evaluating it with the SPKEnn routines.
C
C
C     C
C     C     Get a segment applicable to a specified body and epoch.
C     C
C           CALL SPKSFS ( BODY, ET, HANDLE, DESCR, IDENT, FOUND )
C
C     C
C     C     Look at parts of the descriptor.
C     C
C           CALL DAFUS ( DESCR, 2, 6, DCD, ICD )
C           CENTER = ICD( 2 )
C           REF    = ICD( 3 )
C           TYPE   = ICD( 4 )
C
C           IF ( TYPE .EQ. 10 ) THEN
C
C              CALL SPKR10 ( HANDLE, DESCR, ET, RECORD )
C                  .
C                  .  Look at the RECORD data.
C                  .
C              CALL SPKE10 ( ET, RECORD, STATE )
C                  .
C                  .  Check out the evaluated state.
C                  .
C           END IF
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     W.L. Taber      (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0 18-JUL-1997 (WLT)
C
C-&
 
C$ Index_Entries
C
C     evaluate type_10 spk segment
C
C-&
 
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
      DOUBLE PRECISION      BRCKTD
      DOUBLE PRECISION      PI
      DOUBLE PRECISION      SPD
      DOUBLE PRECISION      TWOPI
 
 
C
C     Local Parameters
C
      INTEGER               START
      PARAMETER           ( START  = 0 )
 
C
C
C     The following parameters give the location of the various
C     geophysical parameters needed for the two line element
C     sets.  We need these only so that we can count how many there
C     are (NGEOCN).
C
C     KJ2  --- location of J2
C     KJ3  --- location of J3
C     KJ4  --- location if J4
C     KKE  --- location of KE = sqrt(GM) in eart-radii**1.5/MIN
C     KQO  --- upper bound of atmospheric model in KM
C     KSO  --- lower bound of atmospheric model in KM
C     KER  --- earth equatorial radius in KM.
C     KAE  --- distance units/earth radius
C
C
      INTEGER               KJ2
      PARAMETER           ( KJ2    = START  + 1 )
 
      INTEGER               KJ3
      PARAMETER           ( KJ3    = KJ2    + 1 )
 
      INTEGER               KJ4
      PARAMETER           ( KJ4    = KJ3    + 1 )
 
      INTEGER               KKE
      PARAMETER           ( KKE    = KJ4    + 1 )
 
      INTEGER               KQO
      PARAMETER           ( KQO    = KKE    + 1 )
 
      INTEGER               KSO
      PARAMETER           ( KSO    = KQO    + 1 )
 
      INTEGER               KER
      PARAMETER           ( KER    = KSO    + 1 )
 
      INTEGER               KAE
      PARAMETER           ( KAE    = KER    + 1 )
 
      INTEGER               NGEOCN
      PARAMETER           ( NGEOCN = KAE )
C
C     An enumeration of the various components of the
C     a two-line element set.  These are needed so that we
C     can locate the epochs in the two sets and so that
C     we can count the number of elements in a two-line
C     element set.
C
C     KNDT20
C     KNDD60
C     KBSTAR
C     KINCL
C     KNODE0
C     KECC
C     KOMEGA
C     KMO
C     KNO
C     KEPOCH
C
 
      INTEGER               KNDT20
      PARAMETER           ( KNDT20 = START  + 1 )
 
      INTEGER               KNDD60
      PARAMETER           ( KNDD60 = KNDT20 + 1 )
 
      INTEGER               KBSTAR
      PARAMETER           ( KBSTAR = KNDD60 + 1 )
 
      INTEGER               KINCL
      PARAMETER           ( KINCL  = KBSTAR + 1 )
 
      INTEGER               KNODE0
      PARAMETER           ( KNODE0 = KINCL  + 1 )
 
      INTEGER               KECC
      PARAMETER           ( KECC   = KNODE0 + 1 )
 
      INTEGER               KOMEGA
      PARAMETER           ( KOMEGA = KECC   + 1 )
 
      INTEGER               KMO
      PARAMETER           ( KMO    = KOMEGA + 1 )
 
      INTEGER               KNO
      PARAMETER           ( KNO    = KMO    + 1 )
 
      INTEGER               KEPOCH
      PARAMETER           ( KEPOCH = KNO    + 1 )
 
      INTEGER               NELEMS
      PARAMETER           ( NELEMS = KEPOCH )
 
C
C     The nutation in obliquity and longitude as well as their rates
C     follow the elements.  So we've got four angles/angle rates
C     following the elements
C
      INTEGER               NANGS
      PARAMETER           ( NANGS = 4 )
 
C
C     The locations of the epochs and the starts of the element
C     sets are given below.
C
      INTEGER               TIME1
      PARAMETER           ( TIME1  = NGEOCN + KEPOCH )
 
      INTEGER               TIME2
      PARAMETER           ( TIME2  = NELEMS + NANGS + TIME1  )
 
      INTEGER               MODEL1
      PARAMETER           ( MODEL1 = NGEOCN + 1      )
 
      INTEGER               ANGS1
      PARAMETER           ( ANGS1 =  MODEL1 + NELEMS )
 
      INTEGER               MODEL2
      PARAMETER           ( MODEL2 = ANGS1 + NANGS )
 
      INTEGER               ANGS2
      PARAMETER           ( ANGS2  = MODEL2 + NELEMS )
 
      INTEGER               MEANMO
      PARAMETER           ( MEANMO = NGEOCN + KNO )
 
C
C     The parameters below give the location of the nutation terms
C     and their rates for the first packet.
C
      INTEGER               NUOB1
      PARAMETER           ( NUOB1 = ANGS1 )
 
      INTEGER               NULN1
      PARAMETER           ( NULN1 = NUOB1   + 1 )
 
      INTEGER               DNUOB1
      PARAMETER           ( DNUOB1 = NULN1  + 1 )
 
      INTEGER               DNULN1
      PARAMETER           ( DNULN1 = DNUOB1 + 1 )
 
 
C
C     The parameters below give the location of the nutation terms
C     and their rates for the second packet.
C
      INTEGER               NUOB2
      PARAMETER           ( NUOB2 = ANGS2 )
 
      INTEGER               NULN2
      PARAMETER           ( NULN2 = NUOB2 + 1 )
 
      INTEGER               DNUOB2
      PARAMETER           ( DNUOB2 = NULN2   + 1 )
 
      INTEGER               DNULN2
      PARAMETER           ( DNULN2 = DNUOB2 + 1 )
 
 
 
C
C     Local variables
C
      DOUBLE PRECISION      ARG
      DOUBLE PRECISION      DARGDT
      DOUBLE PRECISION      DENOM
      DOUBLE PRECISION      DMOB
      DOUBLE PRECISION      DT1
      DOUBLE PRECISION      DT2
      DOUBLE PRECISION      DWDT
      DOUBLE PRECISION      FIVDAY
      DOUBLE PRECISION      M      ( 3, 3 )
      DOUBLE PRECISION      MNRATE
      DOUBLE PRECISION      MOB
      DOUBLE PRECISION      MY2PI
      DOUBLE PRECISION      MYPI
      DOUBLE PRECISION      N0
      DOUBLE PRECISION      NULON
      DOUBLE PRECISION      NULON1
      DOUBLE PRECISION      NULON2
      DOUBLE PRECISION      NUMER
      DOUBLE PRECISION      NUOBL
      DOUBLE PRECISION      NUOBL1
      DOUBLE PRECISION      NUOBL2
      DOUBLE PRECISION      PRECM    ( 3, 3 )
      DOUBLE PRECISION      S1       ( 6 )
      DOUBLE PRECISION      S2       ( 6 )
      DOUBLE PRECISION      T1
      DOUBLE PRECISION      T2
      DOUBLE PRECISION      VCOMP    ( 3 )
      DOUBLE PRECISION      W
 
      LOGICAL               FIRST
      LOGICAL               LOWORB
      SAVE
 
      DATA                  FIRST /.TRUE./
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SPKE10' )
      END IF
 
      IF ( FIRST ) THEN
         FIRST  = .FALSE.
         MYPI   = PI()
         MY2PI  = TWOPI()
         FIVDAY = 5.0D0 * SPD()
      END IF
 
C
C     Fetch the mean motion from the first set of two-line elements
C     stored in the record.
C
      N0     = RECORD(MEANMO)
      MNRATE = MY2PI / 225.0D0
      LOWORB = N0 .GE. MNRATE
 
C
C     Fetch the two epochs stored in the record.
C
      T1 = RECORD(TIME1)
      T2 = RECORD(TIME2)
 
C
C     Evaluate the two states. Call them s_1(t) and s_2(t).
C     Let the position and velocity components be: p_1, v_1, p_2, v_2.
C
C     The final position is a weighted average.
C
C     Let
C
C        W(t) =  0.5 + 0.5*COS( PI*(t-t1)/(t2-t1) )
C
C     then
C
C        p  = W(t)*p_1(t) + (1 - W(t))*p_2(t)
C        v  = W(t)*v_1(t) + (1 - W(t))*v_2(t) + W'(t)*(p_1(t) - p_2(t))
C
C     If t1 = t2, the state is just s(t1).
C
C
C     Note: there are a number of weighting schemes we could have
C     used.  This one has the nice property that
C
C     The graph of W is symmetric about the point
C
C
C        ( (t1+t2)/2,  W( (t1+t2)/2 )
C
C     The range of W is from 1 to 0.  And the derivative of W is
C     symmetric and zero at both t1 and t2.
C
      IF ( T1 .NE. T2 ) THEN
 
         IF ( LOWORB ) THEN
            CALL EV2LIN ( ET, RECORD(1), RECORD(MODEL1), S1 )
            CALL EV2LIN ( ET, RECORD(1), RECORD(MODEL2), S2 )
         ELSE
            CALL DPSPCE ( ET, RECORD(1), RECORD(MODEL1), S1 )
            CALL DPSPCE ( ET, RECORD(1), RECORD(MODEL2), S2 )
         END IF
C
C        Using the different element packets compute nutations in
C        obliquity and longitude.  Note to avoid run-away nutation
C        interpolation we bracket DT1 and DT2 to be between -5 and 5
C        days.
C
         DT1    = BRCKTD ( ET - T1, -FIVDAY, FIVDAY )
         NUOBL1 = RECORD(NUOB1) + DT1*RECORD(DNUOB1)
         NULON1 = RECORD(NULN1) + DT1*RECORD(DNULN1)
 
         DT2    = BRCKTD ( ET - T2, -FIVDAY, FIVDAY )
         NUOBL2 = RECORD(NUOB2) + DT2*RECORD(DNUOB2)
         NULON2 = RECORD(NULN2) + DT2*RECORD(DNULN2)
C
C        Compute the weighting function that we'll need later
C        when we combine states 1 and 2.
C
         NUMER  = ET - T1
         DENOM  = T2 - T1
         ARG    = NUMER*MYPI/DENOM
         DARGDT =       MYPI/DENOM
 
         W      =  0.5D0 + 0.5D0 * COS( ARG )
         DWDT   =        - 0.5D0 * SIN( ARG ) * DARGDT
 
C
C        Use the weighting function to compute the nutation in obliquity
C        longitude.
C
         NUOBL = W*NUOBL1 + (1.0D0-W)*NUOBL2
         NULON = W*NULON1 + (1.0D0-W)*NULON2
 
C
C        Compute the mean obliquity at the epoch ET.
C
         CALL ZZMOBLIQ ( ET, MOB, DMOB )
 
C
C        Construct the transformation from mean of date to true of date.
C
         CALL EUL2M (  -MOB-NUOBL, -NULON, MOB, 1, 3, 1, M )
 
C
C        Use the transpose of the matrix just computed to convert
C        S1 and S2 from true of date to mean of date.
C
         CALL MTXV  ( M, S1(1), S1(1) )
         CALL MTXV  ( M, S1(4), S1(4) )
 
         CALL MTXV  ( M, S2(1), S2(1) )
         CALL MTXV  ( M, S2(4), S2(4) )
 
C
C        Now compute the weighted average of the two true of date
C        states.
C
         CALL VLCOMG ( 6,  W,    S1,       1.D0-W, S2,     STATE    )
         CALL VLCOM  (     DWDT, S1,       -DWDT,  S2,     VCOMP    )
         CALL VADD   (           STATE(4),         VCOMP,  STATE(4) )
 
      ELSE
 
         IF ( LOWORB ) THEN
            CALL EV2LIN ( ET, RECORD(1), RECORD(MODEL1), STATE )
         ELSE
            CALL DPSPCE ( ET, RECORD(1), RECORD(MODEL1), STATE )
         END IF
C
C        Interpolate the nutation in longitude and obliquity.  Note
C        that to avoid run-away linear interpolation of the nutation
C        angles, we bracket DT1 to be between -5 and 5 days.
C
         DT1   = BRCKTD ( ET - T1, -FIVDAY, FIVDAY )
         NUOBL = RECORD(NUOB1) + DT1*RECORD(DNUOB1)
         NULON = RECORD(NULN1) + DT1*RECORD(DNULN1)
 
C
C        Get the current obliquity.
C
         CALL ZZMOBLIQ ( ET, MOB, DMOB )
 
         CALL EUL2M (  -MOB-NUOBL, -NULON, MOB, 1, 3, 1, M )
 
         CALL MTXV  ( M, STATE(1), STATE(1) )
         CALL MTXV  ( M, STATE(4), STATE(4) )
 
      END IF
C
C     Finally, convert the mean of date state to J2000.  First get
C     the rotation from J2000 to mean of date.
C
      CALL ZZEPRCSS ( ET, PRECM )
C
C     Now convert STATE to J2000.
C
      CALL MTXV ( PRECM, STATE(1), STATE(1) )
      CALL MTXV ( PRECM, STATE(4), STATE(4) )
 
 
      CALL CHKOUT ( 'SPKE10' )
      RETURN
      END
