C$Procedure      SRFREC ( Surface to rectangular coordinates )
 
      SUBROUTINE SRFREC ( BODY, LONG, LAT, RECTAN )
 
C$ Abstract
C
C     Convert planetocentric latitude and longitude of a surface
C     point on a specified body to rectangular coordinates.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     KERNEL
C     NAIF_IDS
C
C$ Keywords
C
C     CONVERSION
C     COORDINATES
C     TRANSFORMATION
C
C$ Declarations
 
      INTEGER               BODY
      DOUBLE PRECISION      LONG
      DOUBLE PRECISION      LAT
      DOUBLE PRECISION      RECTAN ( 3 )
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     BODY       I   NAIF integer code of an extended body.
C     LONG       I   Angle of the point from the XZ plane in radians.
C     LAT        I   Angle of the point from the XY plane in radians.
C     RECTAN     O   Rectangular coordinates of the point.
C
C$ Detailed_Input
C
C     BODY           is the NAIF integer code of an extended body
C                    on which a surface point of interest is located.
C
C     LONG           is the planetocentric longitude of the surface
C                    point.  This is the angle of the surface point
C                    from the XZ plane, in radians.
C
C     LAT            is the planetocentric latitude of the surface
C                    point.  This is the angle of the surface point
C                    from the XY plane, in radians.
C
C$ Detailed_Output
C
C      RECTAN        The rectangular coordinates of the input surface
C                    point.  Units are the same as those used to define
C                    the radii of BODY.  Normally, these units are km.
C
C$ Parameters
C
C      None.
C
C$ Exceptions
C
C     1)  If radii for BODY are not found in the kernel pool, the error
C         will be diagnosed by routines called by this routine.
C
C     2)  If radii for BODY are invalid, the error will be diagnosed by
C         routines called by this routine.  The radii should be
C         positive.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     This routine returns the rectangular coordinates of a surface
C     point on an extended body with known radii, where the location
C     of the surface point is specified in planetocentric latitudinal
C     coordinates.
C
C     Latitudinal coordinates are defined by a distance from a central
C     reference point, an angle from a reference meridian, and an angle
C     above the equator of a sphere centered at the central reference
C     point.  In this case, the distance from the central reference
C     point is not required as an input because the fact that the
C     point is on the body's surface allows one to deduce this quantity.
C
C     Below are two tables that demonstrate by example the relationship
C     between rectangular and latitudinal coordinates.
C
C     Listed in the first table (under R, LONG and LAT ) are
C     latitudinal coordinate triples that approximately represent
C     points whose rectangular coordinates are taken from the set
C     {-1, 0, 1}.  (Angular quantities are given in degrees.)
C
C
C          R         LONG       LAT           X(1)       X(2)     X(3)
C         --------------------------         --------------------------
C         0.0000    0.0000    0.0000         0.0000     0.0000   0.0000
C         1.0000    0.0000    0.0000         1.0000     0.0000   0.0000
C         1.0000   90.0000    0.0000         0.0000     1.0000   0.0000
C         1.0000    0.0000   90.0000         0.0000     0.0000   1.0000
C         1.0000  180.0000    0.0000        -1.0000     0.0000   0.0000
C         1.0000  -90.0000    0.0000         0.0000    -1.0000   0.0000
C         1.0000    0.0000  -90.0000         0.0000     0.0000  -1.0000
C         1.4142   45.0000    0.0000         1.0000     1.0000   0.0000
C         1.4142    0.0000   45.0000         1.0000     0.0000   1.0000
C         1.4142   90.0000   45.0000         0.0000     1.0000   1.0000
C         1.7320   45.0000   35.2643         1.0000     1.0000   1.0000
C
C
C     This routine is related to the SPICELIB routine LATREC, which
C     accepts a radius, longitude, and latitude as inputs and produces
C     equivalent rectangular coordinates as outputs.
C
C$ Examples
C
C     1)  Find the rectangular coordinates of the point
C
C            100 degrees planetocentric longitude
C            -35 degrees planetocentric latitude
C
C         on the Earth; then convert these coordinates back to
C         latitudinal coordinates.  We should be able to recover
C         our original longitude and latitude values.
C
C
C                  PROGRAM TEST_SRFREC
C
C                  DOUBLE PRECISION      LAT
C                  DOUBLE PRECISION      LONG
C                  DOUBLE PRECISION      X     ( 3 )
C                  DOUBLE PRECISION      RADIUS
C
C            C
C            C     Load the kernel pool with a PCK file that contains
C            C     values for the radii of the Earth.
C            C
C                  CALL LDPOOL ( 'MYPCK.KER' )
C
C            C
C            C     Find X, the rectangular coordinates of the
C            C     surface point defined by LAT and LONG.  The
C            C     NAIF integer code for the Earth is 399.
C            C     (See the NAIF_IDS required reading file for
C            C     the complete set of codes.)
C            C
C                  LONG  =  100.D0
C                  LAT   =   35.D0
C
C                  WRITE (*,*) 'Original latitudinal coordinates'
C                  WRITE (*,*) ' '
C                  WRITE (*,*) 'Longitude  ', LONG
C                  WRITE (*,*) 'Latitude   ', LAT
C
C                  CALL CONVRT ( 100.D0, 'DEGREES', 'RADIANS', LONG )
C                  CALL CONVRT ( 35.D0,  'DEGREES', 'RADIANS', LAT  )
C
C                  CALL SRFREC ( 399, LONG, LAT, X )
C
C                  WRITE (*,*) ' '
C                  WRITE (*,*) ' '
C                  WRITE (*,*) 'Rectangular coordinates '
C                  WRITE (*,*) ' '
C                  WRITE (*,*) X
C
C            C
C            C     Now try to recover the original latitudinal
C            C     coordinates from the rectangular coordinates
C            C     found by SRFREC.
C            C
C                  CALL RECLAT ( X, RADIUS, LONG, LAT )
C
C                  CALL CONVRT ( LONG, 'RADIANS', 'DEGREES', LONG )
C                  CALL CONVRT ( LAT,  'RADIANS', 'DEGREES', LAT  )
C
C                  WRITE (*,*) ' '
C                  WRITE (*,*) ' '
C                  WRITE (*,*) 'Latitudinal coordinates recovered '   //
C                 .            'from rectangular coordinates'
C                  WRITE (*,*) ' '
C                  WRITE (*,*) 'Longitude        ', LONG
C                  WRITE (*,*) 'Latitude         ', LAT
C                  WRITE (*,*) 'Radius           ', RADIUS
C
C                  END
C
C
C$ Restrictions
C
C     1)  A NAIF text kernel containing the body radius definitions
C         required by this routine must be loaded into the kernel
C         pool prior to any calls to this routine.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman   (JPL)
C     W.L. Taber     (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 03-SEP-1991 (NJB) (WLT)
C
C-&
 
C$ Index_Entries
C
C     convert bodyfixed latitudinal coordinates to rectangular
C     convert surface latitudinal coordinates to rectangular
C     surface point latitudinal coordinates to rectangular
C
C-&
 
 
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
 
C
C     Local variables
C
      DOUBLE PRECISION      ORIGIN ( 3 )
      DOUBLE PRECISION      RADII  ( 3 )
      DOUBLE PRECISION      UVEC   ( 3 )
 
      INTEGER               N
 
      LOGICAL               FOUND
 
C
C     Saved variables
C
      SAVE                  ORIGIN
 
C
C     Initial values
C
      DATA ORIGIN / 3 * 0.D0 /
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SRFREC' )
      END IF
 
C
C     Look up the body's radii.
C
      CALL BODVAR ( BODY, 'RADII', N, RADII )
 
C
C     Find the unit vector pointing from the body center to the
C     input surface point.
C
      CALL LATREC ( 1.D0, LONG, LAT, UVEC )
 
C
C     Find out where the ray defined by this vector intersects the
C     surface.  This intercept is the point we're looking for.
C
      CALL SURFPT ( ORIGIN,
     .              UVEC,
     .              RADII(1),
     .              RADII(2),
     .              RADII(3),
     .              RECTAN,
     .              FOUND     )
 
C
C     You can't miss the surface if you're riding a ray out from the
C     origin, so we don't check the FOUND flag.
C
 
      CALL CHKOUT ( 'SRFREC' )
      RETURN
      END
