C$Procedure      SXFORM ( State Transformation Matrix )
 
      SUBROUTINE SXFORM ( FROM, TO, ET, XFORM )
 
C$ Abstract
C
C     Return the state transformation matrix from one frame to
C     another at a specified epoch.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      FRAMES
C
C$ Declarations
 
      IMPLICIT NONE
      CHARACTER*(*)         FROM
      CHARACTER*(*)         TO
      DOUBLE PRECISION      ET
      DOUBLE PRECISION      XFORM  ( 6, 6 )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     FROM       I   Name of the frame to transform from.
C     TO         I   Name of the frame to transform to.
C     ET         I   Epoch of the state transformation matrix.
C     XFORM      O   A state transformation matrix.
C
C$ Detailed_Input
C
C     FROM        is the name of a reference frame in which a state is
C                 known.
C
C     TO          is the name of a reference frame in which it is
C                 desired to represent the state.
C
C     ET          is the epoch in ephemeris seconds past the epoch of
C                 J2000 (TDB) at which the state transformation matrix
C                 should be evaluated.
C
C$ Detailed_Output
C
C     XFORM       is the state transformation matrix that transforms
C                 states from the reference frame FROM to the frame TO
C                 at epoch ET.  If (x, y, z, dx, dy, dz) is a state
C                 relative to the frame FROM then the vector ( x', y',
C                 z', dx', dy', dz' ) is the same state relative to the
C                 frame TO at epoch ET.  Here the vector ( x', y', z',
C                 dx', dy', dz' ) is defined by the equation:
C
C                   -   -       -          -     -  -
C                  | x'  |     |            |   | x  |
C                  | y'  |     |            |   | y  |
C                  | z'  |  =  |   XFORM    |   | z  |
C                  | dx' |     |            |   | dx |
C                  | dy' |     |            |   | dy |
C                  | dz' |     |            |   | dz |
C                   -   -       -          -     -  -
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If sufficient information has not been supplied via loaded
C        SPICE kernels to compute the transformation between the
C        two frames, the error will be diagnosed by a routine
C        in the call tree to this routine.
C
C     2) If either frame FROM or TO is not recognized the error
C        SPICE(UNKNOWNFRAME) will be signaled.
C
C$ Particulars
C
C     This routine provides the user level interface to computing state
C     transformations from one reference frame to another.
C
C     Note that the reference frames may be inertial or non-inertial.
C     However, the user must take care that sufficient SPICE kernel
C     information is loaded to provide a complete state transformation
C     path from the FROM frame to the TO frame.
C
C$ Examples
C
C     Suppose that you have geodetic coordinates of a station on
C     the surface of the earth and that you need the inertial
C     (J2000) state of this station.  The following code fragment
C     illustrates how to transform the position of the station to
C     a J2000 state.
C
C        CALL BODVAR ( 399, RADII, N, ABC  )
C
C        EQUATR   =  ABC(1)
C        POLAR    =  ABC(3)
C        F        = (POLAR - EQUATR) / EQUATR
C
C        CALL GEOREC ( LONG, LAT, 0.0D0,  EQUATR, F,  ESTATE )
C
C        ESTATE(4) = 0.0D0
C        ESTATE(5) = 0.0D0
C        ESTATE(6) = 0.0D0
C
C        CALL SXFORM ( 'IAU_EARTH', 'J2000',   ET, XFORM  )
C        CALL MXVG   (  XFORM,       ESTATE, 6, 6, JSTATE )
C
C     The state JSTATE is the desired J2000 state of the station.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     C.H. Acton      (JPL)
C     N.J. Bachman    (JPL)
C     W.L. Taber      (JPL)
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 29-JUL-2003 (NJB) (CHA)
C
C        Minor header changes were made to improve clarity.
C
C-    SPICELIB Version 1.0.0, 19-SEP-1995 (WLT)
C
C-&
 
C$ Index_Entries
C
C     Find a state transformation matrix
C
C-&
 
C
C     Spicelib Functions
C
      LOGICAL               RETURN
 
C
C     Local Variables.
C
      INTEGER               FCODE
      INTEGER               TCODE
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN ( 'SXFORM' )
 
 
      CALL NAMFRM ( FROM, FCODE )
      CALL NAMFRM ( TO,   TCODE )
C
C     Only non-zero id-codes are legitimate frame id-codes.  Zero
C     indicates that the frame wasn't recognized.
C
      IF      ( FCODE .NE. 0 .AND. TCODE .NE. 0 ) THEN
 
         CALL FRMCHG ( FCODE, TCODE, ET, XFORM )
 
      ELSE IF ( FCODE .EQ. 0 .AND. TCODE .EQ. 0 ) THEN
 
         CALL SETMSG ( 'Neither of the frames # or # was '
     .   //            'recognized as a known reference frame. ' )
         CALL ERRCH  ( '#', FROM )
         CALL ERRCH  ( '#', TO   )
         CALL SIGERR ( 'SPICE(UNKNOWNFRAME)'  )
 
      ELSE IF ( FCODE .EQ. 0 ) THEN
 
         CALL SETMSG ( 'The frame # was not recognized as a '
     .   //            'known reference frame. ' )
         CALL ERRCH  ( '#', FROM )
         CALL SIGERR ( 'SPICE(UNKNOWNFRAME)'  )
 
      ELSE IF ( TCODE .EQ. 0 ) THEN
 
         CALL SETMSG ( 'The frame # was not recognized as a '
     .   //            'known reference frame. ' )
         CALL ERRCH  ( '#', TO   )
         CALL SIGERR ( 'SPICE(UNKNOWNFRAME)'  )
 
      END IF
 
      CALL CHKOUT ( 'SXFORM' )
      RETURN
      END
