C$Procedure      SYGETD ( Return all components for a symbol )
 
      SUBROUTINE SYGETD ( NAME,
     .                    TABSYM, TABPTR, TABVAL,
     .                    N,      VALUES,
     .                    FOUND                     )
 
C$ Abstract
C
C     Return the dimension and associated values for a particular
C     symbol.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SYMBOLS
C
C$ Keywords
C
C     SYMBOLS
C
C$ Declarations
 
      INTEGER               LBCELL
      PARAMETER           ( LBCELL = -5 )
 
      CHARACTER*(*)         NAME
      CHARACTER*(*)         TABSYM     ( LBCELL:* )
      INTEGER               TABPTR     ( LBCELL:* )
      DOUBLE PRECISION      TABVAL     ( LBCELL:* )
      INTEGER               N
      DOUBLE PRECISION      VALUES     (        * )
      LOGICAL               FOUND
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NAME       I   Name of the symbol whose components are to be
C                    returned.
C     TABSYM,
C     TABPTR,
C     TABVAL     I   Components of the symbol table.
C
C     N          O   Dimension of the symbol.
C     VALUES     O   Values associated with the symbol.
C     FOUND      O   True if the symbol NAME is in the symbol table,
C                    false if it is not.
C
C$ Detailed_Input
C
C     NAME       is the name of the symbol whose components are to be
C                returned. If NAME is not in the symbol table, FOUND is
C                false.
C
C     TABSYM,
C     TABPTR,
C     TABVAL     are the components of a double precision symbol table.
C                The symbol NAME may or may not be in the symbol
C                table. The symbol table is not modified by this
C                subroutine.
C
C$ Detailed_Output
C
C     N          is the dimension of the symbol NAME. The dimension
C                if the number of values associated with the given
C                symbol.
C
C     VALUES     is an array containing the values associated with
C                the symbol. If the array is not large enough to hold
C                all of the values associated with NAME, as many as will
C                fit are returned.
C
C     FOUND      is true if NAME is in the symbol table.
C                If NAME is not in the table, FOUND is false.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     None.
C
C$ Particulars
C
C     This subroutine does not check to see if the output array
C     VALUES is large enough to hold all of the values associated
C     with the symbol NAME. As many values as will fit are returned.
C
C$ Examples
C
C     The contents of the symbol table are:
C
C        BODY4_POLE_RA -->    3.17681D2
C                             1.08D-1
C                             0.0D0
C        DELTA_T_A     -->    3.2184D1
C        K             -->    1.657D-3
C        MEAN_ANOM     -->    6.239996D0
C                             1.99096871D-7
C        ORBIT_ECC     -->    1.671D-2
C
C     Let the dimension of VALUES be 2.
C
C     The calls,
C
C     CALL SYGETD ( 'K',            TABSYM, TABPTR, TABVAL,
C    .               N,             VALUES, FOUND           )
C
C     CALL SYGETD ( 'BODY4_POLE_RA', TABSYM, TABPTR, TABVAL,
C    .               N,              VALUES, FOUND            )
C
C     CALL SYGETD ( 'BODY4_PRIME',   TABSYM, TABPTR, TABVAL,
C    .               N,              VALUES, FOUND           )
C
C
C     return the values for N, VALUES, and FOUND associated with NAME:
C
C
C     NAME            N        VALUES                      FOUND
C     ----------     ---      -----------------------     -------
C     K               1          1.657D-3                   TRUE
C     BODY4_POLE_RA   3          3.17681D2                  TRUE
C                                1.08D-1
C     BODY4_PRIME                                          FALSE
C
C
C     Notice that even though there were three values associated with
C     BODY4_POLE_RA, only as many as would fit into the array VALUES are
C     returned.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     H.A. Neilan     (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (IMU) (HAN)
C
C-&
 
C$ Index_Entries
C
C     fetch all components for a symbol
C
C-&
 
 
 
 
 
C
C     SPICELIB functions
C
      INTEGER               BSRCHC
      INTEGER               CARDC
      LOGICAL               RETURN
      INTEGER               SUMAI
 
C
C     Local variables
C
      INTEGER               NSYM
 
      INTEGER               LOCSYM
      INTEGER               LOCVAL
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SYGETD' )
      END IF
 
C
C     How many symbols to start with?
C
      NSYM = CARDC ( TABSYM )
 
C
C     Is this symbol even in the table?
C
      LOCSYM = BSRCHC ( NAME, NSYM, TABSYM(1) )
 
C
C     If it's not in the table, it's definitely a problem.
C
      IF ( LOCSYM .EQ. 0 ) THEN
 
         FOUND = .FALSE.
 
C
C     Otherwise, we can proceed without fear of error. Merely locate
C     and return the appropriate component from the values table.
C     We trust that the user has supplied enough room.
C
      ELSE
 
         FOUND = .TRUE.
 
         LOCVAL = SUMAI ( TABPTR(1), LOCSYM-1 ) + 1
         N      = TABPTR(LOCSYM)
 
         CALL MOVED ( TABVAL(LOCVAL), N, VALUES )
 
      END IF
 
 
      CALL CHKOUT ( 'SYGETD' )
      RETURN
      END
