C$Procedure      SYSELD ( Select a subset of the values of a symbol )
 
      SUBROUTINE SYSELD ( NAME,
     .                    BEGIN,
     .                    END,
     .                    TABSYM, TABPTR, TABVAL,
     .                    VALUES,
     .                    FOUND                     )
 
C$ Abstract
C
C     Select a subset of the values associated with a particular
C     symbol in a double precision symbol table.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SYMBOLS
C
C$ Keywords
C
C     SYMBOLS
C
C$ Declarations
 
      INTEGER               LBCELL
      PARAMETER           ( LBCELL = -5 )
 
      CHARACTER*(*)         NAME
      INTEGER               BEGIN
      INTEGER               END
      CHARACTER*(*)         TABSYM     ( LBCELL:* )
      INTEGER               TABPTR     ( LBCELL:* )
      DOUBLE PRECISION      TABVAL     ( LBCELL:* )
      DOUBLE PRECISION      VALUES     (        * )
      LOGICAL               FOUND
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NAME       I   Name of the symbol whose associated values are to
C                    be returned.
C     BEGIN      I   Index of the first associated value to be returned.
C     END        I   Index of the last associated value to be returned.
C
C     TABSYM,
C     TABPTR,
C     TABVAL     I   Components of the symbol table.
C
C     VALUES     O   Subset of the values associated with the symbol
C                    NAME.
C     FOUND      O   True if the subset of values exists.
C
C$ Detailed_Input
C
C     NAME       is the name of the symbol whose subset of associated
C                values to be returned. If NAME is not in the symbol
C                table, FOUND is false.
C
C     BEGIN      is the index of the first associated value to be
C                returned. If BEGIN is out of range (BEGIN < 1 or
C                BEGIN > END) FOUND is false.
C
C     END        is the index of the last associated value to be
C                returned. If END is out of range (END < 1 or
C                END > is greater than the dimension of NAME)
C                FOUND is false.
C
C     TABSYM,
C     TABPTR,
C     TABVAL     are components of the double precision symbol table.
C
C$ Detailed_Output
C
C     VALUES     is a subset of the values associated with the
C                symbol NAME. If the subset specified by BEGIN and
C                END exists, as many values as will fit in VALUES
C                are returned. If the subset does not exist, no
C                values are returned and FOUND is false.
C
C     FOUND      is true if the subset of values is exists.
C                FOUND is false if BEGIN < 1, BEGIN > END, END < 1,
C                END > the dimension of NAME, or NAME is not
C                in the symbol table.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     None.
C
C$ Particulars
C
C     FOUND will be false if the bounds of the subset specified by
C     BEGIN and END are out of range. Values of BEGIN and END which
C     specify bounds out of range are BEGIN < 1, BEGIN > END,
C     END < 1, or END > the dimension of NAME. FOUND is also false
C     if the symbol NAME is not in the symbol table.
C
C     VALUES will contain as many of the values of the subset as will
C     fit beginning with the value at index BEGIN.
C
C$ Examples
C
C     The contents of the symbol table are:
C
C        BODY4_POLE_RA -->    3.17681D2
C                             1.08D-1
C                             0.0D0
C        DELTA_T_A     -->    3.2184D1
C        K             -->    1.657D-3
C        MEAN_ANOM     -->    6.239996D0
C                             1.99096871D-7
C        ORBIT_ECC     -->    1.671D-2
C
C     Let the dimension of the array VALUES be 2.
C
C
C     The ouput values of VALUES and FOUND for the input values of
C     NAME, BEGIN, and END are contained in this table:
C
C     NAME            BEGIN    END        VALUES               FOUND
C     -------------   -----    ---    ---------------------   -------
C     MEAN_ANOM         1       2         6.239996D0            TRUE
C                                         1.99096871D-7
C
C     BODY4_POLE_RA     1       3         3.17681D2
C                                         1.08D-1
C
C     BODY4_PRIME       1       3                              FALSE
C
C     MEAN_ANOM         2       1                              FALSE
C
C     ORBIT_ECC         1      -2                              FALSE
C
C     K                 1       5                              FALSE
C     ----------------------------------------------------------------
C
C     Note that even though there were threee values associated with
C     the symbol "BODY4_POLE_RA", only two were returned due to the
C     dimension of the array VALUES.
C
C     Also note that FOUND is FALSE for examples 3 through 6 because:
C
C     In the 3rd example, the symbol "BODY4_PRIME" is not in the symbol
C     table.
C     In the 4th example, BEGIN > END.
C     In the 5th example, END < 0.
C     In the 6th example, END is greater than the dimension of the
C     symbol "K".
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     H.A. Neilan     (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (IMU) (HAN)
C
C-&
 
C$ Index_Entries
C
C     select a subset of the values of a symbol
C
C-&
 
 
 
C
C     SPICELIB functions
C
      INTEGER               BSRCHC
      INTEGER               CARDC
      LOGICAL               RETURN
      INTEGER               SUMAI
 
C
C     Local variables
C
      INTEGER               NSYM
 
      INTEGER               LOCSYM
      INTEGER               LOCVAL
      INTEGER               N
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SYSELD' )
      END IF
 
 
C
C     How many symbols to start with?
C
      NSYM = CARDC ( TABSYM )
 
C
C     Is this symbol even in the table?
C
      LOCSYM = BSRCHC ( NAME, NSYM, TABSYM(1) )
 
C
C     If it's not in the table, it's definitely a problem.
C
      IF ( LOCSYM .EQ. 0 ) THEN
 
         FOUND = .FALSE.
 
      ELSE
 
C
C        We could still have a problem: do these components exist?
C        Does this request even make sense?
C
         N = TABPTR(LOCSYM)
 
         IF (       BEGIN .GE. 1
     .        .AND. BEGIN .LE. N
     .        .AND. END   .GE. 1
     .        .AND. END   .LE. N
     .        .AND. BEGIN .LE. END ) THEN
 
            FOUND  = .TRUE.
            LOCVAL = SUMAI ( TABPTR(1), LOCSYM-1 ) + 1
 
            CALL MOVED ( TABVAL(LOCVAL+BEGIN-1), END-BEGIN+1, VALUES )
 
         ELSE
            FOUND = .FALSE.
         END IF
 
      END IF
 
 
      CALL CHKOUT ( 'SYSELD' )
      RETURN
      END
