C$Procedure      SYTRND (Transpose two values associated with a symbol)
 
      SUBROUTINE SYTRND ( NAME, I, J, TABSYM, TABPTR, TABVAL )
 
C$ Abstract
C
C     Transpose two values associated with a particular symbol in a
C     double precision symbol table.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SYMBOLS
C
C$ Keywords
C
C     SYMBOLS
C
C$ Declarations
 
      INTEGER               LBCELL
      PARAMETER           ( LBCELL = -5 )
 
      CHARACTER*(*)         NAME
      INTEGER               I
      INTEGER               J
      CHARACTER*(*)         TABSYM     ( LBCELL:* )
      INTEGER               TABPTR     ( LBCELL:* )
      DOUBLE PRECISION      TABVAL     ( LBCELL:* )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NAME       I   Name of the symbol whose associated values are to
C                    be transposed.
C     I          I   Index of the first associated value to be
C                    transposed.
C     J          I   Index of the second associated value to be
C                    transposed.
C
C     TABSYM,
C     TABPTR,
C     TABVAL    I/O  Components of the symbol table.
C
C$ Detailed_Input
C
C     NAME       is the name of the symbol whose associated values are
C                to be transposed. If NAME is not in the symbol table,
C                the symbol tables are not modified.
C
C     I          is the index of the first associated value to be
C                transposed. If this index is not valid ( I < 1 or
C                I > the dimension of NAME) the symbol table is not
C                modified.
C
C     J          is the index of the second associated value to be
C                transposed. If this index is not valid ( J < 1 or
C                J > the dimension of NAME) the symbol table is not
C                modified.
C
C     TABSYM,
C     TABPTR,
C     TABVAL     are components of the double precision symbol table.
C
C$ Detailed_Output
C
C     TABSYM,
C     TABPTR,
C     TABVAL     are components of the double precision symbol table.
C                If the symbol NAME is not in the symbol table
C                the symbol tables are not modified. Otherwise,
C                the values that I and J refer to are transposed
C                in the value table.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1) If I < 1, J < 1, I > the dimension of NAME, or J > the
C        dimension of NAME, the error SPICE(INVALIDINDEX) is signalled.
C
C     2) If NAME is not in the symbol table, the symbol tables are not
C        modified.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C     The contents of the symbol table are:
C
C        DELTA_T_A     -->    3.2184D1
C        K             -->    1.657D-3
C        MEAN_ANOM     -->    6.239996D0
C                             1.99096871D-7
C        ORBIT_ECC     -->    1.671D-2
C        BODY4_POLE_RA -->    3.17681D2
C                             1.08D-1
C                             0.0D0
C
C     The call,
C
C     CALL SYTRND ( 'BODY4_POLE_RA', 2, 3, TABSYM, TABPTR, TABVAL )
C
C     modifies the contents of the symbol table to be:
C
C        DELTA_T_A     -->    3.2184D1
C        K             -->    1.657D-3
C        MEAN_ANOM     -->    6.239996D0
C                             1.99096871D-7
C        ORBIT_ECC     -->    1.671D-2
C        BODY4_POLE_RA -->    3.17681D2
C                             0.0D0C
C                             1.08D-1
C     The next call,
C
C     CALL SYTRND ( 'MEAN_ANOM', 2, 4, TABSYM, TABPTR, TABVAL )
C
C     causes the error SPICE(INVALIDINDEX) to be signalled.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     H.A. Neilan     (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (IMU) (HAN)
C
C-&
 
C$ Index_Entries
C
C     transpose two values associated with a symbol
C
C-&
 
 
 
C$ Revisions
C
C-     Beta Version 2.0.0, 16-JAN-1989 (HAN)
C
C         If one of the indices of the values to be transposed is
C         invalid, an error is signalled and the symbol table is
C         not modified.
C
C-&
 
 
 
C
C     SPICELIB functions
C
      INTEGER               BSRCHC
      INTEGER               CARDC
      LOGICAL               RETURN
      INTEGER               SUMAI
 
C
C     Local variables
C
      INTEGER               NSYM
 
      INTEGER               LOCSYM
      INTEGER               LOCVAL
      INTEGER               N
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SYTRND' )
      END IF
 
C
C     How many symbols?
C
      NSYM = CARDC ( TABSYM )
 
C
C     Is this symbol even in the table?
C
      LOCSYM = BSRCHC ( NAME, NSYM, TABSYM(1) )
 
      IF ( LOCSYM .GT. 0 ) THEN
 
C
C        Are there enough values associated with the symbol?
C
         N = TABPTR(LOCSYM)
 
C
C        Are the indices valid?
C
 
         IF (        I .GE. 1
     .        .AND.  I .LE. N
     .        .AND.  J .GE. 1
     .        .AND.  J .LE. N ) THEN
 
C
C           Exchange the values in place.
C
            LOCVAL = SUMAI ( TABPTR(1), LOCSYM-1 ) + 1
 
            CALL SWAPD ( TABVAL(LOCVAL+I-1), TABVAL(LOCVAL+J-1) )
 
         ELSE
 
            CALL SETMSG ( 'The first index was *. '                   //
     .                    'The second index was *.'                    )
            CALL ERRINT ( '*', I )
            CALL ERRINT ( '*', J )
            CALL SIGERR ( 'SPICE(INVALIDINDEX)' )
 
         END IF
 
      END IF
 
 
      CALL CHKOUT ( 'SYTRND' )
      RETURN
      END
