C$Procedure      TKFRAM (Text kernel frame transformation)
 
      SUBROUTINE TKFRAM ( ID, ROT, FRAME, FOUND )
 
C$ Abstract
C
C     This routine returns the rotation from the input frame
C     specified by ID to the associated frame given by FRAME.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      FRAMES
C
C$ Keywords
C
C       POINTING
C
C$ Declarations
 
      IMPLICIT NONE
      INTEGER               ID
      DOUBLE PRECISION      ROT   ( 3, 3 )
      INTEGER               FRAME
      LOGICAL               FOUND
 
      INTEGER               BUFSIZ
      PARAMETER           ( BUFSIZ = 20 )
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  ----------------------------------------------
C      ID         I   Class identification code for the instrument
C      ROT        O   The rotation from ID to FRAME.
C      FRAME      O   The integer code of some reference frame.
C      FOUND      O   TRUE if the rotation could be determined.
C
C$ Detailed_Input
C
C     ID          The identification code used to specify an
C                 instrument in the SPICE system.
C
C$ Detailed_Output
C
C     ROT         is a rotation matrix that gives the transformation
C                 from the frame specified by ID to the frame
C                 specified by FRAME.
C
C     FRAME       is the id code of the frame used to define the
C                 orientation of the frame given by ID.  ROT gives
C                 the transformation from the IF frame to
C                 the frame specified by FRAME.
C
C     FOUND       is a logical indicating whether or not a frame
C                 definition for frame ID was constructed from
C                 kernel pool data.  If ROT and FRAME were constructed
C                 FOUND will be returned with the value TRUE.
C                 Otherwise it will be returned with the value FALSE.
C
C$ Parameters
C
C      BUFSIZ     is the number of rotation, frame id pairs that
C                 can have their instance data buffered for the
C                 sake of improving run-time performance.  This
C                 value MUST be positive and should probably be
C                 at least 10.
C
C$ Files
C
C      This routine makes use of the loaded text kernels to
C      determine the rotation from a constant offset frame
C      to its defining frame.
C
C$ Exceptions
C
C     1)  If some instance value associated with this frame
C         cannot be located, or does not have the proper type
C         or dimension, the error will be diagnosed by the
C         routine BADKPV. In such a case FOUND will be set to .FALSE.
C
C     2)  If the input ID has the value 0, the error
C         'SPICE(ZEROFRAMEID)' will be signalled. FOUND will be set
C         to FALSE.
C
C     3) If the name of the frame corresponding to ID cannot be
C        determined, the error 'SPICE(INCOMPLETEFRAME)' is signalled.
C
C     4) If the frame given by ID is defined relative to a frame
C        that is unrecognized, the error SPICE(BADFRAMESPEC)
C        will be signalled.  FOUND will be set to FALSE.
C
C     5) If the kernel pool specification for ID is not one of
C        MATRIX, ANGLES, or QUATERNION, then the error
C        'SPICE(UNKNOWNFRAMESPEC)' will be signalled. FOUND will be
C        set to FALSE.
C
C
C$ Particulars
C
C     This routine is used to construct the rotation from some frame
C     that is a constant rotation offset from some other reference
C     frame. This rotation is derived from data stored in the kernel
C     pool.
C
C     It is considered to be an low level routine that
C     will need to be called directly only by persons performing
C     high volume processing.
C
C$ Examples
C
C     This is intended to be used as a low level routine by
C     the frame system software.  However, you could use this
C     routine to directly retrieve the rotation from an offset
C     frame to it's relative frame.  One instance in which you
C     might do this is if you have a properly specified topocentric
C     frame for some site on earth and you wish to determine
C     the geodetic latitude and longitude of the site.  Here's how.
C
C        Suppose the name of the topocentric frame is: 'MYTOPO'.
C        First we get the id-code of the topocentric frame.
C
C        CALL NAMFRM ( 'MYTOPO', FRCODE )
C
C        Next get the rotation from the topocentric frame to
C        the bodyfixed frame.
C
C        CALL TKFRAM ( FRCODE, ROT, FRAME, FOUND )
C
C        Make sure the topoframe is relative to one of the earth
C        fixed frames.
C
C        CALL FRMNAM( FRAME, TEST )
C
C        IF (       TEST .NE. 'IAU_EARTH'
C       .     .AND. TEST .NE. 'EARTH_FIXED'
C       .     .AND. TEST .NE. 'ITRF93'  ) THEN
C
C           WRITE (*,*) 'The frame MYTOPO does not appear to be '
C           WRITE (*,*) 'defined relative to an earth fixed frame.'
C           STOP
C
C        END IF
C
C        Things look ok. Get the location of the Z-axis in the
C        topocentric frame.
C
C        Z(1) = ROT(1,3)
C        Z(2) = ROT(2,3)
C        Z(3) = ROT(3,3)
C
C        Convert the Z vector to latitude longitude and radius.
C
C        CALL RECLAT ( Z, LAT, LONG, RAD )
C
C        WRITE (*,*) 'The geodetic coordinates of the center of'
C        WRITE (*,*) 'the topographic frame are: '
C        WRITE (*,*)
C        WRITE (*,*) 'Latitude  (deg): ', LAT *DPR()
C        WRITE (*,*) 'Longitude (deg): ', LONG*DPR()
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C      W.L. Taber      (JPL)
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 21-NOV-2001 (FST)
C
C        Updated this routine to dump the buffer of frame ID codes
C        it saves when it or one of the modules in its call tree signals
C        an error.  This fixes a bug where a frame's ID code is
C        buffered, but the matrix and kernel pool watcher were not set
C        properly.
C
C-    SPICELIB Version 1.0.0, 18-NOV-1996 (WLT)
C
C-&
 
C$ Index_Entries
C
C     Fetch the rotation and frame of a text kernel frame
C     Fetch the rotation and frame of a constant offset frame.
C
C-&
 
 
C
C     Spicelib Functions
C
      DOUBLE PRECISION      VDOT
      INTEGER               RTRIM
      LOGICAL               BADKPV
      LOGICAL               FAILED
      LOGICAL               RETURN
 
C
C     Local Parameters
C
 
      INTEGER               WDSIZE
      PARAMETER           ( WDSIZE = 32 )
 
      INTEGER               NITEMS
      PARAMETER           ( NITEMS = 14 )
 
      INTEGER               NVARS
      PARAMETER           ( NVARS = 14 )
 
      INTEGER               NDPS
      PARAMETER           ( NDPS  =  9 )
 
      INTEGER               NINTS
      PARAMETER           ( NINTS =  1 )
 
 
      INTEGER               LBPOOL
      PARAMETER           ( LBPOOL = -5 )
 
      INTEGER               SMWDSZ
      PARAMETER           ( SMWDSZ = 8 )
 
      CHARACTER*(1)         TYPE
 
      CHARACTER*(WDSIZE)    AGENT
      CHARACTER*(WDSIZE)    ALTNAT
      CHARACTER*(WDSIZE)    ALT    ( NITEMS )
      CHARACTER*(WDSIZE)    ITEM   ( NVARS  )
      CHARACTER*(WDSIZE)    UNITS
      CHARACTER*(WDSIZE)    SPEC
      CHARACTER*(WDSIZE)    NAME
      CHARACTER*(WDSIZE)    FRNAME
      CHARACTER*(WDSIZE)    IDSTR
 
      CHARACTER*(SMWDSZ)    VERSN
 
 
      DOUBLE PRECISION      ANGLES ( 3 )
      DOUBLE PRECISION      BUFFD  ( NDPS, BUFSIZ )
      DOUBLE PRECISION      MATRIX ( 3,    3      )
      DOUBLE PRECISION      QUATRN ( 0:3 )
 
      INTEGER               AR
      INTEGER               AT
      INTEGER               AXES   ( 3 )
      INTEGER               BUFFI  ( NINTS,     BUFSIZ )
      INTEGER               I
      INTEGER               IDENTS ( 1,         BUFSIZ )
      INTEGER               IDNT   ( 1 )
      INTEGER               N
      INTEGER               POOL   ( 2, LBPOOL: BUFSIZ )
      INTEGER               R
 
      LOGICAL               FIRST
      LOGICAL               FND
      LOGICAL               BUFFRD
      LOGICAL               UPDATE
 
 
      SAVE
C
C     Initial Values.
C
      DATA                  FIRST  / .TRUE.  /
 
 
 
C
C     Before we even check in, if N is less than 1 we can
C     just return.
C
C
C     Perform any initializations that might be needed for this
C     routine.
C
      IF ( FIRST ) THEN
 
         FIRST = .FALSE.
         VERSN = '1.0.0'
 
         CALL LNKINI ( BUFSIZ, POOL )
 
      END IF
 
C
C     Now do the standard SPICE error handling.  Sure this is
C     a bit unconventional, but nothing will be hurt by doing
C     the stuff above first.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF
 
      CALL CHKIN ( 'TKFRAM' )
 
C
C     So far, we've not FOUND the rotation to the specified frame.
C
      FOUND = .FALSE.
 
C
C     Check the ID to make sure it is non-zero.
C
      IF (  ID .EQ. 0 ) THEN
 
         CALL LNKINI ( BUFSIZ, POOL )
 
         CALL SETMSG ( 'Frame identification codes are '
     .   //            'required to be non-zero.  You''ve '
     .   //            'specified a frame with ID value '
     .   //            'zero. ' )
         CALL SIGERR ( 'SPICE(ZEROFRAMEID)'  )
         CALL CHKOUT ( 'TKFRAM' )
         RETURN
 
      END IF
 
 
C
C     Lookup the address of the instance data.
C
      IDNT(1) = ID
      CALL LOCATI ( IDNT, 1, IDENTS, POOL, AT, BUFFRD )
C
C     Until we have better information we put the identity matrix
C     into the output rotation and set FRAME to zero.
C
      CALL IDENT ( ROT )
      FRAME = 0
 
C
C     If we have to look up the data for our frame, we do
C     it now and perform any conversions and computations that
C     will be needed when it's time to convert coordinates to
C     directions.
C
C     Construct the name of the agent associated with the
C     requested frame.  (Each frame has its own agent).
C
      CALL INTSTR ( ID, IDSTR  )
      CALL FRMNAM ( ID, FRNAME )
 
      IF ( FRNAME .EQ. ' ' ) THEN
 
         CALL LNKINI ( BUFSIZ, POOL )
 
         CALL SETMSG ( 'The Text Kernel (TK) frame with id-code '
     .   //            '# does not have a recognized name. ' )
         CALL ERRINT ( '#', ID                  )
         CALL SIGERR ( 'SPICE(INCOMPLETFRAME)'  )
         CALL CHKOUT ( 'TKFRAM'                 )
         RETURN
 
      END IF
 
      AGENT  = 'TKFRAME_' // IDSTR
      R      =  RTRIM(AGENT)
 
      ALTNAT = 'TKFRAME_' // FRNAME
      AR     =  RTRIM(ALTNAT)
 
C
C     If the frame is buffered, we check the kernel pool to
C     see if there has been an update to this frame.
C
      IF ( BUFFRD ) THEN
 
         CALL CVPOOL ( AGENT(1:R), UPDATE )
 
C
C     If the frame is not buffered we definitely need to update
C     things.
C
      ELSE
 
         UPDATE = .TRUE.
 
      END IF
 
 
      IF ( .NOT. UPDATE ) THEN
C
C        Just look up the rotation matrix and relative-to
C        information from the local buffer.
C
         ROT(1,1)  = BUFFD ( 1, AT )
         ROT(2,1)  = BUFFD ( 2, AT )
         ROT(3,1)  = BUFFD ( 3, AT )
         ROT(1,2)  = BUFFD ( 4, AT )
         ROT(2,2)  = BUFFD ( 5, AT )
         ROT(3,2)  = BUFFD ( 6, AT )
         ROT(1,3)  = BUFFD ( 7, AT )
         ROT(2,3)  = BUFFD ( 8, AT )
         ROT(3,3)  = BUFFD ( 9, AT )
 
         FRAME     = BUFFI ( 1, AT )
 
      ELSE
C
C        Determine how the frame is specified and what it
C        is relative to.  The variables that specify
C        how the frame is represented and what it is relative to
C        are TKFRAME_#_SPEC and TKFRAME_#_RELATIVE where # is
C        replaced by the text value of ID or the frame name.
C
         ITEM(1) = AGENT(1:R) // '_SPEC'
         ITEM(2) = AGENT(1:R) // '_RELATIVE'
 
         ALT (1) = ALTNAT(1:AR) // '_SPEC'
         ALT (2) = ALTNAT(1:AR) // '_RELATIVE'
C
C        See if the friendlier version of the kernel pool variables
C        are available.
C
         DO I = 1, 2
            CALL DTPOOL ( ALT(1), FOUND, N, TYPE )
 
            IF ( FOUND ) THEN
               ITEM(I) = ALT(I)
            END IF
         END DO
 
 
C
C        If either the SPEC or RELATIVE frame are missing from
C        the kernel pool, we simply return.
C
         IF (     BADKPV ( 'TKFRAM', ITEM(1), '=', 1, 1, 'C' )
     .       .OR. BADKPV ( 'TKFRAM', ITEM(2), '=', 1, 1, 'C' ) ) THEN
 
            CALL LNKINI ( BUFSIZ, POOL )
            FRAME =  0
            CALL IDENT  (  ROT     )
            CALL CHKOUT ( 'TKFRAM' )
            RETURN
 
         END IF
 
C
C        If we make it this far, look up the SPEC and RELATIVE frame.
C
         CALL GCPOOL ( ITEM(1),   1, 1, N, SPEC,  FND )
         CALL GCPOOL ( ITEM(2),   1, 1, N, NAME,  FND )
 
C
C        Look up the id-code for this frame.
C
         CALL NAMFRM ( NAME, FRAME )
 
         IF ( FRAME .EQ. 0 ) THEN
 
            CALL LNKINI ( BUFSIZ, POOL )
 
            CALL SETMSG ( 'The frame to which frame # is '
     .      //            'relatively defined is not '
     .      //            'recognized. The kernel pool '
     .      //            'specification of the relative '
     .      //            'frame is ''#''.  This is not a '
     .      //            'recognized frame. ' )
            CALL ERRINT ( '#', ID   )
            CALL ERRCH  ( '#', NAME )
            CALL SIGERR ( 'SPICE(BADFRAMESPEC)' )
            CALL CHKOUT ( 'TKFRAM' )
            RETURN
 
         END IF
C
C        Convert SPEC to upper case so that we can easily check
C        to see if this is one of the expected specification types.
C
         CALL UCASE ( SPEC, SPEC )
 
         IF      ( SPEC .EQ. 'MATRIX' ) THEN
C
C           This is the easiest case.  Just grab the matrix
C           from the kernel pool (and polish it up a bit just
C           to make sure we have a rotation matrix).
C
C           We give preference to the the kernel pool variable
C           TKFRAME_<name>_MATRIX if it is available.
C
            ITEM(3) = AGENT (1:R)  // '_MATRIX'
            ALT (3) = ALTNAT(1:AR) // '_MATRIX'
 
            CALL DTPOOL ( ALT(3), FOUND, N, TYPE )
 
            IF ( FOUND ) THEN
               ITEM(3) = ALT(3)
            END IF
 
            IF ( BADKPV ( 'TKFRAM', ITEM(3), '=', 9, 1, 'N' ) ) THEN
 
               CALL LNKINI ( BUFSIZ, POOL )
               FRAME =  0
               CALL IDENT  ( ROT      )
               CALL CHKOUT ( 'TKFRAM' )
               RETURN
 
            END IF
 
C
C           The variable meets current expectations, look it up
C           from the kernel pool.
C
            CALL GDPOOL ( ITEM(3),   1, 9, N, MATRIX,  FND )
 
C
C           In this case the full transformation matrix has been
C           specified.  We simply polish it up a bit.
C
            CALL VHAT ( MATRIX(1,1), ROT   (1,1)           )
            CALL UCRSS( MATRIX(1,1), MATRIX(1,2), ROT(1,3) )
            CALL UCRSS( ROT   (1,3), ROT   (1,1), ROT(1,2) )
 
            IF ( VDOT( ROT(1,2), MATRIX(1,2) ) .LT. 0.0D0 ) THEN
               CALL VSCL ( -1.0D0, ROT(1,2), ROT(1,2) )
            END IF
 
            IF ( VDOT( ROT(1,3), MATRIX(1,3) ) .LT. 0.0D0 ) THEN
               CALL VSCL ( -1.0D0, ROT(1,3), ROT(1,3) )
            END IF
 
 
         ELSE IF ( SPEC .EQ. 'ANGLES' ) THEN
C
C           Look up the angles, their units and axes for the
C           frame specified by ID. (Note that UNITS are optional).
C           As in the previous case we give preference to the
C           form TKFRAME_<name>_<item> over TKFRAME_<id>_<item>.
C
            ITEM(3) = AGENT(1:R)   // '_ANGLES'
            ITEM(4) = AGENT(1:R)   // '_AXES'
            ITEM(5) = AGENT(1:R)   // '_UNITS'
 
            ALT (3) = ALTNAT(1:AR) // '_ANGLES'
            ALT (4) = ALTNAT(1:AR) // '_AXES'
            ALT (5) = ALTNAT(1:AR) // '_UNITS'
C
C           Again, we give preference to the more friendly form
C           of TKFRAME specification.
C
            DO I = 3, 5
 
               CALL DTPOOL ( ALT(I), FOUND, N, TYPE )
 
               IF ( FOUND ) THEN
                  ITEM(I) = ALT(I)
               END IF
 
            END DO
 
 
            IF (    BADKPV( 'TKFRAM', ITEM(3), '=', 3, 1, 'N' )
     .         .OR. BADKPV( 'TKFRAM', ITEM(4), '=', 3, 1, 'N' ) )
     .      THEN
 
               CALL LNKINI ( BUFSIZ, POOL )
               FRAME =  0
               CALL IDENT  ( ROT      )
               CALL CHKOUT ( 'TKFRAM' )
               RETURN
 
            END IF
 
            UNITS = 'RADIANS'
 
            CALL GDPOOL ( ITEM(3),   1, 3, N, ANGLES, FND )
            CALL GIPOOL ( ITEM(4),   1, 3, N, AXES,   FND )
            CALL GCPOOL ( ITEM(5),   1, 1, N, UNITS,  FND )
 
C
C           Convert angles to radians.
C
            CALL CONVRT ( ANGLES(1), UNITS, 'RADIANS', ANGLES(1) )
            CALL CONVRT ( ANGLES(2), UNITS, 'RADIANS', ANGLES(2) )
            CALL CONVRT ( ANGLES(3), UNITS, 'RADIANS', ANGLES(3) )
 
            IF ( FAILED() ) THEN
               CALL LNKINI ( BUFSIZ, POOL )
               FRAME =  0
               CALL IDENT  ( ROT      )
               CALL CHKOUT ( 'TKFRAM' )
               RETURN
            END IF
 
C
C           Compute the rotation from instrument frame to CK frame.
C
            CALL EUL2M (  ANGLES(1),   ANGLES(2),   ANGLES(3),
     .                    AXES  (1),   AXES  (2),   AXES  (3), ROT )
 
 
         ELSE IF ( SPEC .EQ. 'QUATERNION' ) THEN
C
C           Look up the quaternion and convert it to a rotation
C           matrix. Again there are two possible variables that
C           may point to the quaternion. We give preference to
C           the form TKFRAME_<name>_Q over the form TKFRAME_<id>_Q.
C
            ITEM(3) = AGENT(1:R)   // '_Q'
            ALT (3) = ALTNAT(1:AR) // '_Q'
 
            CALL DTPOOL ( ALT(3), FOUND, N, TYPE )
 
            IF ( FOUND ) THEN
               ITEM(3) = ALT(3)
            END IF
 
 
            IF ( BADKPV ( 'TKFRAM', ITEM(3), '=', 4, 1, 'N' ) ) THEN
               CALL LNKINI ( BUFSIZ, POOL )
               FRAME =  0
               CALL IDENT  ( ROT      )
               CALL CHKOUT ( 'TKFRAM' )
               RETURN
            END IF
C
C           In this case we have the quaternion representation.
C           Again, we do a small amount of polishing of the input.
C
            CALL GDPOOL ( ITEM(3), 1, 4, N, QUATRN, FND )
            CALL VHATG  ( QUATRN,  4,       QUATRN )
            CALL Q2M    ( QUATRN,           ROT    )
 
         ELSE
C
C           We don't recognize the SPEC for this frame.  Say
C           so.  Also note that perhaps the user needs to upgrade
C           the toolkit.
C
            CALL LNKINI ( BUFSIZ, POOL )
 
            CALL SETMSG ( 'The frame specification "# = '
     .      //            '''#''" is not one of the reconized '
     .      //            'means of specifying a text-kernel '
     .      //            'constant offset frame (as of '
     .      //            'version # of the routine TKFRAM). '
     .      //            'This may reflect a typographical '
     .      //            'error or may indicate that you '
     .      //            'need to consider updating your '
     .      //            'version of the SPICE toolkit. ' )
 
 
 
            CALL ERRCH  ( '#', ITEM(1) )
            CALL ERRCH  ( '#', SPEC    )
            CALL ERRCH  ( '#', VERSN   )
            CALL SIGERR ( 'SPICE(UNKNOWNFRAMESPEC)' )
            CALL CHKOUT ( 'TKFRAM' )
            RETURN
 
         END IF
 
C
C        Buffer the identifier, relative frame and rotation matrix.
C
 
         BUFFD ( 1, AT ) = ROT(1,1)
         BUFFD ( 2, AT ) = ROT(2,1)
         BUFFD ( 3, AT ) = ROT(3,1)
         BUFFD ( 4, AT ) = ROT(1,2)
         BUFFD ( 5, AT ) = ROT(2,2)
         BUFFD ( 6, AT ) = ROT(3,2)
         BUFFD ( 7, AT ) = ROT(1,3)
         BUFFD ( 8, AT ) = ROT(2,3)
         BUFFD ( 9, AT ) = ROT(3,3)
 
         BUFFI ( 1, AT ) = FRAME
 
C
C        If these were not previously buffered, we need to set
C        a watch on the various items that might be used to define
C        this frame.
C
         IF ( .NOT. BUFFRD ) THEN
C
C           Immediately check for an update so that we will
C           not redundantly look for this item the next time this
C           routine is called.
C
            ITEM( 1) = AGENT(1:R)   // '_RELATIVE'
            ITEM( 2) = AGENT(1:R)   // '_SPEC'
            ITEM( 3) = AGENT(1:R)   // '_AXES'
            ITEM( 4) = AGENT(1:R)   // '_MATRIX'
            ITEM( 5) = AGENT(1:R)   // '_Q'
            ITEM( 6) = AGENT(1:R)   // '_ANGLES'
            ITEM( 7) = AGENT(1:R)   // '_UNITS'
 
            ITEM( 8) = ALTNAT(1:AR) // '_RELATIVE'
            ITEM( 9) = ALTNAT(1:AR) // '_SPEC'
            ITEM(10) = ALTNAT(1:AR) // '_AXES'
            ITEM(11) = ALTNAT(1:AR) // '_MATRIX'
            ITEM(12) = ALTNAT(1:AR) // '_Q'
            ITEM(13) = ALTNAT(1:AR) // '_ANGLES'
            ITEM(14) = ALTNAT(1:AR) // '_UNITS'
 
            CALL SWPOOL ( AGENT, 14, ITEM )
            CALL CVPOOL ( AGENT, UPDATE      )
 
         END IF
 
      END IF
 
      IF ( FAILED() ) THEN
         CALL LNKINI ( BUFSIZ, POOL )
         CALL CHKOUT ( 'TKFRAM' )
         RETURN
      END IF
 
C
C     All errors cause the routine to exit before we get to this
C     point.  If we reach this point we didn't have an error and
C     hence did find the rotation from ID to FRAME.
C
      FOUND = .TRUE.
 
C
C     That's it
C
      CALL CHKOUT ( 'TKFRAM' )
      RETURN
 
      END
