C$Procedure      VPERP ( Perpendicular component of a 3-vector)
 
      SUBROUTINE VPERP ( A, B, P )
 
C$ Abstract
C
C      Find the component of a vector that is perpendicular to a second
C      vector.  All vectors are 3-dimensional.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C      VECTOR
C
C$ Declarations
 
      DOUBLE PRECISION   A ( 3 )
      DOUBLE PRECISION   B ( 3 )
      DOUBLE PRECISION   P ( 3 )
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C       A         I    The vector whose orthogonal component is sought.
C       B         I    The vector used as the orthogonal reference.
C       P         O    The component of A orthogonal to B.
C
C$ Detailed_Input
C
C      A     is a double precision, 3-dimensional vector.  It the vector
C            whose component orthogonal to B is sought. (There is a
C            unique decomposition of A into a sum V + P, where V is
C            parallel to B and P is orthogonal to B.  We want the
C            component P.)
C
C      B     is a double precision, 3-dimensional vector.  This
C            vector is the vector used as a reference for the
C            decomposition of A.
C
C
C$ Detailed_Output
C
C      P     is a double precision, 3-dimensional vector containing
C            the the component of A that is orthogonal to B.
C            P may overwrite either A or B.
C
C$ Parameters
C
C     None.
C
C$ Particulars
C
C      Given and non-zero vector B and a vector A, there is a unique
C      decomposition of A as a sum V + P such that P is orthogonal
C      to B and V is parallel to B.  This routine finds the vector P.
C
C      If B is a zero vector, P will be identical to A.
C
C$ Examples
C
C      The following table gives sample inputs and results from calling
C      VPERP.
C
C      A                  B                 P
C      ------------------------------------------
C      (6, 6, 6)      ( 2, 0, 0)        (0, 6, 6)
C      (6, 6, 6)      (-3, 0, 0)        (0, 6, 6)
C      (6, 6, 0)      ( 0, 7, 0)        (6, 0, 0)
C      (6, 0, 0)      ( 0, 0, 9)        (6, 0, 0)
C
C$ Restrictions
C
C      None.
C
C$ Exceptions
C
C      Error free.
C
C$ Files
C
C      None.
C
C$ Author_and_Institution
C
C      W.L. Taber      (JPL)
C
C$ Literature_References
C
C      REFERENCE: Any reasonable calculus text (for example Thomas)
C
C$ Version
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (WLT)
C
C-&
 
C$ Index_Entries
C
C     perpendicular component of a 3-vector
C
C-&
 
 
      DOUBLE PRECISION      BIGA
      DOUBLE PRECISION      BIGB
      DOUBLE PRECISION      R(3)
      DOUBLE PRECISION      T(3)
      DOUBLE PRECISION      V(3)
 
      BIGA = MAX ( DABS(A(1)), DABS(A(2)), DABS(A(3)) )
      BIGB = MAX ( DABS(B(1)), DABS(B(2)), DABS(B(3)) )
C
C     If A is the zero vector, just set P to zero and return.
C
      IF ( BIGA .EQ. 0.0D0 ) THEN
         P(1) = 0.0D0
         P(2) = 0.0D0
         P(3) = 0.0D0
         RETURN
      END IF
 
C
C     If B is the zero vector, then set P equal to A
C
      IF ( BIGB .EQ. 0.0D0 ) THEN
 
         P(1) = A(1)
         P(2) = A(2)
         P(3) = A(3)
         RETURN
 
      END IF
 
      T(1) = A(1) / BIGA
      T(2) = A(2) / BIGA
      T(3) = A(3) / BIGA
 
      R(1) = B(1) / BIGB
      R(2) = B(2) / BIGB
      R(3) = B(3) / BIGB
 
      CALL VPROJ ( T,    R, V )
      CALL VSUB  ( T,    V, P )
      CALL VSCL  ( BIGA, P, P )
 
      RETURN
 
 
      END
