C$Procedure      VPROJ ( Vector projection, 3 dimensions )
 
      SUBROUTINE VPROJ ( A, B, P )
 
C$ Abstract
C
C      VPROJ finds the projection of one vector onto another vector.
C      All vectors are 3-dimensional.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C      VECTOR
C
C$ Declarations
 
      DOUBLE PRECISION   A ( 3 )
      DOUBLE PRECISION   B ( 3 )
      DOUBLE PRECISION   P ( 3 )
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C       A         I    The vector to be projected.
C       B         I    The vector onto which A is to be projected.
C       P         O    The projection of A onto B.
C
C$ Detailed_Input
C
C      A     is a double precision, 3-dimensional vector.  This
C            vector is to be projected onto the vector B.
C
C      B     is a double precision, 3-dimensional vector.  This
C            vector is the vector which receives the projection.
C
C$ Detailed_Output
C
C      P     is a double precision, 3-dimensional vector containing
C            the projection of A onto B.  P may overwrite either
C            A or B.  (P is necessarily parallel to B.)  If B is
C            the zero vector then P will be returned as the zero vector.
C
C$ Parameters
C
C     None.
C
C$ Particulars
C
C      The given any vectors A and B there is a unique decomposition
C      of A as a sum V + P such that V  the dot product of V and B
C      is zero, and the dot product of P with B is equal the product
C      of the lengths of P and B.  P is called the projection of
C      A onto B.  It can be expressed mathematically as
C
C            DOT(A,B)
C            -------- * B
C            DOT(B,B)
C
C      (This is not necessarily the prescription used to compute
C      the projection. It is intended only for descriptive purposes.)
C
C$ Examples
C
C      The following table gives sample inputs and results from calling
C      VPROJ.
C
C      A                  B           NDIM               P
C      -----------------------------------------------------------------
C      (6, 6, 6)      ( 2, 0, 0)        3            (6, 0, 0)
C      (6, 6, 6)      (-3, 0, 0)        3            (6, 0, 0)
C      (6, 6, 0)      ( 0, 7, 0)        3            (0, 6, 0)
C      (6, 0, 0)      ( 0, 0, 9)        3            (0, 0, 0)
C
C$ Restrictions
C
C      None.
C
C$ Exceptions
C
C      Error free.
C
C$ Files
C
C      None.
C
C$ Author_and_Institution
C
C      W.L. Taber      (JPL)
C
C$ Literature_References
C
C      REFERENCE: Any reasonable calculus text (for example Thomas)
C
C$ Version
C
C-     SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C         Comment section for permuted index source lines was added
C         following the header.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (WLT)
C
C-&
 
C$ Index_Entries
C
C     3-dimensional vector projection
C
C-&
 
      DOUBLE PRECISION VDOT
C
      DOUBLE PRECISION      BIGA
      DOUBLE PRECISION      BIGB
      DOUBLE PRECISION      R(3)
      DOUBLE PRECISION      T(3)
      DOUBLE PRECISION      SCALE
C
      BIGA = MAX ( DABS(A(1)), DABS(A(2)), DABS(A(3)) )
      BIGB = MAX ( DABS(B(1)), DABS(B(2)), DABS(B(3)) )
 
      IF ( BIGA .EQ. 0 ) THEN
         P(1) = 0.0D0
         P(2) = 0.0D0
         P(3) = 0.0D0
         RETURN
      END IF
 
      IF (  BIGB .EQ. 0 ) THEN
         P(1) = 0.0D0
         P(2) = 0.0D0
         P(3) = 0.0D0
         RETURN
      END IF
 
      R(1)  = B(1) / BIGB
      R(2)  = B(2) / BIGB
      R(3)  = B(3) / BIGB
 
      T(1)  = A(1) / BIGA
      T(2)  = A(2) / BIGA
      T(3)  = A(3) / BIGA
 
      SCALE = VDOT (T,R) * BIGA  / VDOT (R,R)
 
      CALL VSCL ( SCALE, R, P )
 
      RETURN
      END
