C$Procedure ZZASCII ( determine/verify EOL terminators in a text file )
 
      SUBROUTINE ZZASCII ( FILE, LINE, CHECK, TERMIN )
 
      IMPLICIT NONE
 
C$ Abstract
C
C     Returns a string indicating the line terminators of an ASCII file
C     and, if requested, stops execution if the terminator does match
C     the one that is native to the platform on which the toolkit was
C     compiled.
C
C$ Copyright
C
C     Copyright (2004), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     FILE TYPE
C
C$ Declarations
 
      CHARACTER*(*)         FILE
      CHARACTER*(*)         LINE
      LOGICAL               CHECK
      CHARACTER*(*)         TERMIN
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     FILE       I   Name of the text file to scan.
C     LINE       I   The work string for file reads.
C     CHECK      I   Flag directing to check for mismatched EOL.
C     TERMIN     0   The deduced terminator ID.
C
C$ Detailed_Input
C
C     FILE       the name of the ASCII file to scan for a line
C                terminator
C
C     LINE       a character string of sufficient length to perform the
C                line reads from FILE.
C
C     CHECK      a logical flag that, if set to .TRUE., instructs this
C                routine to check terminator that has been determined
C                against the one that is native to the platform, on
C                which the toolkit was compiled, and to generate error
C                if it was not the case. If set to .FALSE., instructs
C                the routine to bypass the check.
C
C$ Detailed_Output
C
C     TERMIN     the terminator ID extracted from FILE. The possible
C                values:
C
C                'CR'    - carriage return (Mac classic)
C                'LF'    - line feed (Unix)
C                'CR-LF' - carriage return and line feed (DOS)
C                '?'     - unable to determine, possibly
C                          due to an error event
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) A SPICE(STRINGTOOSHORT) error signals if LINE has length less
C        than 3.
C
C     2) A SPICE(FILEOPENFAILED) error signals if the file of interest
C        fails to open, i.e. IOSTAT < 0.
C
C     3) A text kernel found to contain non-native line terminators
C        and abort of the run was requested by causes this routine to
C        signal the error SPICE(INCOMPATIBLEEOL).
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     The function scans a string read from a text file to determine
C     the native platform of the file. The functions response is
C     unpredictable if it scans a binary file.
C
C$ Examples
C
C     To return EOL terminator for a given file:
C
C         CHARACTER*(5)    TERMIN
C         CHARACTER*(64)   LINE
C
C          ... given a file name
C          ... and a line long enough to hold a text string
C              from FILE
C
C         CALL ZZASCII( FILE, LINE, .FALSE., TERMIN )
C
C         CALL TOSTDO( 'FOUND FILE TERMINATOR '//TERMIN )
C
C     To stop if EOL terminator for a given file, if detected
C     successfully, is not native to this platform:
C
C         CHARACTER*(5)    TERMIN
C         CHARACTER*(64)   LINE
C
C          ... given a file name
C          ... and a line long enough to hold a text string
C              from FILE
C
C         CALL ZZASCII( FILE, LINE, .TRUE., TERMIN )
C
C     If the EOL terminator was not native, the call will generate
C     SPICE(INCOMPATIBLEEOL) error.
C
C$ Restrictions
C
C     1) The terminator detection is not performed if the read from
C        the file fails because the file is smaller than the allocated
C        LINE size or for any other reason.
C
C     2) The terminator detection is not possible if the length of the
C        first text line in the file exceeds the length of the LINE
C        work space.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     E.D. Wright      (JPL)
C     B.V. Semenov     (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.2.0, 03-JAN-2005 (BVS)
C
C        Updated for PC-CYGWIN_C.
C
C-    SPICELIB Version 1.1.0, 03-JAN-2005 (BVS)
C
C        Updated for PC-CYGWIN.
C
C-    SPICELIB Version 1.0.0, 17-FEB-2004 (EDW) (BVS)
C
C-&
 
C$ Index_Entries
C
C     determine ascii text file end-of-line type
C
C-&
 
C
C     SPICELIB functions.
C
      LOGICAL               EQSTR
      LOGICAL               RETURN
      INTEGER               RTRIM
 
C
C     Local parameters.
C
      INTEGER               EOLTSZ
      PARAMETER           ( EOLTSZ = 5 )
 
C
C     Local variables.
C
      CHARACTER*(EOLTSZ)    NATIVE
 
      INTEGER               DOSCNT
      INTEGER               I
      INTEGER               IOSTAT
      INTEGER               MACCNT
      INTEGER               NUMBER
      INTEGER               RECLEN
      INTEGER               UNXCNT
 
C
C     Discovery check-in. Can't determine the terminator in RETURN
C     mode.
C
      IF ( RETURN() ) THEN
         TERMIN = '?'
         RETURN
      END IF
 
C
C     Check-in to the error system.
C
      CALL CHKIN( 'ZZASCII' )
 
C
C     Retrieve the native line terminator.
C
      CALL ZZPLATFM ( 'TEXT_FORMAT', NATIVE )
 
C
C     If it is VAX, return immediately with undefined terminator.
C
      IF ( EQSTR( NATIVE, 'VAX' ) ) THEN
 
         TERMIN = '?'
 
         CALL CHKOUT ( 'ZZASCII' )
         RETURN
 
      END IF
 
C
C     Set the record lenght that will be used to read data from
C     the file.
C
      RECLEN  = LEN(LINE)
 
      IF ( MOD( RECLEN, 4 ) .NE. 0 ) THEN
         RECLEN = RECLEN / 4 + 1
      ELSE
         RECLEN = RECLEN / 4
      END IF
 
C
C     Check the length of the work string is sufficient to perform the
C     operations. Less than 3 is a no-op.
C
      IF ( LEN(LINE) .LT. 3 ) THEN
 
         TERMIN = '?'
 
         CALL SETMSG ( 'Work string lacks sufficient length'
     .              // ' to perform operation.')
         CALL SIGERR ( 'SPICE(STRINGTOOSHORT)' )
         CALL CHKOUT ( 'ZZASCII' )
         RETURN
 
      END IF
 
C
C     Find a free logical unit for file access.
C
      CALL GETLUN ( NUMBER )
 
C
C     Open the file for DIRECT access.
C
      OPEN ( UNIT               = NUMBER  ,
     .       FILE               = FILE( :RTRIM(FILE) ),
     .       ACCESS             = 'DIRECT',
     .       STATUS             = 'OLD'   ,
     .       RECL               = RECLEN  ,
     .       READONLY,
     .       IOSTAT             = IOSTAT  )
 
      IF ( IOSTAT .NE. 0 ) THEN
 
C
C        The open failed, can't determine the terminator if the routine
C        can't open the file.
C
         TERMIN = '?'
 
C
C        Execute a close, J.I.C.
C
         CLOSE( NUMBER )
 
         CALL SETMSG ( 'File open failed for file ''$1''. IOSTAT '
     .               //' value $2.' )
         CALL ERRCH  ( '$1', FILE            )
         CALL ERRINT ( '$2', IOSTAT          )
         CALL SIGERR ( 'SPICE(FILEOPENFAIL)' )
         CALL CHKOUT ( 'ZZASCII' )
         RETURN
 
      END IF
 
C
C     Read a line into the LINE variable assigned by the user.
C
      LINE = ' '
 
      READ ( NUMBER, REC=1, IOSTAT=IOSTAT ) LINE
 
      IF ( IOSTAT .NE. 0 ) THEN
 
C
C        If something went wrong during this read, a part or the whole
C        returned line may contain garbage. Instead of examining it and
C        making wrong determination based on it, set terminator to
C        undefined and return.
C
         TERMIN = '?'
 
C
C        Execute a close, J.I.C.
C
         CLOSE( NUMBER )
 
         CALL CHKOUT ( 'ZZASCII' )
         RETURN
 
      END IF
 
C
C     We have a line of text data. Use ICHAR to scan for carriage
C     returns and line feeds and count how may of various recognized
C     line termination sequences are in this line.
C
      DOSCNT = 0
      UNXCNT = 0
      MACCNT = 0
 
      I      = 1
 
      DO WHILE ( I .LT. LEN(LINE) )
 
C
C        Check for ICHAR values of 10 (LF) and 13 (CR).
C
         IF      ( ICHAR(LINE(I:I)) .EQ. 10 ) THEN
 
C
C           Found a UNIX line terminator LF.
C
            UNXCNT = UNXCNT + 1
 
         ELSE IF ( ICHAR(LINE(I:I)) .EQ. 13 ) THEN
 
C
C           Found CR, increment character counter and check
C           the next character.
C
            I = I + 1
 
            IF ( ICHAR(LINE(I:I)) .EQ. 10 ) THEN
 
C
C              Found a DOS line terminator CR+LF.
C
               DOSCNT = DOSCNT + 1
 
            ELSE
 
C
C              Found a Classic Mac line terminator CR.
C
               MACCNT = MACCNT + 1
 
            END IF
 
         END IF
 
         I = I + 1
 
      END DO
 
C
C     Examine the counters.
C
      IF      ( DOSCNT .GT. 0 .AND.
     .          UNXCNT .EQ. 0 .AND.
     .          MACCNT .EQ. 0       ) THEN
 
C
C        Only DOS terminator counter is non-zero. ID the file as DOS.
C
         TERMIN = 'CR-LF'
 
      ELSE IF ( DOSCNT .EQ. 0 .AND.
     .          UNXCNT .GT. 0 .AND.
     .          MACCNT .EQ. 0       ) THEN
 
 
C
C        Only Unix terminator counter is non-zero. ID the file as UNIX.
C
         TERMIN =  'LF'
 
      ELSE IF ( DOSCNT .EQ. 0 .AND.
     .          UNXCNT .EQ. 0 .AND.
     .          MACCNT .GT. 0       ) THEN
 
C
C        Only Mac terminator counter is non-zero. ID the file as Mac
C        Classic.
C
         TERMIN =  'CR'
 
      ELSE
 
C
C        We can get here in two cases. First if the line did not
C        contain any CRs or LFs. Second if the line contained more than
C        one kind of terminators. In either case the format of the file
C        is unclear.
C
         TERMIN = '?'
 
      END IF
 
C
C     Close the file.
C
      CLOSE ( NUMBER )
 
C
C     If we were told check the terminator against the native one, do
C     it.
C
      IF ( CHECK ) THEN
 
C
C        If the terminator was identified and does not match the native
C        one, error out.
C
         IF ( .NOT. EQSTR( TERMIN, NATIVE ) .AND.
     .        .NOT. EQSTR( TERMIN, '?'    )       ) THEN
 
            CALL SETMSG ( 'Text file ''$1'' contains lines '
     .               //   'terminated with ''$2'' while '
     .               //   'the expected terminator for this '
     .               //   'platform is ''$3''. This text '
     .               //   'file cannot be processed.'       )
 
            CALL ERRCH  ( '$1', FILE   )
            CALL ERRCH  ( '$2', TERMIN )
            CALL ERRCH  ( '$3', NATIVE )
            CALL SIGERR ( 'SPICE(INCOMPATIBLEEOL)' )
            CALL CHKOUT ( 'ZZASCII' )
            RETURN
 
         END IF
 
      END IF
 
      CALL CHKOUT( 'ZZASCII' )
 
      RETURN
      END
