C$Procedure    ZZEKILLE ( EK, indirect, last elt less than or equal to )
 
      INTEGER FUNCTION ZZEKILLE (  HANDLE,  SEGDSC,  COLDSC,  NROWS,
     .                             DTYPE,   CVAL,    DVAL,    IVAL    )
 
C$ Abstract
C
C     Find the ordinal position of the row, in an specified EK segment,
C     whose value in a specified column is the last last element less
C     than or equal to a specified value, where the order relation is
C     given by an order vector in a specified DAS file.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     DAS
C     EK
C
C$ Keywords
C
C     ARRAY
C     FILES
C     SORT
C     UTILITY
C
C$ Declarations
 
 
      INCLUDE 'ekcoldsc.inc'
      INCLUDE 'ekopcd.inc'
      INCLUDE 'eksegdsc.inc'
      INCLUDE 'ektype.inc'
 
      INTEGER               HANDLE
      INTEGER               SEGDSC ( SDSCSZ )
      INTEGER               COLDSC ( CDSCSZ )
      INTEGER               NROWS
      INTEGER               DTYPE
      CHARACTER*(*)         CVAL
      DOUBLE PRECISION      DVAL
      INTEGER               IVAL
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     HANDLE     I   HANDLE of EK file.
C     SEGDSC     I   Segment descriptor.
C     COLDSC     I   Descriptor of column to be searched.
C     NROWS      I   Number of rows in column.
C     DTYPE      I   Data type of input value.
C     CVAL       I   Character string value.
C     DVAL       I   Double precision value.
C     IVAL       I   Integer value.
C
C     The function returns the index of the last order vector element
C     that points to an array element that is less than or equal to
C     the input value of the same data type as the column.
C
C$ Detailed_Input
C
C     HANDLE         is the file handle of the EK containing the
C                    segment of interest.
C
C     SEGDSC         is the segment descriptor of the EK
C                    segment of interest.
C
C     COLDSC         is a column descriptor for the column whose
C                    entries are to be compared with an input scalar
C                    value.  The column must be indexed.
C
C     NROWS          is the number of rows in the segment of interest.
C
C     DTYPE          is the data type of the input scalar value.
C
C     CVAL,
C     DVAL,
C     IVAL           are a set of scalar variables of character,
C                    double precision, and integer type.  Whichever
C                    of these has the same data type as the column
C                    indicated by COLDSC is used to compare rows
C                    against.  If COLDSC has data type TIME, DVAL
C                    is used in the comparison.
C
C$ Detailed_Output
C
C     The function returns the index of the last order vector element
C     that points to a column entry that is less than or equal to
C     whichever of CVAL, DVAL, or IVAL has the same data type as the
C     input column.  If the least element of the column is greater than
C     the input value of the matching type, the function returns the
C     value zero.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1)  If the array size NROWS is non-positive, the error
C         SPICE(INVALIDSIZE) will be signalled.
C
C     2)  If HANDLE is invalid, the error will be diagnosed by routines
C         called by this routine.
C
C     3)  If an I/O error occurs during any access to the file
C         specified by HANDLE, the error will be diagnosed by routines
C         called by this routine.
C
C     4)  If any of SEGBAS, COLDSC, or NROWS are invalid, this routine
C         may fail in unpredictable, but possibly spectacular, ways.
C         Except as described in this header section, no attempt is
C         made to handle these errors.
C
C$ Files
C
C     See the description of the argument HANDLE in $Detailed_Input.
C
C$ Particulars
C
C     This routine supports allow rapid look-up of elements in indexed
C     EK columns.
C
C$ Examples
C
C     See ZZEKKEY.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman   (JPL)
C
C$ Version
C
C-    Beta Version 1.0.0, 10-OCT-1995 (NJB)
C
C-&
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
 
C
C     Local variables
C
      DOUBLE PRECISION      DNUM
 
      INTEGER               COLTYP
      INTEGER               INUM
      INTEGER               REC
 
C
C     Initialize the function's return value.
C
      ZZEKILLE  =  0
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'ZZEKILLE' )
      END IF
 
C
C     Validate the number of rows in the column.
C
      IF ( NROWS .LT. 1 ) THEN
C
C        There's nobody home---that is, there is nothing in the array
C        to compare against.  Zero is the only sensible thing to return.
C
         ZZEKILLE = 0
         CALL SETMSG ( 'Number of rows must be positive; was #.' )
         CALL ERRINT ( '#',  NROWS                               )
         CALL SIGERR ( 'SPICE(INVALIDSIZE)'                      )
         CALL CHKOUT ( 'ZZEKILLE'                                )
         RETURN
      END IF
 
C
C     Hand off the problem to the LLE routine of the correct type.
C
      COLTYP  =  COLDSC ( TYPIDX )
 
      IF ( COLTYP .EQ. CHR ) THEN
 
         CALL ZZEKLLEC ( HANDLE, SEGDSC, COLDSC, CVAL, ZZEKILLE, REC )
 
 
      ELSE IF ( COLTYP .EQ. DP ) THEN
 
         IF ( DTYPE .EQ. DP ) THEN
            DNUM = DVAL
         ELSE
            DNUM = IVAL
         END IF
 
         CALL ZZEKLLED ( HANDLE, SEGDSC, COLDSC, DNUM, ZZEKILLE, REC )
 
 
      ELSE IF ( COLTYP .EQ. TIME ) THEN
 
         CALL ZZEKLLED ( HANDLE, SEGDSC, COLDSC, DVAL, ZZEKILLE, REC )
 
 
      ELSE IF ( COLTYP .EQ. INT ) THEN
 
         IF ( DTYPE .EQ. DP ) THEN
            INUM = NINT ( DVAL )
         ELSE
            INUM = IVAL
         END IF
 
         CALL ZZEKLLEI ( HANDLE, SEGDSC, COLDSC, INUM, ZZEKILLE, REC )
 
      ELSE
 
         CALL SETMSG ( 'The data type # is not supported.' )
         CALL ERRINT ( '#',  COLTYP                        )
         CALL SIGERR ( 'SPICE(INVALIDSIZE)'                )
         CALL CHKOUT ( 'ZZEKILLE'                          )
         RETURN
 
      END IF
 
 
      CALL CHKOUT ( 'ZZEKILLE' )
      RETURN
      END
