C$Procedure      EXPFNM_1 ( Expand a filename )
 
      SUBROUTINE EXPFNM_1 ( INFIL, OUTFIL )
 
C$ Abstract
C
C     Given a filename, expand it to be a full filename.
C
C$ Copyright
C
C     Copyright (1995), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     FILES
C     UTILITY
C
C$ Declarations
 
 
      CHARACTER*(*)         INFIL
      CHARACTER*(*)         OUTFIL
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     INFIL      I   The filename to be expanded.
C     OUTFIL     O   The expanded filename.
C
C$ Detailed_Input
C
C     INFIL      is the filename to be expanded.
C
C$ Detailed_Output
C
C     OUTFIL     is the expanded filename. If no expansion could be
C                done, the value of OUTFIL is equal to the value of
C                INFIL. OUTFIL may not overwrite INFIL.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If the input filename is blank, begins with blank characters,
C        or has embedded blanks in it, the error SPICE(BADFILENAME)
C        is signalled.
C
C     2) If the expanded filename is too long to fit into the
C        output string, the error SPICE(STRINGTOOSMALL) is signalled.
C
C     3) The output string may not overwrite the input string.
C
C     4) If no expansion of the input filename can be done, the
C        output filename is assigned the value of the input filename.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     The input filename may not be blank, begin with blank characters,
C     nor may it it contain embedded blanks. As a general rule,
C     SPICELIB routines do not allow blank characters as part of a
C     filename.
C
C     Unix platforms:
C
C     On the Unix platforms, a filename containing an environment
C     variable must be expanded completely before FORTRAN can do
C     anything with it. FORTRAN interacts directly with the kernel, and
C     as a result does not pass input filenames through the shell
C     for expansion of environment variables.
C
C     VAX/VMS, Alpha/OpenVMS platforms:
C
C     The operating system does filname expansion itself, so this
C     routine currently does not expand the name.
C
C$ Examples
C
C     None.
C
C$ Restrictions
C
C     Unix platforms:
C
C     This routine cannot be used to expand a file name whose form
C     is '~xxx', where xxx is an account name.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     H.A. Neilan    (JPL)
C
C$ Version
C
C-    Beta Version 3.2.0, 03-JAN-2005 (BVS)
C
C        Updated for PC-CYGWIN_C.
C
C-    Beta Version 3.1.0, 03-JAN-2005 (BVS)
C
C        Updated for PC-CYGWIN.
C
C-    Beta Version 3.0.5, 17-JUL-2002 (BVS)
C
C        Added MAC-OSX environments.
C
C-    Beta Version 3.0.4, 08-OCT-1999 (WLT)
C
C        The environment lines were expanded so that the supported
C        environments are now explicitly given.  New
C        environments are WIN-NT
C
C-    Beta Version 3.0.3, 21-SEP-1999 (NJB)
C
C        CSPICE environments were added.  Some typos were corrected.
C
C-    Beta Version 3.0.2, 28-JUL-1999 (WLT)
C
C        The environment lines were expanded so that the supported
C        environments are now explicitly given.  New
C        environments are PC-DIGITAL, SGI-O32 and SGI-N32.
C
C-    Beta Version 3.0.1, 18-MAR-1999 (WLT)
C
C        The environment lines were expanded so that the supported
C        environments are now explicitly given.  Previously,
C        environments such as SUN-SUNOS and SUN-SOLARIS were implied
C        by the environment label SUN.
C
C-    Beta Version 3.0.0, 05-APR-1998 (NJB)
C
C        Added references to the PC-LINUX environment.
C
C-    Beta Version 2.1.0, 5-JAN-1995 (HAN)
C
C        Removed Sun Solaris environment since it is now the same
C        as the Sun OS 4.1.x environment.
C        Removed DEC Alpha/OpenVMS environment since it is now the
C        same as the VAX environment.
C
C-    Beta Version 2.0.0, 08-JUL-1994 (HAN)
C
C        The capability of resolving a Unix filename that contains
C        an environment variable directory specificiation plus a
C        filename has been added.
C
C-    Beta Version 1.0.0, 06-APR-1992 (HAN)
C
C-&
 
C$ Index_Entry
C
C     expand a filename
C
C-&
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
      INTEGER               POS
      INTEGER               RTRIM
 
 
C
C     Local variables
C
      INTEGER               BLANK
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'EXPFNM_1' )
      END IF
 
 
C
C     If the input filename is blank, that's an error.
C
      IF ( INFIL .EQ. ' ' ) THEN
         CALL SETMSG ( 'The input filename ''#'' was blank.' )
         CALL ERRCH  ( '#', INFIL                            )
         CALL SIGERR ( 'SPICE(BADFILENAME)'                  )
         CALL CHKOUT ( 'EXPFNM_1'                            )
         RETURN
      END IF
 
C
C     If there are blanks anywhere in the filename, SPICELIB
C     considers the filename to be invalid.
C
      BLANK = POS( INFIL(1:RTRIM(INFIL)), ' ', 1 )
 
      IF ( BLANK .NE. 0 ) THEN
         CALL SETMSG ( 'The input filename ''#'' had blank '         //
     .                 'characters in it.'                            )
         CALL ERRCH  ( '#', INFIL                                     )
         CALL SIGERR ( 'SPICE(BADFILENAME)'                           )
         CALL CHKOUT ( 'EXPFNM_1'                                     )
         RETURN
      END IF
 
C
C     Because the operating system takes care of any necessary filename
C     translations, just return the input filename.
C
      OUTFIL = INFIL
      CALL CHKOUT ( 'EXPFNM_1' )
      RETURN
      END
