 
C$Procedure NCODEI ( Encode integer value into integer item )
 
      SUBROUTINE NCODEI ( VALUE, ITEM )
      IMPLICIT NONE
 
C$ Abstract
C
C     Encode an integer value into an integer item.
C
C$ Copyright
C
C     Copyright (1996), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     CELLS
C
C$ Keywords
C
C     CELLS
C
C$ Declarations
 
      INTEGER               VALUE
      INTEGER               ITEM
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     VALUE      I   Non-negative integer value.
C     ITEM       O   Item into which VALUE has been encoded.
C
C$ Detailed_Input
C
C     VALUE       is an arbitrary non-negative integer value.
C
C$ Detailed_Output
C
C     ITEM        is an integer item, into which the value has been
C                 been encoded. The value can be recovered by calling
C                 subroutine DCODE.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If the number to be encoded is negative, the error
C        'SPICE(OUTOFRANGE)' is signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     NCODE (and its inverse, DCODE) provide a uniform way to store
C     and retrieve values from the control areas of cells and cell-
C     based data types. This makes it possible to write templates
C     for a generic data type, suitable for instantiation by a
C     pre-compiler.
C
C$ Examples
C
C     The following code fragment illustrates how NCODE and DCODE
C     can be used to create instantiable subroutine templates.
C
C        C
C        C     Check the bolzat counter, to see if the elements
C        C     have been freebished; if not, do it now, and set
C        C     the counter to zero.
C        C
C              CALL DCODE@ ( STRUCT(-4), BCOUNT )
C
C              IF ( BCOUNT .GT. 0 ) THEN
C                 CALL FREEB@ ( CARD@ ( STRUCT ), STRUCT(1)  )
C                 CALL NCODE@ (                0, STRUCT(-4) )
C              END IF
C
C     By replacing all occurrences of `@' with the appropriate
C     type ending (C, D, or I), this single template can give
C     rise to three separate pieces of type-dependent code.
C
C     The alternative to using NCODE and DCODE is to use simple
C     assignments for numeric cells, and calls to ENCHAR and
C     DECHAR for character cells, destroying the symmetry inherent
C     in the rest of the code.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     W.L. Taber     (JPL)
C     I.M. Underwood (JPL)
C
C$ Version
C
C-    Beta Version 1.0.0, 15-JUL-1989 (WLT) (IMU)
C
C-&
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'NCODEI' )
      END IF
 
      IF ( VALUE .GE. 0 ) THEN
         ITEM = VALUE
 
      ELSE
         CALL SETMSG ( 'Cannot encode #; must be non-negative.' )
         CALL ERRINT ( '#', VALUE                               )
         CALL SIGERR ( 'SPICE(OUTOFRANGE)'                      )
      END IF
 
      CALL CHKOUT ( 'NCODEI' )
      RETURN
 
 
 
 
C$Procedure DCODEI ( Decode integer value from integer item )
 
      ENTRY DCODEI ( ITEM, VALUE )
 
C$ Abstract
C
C     Decode the integer value stored in an integer item by a
C     previous call to NCODEI.
C
C$ Required_Reading
C
C     CELLS
C
C$ Keywords
C
C     CELLS
C
C$ Declarations
C
C     INTEGER               ITEM
C     INTEGER               VALUE
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     ITEM       I   Item into which an integer value has been encoded.
C     VALUE      O   Encoded value.
C
C$ Detailed_Input
C
C     ITEM        is an integer item, into which an integer value
C                 has been encoded by a previous call to NCODEI.
C
C$ Detailed_Output
C
C     VALUE       is the encoded value.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     NCODE (and its inverse, DCODE) provide a uniform way to store
C     and retrieve values from the control areas of cells and cell-
C     based data types. This makes it possible to write templates
C     for a generic data type, suitable for instantiation by a
C     pre-compiler.
C
C$ Examples
C
C     The following code fragment illustrates how NCODE and DCODE
C     can be used to create instantiable subroutine templates.
C
C        C
C        C     Check the bolzat counter, to see if the elements
C        C     have been freebished; if not, do it now, and set
C        C     the counter to zero.
C        C
C              CALL DCODE@ ( STRUCT(-4), BCOUNT )
C
C              IF ( BCOUNT .GT. 0 ) THEN
C                 CALL FREEB@ ( CARD@ ( STRUCT ), STRUCT(1)  )
C                 CALL NCODE@ (                0, STRUCT(-4) )
C              END IF
C
C     By replacing all occurrences of `@' with the appropriate
C     type ending (C, D, or I), this single template can give
C     rise to three separate pieces of type-dependent code.
C
C     The alternative to using NCODE and DCODE is to use simple
C     assignments for numeric cells, and calls to ENCHAR and
C     DECHAR for character cells, destroying the symmetry inherent
C     in the rest of the code.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     W.L. Taber     (JPL)
C     I.M. Underwood (JPL)
C
C$ Version
C
C-    Beta Version 1.0.0, 15-JUL-1989 (WLT) (IMU)
C
C-&
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'DCODEI' )
      END IF
 
      VALUE = ITEM
 
      CALL CHKOUT ( 'DCODEI' )
      RETURN
      END
