C$Procedure      PODBGC ( Pod, begin group, character )
 
      SUBROUTINE PODBGC ( POD )
      IMPLICIT NONE
 
C$ Abstract
C
C     Begin a new (empty) group within a pod.
C
C$ Copyright
C
C     Copyright (1996), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     PODS
C
C$ Keywords
C
C     ARRAYS
C     CELLS
C     PODS
C
C$ Declarations
 
      INTEGER               LBCELL
      PARAMETER           ( LBCELL = -5 )
 
      CHARACTER*(*)         POD ( LBCELL : * )
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     POD       I,O  Pod.
C
C$ Detailed_Input
C
C     POD       on input, is an arbitrary pod.
C
C$ Detailed_Output
C
C     POD       on output, is the same pod, in which the active
C               group has been sealed, and a new active group
C               (containing no elements) begun.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If POD does not have sufficient free space to create a new
C        group with room for at least one element, the pod is not
C        changed, and the error SPICE(TOOMANYPEAS) is signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     There are two ways to create a new group within a pod.
C     PODBG (begin group) seals the current contents of the pod,
C     and creates a new active group containing no elements.
C     PODDG (duplicate group) also seals the current contents
C     of the pod, but places a copy of the previous active
C     group into the new active group.
C
C     In both cases, the active group and all previous groups are
C     unavailable so long as the new group exists.
C
C     The active group of a pod may be removed by any of the
C     following routines: PODEG (end group), PODCG (close group),
C     or PODRG (replace group).
C
C$ Examples
C
C     Let the active group of POD be located in elements 21
C     through 40. Then following the call
C
C        CALL PODBGC ( POD )
C
C     the active group is located in elements 42 through 41.
C     In other words, element 41 has been appropriated by the
C     pod itself, and the active group is empty.
C
C     However, following the call
C
C        CALL PODDG ( POD )
C
C     the active group is located in elements 42 through 61,
C     and contains the same elements as the previous active
C     group.
C
C$ Restrictions
C
C     1) In any pod, only the active group should be accessed,
C        and its location should always be determined by PODBE
C        or PODON. Never assume that the active group begins
C        at POD(1).
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     W.L. Taber     (JPL)
C     I.M. Underwood (JPL)
C
C$ Version
C
C-    Beta Version 1.0.0, 15-JUL-1989 (WLT) (IMU)
C
C
C-&
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
      INTEGER               CARDC
      INTEGER               SIZEC
 
C
C     Local parameters
C
      INTEGER               CRDLOC
      PARAMETER           ( CRDLOC =  0 )
 
      INTEGER               GRPOFF
      PARAMETER           ( GRPOFF = -2 )
 
C
C     Local variables
C
      INTEGER               HAVE
      INTEGER               NEED
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'PODBGC' )
      END IF
 
C
C     There must be at least two spaces at the end of the pod:
C     one for bookkeeping, and one for the first element of
C     the new group.
C
      HAVE = SIZEC ( POD )
      NEED = CARDC ( POD ) + 2
 
      IF ( HAVE .LT. NEED ) THEN
         CALL SIGERR ( 'SPICE(TOOMANYPEAS)' )
         CALL CHKOUT ( 'PODBGC'             )
         RETURN
      END IF
 
C
C     Okay: go ahead and create the group. The offset of the active
C     group is stored in the first empty slot of the pod; when the
C     new group is removed, this will be reinstated as the offset of
C     the active group.
C
      POD(CARDC(POD) + 1) = POD(GRPOFF)
 
C
C     This requires the cardinality of the pod to increase by one.
C
      CALL SCARDC ( CARDC(POD) + 1, POD )
 
C
C     Surprise! The new cardinality is the same as the offset of
C     the new group!
C
      POD(GRPOFF) = POD(CRDLOC)
 
      CALL CHKOUT ( 'PODBGC' )
      RETURN
      END
