C$Procedure      CHCKSC ( Check scalar character string )
C
      SUBROUTINE CHCKSC ( NAME, VAL, COMP, EXP,  OK )
C
C$ Abstract
C
C     Check an string scalar value against some expected value.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C       TESTING
C
C$ Declarations
C
      IMPLICIT NONE
      CHARACTER*(*)         NAME
      CHARACTER*(*)         VAL
      CHARACTER*(*)         EXP
      CHARACTER*(*)         COMP
      LOGICAL               OK
 
C
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      NAME       I   the name of the variable to be examined.
C      VAL        I   the actual variable
C      EXP        I   the comparison value for the variable
C      COMP       I   the kind of comparison to perform.
C      TOL        I   the tolerance allowed in comparing.
C      OK         O   TRUE if the test passes, FALSE otherwise..
C
C$ Detailed_Input
C
C     NAME        is the string used to give the name of a variable
C
C     VAL         is the actual character variable to be examined
C
C     EXP         an expected value or bound on the value VAL.
C
C                    =    ---   check for strict equality
C                    >    ---   check for VAL >  EXP
C                    <    ---   check for VAL <  EXP
C                    >=   ---   check for VAL >= EXP ( LGE (VAL, EXP) )
C                    <=   ---   check for VAL <= EXP ( LLE (VAL, EXP) )
C                    !=   ---   check for VAL != EXP ( VAL .NE. EXP   )
C
C                    The following are the case insensitive versions
C                    of the previous checks.
C
C                    ~=    ---   check for strict equality
C                    ~>    ---   check for VAL >  EXP
C                    ~<    ---   check for VAL <  EXP
C                    ~>=   ---   check for VAL >= EXP
C                    ~<=   ---   check for VAL <= EXP
C                    ~!=   ---   check for VAL != EXP
C
C$ Detailed_Output
C
C     OK         if the check of the input variable is successful then
C                OK is given the value TRUE.  Otherwise OK is given the
C                value FALSE and a diagnostic message is sent to the
C                test logger.
C
C$ Parameters
C
C      None.
C
C$ Files
C
C      The result of a failure is automatically logged in the testing
C      log file and failure file.  Success is logged only if verbose
C      testing has been enabled.
C
C
C$ Exceptions
C
C     Error free.
C
C$ Particulars
C
C     This routine handles a wide variety of comparisons between
C     scalar string values.
C
C$ Examples
C
C     Suppose that you have just made a call to a subroutine that
C     you wish to test (call the routine SPUD) and you would like
C     to test an output string against an expected value.  Using
C     this routine you can automatically have the test result logged
C     in via the testing utitities.
C
C        CALL SPUD   (  INPUT,   OUTPUT )
C        CALL CHCKSC ( 'OUTPUT', OUTPUT, '=', EXPECT, OK )
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C      W.L. Taber      (JPL)
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-    Testing Utilities Version 1.0.0, 8-Nov-1994 (WLT)
C
C
C-&
C
C     Test Utility functions
C
      LOGICAL               VERBOS
 
 
C
C     Spicelib functions
C
      INTEGER               RTRIM
 
C
C     Local Variables
C
      INTEGER               LNSIZE
      PARAMETER           ( LNSIZE = 320 )
 
      CHARACTER*(LNSIZE)    MESSGE
      LOGICAL               FAIL
 
      INTEGER               LVAL
      INTEGER               LEXP
 
      INTEGER               MAXLEN
      PARAMETER           ( MAXLEN = 4000 )
 
      CHARACTER*(MAXLEN)    MYVAL
      CHARACTER*(MAXLEN)    MYEXP
      CHARACTER*(4)         MYCOMP
 
      INTEGER               STYSIZ
      PARAMETER           ( STYSIZ = 120 )
 
      CHARACTER*(STYSIZ)    GOOD
      CHARACTER*(STYSIZ)    BAD
 
      CALL TSTSTY ( GOOD, BAD )
      CALL TSTLGS ( 'LEFT 3 RIGHT 75 NEWLINE /cr ',
     .              'LEFT 3 RIGHT 75 NEWLINE /cr FLAG --- LEADER ---' )
 
 
 
      LVAL  = LEN ( VAL )
      LEXP  = LEN ( EXP )
      MYVAL = VAL
      MYEXP = EXP
 
 
      IF ( COMP(1:1) .EQ. '~' ) THEN
         MYCOMP = COMP(2:)
         CALL UCASE( MYVAL, MYVAL )
         CALL UCASE( MYEXP, MYEXP )
      ELSE
         MYCOMP = COMP
      END IF
 
 
      FAIL = .FALSE.
 
      IF      ( MYCOMP .EQ. '='  ) THEN
 
         FAIL = MYVAL(1:LVAL) .NE. MYEXP(1:LEXP)
 
         MESSGE = 'The value of # was not the value expected./cr'
     .   //       'The value was:/cr(3:) ''#'' /cr(-3:) '
     .   //       'the expected value was:/cr(3:) ''#''. '
     .
         CALL REPMC ( MESSGE, '#', NAME(1:RTRIM(NAME)), MESSGE )
         CALL REPMC ( MESSGE, '#', MYVAL(1:LVAL),       MESSGE )
         CALL REPMC ( MESSGE, '#', MYEXP(1:LEXP),       MESSGE )
 
 
      ELSE IF ( MYCOMP .EQ. '<'  ) THEN
 
         FAIL   =  LGE( MYVAL(1:LVAL), MYEXP(1:LEXP) )
 
         MESSGE = 'The value of # was not less than ''#'' as was '
     .   //       'expected. /cr The value was: ''#''. '
 
 
         CALL REPMC ( MESSGE, '#', NAME(1:RTRIM(NAME)), MESSGE )
         CALL REPMC ( MESSGE, '#', MYEXP(1:LEXP),       MESSGE )
         CALL REPMC ( MESSGE, '#', MYVAL(1:LVAL),       MESSGE )
 
      ELSE IF ( MYCOMP .EQ. '>'  ) THEN
 
         FAIL   =  LLE( MYVAL(1:LVAL), MYEXP(1:LEXP) )
 
         MESSGE = 'The value of # was not greater than ''#'' as '
     .   //       'was expected. /cr The value was: ''#''. '
 
 
         CALL REPMC ( MESSGE, '#', NAME(1:RTRIM(NAME)), MESSGE )
         CALL REPMC ( MESSGE, '#', MYEXP(1:LEXP),       MESSGE )
         CALL REPMC ( MESSGE, '#', MYVAL(1:LVAL),       MESSGE )
 
      ELSE IF ( MYCOMP .EQ. '>=' ) THEN
 
         FAIL   =  LLT( MYVAL(1:LVAL), MYEXP(1:LEXP) )
 
         MESSGE = 'The value of # was not greater than ''#'' as '
     .   //       'was expected. /cr The value was: ''#''. '
 
 
         CALL REPMC ( MESSGE, '#', NAME(1:RTRIM(NAME)), MESSGE )
         CALL REPMC ( MESSGE, '#', MYEXP(1:LEXP),       MESSGE )
         CALL REPMC ( MESSGE, '#', MYVAL(1:LVAL),       MESSGE )
 
      ELSE IF ( MYCOMP .EQ. '<=' ) THEN
 
         FAIL   =  LGT( MYVAL(1:LVAL), MYEXP(1:LEXP) )
 
         MESSGE = 'The value of # was not less than ''#'' as was '
     .   //       'expected. /cr The value was: ''#''. '
 
 
         CALL REPMC ( MESSGE, '#', NAME(1:RTRIM(NAME)), MESSGE )
         CALL REPMC ( MESSGE, '#', MYEXP(1:LEXP),       MESSGE )
         CALL REPMC ( MESSGE, '#', MYVAL(1:LVAL),       MESSGE )
 
      ELSE IF ( MYCOMP .EQ. '!=' ) THEN
 
         FAIL   = MYVAL(1:LVAL) .EQ. MYEXP(1:LEXP)
 
         MESSGE = 'The value of # should have been different '
     .   //       'from ''#''./cr It wasn''t. '
 
 
         CALL REPMC ( MESSGE, '#', NAME(1:RTRIM(NAME)), MESSGE )
         CALL REPMC ( MESSGE, '#', MYEXP(1:LEXP),       MESSGE )
 
 
      ELSE
 
         FAIL = .TRUE.
 
         MESSGE = 'The comparison # is not recognized. '
         CALL REPMC ( MESSGE, '#', COMP(1:RTRIM(COMP)), MESSGE )
 
      END IF
 
      IF ( .NOT. FAIL ) THEN
 
         MESSGE = 'The comparison/cr/cr(3:) # # ''#'' /cr/cr(-3:)'
     .   //       'was satisfied./cr'
     .   //       'The value of # was ''#''. '
 
         CALL REPMC ( MESSGE, '#', NAME(1:RTRIM(NAME)), MESSGE )
         CALL REPMC ( MESSGE, '#', COMP(1:RTRIM(COMP)), MESSGE )
         CALL REPMC ( MESSGE, '#', MYEXP(1:LEXP),       MESSGE )
         CALL REPMC ( MESSGE, '#', NAME(1:RTRIM(NAME)), MESSGE )
         CALL REPMC ( MESSGE, '#', MYVAL(1:LVAL),       MESSGE )
 
         IF ( VERBOS() ) THEN
 
            CALL TSTLOG ( ' ',    FAIL )
            CALL TSTLOG ( MESSGE, FAIL )
 
         END IF
 
      ELSE
 
         CALL TSTLOG ( ' ',    FAIL )
         CALL TSTLOG ( MESSGE, FAIL )
 
      END IF
 
      CALL TSTLGS ( GOOD, BAD )
      OK = .NOT. FAIL
 
      RETURN
      END
 
 
 
 
 
 
