C$Procedure   NATPCK (Create a text PCK for Nat's solar system)
 
      SUBROUTINE NATPCK ( NAMEPC, LOADPC, KEEPPC )
 
C$ Abstract
C
C     Create and if appropriate load a text PCK kernel for Nat's
C     solar system.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     TESTING
C
C$ Declarations
 
      IMPLICIT NONE
      CHARACTER*(*)         NAMEPC
      LOGICAL               LOADPC
      LOGICAL               KEEPPC
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NAMEPC     I   The name of the PC-kernel to create.
C     LOADPC     I   Load the PC-kernel if .TRUE.
C     KEEPPC     I   Keep the PC-kernel if .TRUE., else delete it.
C
C$ Detailed_Input
C
C     NAMEPC      is the name of a PCK to create and load if LOADPC is
C                 set to .TRUE.  If a PCK of the same name already
C                 exists it is deleted.
C
C     LOADPC      is a logical that indicates whether or not the PCK
C                 file should be loaded after it is created.  If LOADPC
C                 has the value .TRUE. the PCK is loaded after it is
C                 created.  Otherwise it is left un-opened.
C
C     KEEPPC      is a logical that indicates whether or not the PCK
C                 file should be deleted after it is loaded.  If KEEPPC
C                 is .TRUE. the file is not deleted.  If KEEPPC is
C                 .FALSE. the file is deleted after it is loaded.
C
C                 Note this behavior is different from that of TSTPCK,
C                 which always keeps the file if the file is not loaded,
C                 regardless of the value of KEEPPC.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     This routine creates a PCK file meant for use with the SPK
C     file created by the test utility NATSPK.  If that routine
C     is changed, the body radii defined here may need to be
C     changed as well.  NATSPK contains commented-out lines of code
C     that generate appropriate body radii.
C
C     The PCK created by this routine contains
C
C        - Name-ID mappings for bodies ALPHA and BETA
C
C        - Body radii for bodies ALPHA and BETA
C
C        - Rotational elements for bodies ALPHA and BETA
C
C        - PCK frame definitions for the body-fixed frames 
C
C             ALPHAFIXED
C             BETAFIXED
C
C        - Associations of the bodyfixed frames with the
C          bodies ALPHA and BETA
C
C        - GM and radii for the "sun" of this solar system.  The
C          GM value is computed by NATSPK; normally the lines of
C          code that write out the value are commented out.
C
C          The smaller radii of the sun are provided because the
C          original values make the sun too large relative to the
C          orbital radii of the planets.
C
C$ Exceptions
C
C     1) Any I/O errors occurring in the creation of the PCK file
C        will be signaled by routines in the call tree of this
C        routine.  
C
C     Since this routine is normally used within the TSPICE system,
C     it's up the the caller to call CHCKXC to catch errors signaled by
C     this routine.
C 
C$ Particulars
C
C     This routine creates a planetary constants file for use in
C     testing.
C
C$ Examples
C
C     The normal way to use this routine is shown below.
C
C     CALL NATSPK ( 'nat.bsp', .TRUE., HANDLE  )
C     CALL NATPCK ( 'nat.tpc', .TRUE., .FALSE. )
C
C        [perform some tests and computations]
C
C     CALL SPKUEF ( HANDLE )
C     CALL KILFIL ( 'nat.bsp' )
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    Test Utilities 1.0.0, 29-SEP-2004 (NJB)
C
C-&
 
C$ Index_Entries
C
C     Create a "Nat's solar system" PCK file
C
C-&
 
 
C
C     Test Utility Functions
C
      CHARACTER*(16)        BEGDAT
      CHARACTER*(16)        BEGTXT
 
C
C     Local Parameters
C
      INTEGER               LNSIZE
      PARAMETER           ( LNSIZE = 80 )
 
      INTEGER               NLINES
      PARAMETER           ( NLINES = 48 )
 
C
C     Local Variables
C
      CHARACTER*(LNSIZE)    PCK  ( NLINES )
 
      INTEGER               I
      INTEGER               UNIT
 


      CALL CHKIN ( 'NATPCK' )

C
C     Delete any existing file of the same name.
C
      CALL KILFIL ( NAMEPC )
 
C
C     Fill the PCK buffer with the PCK kernel variable assignments.
C
      PCK(  1 ) = BEGDAT()
      PCK(  2 ) = ' '
      PCK(  3 ) = '      NAIF_BODY_NAME += ( ''ALPHA'', ''BET'
     .//          'A'' )'
      PCK(  4 ) = '      NAIF_BODY_CODE += ( 1000,     2000  '
     .//          ')'
      PCK(  5 ) = ' '
      PCK(  6 ) = ' '
      PCK(  7 ) = '      BODY1000_RADII = ( 0.366246987669377'
     .//          '12E+05,'
      PCK(  8 ) = '                         0.366246987669377'
     .//          '12E+05,'
      PCK(  9 ) = '                         0.366246987669377'
     .//          '12E+05  )'
      PCK( 10 ) = ' '
      PCK( 11 ) = '      BODY2000_RADII = ( 0.228915262710469'
     .//          '37E+04,'
      PCK( 12 ) = '                         0.228915262710469'
     .//          '37E+04,'
      PCK( 13 ) = '                         0.228915262710469'
     .//          '37E+04, )'
      PCK( 14 ) = ' '
      PCK( 15 ) = ' '
      PCK( 16 ) = '      BODY1000_POLE_RA        = (    0.   '
     .//          '    0.         0. )'
      PCK( 17 ) = '      BODY1000_POLE_DEC       = (  +90.   '
     .//          '    0.         0. )'
      PCK( 18 ) = '      BODY1000_PM             = (    0.   '
     .//          '    0.         0. )'
      PCK( 19 ) = ' '
      PCK( 20 ) = '      BODY2000_POLE_RA        = (    0.   '
     .//          '    0.         0. )'
      PCK( 21 ) = '      BODY2000_POLE_DEC       = (  +90.   '
     .//          '    0.         0. )'
      PCK( 22 ) = '      BODY2000_PM             = (    0.   '
     .//          '    0.         0. )'
      PCK( 23 ) = ' '
      PCK( 24 ) = ' '
      PCK( 25 ) = '      FRAME_ALPHAFIXED        =  1000001'
      PCK( 26 ) = '      FRAME_1000001_NAME      = ''ALPHAFIX'
     .//          'ED'''
      PCK( 27 ) = '      FRAME_1000001_CLASS     =  2'
      PCK( 28 ) = '      FRAME_1000001_CLASS_ID  =  1000'
      PCK( 29 ) = '      FRAME_1000001_CENTER    =  1000'
      PCK( 30 ) = ' '
      PCK( 31 ) = '      OBJECT_1000_FRAME       = ''ALPHAFIX'
     .//          'ED'''
      PCK( 32 ) = ' '
      PCK( 33 ) = ' '
      PCK( 34 ) = '      FRAME_BETAFIXED         =  1000002'
      PCK( 35 ) = '      FRAME_1000002_NAME      = ''BETAFIXE'
     .//          'D'''
      PCK( 36 ) = '      FRAME_1000002_CLASS     =  2'
      PCK( 37 ) = '      FRAME_1000002_CLASS_ID  =  2000'
      PCK( 38 ) = '      FRAME_1000002_CENTER    =  2000'
      PCK( 39 ) = ' '
      PCK( 40 ) = '      OBJECT_2000_FRAME       = ''BETAFIXE'
     .//          'D'''
      PCK( 41 ) = ' '
      PCK( 42 ) = ' '
      PCK( 43 ) = '      BODY10_RADII            = ( 1000, 10'
     .//          '00, 1000 )'
      PCK( 44 ) = ' '
      PCK( 45 ) = '      BODY10_GM               =  0.9974529'
     .//          '0739151156E+09'
      PCK( 46 ) = ' '
      PCK( 47 ) = BEGTXT()
      PCK( 48 ) = ' '



C
C     Create the PCK.
C
      CALL TXTOPN ( NAMEPC, UNIT )
 
      DO I = 1, NLINES
         CALL WRITLN ( PCK(I), UNIT )
      END DO
 
      CLOSE ( UNIT )

C
C     If this file needs to be loaded, do it. 
C
      IF ( LOADPC ) THEN
         CALL LDPOOL ( NAMEPC )
      END IF
 
      IF ( KEEPPC ) THEN
C
C        If we are keeping this file, we need to register it
C        with FILREG.
C
         CALL TFILES ( NAMEPC )

      ELSE
C
C        Lose the file.
C
         CALL KILFIL ( NAMEPC )

      END IF

      CALL CHKOUT ( 'NATPCK' )
      RETURN
      END
