C$Procedure      T_STAT (Test Status)
 
      SUBROUTINE T_STAT ( ACT, NAME, OK, NUMBER )
 
C$ Abstract
C
C     This routine provides an umbrella for a number of entry points
C     used for setting and retrieving the status of a family of
C     related test cases.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C       TESTING
C
C$ Declarations
 
      IMPLICIT NONE
      CHARACTER*(*)         ACT
      CHARACTER*(*)         NAME
      LOGICAL               OK
      INTEGER               NUMBER
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      NAME      I/O  is the name of a test family or test case.
C      OK         O   the success of a single case or family of cases.
C      NUMBER     O   the number of the current test case.
C
C$ Detailed_Input
C
C     See individual entry points for descriptions
C
C$ Detailed_Output
C
C     See individual entry points for descriptions
C
C$ Parameters
C
C      None.
C
C$ Files
C
C      None.
C
C$ Exceptions
C
C     Error free.
C
C     This routine has no calls to any external routines.
C
C$ Particulars
C
C     This routine maintains the status of a family of related test
C     routines (provided of course that users) make proper use
C     of the various entry points.  The entry points are:
C
C     T_BEGIN ( NAME )  --- set the NAME of the current family of
C                           tests. The family can have a name of up to
C                           12 characters
C
C     T_NAME  ( NAME )  --- return the name of the current family of
C                           tests
C
C     T_SUCCESS ( OK )  --- have all tests in this family passed to
C                           this point.
C
C     T_CASE  ( NAME )  --- gives a name (possibly blank) to the current
C                           test case. Set the current case success to
C                           .TRUE.  The name may be up to 60 characters.
C
C     T_CFAIL ( )       --- set the success of the current case to
C                          .FALSE.
C
C     T_CPASS ( OK )    --- return the success of the current test
C                           case.
C
C     T_CNUM  (NUMBER)  --- return the current test case number.
C
C     T_CNAME ( NAME )  --- return the full name of the current test
C                           case.  The full name returned may have up
C                           to 80 characters.
C
C     T_FCOUNT(NUMBER)  --- the number of test cases that failed.
C
C     T_TRACE ( ACT, NAME ) --- set or get the last stored value of
C                               the SPICELIB traceback.
C
C
C     Note that most of the routines are meant to be used in building
C     other somewhat higher test utilities.
C
C$ Examples
C
C     A sample test case program would be outlined as shown here:
C
C     First establish the test name.
C
C     CALL T_BEGIN ( 'MY_ROUTINE' )
C
C
C        perform any test initializations.
C
C
C     CALL T_CASE ( 'Normal Inputs' )
C
C     set up the inputs and expected outputs.
C
C     IF ( not success ) THEN
C        CALL T_CFAIL ()
C     END IF
C
C     All the bookkeeping about the success or failure
C     is maintained in one place we can deal with a failure
C     (even though we may have several tests above) here.
C
C     CALL T_CPASS ( OK )
C
C     IF ( .NOT. OK ) THEN
C        CALL T_CNAME ( NAME )
C
C        WRITE (*,*) NAME
C        WRITE (*,*) 'Did not pass. '
C
C     END IF
C
C
C     DO I = 1, NEXCEP
C
C        CALL T_CASE ( 'Exception' )
C
C
C        set up this case
C
C        IF ( not success ) THEN
C           CALL T_CFAIL()
C        END IF
C
C
C        CALL T_CPASS ( OK )
C
C        IF ( .NOT. OK ) THEN
C           CALL T_CNAME ( NAME )
C
C           WRITE (*,*) NAME
C           WRITE (*,*) 'Did not pass. '
C
C        END IF
C
C     END DO
C
C
C     CALL T_SUCCESS ( OK   )
C     CALL T_NAME    ( NAME )
C
C     IF ( .NOT. OK ) THEN
C
C        WRITE (*,*) 'The ', NAME, ' family of tests failed. '
C        WRITE (*,*)
C     ELSE
C        WRITE (*,*) 'All tests of ', NAME, ' passed.'
C     END IF
C
C
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C      W.L. Taber      (JPL)
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-    Testing Utilities 2.1.0, 18-JUN-1999 (WLT)
C
C        Added RETURN before first entry point.
C
C-    Testing Utilities 2.0.0, 20-JUN-1997 (WLT)
C
C        Added the entry point that allows users to set or get
C        the current traceback.
C
C-    TESTLIB Version 1.0.0, 25-OCT-1994 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Keep track of testing status.
C
C-&
 
 
 
C
C     Local Variables
C
      LOGICAL               SUCCES
      LOGICAL               CASEOK
      INTEGER               CASENO
      INTEGER               NFAIL
 
      INTEGER               LNSIZE
      PARAMETER           ( LNSIZE = 500 )
 
      CHARACTER*(LNSIZE)    DESCR
 
      INTEGER               LONGSZ
      PARAMETER           ( LONGSZ = 520 )
 
      INTEGER               LNGSTR
      PARAMETER           ( LNGSTR = 1200 )
 
      INTEGER               WDSIZE
      PARAMETER           ( WDSIZE = 32 )
 
      CHARACTER*(LONGSZ)    MYNAME
      CHARACTER*(WDSIZE)    MYACT
      CHARACTER*(LNGSTR)    TRACE
 
      CHARACTER*(12)        RNAME
C
C     We enough digits to have unique text versions of case numbers
C     for 10**7 cases.
C
      INTEGER               MAXPLC
      PARAMETER           ( MAXPLC = 7 )
 
      CHARACTER*(1)         DIGIT( 10, MAXPLC )
      INTEGER               D    (     MAXPLC )
      INTEGER               I
      INTEGER               J
      LOGICAL               INCR
      LOGICAL               ANYBAD
      SAVE
 
      DATA                  ANYBAD   / .FALSE. /
      DATA                  SUCCES   / .TRUE.  /
      DATA                  CASEOK   / .TRUE.  /
      DATA                  CASENO   /  0      /
      DATA                  NFAIL    /  0      /
      DATA                  MYNAME   / ' '     /
      DATA                  DESCR    / ' '     /
      DATA                  RNAME    / ' '     /
 
      RETURN
 
 
C
C     The entry point T_BEGIN sets up all of the needed local
C     machinery for tracking the test cases for the current
C     set of tests.
C
      ENTRY T_BEGIN ( NAME )
C
C        Until something goes wrong.  All tests are assumed to pass.
C        Store the name and current case number.
C
         SUCCES = .TRUE.
         RNAME  =  NAME
         CASENO =  0
         NFAIL  =  0
C
C        Set up the "odometer" for the case counter.  Note we put
C        blanks instead of zeros for every digit but the least
C        significant digit.
C
C        Here's the idea.  At each test case, we increment CASENO
C        and flip over one digit on the case odometer.  When we
C        reach the 10 on any digit, we flip back to zero and
C        cause the digit to the left to increment.  The actual
C        text version for the current case number will be:
C
C        DIGIT(D(1),1)//DIGIT(D(2),2)// ... //DIGIT(D(MAXPLC),MAXPLC).
C
 
         DO I = 1, MAXPLC
            DIGIT (  1, I) = ' '
            DIGIT (  2, I) = '1'
            DIGIT (  3, I) = '2'
            DIGIT (  4, I) = '3'
            DIGIT (  5, I) = '4'
            DIGIT (  6, I) = '5'
            DIGIT (  7, I) = '6'
            DIGIT (  8, I) = '7'
            DIGIT (  9, I) = '8'
            DIGIT ( 10, I) = '9'
            D(I)           =  1
         END DO
 
         DIGIT( 1, MAXPLC ) = '0'
 
 
         RETURN
C
C     Return the name of the current family of tests.
C
      ENTRY T_NAME    ( NAME   )
 
         NAME = RNAME
         RETURN
 
C
C     Return information about whether all test cases since set up
C     via T_BEGIN have passed.
C
      ENTRY T_SUCCESS ( OK )
 
         OK = SUCCES
         RETURN
 
      ENTRY T_FCOUNT  ( NUMBER )
 
         NUMBER = NFAIL
         RETURN
 
C
C     Set up for the next test case.
C
      ENTRY T_CASE ( NAME )
C
C        Store the name, set this case success value to .TRUE. and
C        increment the case number.
C
         DESCR  =  NAME
         CASEOK = .TRUE.
         CASENO =  CASENO + 1
C
C        Reset the error handling.
C
         CALL RESET ()
 
C
C        Next we increment the odometer.  We always increment the least
C        significant digit.
C
         INCR   = .TRUE.
 
         DO I = MAXPLC, 1, -1
 
            IF ( INCR ) THEN
C
C              If we incremented this digit, we will never want to use
C              a blank (while in this test case) for this digit again.
C              Set the first digit to zero "0".
C
               D(I)       =  D(I) + 1
               DIGIT(1,I) = '0'
C
C              If the current digit is not greater than 10, we need
C              to set it back to 1 and cascade up to the next digit.
C              That is, make sure INCR stays .TRUE.
C
               IF ( D(I) .GT. 10 ) THEN
 
                  D(I) =  1
 
               ELSE
C
C                 In this case we do not need to increment any digit
C                 that is more significant than the current digit.
C
                  INCR = .FALSE.
 
               END IF
 
            END IF
 
         END DO
         RETURN
 
C
C     This entry point merely records that some aspect of this
C     test case has failed.
C
      ENTRY T_CFAIL ( )
 
         IF ( CASEOK ) THEN
            NFAIL = NFAIL + 1
         END IF
 
         ANYBAD = .TRUE.
         CASEOK = .FALSE.
         SUCCES = .FALSE.
         RETURN
 
C
C     Return information about whether or not the current test case
C     passed.
C
      ENTRY T_CPASS   ( OK )
 
         OK = CASEOK
         RETURN
 
C
C     Return the number of the current test case.
C
      ENTRY T_CNUM    ( NUMBER )
 
         NUMBER = CASENO
         RETURN
 
C
C     The entry point T_NAME gets the name of the current test case.
C     This has the form "Test Case #. <description> " where # is
C     the text decimal for the current case and <description> is the
C     description provided at the entry T_CASE.
C
      ENTRY T_CNAME ( NAME )
 
         MYNAME = 'Test Case '
         J      =  10
C
C        Append each non-blank digit (starting with the most
C        significant) to the current value of MYNAME.
C
         DO I = 1, MAXPLC
            IF ( DIGIT(D(I),I) .NE. ' ' ) THEN
               J         = J + 1
               MYNAME(J:J) = DIGIT(D(I),I)
            END IF
         END DO
C
C        Place a period after the case number.
C
         J           = J + 1
         MYNAME(J:J) = '.'
C
C        Follow that up with the case description.
C
         J          = J + 2
         MYNAME(J:) = DESCR
         NAME       = MYNAME
         RETURN
 
 
      ENTRY T_ANYBAD ( OK )
 
         OK = ANYBAD
         RETURN
 
      ENTRY T_TRACE ( ACT, NAME )
 
         CALL LJUST ( ACT,   MYACT )
         CALL UCASE ( MYACT, MYACT )
 
         IF ( MYACT .EQ. 'SET' ) THEN
            TRACE = NAME
         ELSE IF ( MYACT .EQ. 'GET' ) THEN
            NAME = TRACE
         END IF
 
         RETURN
 
      END
