C$Procedure      TSTCKN (Test CK No angular velocity)
 
      SUBROUTINE TSTCKN ( CKNM, SCLKNM, LOADCK, LOADSC, KEEPSC, HANDLE )
 
C$ Abstract
C
C     Create and if appropriate load a test type 03 C-kernel and
C     associated S-clock kernel file.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C       TESTING
C
C$ Declarations
 
      IMPLICIT NONE
      CHARACTER*(*)         CKNM
      CHARACTER*(*)         SCLKNM
      LOGICAL               LOADCK
      LOGICAL               LOADSC
      LOGICAL               KEEPSC
      INTEGER               HANDLE
 
 
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      CKNM       I   The name of the C-kernel to create
C      SCLKNM     I   The name of the S-clock kernel to create.
C      LOADCK     I   Load the C-kernel if TRUE
C      LOADSC     I   Load the S-clock kernel if TRUE
C      KEEPSC     I   Keep the S-clock kernel if TRUE, else delete it.
C      HANDLE     O   Handle of the c-kernel if it is loaded.
C
C$ Detailed_Input
C
C     CKNM        is the name of a C-kernel to create and load if
C                 LOADCK is set to TRUE.  If a C-kernel of the same
C                 name already exists it is deleted.
C
C     SCLKNM      is the name of an S-clock Kernel to create and load
C                 if LOADSC is set to TRUE.  If an S-clock kernel of
C                 the same name already exists, delete the existing
C                 kernel before creating this one.
C
C     LOADCK      is a logical that indicates whether or not the CK
C                 file should be loaded after it is created.  If it
C                 has the value TRUE the C-kernel is loaded after
C                 it is created.  Otherwise it is left un-opened.
C
C     LOADSC      is a logical that indicates whether or not the SCLK
C                 file should be loaded into the kernel pool.  If it
C                 has the value TRUE the SCLK file is loaded, otherwise
C                 it is left un-opened.
C
C     KEEPSC      is a logical that indicates whether or not the SCLK
C                 file should be deleted after it is loaded.  If KEEPSC
C                 is TRUE the file is not deleted.  If KEEPSC is FALSE
C                 the file is deleted after it is loaded.  NOTE that
C                 unless LOADSC is TRUE, the SCLK file is not deleted
C                 by this routine.  This routine deletes the SCLK kernel
C                 only if it LOADSC is TRUE and KEEPSC is FALSE.
C
C$ Detailed_Output
C
C     HANDLE      is the handle attached to the created C-kernel if
C                 the kernel is loaded because LOADCK has a value of
C                 TRUE.  Otherwise the value of HANDLE has no meaning.
C
C$ Parameters
C
C      None.
C
C$ Files
C
C      This routine creates two files a C-kernel with a three type 03
C      segments and an associated SCLK kernel that contains all of the
C      connection information about the CK file and its associated
C      ephemeris and S-clock.  See C$ Particulars for more details.
C
C$ Exceptions
C
C     None.
C
C$ Particulars
C
C     This routine creates two files.
C
C     1) A C-kernel without angular velocity for the fictional
C        objects with ID codes -9999, -10000, and -10001.
C
C     2) A SCLK kernel to be associated with the C-kernel.
C
C     The C-kernel contains a single segment for each of the
C     fictional objects.  These segments give continous attitude
C     over the time interval
C     from 1980 JAN 1, 00:00:00.000 (ET)
C     to   2011 SEP 9, 01:46:40.000 (ET)
C     (a span of exactly 1 billion seconds).
C
C
C     The frames of the objects are
C
C     Object    Frame
C     -------   --------
C     -9999     Galactic
C     -10000    FK5
C     -10001    J2000
C
C     All three objects rotate  at a rate of 1 radian per 10 million
C     seconds. The axis of rotation changes every 100 million seconds.
C
C     At various epochs the axes of the objects are exactly aligned
C     with their associated reference frame.
C
C     Object     Aligned with reference frame at epoch
C     ------     -------------------------------------
C     -9999      Epoch of the J2000 frame
C     -10000     Epoch of J2000
C     -10001     Epoch of J2000
C
C     At the moment when the frames are aligned. The are rotating
C     around the direction (2, 1, 3) in their associated frames.
C
C     The C-kernel contains 606 attitude instances.
C
C     The attitude  produced by the CK software should very nearly
C     duplicate the results returned by the test routine TSTATD.
C
C     More specifically suppose we set up the arrays:
C
C        ID(1)     = -9999
C        ID(2)     = -10000
C        ID(3)     = -10001
C
C
C        FRAME(1)  = 'GALACTIC'
C        FRAME(2)  = 'FK4'
C        FRAME(3)  = 'J2000'
C
C
C     Then the two methods of getting ROT  below should
C     produce results that agree to nearly roundoff.
C
C     Method 1.
C
C        CALL SCE2T ( -9, ET, TICK )
C        CALL CKGP  ( ID(I), TICK, 0.0D0, FRAME(I), ROT, OUT, FND )
C
C     Method 2.
C
C        CALL TSTATD ( ET , ROT, AV )
C
C
C$ Examples
C
C     This is intended to be used in those instances when you
C     need a well defined C-kernel whose attitude can be accurately
C     predicted in advance.
C
C     The routine TSTATD returns the continuous attitude and angular
C     velocity of the C-kernel for all time.  As such it provides
C     a convenient method for testing the CK software for an individual
C     segment.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C      W.L. Taber      (JPL)
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-    Test Utilities 1.1.0, 28-JUL-1999 (WLT)
C
C        Added code so that the files created are "registered" with
C        the Test Utilities File Registry ( FILREG ).  By doing so
C        files will be deleted automatically at the end of a test
C        family.
C
C-    Test Utilities 1.0.0, 05-APR-1999 (WLT)
C
C
C-&
 
C$ Index_Entries
C
C     Create test CK and SCLK files.
C
C-&
 
C
C     Spicelib Functions
C
      INTEGER               RTRIM
 
C
C     Test Utility Functions
C
      CHARACTER*(16)        BEGDAT
 
C
C     Inline functions.
C
      DOUBLE PRECISION      TICKS
      DOUBLE PRECISION      SECNDS
C
C     Local Variables.
C
 
      INTEGER               LNSIZE
      PARAMETER           ( LNSIZE = 80 )
 
      INTEGER               NLINES
      PARAMETER           ( NLINES = 98 )
 
      INTEGER               MAXTIK
      PARAMETER           ( MAXTIK = 205 )
 
      INTEGER               WDSIZE
      PARAMETER           ( WDSIZE = 32 )
 
      CHARACTER*(WDSIZE)    REF
      CHARACTER*(WDSIZE)    SEGID
 
      CHARACTER*(LNSIZE)    SCLKBF ( NLINES )
      CHARACTER*(LNSIZE)    ERROR
 
      DOUBLE PRECISION      BEGTIM
      DOUBLE PRECISION      ENDTIM
      DOUBLE PRECISION      ET
      DOUBLE PRECISION      TK
      DOUBLE PRECISION      ZEROPT
      DOUBLE PRECISION      MAXET
 
      DOUBLE PRECISION      ROT    ( 3, 3 )
      DOUBLE PRECISION      STARTS ( 1         )
      DOUBLE PRECISION      TICK   (    MAXTIK )
      DOUBLE PRECISION      QUATRN ( 4, MAXTIK )
      DOUBLE PRECISION      ANGVEL ( 3, MAXTIK )
 
      INTEGER               I
      INTEGER               INST
      INTEGER               J
      INTEGER               NINTS
      INTEGER               NTICKS
      INTEGER               UNIT
      INTEGER               R
 
      LOGICAL               AVFLG
 
C
C     Definitions of inline functions.
C
      TICKS ( ET ) =  ( ET - ZEROPT )*10000.0D0
      SECNDS( TK ) =    ZEROPT + (TK/10000.0D0)
 
C
C     The first order of business is to wipe out any existing
C     files with the same name.
C
      CALL KILFIL ( CKNM   )
      CALL KILFIL ( SCLKNM )
C
C     Fill up the buffer SCLKBF with the text that will make up
C     our test SCLK file.
C
 
      SCLKBF(  1) = 'TEST SPICE SCLK Kernel '
      SCLKBF(  2) = '------------------------------------------- '
     .//            '------------- '
      SCLKBF(  3) = ' '
      SCLKBF(  4) = 'This file contains the data necessary for '
     .//            'converting from '
      SCLKBF(  5) = 'ET to ticks for the fictional spacecraft '
     .//            '-9999.  It is produced '
      SCLKBF(  6) = 'by the Test Utility routine TSTCKN. '
      SCLKBF(  7) = ' '
      SCLKBF(  8) = 'This SCLK kernel is intended to be used '
     .//            'with the test CK file '
      SCLKBF(  9) = 'produced by that same routine.  The '
     .//            'internal file name of the '
      SCLKBF( 10) = 'test C-Kernel is ZZ-TEST-CK-ZZ.  It '
     .//            'contains a single type 03 '
      SCLKBF( 11) = 'segment. '
      SCLKBF( 12) = ' '
      SCLKBF( 13) = 'This fictional clock begins a 1 JAN 1980 '
     .//            '00:00:00 ET and continues '
      SCLKBF( 14) = 'for 1 billion seconds.  (i.e. until 2011 '
     .//            'SEP 9, 01:46:40.000 ET ). '
      SCLKBF( 15) = ' '
      SCLKBF( 16) = 'This is intended for test purposes only and '
     .//            'can be easily rebuilt '
      SCLKBF( 17) = 'by calling the routine TSTCKN. '
      SCLKBF( 18) = ' '
      SCLKBF( 19) = 'If you have any questions about this file '
     .//            'that these comments don''t '
      SCLKBF( 20) = 'answer, contact Bill Taber at NAIF. '
      SCLKBF( 21) = ' '
      SCLKBF( 22) = '(818) 354-4279 '
      SCLKBF( 23) = 'btaber@spice.jpl.nasa.gov '
      SCLKBF( 24) = ' '
      SCLKBF( 25) = ' '
      SCLKBF( 26) = ' '
      SCLKBF( 27) = 'Implementation notes '
      SCLKBF( 28) = '-------------------------------------------'
     .//            '------------- '
      SCLKBF( 29) = ' '
      SCLKBF( 30) = 'This SCLK file is constructed so that the '
     .//            'valid SCLK strings '
      SCLKBF( 31) = 'are simply the number of TDB seconds that '
     .//            'have passed '
      SCLKBF( 32) = 'since the Ephemeris epoch 1 Jan 1980 '
     .//            '00:00:00 '
      SCLKBF( 33) = ' '
      SCLKBF( 34) = 'So that 1/ 288929292.82017  simply '
     .//            'represents the epoch that occurs '
      SCLKBF( 35) = '288929292.82017 TDB seconds past the ET '
     .//            'epoch 1 Jan 1980. '
      SCLKBF( 36) = ' '
      SCLKBF( 37) = 'For all time, the clock runs at the same '
     .//            'rate as TDB. There is only '
      SCLKBF( 38) = 'one partition for this clock. '
      SCLKBF( 39) = ' '
      SCLKBF( 40) = 'You must load this file into the kernel '
     .//            'pool before using any of the '
      SCLKBF( 41) = 'SPICELIB SCLK routines. The code fragment '
      SCLKBF( 42) = ' '
      SCLKBF( 43) = 'CALL LDPOOL ( < name of this file > ) '
      SCLKBF( 44) = ' '
      SCLKBF( 45) = 'performs this task. To convert between ET '
     .//            'and UTC, you will also need '
      SCLKBF( 46) = 'to load a leapseconds kernel. The '
     .//            'additional call to LDPOOL, '
      SCLKBF( 47) = ' '
      SCLKBF( 48) = 'CALL LDPOOL ( < name of your leapsecond '
     .//            'file > ) '
      SCLKBF( 49) = ' '
      SCLKBF( 50) = 'will accomplish this. Note that you must '
     .//            'supply the actual names of '
      SCLKBF( 51) = 'the files used on your system as arguments '
     .//            'to LDPOOL. Because the file '
      SCLKBF( 52) = 'names are system dependent, we do not list '
     .//            'them here. '
      SCLKBF( 53) = ' '
      SCLKBF( 54) = 'For more information, consult your SPICELIB '
     .//            'required reading files. '
      SCLKBF( 55) = 'The following areas are covered: '
      SCLKBF( 56) = ' '
      SCLKBF( 57) = 'SCLK system                     SCLK '
     .//            'required reading '
      SCLKBF( 58) = 'Time systems and conversion     TIME '
     .//            'required reading '
      SCLKBF( 59) = 'Kernel pool                     KERNEL '
     .//            'required reading '
      SCLKBF( 60) = ' '
      SCLKBF( 61) = ' '
      SCLKBF( 62) = 'Kernel data '
      SCLKBF( 63) = '-------------------------------------------'
     .//            '------------- '
      SCLKBF( 64) = ' '
      SCLKBF( 65) = ' '
      SCLKBF( 66) = BEGDAT()
      SCLKBF( 67) = ' '
      SCLKBF( 68) = ' '
      SCLKBF( 69) = 'SCLK_KERNEL_ID                = ( '
     .//            '@28-OCT-1994        ) '
      SCLKBF( 70) = ' '
      SCLKBF( 71) = 'SCLK_DATA_TYPE_9              = ( '
     .//            '1 ) '
      SCLKBF( 72) = ' '
      SCLKBF( 73) = 'SCLK01_TIME_SYSTEM_9          = ( '
     .//            '1 ) '
      SCLKBF( 74) = 'SCLK01_N_FIELDS_9             = ( '
     .//            '2 ) '
      SCLKBF( 75) = 'SCLK01_MODULI_9               = ( '
     .//            '1000000000     10000 ) '
      SCLKBF( 76) = 'SCLK01_OFFSETS_9              = ( '
     .//            '0         0 ) '
      SCLKBF( 77) = 'SCLK01_OUTPUT_DELIM_9         = ( '
     .//            '1 ) '
      SCLKBF( 78) = ' '
      SCLKBF( 79) = 'SCLK_PARTITION_START_9        = ( '
     .//            '0.0000000000000E+00 ) '
      SCLKBF( 80) = 'SCLK_PARTITION_END_9          = ( '
     .//            '1.00000000E+14      ) '
      SCLKBF( 81) = 'SCLK01_COEFFICIENTS_9         = ( '
     .//            '0.00000000E+00 '
      SCLKBF( 82) = '@01-JAN-1980-00:00:00.000 '
      SCLKBF( 83) = '1  ) '
      SCLKBF( 84) = ' '
      SCLKBF( 85) = ' '
      SCLKBF( 86) = 'DELTET/DELTA_T_A       =   32.184 '
      SCLKBF( 87) = 'DELTET/K               =    1.657D-3 '
      SCLKBF( 88) = 'DELTET/EB              =    1.671D-2 '
      SCLKBF( 89) = 'DELTET/M               = (  6.239996D0 '
     .//            '1.99096871D-7 ) '
      SCLKBF( 90) = ' '
      SCLKBF( 91) = 'CK_-9999_SCLK          =   -9 '
      SCLKBF( 92) = 'CK_-9999_SPK           =   -9 '
      SCLKBF( 93) = ' '
      SCLKBF( 94) = 'CK_-10000_SCLK         =   -9 '
      SCLKBF( 95) = 'CK_-10000_SPK          =   -9 '
      SCLKBF( 96) = ' '
      SCLKBF( 97) = 'CK_-10001_SCLK         =   -9 '
      SCLKBF( 98) = 'CK_-10001_SPK          =   -9 '
 
 
C
C     Create the SCLK kernel.
C
      CALL TXTOPN ( SCLKNM, UNIT )
 
      DO I = 1, NLINES
 
         R =     RTRIM ( SCLKBF(I) )
         WRITE (UNIT,*)  SCLKBF(I)(1:R)
 
      END DO
 
      CLOSE ( UNIT )
 
C
C     Next create the C-kernel. Recall the relationship between
C     ET and encoded SCLK ticks.  There are 10000 ticks/second.
C     The zero point of the clock is 1 Jan 1980 TDB.
C
      CALL TPARSE ( '1 Jan 1980', ZEROPT, ERROR )
 
      MAXET = ZEROPT + 1000000000.0D0
      ET    =  0.0D0
      J     = 0
 
      DO WHILE ( ET .GT. ZEROPT )
 
         J       = J + 1
         TICK(J) = TICKS(ET)
 
         J       = J + 1
         TICK(J) = TICK(J-1) - 1.0D0
 
         ET      = ET - 10000000.0D0
 
      END DO
 
      J        = J + 1
      TICK(J)  = 0.0D0
 
 
      ET = 10000000.0D0
 
      DO WHILE ( ET .LT. MAXET )
 
         J       = J + 1
         TICK(J) = TICKS(ET)
 
         J       = J + 1
         TICK(J) = TICK(J-1) - 1.0D0
 
         ET      = ET + 10000000.0D0
 
      END DO
 
      J       = J + 1
      TICK(J) = TICKS(MAXET)
 
      NTICKS = J
C
C     Sort the ticks.
C
      CALL SHELLD ( NTICKS, TICK )
 
      BEGTIM = TICK(1)
      ENDTIM = TICK(NTICKS)
 
      INST   = -9999
      REF    = 'GALACTIC'
      AVFLG  = .FALSE.
      SEGID  = 'Test Segment for object -9999'
 
 
      DO I = 1, NTICKS
         TK = TICK(I)
         ET = SECNDS ( TK )
         CALL TSTATD ( ET, ROT, ANGVEL(1,I) )
         CALL M2Q    (     ROT, QUATRN(1,I) )
      END DO
 
 
      NINTS     = 1
      STARTS(1) = 0.0D0
 
      CALL SPCOPN ( CKNM, 'Test C-kernel',  HANDLE )
      CALL CKW03  ( HANDLE,
     .              BEGTIM, ENDTIM, INST, REF, AVFLG,
     .              SEGID,  NTICKS, TICK, QUATRN, ANGVEL,
     .              NINTS,  STARTS )
C
C     Now create a second segment by simply taking that attitude
C     10 million seconds later than those for body -9999
C
      BEGTIM = TICK(1)
      ENDTIM = TICK(NTICKS)
 
      INST   = -10000
      REF    = 'FK4'
      AVFLG  = .FALSE.
      SEGID  = 'Object -10000'
 
 
      DO I = 1, NTICKS
         TK = TICK(I)
         ET = SECNDS ( TK )
         CALL TSTATD ( ET, ROT, ANGVEL(1,I) )
         CALL M2Q    (     ROT, QUATRN(1,I) )
      END DO
 
 
      NINTS     = 1
      STARTS(1) = 0.0D0
 
      CALL CKW03  ( HANDLE,
     .              BEGTIM, ENDTIM, INST, REF, AVFLG,
     .              SEGID,  NTICKS, TICK, QUATRN, ANGVEL,
     .              NINTS,  STARTS )
 
C
C     Finally for the third segment take take the same attitudes
C     but 100 million seconds later than those for object -9999
C
      BEGTIM = TICK(1)
      ENDTIM = TICK(NTICKS)
 
      INST   = -10001
      REF    = 'J2000'
      AVFLG  = .FALSE.
      SEGID  = 'Test Segment for object -10001'
 
 
      DO I = 1, NTICKS
         TK = TICK(I)
         ET = SECNDS ( TK )
         CALL TSTATD ( ET, ROT, ANGVEL(1,I) )
         CALL M2Q    (     ROT, QUATRN(1,I) )
      END DO
 
 
      NINTS     = 1
      STARTS(1) = 0.0D0
 
      CALL CKW03  ( HANDLE,
     .              BEGTIM, ENDTIM, INST, REF, AVFLG,
     .              SEGID,  NTICKS, TICK, QUATRN, ANGVEL,
     .              NINTS,  STARTS )
 
 
 
      CALL DAFCLS ( HANDLE )
C
C     Now take care of loading the test kernels if they are needed.
C
      IF ( LOADSC ) THEN
 
         CALL LDPOOL ( SCLKNM )
 
 
         IF ( .NOT. KEEPSC ) THEN
 
            CALL KILFIL ( SCLKNM )
 
         END IF
 
      END IF
 
      IF ( LOADCK ) THEN
 
         CALL CKLPF ( CKNM, HANDLE )
 
 
      END IF
 
      CALL TFILES ( CKNM )
      CALL TFILES ( SCLKNM )
 
      RETURN
      END
