C$Procedure      CKGP_O( C-kernel, get pointing )
 
      SUBROUTINE CKGP_O( INST, SCLKDP, TOL, REF, CMAT, CLKOUT, FOUND )
 
C$ Abstract
C
C     Get inertially referenced instrument pointing for a specified
C     spacecraft clock time.
C
C$ Required_Reading
C
C     CK
C
C$ Keywords
C
C     POINTING
C
C$ Declarations
 
      INTEGER               INST
      DOUBLE PRECISION      SCLKDP
      DOUBLE PRECISION      TOL
      CHARACTER*(*)         REF
      DOUBLE PRECISION      CMAT   ( 3, 3 )
      DOUBLE PRECISION      CLKOUT
      LOGICAL               FOUND
 
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     INST       I   NAIF instrument ID.
C     SCLKDP     I   Spacecraft clock time.
C     TOL        I   Time tolerance.
C     REF        I   Inertial reference frame.
C     CMAT       O   C-matrix pointing data.
C     CLKOUT     O   Output spacecraft clock time.
C     FOUND      O   True when requested pointing is available.
C
C$ Detailed_Input
C
C     INST       is the unique NAIF integer ID for the spacecraft
C                instrument for which pointing is being requested.
C
C     SCLKDP     is the encoded spacecraft clock time for which
C                pointing is being requested.  The SPICELIB routines
C                SCENCD and SCDECD encode and decode SCLK times.
C
C     TOL        is a time tolerance in ticks, the units of encoded
C                spacecraft clock time.  The SPICELIB routine SCTIKS
C                converts a spacecraft clock tolerance time from its
C                character string representation to ticks.  SCFMT
C                performs the inverse conversion.
C
C                The C-matrix returned by CKGP_Ois the one whose time
C                tag is closest to SCLKDP and within TOL units of
C                SCLKDP.  (More in Particulars, below.)
C
C
C     REF        is the desired inertial reference frame for the
C                returned pointing.
C
C                See the SPICELIB routine CHGIRF for a complete list of
C                those frames supported by NAIF.  For example, 'J2000',
C                'B1950', 'FK4', and so on.
C
C$ Detailed_Output
C
C     CMAT       is a rotation matrix that transforms the components of
C                of a vector expressed in the inertial frame specified
C                by REF to components expressed in the instrument
C                fixed frame at time CLKOUT.
C
C                Thus, if a vector v has components x, y, z in the
C                inertial frame, then v has components x', y', z' in the
C                instrument fixed frame at time CLKOUT:
C
C                     [ x' ]     [          ] [ x ]
C                     | y' |  =  |   CMAT   | | y |
C                     [ z' ]     [          ] [ z ]
C
C                If you know x', y', z', use the transpose of the
C                C-matrix to determine x, y, z as follows:
C
C                     [ x ]      [          ]T    [ x' ]
C                     | y |  =   |   CMAT   |     | y' |
C                     [ z ]      [          ]     [ z' ]
C                             (Transpose of CMAT)
C
C     CLKOUT     is the encoded spacecraft clock time associated with
C                the returned C-matrix. This value may differ from the
C                requested time, but never by more than the input
C                tolerance.
C
C                The particulars section below describes the search
C                algorithm used by CKGP_Oto satisfy a pointing request.
C                This algorithm determines the pointing instance
C                (and therefore the associated time value) that is
C                returned.
C
C     FOUND      is true if a record was found to satisfy the pointing
C                request.  FOUND will be false otherwise.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1)  If a C-kernel file is not loaded using CKLPF prior to calling
C         this routine, an error is signalled by a routine that this
C         routine calls.
C
C     2)  If TOL is negative, FOUND is false.
C
C     3)  If REF is not one of the supported inertial reference frames,
C         an error is signalled by a routine that this routine calls and
C         FOUND is false.
C
C$ Files
C
C     CKGP_Osearches through files loaded by CKLPF to locate a segment
C     that can satisfy the request for pointing data for instrument
C     INST at time SCLKDP.  You must load a C-kernel file using CKLPF
C     before calling this routine.
C
C$ Particulars
C
C     CKGP_Osearches through files loaded by CKLPF to satisfy a pointing
C     request. Last-loaded files are searched first, and individual
C     files are searched in backwards order, giving priority to
C     segments that were added to a file later than the others. CKGP
C     doesn't care if segments contain angular velocity data or not,
C     since it returns pointing C-matrices only.
C
C     The search ends when a segment is found that can give pointing
C     for the specified instrument at a time falling within the
C     specified tolerance on either side of the request time. Within
C     that segment, the instance closest to the input time is located
C     and returned.
C
C     The following example illustrates this search procedure. Segments
C     A and B are in the same file, with segment A located further
C     towards the end of the file than segment B. Both segment A and B
C     contain discrete pointing data.
C
C                              SCLKDP
C                                   \   TOL
C                                    | /
C                                    |/\
C     Request 1                   [--+--]
C                                 .  .  .
C                                 .  .  .
C     Segment A          (0-----------------0--------0--0-----0)
C                                 .  .  .
C                                 .  .  .
C     Segment B    (0--0--0--0--0--0--0--0--0--0--0--0--0--0--0--0--0)
C                                     ^
C                                     |
C                         CKGP_Oreturns this instance
C
C
C
C                                           SCLKDP
C                                          /
C                                         |  TOL
C                                         | /
C                                         |/\
C     Request 2                        [--+--]
C                                      .  .  .
C                                      .  .  .
C     Segment A          (0-----------------0--------0--0-----0)
C                                           ^
C                                           |
C                                           |
C                               CKGP_Oreturns this instance
C
C     Segment B    (0--0--0--0--0--0--0--0--0--0--0--0--0--0--0--0--0)
C
C
C     Segments that contain continuous pointing data are searched in
C     the same manner as discrete segments. For request times that
C     fall within the bounds of continuous intervals, CKGP_Owill return
C     pointing at the request time. When the request time does not fall
C     within an interval, then a time at an endpoint of an interval
C     may be returned if it is the closest time in the segment to the
C     user request time and also within the tolerance.
C
C     In the following examples segment A contains discrete pointing
C     data and segment C contains continuous data.
C
C
C                                      SCLKDP
C                                     /
C                                    |  TOL
C                                    | /
C                                    |/\
C     Request 3                   [--+--]
C                                 .  .  .
C                                 .  .  .
C     Segment A          (0-----------------0--------0--0-----0)
C                                 .  .  .
C                                 .  .  .
C     Segment C          (--[=============]---[====]------[=]--)
C                                    ^
C                                    |
C                                    |
C                         CKGP_Oreturns this instance
C
C
C     In the next example assume that the order of segment A and C in
C     file are reversed.
C
C
C                                             SCLKDP
C                                            /
C                                           |  TOL
C                                           | /
C                                           |/\
C     Request 4                          [--+--]
C                                        .  .  .
C                                        .  .  .
C     Segment C          (--[=============]---[====]------[=]--)
C                                             ^
C                                             |
C                                CKGP_Oreturns this instance
C
C     Segment A          (0-----------------0--------0--0-----0)
C
C$ Examples
C
C
C     Suppose you have two C-kernel files containing pointing for the
C     Voyager 2 narrow angle camera.  One file contains predict values,
C     and the other contains corrected pointing for a selected group of
C     images, that is, for a subset of images from the first file.
C
C     The following code fragment uses CKGP_Oto get C-matrices for a
C     set of images whose SCLK counts (un-encoded character string
C     versions) are contained in the array SCLKCH.
C
C     If available, the program will get the corrected pointing values.
C     Otherwise, predict values will be used.
C
C     For each C-matrix, a unit inertial pointing vector is constructed
C     and printed.
C
C
C     C
C     C     Constants for this program.
C     C
C     C     -- The code for the Voyager 2 mission is -32
C     C
C     C     -- The code for the narrow angle camera on the Voyager 2
C     C        spacecraft is -32001.
C     C
C     C    --  Spacecraft clock times for successive images always
C     C        differ by more than 0:0:400.  This is an acceptable
C     C        tolerance, and must be converted to 'ticks' (units
C     C        of encoded SCLK) for input to CKGP.
C     C
C     C     -- The reference frame we want is FK4.
C     C
C     C     -- The narrow angle camera boresight defines the third
C     C        axis of the instrument-fixed coordinate system.
C     C        Therefore, the vector ( 0, 0, 1 ) always represents
C     C        the boresight direction in camera-fixed coordinates.
C     C
C           FILE1      = 'PREDICT_CK.BC'
C           FILE2      = 'CORRECTED_CK.BC'
C           SC         = -32
C           INST       = -32001
C           TOLVGR     = '0:0:400'
C           REF        = 'FK4'
C           VCFIX( 1 ) =  0.D0
C           VCFIX( 2 ) =  0.D0
C           VCFIX( 3 ) =  1.D0
C
C     C
C     C     Loading the files in this order ensures that the
C     C     corrected file will get searched first.
C     C
C           CALL CKLPF ( FILE1, HANDL1 )
C           CALL CKLPF ( FILE2, HANDL2 )
C     C
C     C     Need to load a Voyager 2 SCLK kernel to convert from
C     C     clock string to ticks.
C     C
C           CALL LDPOOL ( 'VGR_SCLK.TSC' )
C     C
C     C     Convert tolerance from VGR formatted character string
C     C     SCLK to ticks which are units of encoded SCLK.
C     C
C           CALL SCTIKS ( SC, TOLVGR, TOLTIK )
C
C
C           DO I = 1, NPICS
C
C     C
C     C        CKGP_Orequires encoded spacecraft clock time.
C     C
C              CALL SCENCD ( SC, SCLKCH( I ), SCLKDP )
C
C              CALL CKGP_O( INST, SCLKDP, TOLTIK, REF, CMAT, CLKOUT,
C          .               FOUND )
C
C              IF ( FOUND ) THEN
C
C     C
C     C           Use the transpose of the C-matrix to transform the
C     C           boresight vector from camera-fixed to inertial
C     C           coordinates.
C     C
C                 CALL MTXV   ( CMAT, VCFIX,  VINERT )
C                 CALL SCDECD ( SC,   CLKOUT, CLKCH  )
C
C                 WRITE (*,*) 'Time:            ', CLKCH
C                 WRITE (*,*) 'Pointing vector: ', VINERT
C
C              ELSE
C
C                 WRITE (*,*) 'Pointing not found for time ', SCLKCH(I)
C
C              END IF
C
C           END DO
C
C
C$ Restrictions
C
C     A C-kernel file must be loaded using the routine CKLPF.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     J.M. Lynch     (JPL)
C     M.J. Spencer   (JPL)
C     R.E. Thurman   (JPL)
C     I.M. Underwood (JPL)
C
C$ Version
C
C-    SPICELIB Version 2.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 2.0.0, 30-AUG-1991 (JML)
C
C        The Particulars section was updated to show how the
C        search algorithm processes segments with continuous
C        pointing data.
C
C        The example program now loads an SCLK kernel.
C
C        FAILED is checked after the call to IRFROT to handle the
C        case where the reference frame is invalid and the error
C        handling is not set to abort.
C
C        FAILED is checked in the DO WHILE loop to handle the case
C        where an error is detected by a SPICELIB routine inside the
C        loop and the error handling is not set to abort.
C
C-    SPICELIB Version 1.0.1, 02-NOV-1990 (JML)
C
C        The restriction that a C-kernel file must be loaded
C        was explicitly stated.
C
C
C-    SPICELIB Version 1.0.0, 07-SEP-1990 (RET) (IMU)
C
C-&
 
C$ Index_Entries
C
C     get ck pointing
C
C-&
 
 
C$ Revisions
C
C-    SPICELIB Version 1.0.2, 30-AUG-1991 (JML)
C
C        1) The Particulars section was updated to show how the
C           search algorithm processes segments with continuous
C           pointing data.
C
C        2) The example program now loads an SCLK kernel.
C
C        3) FAILED is checked after the call to IRFROT to handle the
C           case where the reference frame is invalid and the error
C           handling is not set to abort.
C
C        4) FAILED is checked in the DO WHILE loop to handle the case
C           where an error is detected by a SPICELIB routine inside the
C           loop and the error handling is not set to abort.
C
C-    SPICELIB Version 1.0.1, 02-NOV-1990 (JML)
C
C        1) The restriction that a C-kernel file must be loaded
C           was explicitly stated.
C        2) Minor changes were made to the wording of the header.
C
C
C-    Beta Version 1.1.0, 29-AUG-1990 (MJS)
C
C        The following changes were made as a result of the
C        NAIF CK Code and Documentation Review:
C
C        1) The variable SCLK was changed to SCLKDP.
C        2) The variable INSTR was changed to INST.
C        3) The variable IDENT was changed to SEGID.
C        4) The declarations for the parameters NDC, NIC, NC, and
C           IDLEN were moved from the "Declarations" section of the
C           header to the "Local parameters" section of the code below
C           the header. These parameters are not meant to modified by
C           users.
C        5) The header was updated to reflect the changes.
C
C-    Beta Version 1.0.0, 04-MAY-1990 (RET) (IMU)
C
C-&
 
 
 
 
C
C     SPICELIB functions
C
      LOGICAL               RETURN
 
      LOGICAL               FAILED
 
C
C     Local parameters
C
C        NDC        is the number of double precision components in an
C                   unpacked C-kernel segment descriptor.
C
C        NIC        is the number of integer components in an unpacked
C                   C-kernel segment descriptor.
C
C        NC         is the number of components in a packed C-kernel
C                   descriptor.  All DAF summaries have this formulaic
C                   relationship between the number of its integer and
C                   double precision components and the number of packed
C                   components.
C
C        IDLEN      is the length of the C-kernel segment identifier.
C                   All DAF names have this formulaic relationship
C                   between the number of summary components and
C                   the length of the name (You will notice that
C                   a name and a summary have the same length in bytes.)
C
 
      INTEGER               NDC
      PARAMETER           ( NDC = 2 )
 
      INTEGER               NIC
      PARAMETER           ( NIC = 6 )
 
      INTEGER               NC
      PARAMETER           ( NC = NDC + ( NIC + 1 )/2 )
 
      INTEGER               IDLEN
      PARAMETER           ( IDLEN = NC * 8 )
 
C
C     Local variables
C
      INTEGER               HANDLE
      INTEGER               ICD      ( NIC  )
      INTEGER               REFREQ
      INTEGER               REFSEG
 
      DOUBLE PRECISION      DESCR    ( NC   )
      DOUBLE PRECISION      DCD      ( NDC  )
      DOUBLE PRECISION      AV       ( 3    )
      DOUBLE PRECISION      ROT      ( 3, 3 )
 
      CHARACTER*(IDLEN)     SEGID
 
      LOGICAL               NEEDAV
      LOGICAL               SFND
      LOGICAL               PFND
 
 
 
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'CKGP_O')
      END IF
 
C
C     Don't need angular velocity data.
C     Assume the segment won't be found until it really is.
C
      NEEDAV = .FALSE.
      FOUND  = .FALSE.
 
C
C     Begin a search for this instrument and time, and get the first
C     applicable segment.
C
      CALL CKBSS ( INST,   SCLKDP, TOL,   NEEDAV )
      CALL CKSNS ( HANDLE, DESCR,  SEGID, SFND   )
 
 
C
C     Keep trying candidate segments until a segment can produce a
C     pointing instance within the specified time tolerance of the
C     input time.
C
C     Check FAILED to prevent an infinite loop if an error is detected
C     by a SPICELIB routine and the error handling is not set to abort.
C
 
      DO WHILE ( ( SFND ) .AND. ( .NOT. FAILED () ) )
 
         CALL CKPFS ( HANDLE, DESCR, SCLKDP, TOL, NEEDAV,
     .                CMAT,   AV,    CLKOUT, PFND        )
 
         IF ( PFND ) THEN
 
C
C           Found one. If the C-matrix doesn't already rotate from the
C           requested inertial frame, convert it to one that does.
C
            CALL DAFUS  ( DESCR, NDC, NIC, DCD, ICD )
            REFSEG = ICD( 2 )
 
            CALL IRFNUM ( REF, REFREQ )
 
            IF ( REFREQ .NE. REFSEG ) THEN
 
               CALL IRFROT ( REFREQ, REFSEG, ROT  )
C
C              If IRFROT detects that the reference frame is invalid
C              then return from this routine with FOUND equal to false.
C
               IF ( FAILED () ) THEN
                  CALL CHKOUT ( 'CKGP_O')
                  RETURN
               END IF
 
               CALL MXM    ( CMAT,   ROT,    CMAT )
 
            END IF
 
            FOUND = .TRUE.
 
            CALL CHKOUT ( 'CKGP_O')
            RETURN
 
         END IF
 
         CALL CKSNS ( HANDLE, DESCR, SEGID, SFND )
 
      END DO
 
 
      CALL CHKOUT ( 'CKGP_O')
      RETURN
      END
