C$Procedure F_DYN04 ( Dynamic Frame Test Family 04 )
 
      SUBROUTINE F_DYN04 ( OK )
 
C$ Abstract
C
C     Test family to exercise the logic and code in the fourth
C     subset of dynamic frame routines.  Only Euler frames
C     are used in these tests.
C
C$ Copyright
C
C     Copyright (2004), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     TEST FAMILY
C
C$ Declarations
 
      IMPLICIT NONE
 
      INCLUDE 'zzdyn.inc'
 
      LOGICAL               OK
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     OK         O   logical indicating test status.
C
C$ Detailed_Input
C
C     None.
C
C$ Detailed_Output
C
C     OK         is a logical that indicates the test status to the
C                caller.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     This test family uses the following kernels:
C
C        LSK generated by tstlsk
C        PCK generated by t_pck08
C        SPK generated by tstspk
C
C$ Exceptions
C
C     This routine does not generate any errors. Routines in its
C     call tree may generate errors that are either intentional and
C     trapped or unintentional and need reporting.  The test family
C     utilities manage this.
C
C$ Particulars
C
C     As the set of Euler frame features grows, this routine should
C     be updated to exercise each supported feature.
C
C     Tests done here include, but are not necessarily limited to:
C
C        - Test state transformation from SXFORM:
C
C            o Make sure upper left and lower right blocks match.
C            o Test determinant of rotation blocks
C            o Test norms of rows and columns
C
C        - Compare SXFORM and PXFORM results.
C
C        - Compare SXFORM derivative to discrete derivative
C          computed using PXFORM.  Check absolute and relative
C          derivative errors.
C
C        - Compute state transformation via a local computation.
C          Compare rotation portion of transformation matrix
C          from SXFORM to locally computed result.
C
C        - Make sure that when rotation state is inertial, when a
C          matrix mapping from base frame to an inertial frame is
C          right-multiplied by matrix produced by SXFORM, resulting
C          matrix has zero derivative.
C
C        - Make sure that when rotation state is frozen, when a
C          matrix produced by SXFORM has zero derivative.
C
C
C     Tests are done using a variety of frame definitions.  Each
C     combination of base frame and rotation state is used, where
C     the types of base frames include:
C
C        - inertial
C        - rotating non-dynamic
C        - dynamic
C
C
C$ Examples
C
C     None.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman     (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    TSPICE Version 1.0.0, 18-DEC-2004 (NJB)
C
C
C-&
 
C
C     SPICELIB functions
C
      DOUBLE PRECISION      JYEAR
      DOUBLE PRECISION      VNORMG
 
      INTEGER               PRODAI
      INTEGER               RTRIM
 
      LOGICAL               EQSTR
 
C
C     Local Parameters
C
      CHARACTER*(*)         PCK
      PARAMETER           ( PCK    = 'test_dyn.tpc' )
 
      CHARACTER*(*)         SPK
      PARAMETER           ( SPK    = 'test_dyn.bsp' )
 
      CHARACTER*(*)         FRZTOK
      PARAMETER           ( FRZTOK = '@2010-JAN-1/00:00:00' )
 
C
C     Tolerance levels for various tests.
C
      DOUBLE PRECISION      DM19
      PARAMETER           ( DM19   = 1.D-19 )
 
      DOUBLE PRECISION      DM16
      PARAMETER           ( DM16   = 1.D-16 )
 
      DOUBLE PRECISION      DM14
      PARAMETER           ( DM14   = 1.D-14 )
 
      DOUBLE PRECISION      DM12
      PARAMETER           ( DM12   = 1.D-12 )
 
      DOUBLE PRECISION      DM11
      PARAMETER           ( DM11   = 1.D-11 )
 
      DOUBLE PRECISION      DM6
      PARAMETER           ( DM6    = 1.D-6  )
 
      DOUBLE PRECISION      DM4
      PARAMETER           ( DM4    = 1.D-4  )
 
 
      INTEGER               NAMLEN
      PARAMETER           ( NAMLEN = 32 )
 
      INTEGER               LNSIZE
      PARAMETER           ( LNSIZE = 80 )
 
      INTEGER               MAXDEF
      PARAMETER           ( MAXDEF = 50 )
 
      INTEGER               MAXTOK
      PARAMETER           ( MAXTOK = 7 )
 
      INTEGER               TIMLEN
      PARAMETER           ( TIMLEN = 50 )
 
      INTEGER               MSGLEN
      PARAMETER           ( MSGLEN = 240 )
 
C
C     Parameters controlling frame definitions:
C
 
C
C     The dimensions of the test parameter space are:
C
C         1) Coefficient set
C         2) Axis sequence
C         3) Units
C         4) Epoch
C         5) Base frame
C         6) Center
C         7) Rotation state
C
C     Number of dimensions of the test parameter space:
C
      INTEGER               NDIMS
      PARAMETER           ( NDIMS  = 7 )
 
C
C     Number of coefficient sets:
C
      INTEGER               NCOSET
      PARAMETER           ( NCOSET = 3 )
 
C
C     The constant COFIDX refers to the ordinal position of the
C     dimension corresponding to the coefficient set in the parameter
C     space.
C
      INTEGER               COFIDX
      PARAMETER           ( COFIDX = 1 )
 
C
C     Index of coefficient set for Mars test case:
C
      INTEGER               MCOIDX
      PARAMETER           ( MCOIDX = 3 )
 
C
C     Number of axis sequences:
C
      INTEGER               NSEQ
      PARAMETER           ( NSEQ   = 2 )
 
      INTEGER               SEQIDX
      PARAMETER           ( SEQIDX  = COFIDX + 1 )
 
C
C     Index of axis sequence for Mars test case:
C
      INTEGER               MSQIDX
      PARAMETER           ( MSQIDX = 1 )
 
C
C     Number of unit selections:
C
      INTEGER               NUNITS
      PARAMETER           ( NUNITS = 2 )
 
      INTEGER               UNTIDX
      PARAMETER           ( UNTIDX = SEQIDX + 1 )
 
C
C     Number of epoch selections:
C
      INTEGER               NEPOCS
      PARAMETER           ( NEPOCS = 2 )
 
      INTEGER               EPCIDX
      PARAMETER           ( EPCIDX = UNTIDX + 1 )
 
C
C     Number of base frame cases:
C
      INTEGER               NBASEF
      PARAMETER           ( NBASEF = 3 )
 
C
C     The constant BFRIDX refers to the ordinal position of the
C     dimension corresponding to base frame in the parameter
C     space.
C
      INTEGER               BFRIDX
      PARAMETER           ( BFRIDX = EPCIDX + 1 )
 
C
C     Number of frame center cases:
C
      INTEGER               NCENTR
      PARAMETER           ( NCENTR = 2 )
 
      INTEGER               CTRIDX
      PARAMETER           ( CTRIDX = BFRIDX + 1 )
 
C
C     Number of rotation states:
C
      INTEGER               NRSTAT
      PARAMETER           ( NRSTAT = 3 )
 
C
C     The constant RSTIDX refers to the ordinal position of the
C     dimension corresponding to the rotation state in the parameter
C     space.
C
      INTEGER               RSTIDX
      PARAMETER           ( RSTIDX = CTRIDX + 1 )
 
C
C     Local Variables
C
      CHARACTER*(LNSIZE)    ANGCOF ( 3 )
      CHARACTER*(LNSIZE)    AXESTR
      CHARACTER*(NAMLEN)    BASEFR ( NBASEF )
      CHARACTER*(NAMLEN)    BFRAME
      CHARACTER*(MSGLEN)    CASMSG
      CHARACTER*(LNSIZE)    COFSTR ( 3, NCOSET )
      CHARACTER*(LNSIZE)    DEFTXT ( MAXDEF )
      CHARACTER*(LNSIZE)    EPCSTR ( NEPOCS )
      CHARACTER*(LNSIZE)    EPOCH
      CHARACTER*(TIMLEN)    FRZEPC
      CHARACTER*(NAMLEN)    FRNAME
      CHARACTER*(LNSIZE)    INUNIT
      CHARACTER*(LNSIZE)    KEYWRD
      CHARACTER*(LNSIZE)    RSTATE ( NRSTAT )
      CHARACTER*(NAMLEN)    RSTSTR
      CHARACTER*(LNSIZE)    TIMSTR
      CHARACTER*(LNSIZE)    TOKENS ( MAXTOK )
      CHARACTER*(LNSIZE)    UNITS  ( NUNITS )
      CHARACTER*(LNSIZE)    UNTSTR
 
C
C     We'll use the kernel variable name "stem"
C
C        FRAME_<ID code>_
C
C     The length declared below (18) is the length of such a string
C     where the ID code contains 11 digits.
C
      CHARACTER*(18)        FRSTEM
 
      DOUBLE PRECISION      COEFFS ( 0: MAXTOK-2 )
      DOUBLE PRECISION      DELTA
      DOUBLE PRECISION      DETERR
      DOUBLE PRECISION      DMAG
      DOUBLE PRECISION      DP
      DOUBLE PRECISION      DRDIFF ( 3, 3 )
      DOUBLE PRECISION      DRLERR
      DOUBLE PRECISION      DRV2   ( 3, 3 )
      DOUBLE PRECISION      DRVBLK ( 3, 3 )
      DOUBLE PRECISION      DRVERR
      DOUBLE PRECISION      ET
      DOUBLE PRECISION      EULANG ( 6 )
      DOUBLE PRECISION      NRMERR
      DOUBLE PRECISION      P
      DOUBLE PRECISION      R      ( 3, 3 )
      DOUBLE PRECISION      R2     ( 3, 3 )
      DOUBLE PRECISION      REFEPC
      DOUBLE PRECISION      RMINUS ( 3, 3 )
      DOUBLE PRECISION      RPLUS  ( 3, 3 )
      DOUBLE PRECISION      T
      DOUBLE PRECISION      T0
      DOUBLE PRECISION      TIPM   ( 3, 3 )
      DOUBLE PRECISION      TOL
      DOUBLE PRECISION      TSIPM  ( 6, 6 )
      DOUBLE PRECISION      XF2    ( 6, 6 )
      DOUBLE PRECISION      XFB2J  ( 6, 6 )
      DOUBLE PRECISION      XFF2J  ( 6, 6 )
      DOUBLE PRECISION      XFORM  ( 6, 6 )
      DOUBLE PRECISION      ZR     ( 3, 3 )
 
 
      INTEGER               AXES   ( 3 )
      INTEGER               AXESEQ ( 3, NSEQ )
      INTEGER               CASE
      INTEGER               COORDS ( NDIMS  )
      INTEGER               CENTER
      INTEGER               CTRCDE ( NCENTR )
      INTEGER               DIMS   ( NDIMS  )
      INTEGER               DX
      INTEGER               FRCODE
      INTEGER               HANDLE
      INTEGER               I
      INTEGER               J
      INTEGER               N
      INTEGER               NCART
 
      LOGICAL               ISMARS
      LOGICAL               GO
 
C
C     Saved variables
C
      SAVE                  AXESEQ
      SAVE                  BASEFR
      SAVE                  COFSTR
      SAVE                  CTRCDE
      SAVE                  DIMS
      SAVE                  EPCSTR
      SAVE                  RSTATE
      SAVE                  UNITS
      SAVE                  ZR
 
C
C     Initial values
C
 
 
C
C     Euler angle axis sequences:
C
      DATA                  AXESEQ                                    /
     .                      3, 1, 3,
     .                      2, 3, 1                                   /
 
C
C     We use both inertial, PCK-based, and dynamic base frames.
C
      DATA                  BASEFR                                    /
     .                      'J2000',
     .                      'IAU_MARS',
     .                      'GSE'                                     /
 
C
C     Polynomial coefficient sets:
C
C     The last set is based on the coefficient for Mars from
C     pck00008.tpc.  That PCK contains the assignments
C
C        BODY499_POLE_RA          = (  317.68143   -0.1061      0.  )
C        BODY499_POLE_DEC         = (   52.88650   -0.0609      0.  )
C        BODY499_PM               = (  176.630    350.89198226  0.  )
C
C     The angles shown here are obtained from those angles via
C     the transformations
C
C        Angle 1 is -Pi/2 - RA, mapped into the range [0, 360).
C        Angle 2 is -Pi/2 + Dec.
C        Angle 3 is -PM.
C
C     Units have been changed as well:
C
C        RA/Dec terms in the PCK are in degrees and degrees/century;
C        the rates here have been converted to degrees/sec. Prime
C        meridian terms in the PCK are in degrees and degrees/day; the
C        rate here has been converted to degrees/sec.
C
C
      DATA                  COFSTR                                    /
     .                      '( 1           2E-8        3E-13 )',
     .                      '( 10          2E-7        3E-12 )',
     .                      '( 100         2E-6        3E-11 )',
     .                      '( 2                             )',
     .                      '( 4           5E-7              )',
     .                      '( 6           7E-7        8E-12 )',
     .                      '(  -47.68143  0.33621061170684714E-10 )',
     .                      '(  -37.1135  -0.19298045478743630E-10 )',
     .                      '( -176.630   -0.40612497946759260E-02 )'  /
 
C
C     ID codes of frame centers:
C
      DATA                  CTRCDE                                    /
     .                      399,
     .                      499                                       /
 
C
C     Reference epochs for polynomial definitions:
C
      DATA                  EPCSTR                                    /
     .                      '@2000-JAN-1/12:00',
     .                      '@2001-JAN-1/12:00'                       /
 
C
C     Angular units:
C
      DATA                  UNITS                                     /
     .                      '''DEGREES''',
     .                      '''RADIANS'''                             /
 
 
C
C     Rotation states:
C
      DATA                  RSTATE                                    /
     .                      KVROTG,
     .                      KVINRT,
     .                      'FROZEN'                                  /
 
C
C     DIMS defines the dimensions of the cartesian product of the
C     input parameters.  The cardinality of the set comprising the
C     Nth "factor" of the cartesian product is DIMS(N).  The cardinality
C     of the product itself is the product of the elements of DIMS.
C
      DATA                  DIMS                                      /
     .                      NCOSET,
     .                      NSEQ,
     .                      NUNITS,
     .                      NEPOCS,
     .                      NBASEF,
     .                      NCENTR,
     .                      NRSTAT                                    /
 
C
C     Initial values
C
      DATA                  ZR                                       /
     .                      9 * 0.D0                                 /
 
C
C     Open the test family.
C
      CALL TOPEN ( 'F_DYN04' )
 
 
C
C --- Case: ------------------------------------------------------
C
      CALL TCASE ( 'Create test inputs for comprehensive Euler ' //
     .             'frame test.'                                    )
 
C
C     Create and load kernels.
C
      CALL TSTLSK
      CALL CHCKXC  ( .FALSE., ' ', OK )
 
      CALL TSTSPK  ( SPK, .TRUE., HANDLE  )
      CALL CHCKXC  ( .FALSE., ' ', OK )
 
      CALL T_PCK08 ( PCK, .TRUE., .FALSE. )
      CALL CHCKXC  ( .FALSE., ' ', OK )
 
C
C     Parse the constant string used for the freeze epoch.
C     First remove the '@' and '/' characters from the string.
C     Add TDB token.
C
      CALL CMPRSS ( '@',     0,  FRZTOK, FRZEPC )
      CALL CHCKXC ( .FALSE., ' ', OK )
 
      CALL REPLCH ( FRZEPC, '/', ' ',    FRZEPC )
      CALL CHCKXC ( .FALSE., ' ', OK )
 
      CALL SUFFIX ( 'TDB',   1,          FRZEPC )
      CALL CHCKXC ( .FALSE., ' ', OK )
 
      CALL STR2ET ( FRZEPC, T0 )
      CALL CHCKXC ( .FALSE., ' ', OK )
 
C
C     Define the GSE frame.
C
      DEFTXT(1) =
     .'FRAME_GSE                        =  2399001'
      DEFTXT(2) =
     .'FRAME_2399001_NAME               = ''GSE'''
      DEFTXT(3) =
     .'FRAME_2399001_CLASS              =  5'
      DEFTXT(4) =
     .'FRAME_2399001_CLASS_ID           =  2399001'
      DEFTXT(5) =
     .'FRAME_2399001_CENTER             =  399'
      DEFTXT(6) =
     .'FRAME_2399001_RELATIVE           = ''J2000'''
      DEFTXT(7) =
     .'FRAME_2399001_'//KWSTYL//'       = '''// KVPARM //''''
      DEFTXT(8) =
     .'FRAME_2399001_'//KWFFAM//'             = '''// KV2VEC //''''
      DEFTXT(9) =
     .'FRAME_2399001_'//KWPRI//KWVAXI//'       = '''//KWX//''''
      DEFTXT(10) =
     .'FRAME_2399001_'//KWPRI//KWVCDF//'       = '''//KVPOSV//''''
      DEFTXT(11) =
     .'FRAME_2399001_'//KWPRI//KWVOBS//'       = ''EARTH'''
      DEFTXT(12) =
     .'FRAME_2399001_'//KWPRI//KWVTRG//'         = ''SUN'''
      DEFTXT(13) =
     .'FRAME_2399001_'//KWPRI//KWVABC//'         = ''NONE'''
      DEFTXT(14) =
     .'FRAME_2399001_'//KWSEC//KWVAXI//'       = ''-'//KWY//''''
      DEFTXT(15) =
     .'FRAME_2399001_'//KWSEC//KWVCDF//'       =  '''// KVVELV //''''
      DEFTXT(16) =
     .'FRAME_2399001_'//KWSEC//KWVOBS//'       = ''SUN'''
      DEFTXT(17) =
     .'FRAME_2399001_'//KWSEC//KWVTRG//'       = ''EARTH'''
      DEFTXT(18) =
     .'FRAME_2399001_'//KWSEC//KWVABC//'         = ''NONE'''
      DEFTXT(19) =
     .'FRAME_2399001_'//KWSEC//KWVFRM//'          = ''J2000'''
      DEFTXT(20) =
     .'FRAME_2399001_'//KWRSTA//'       =  '''// KVROTG //''''
 
C
C     Load the GSE frame definition.
C
      CALL LMPOOL ( DEFTXT, 20 )
      CALL CHCKXC ( .FALSE., ' ', OK )
 
C
C     We use MULTIX and the array DIMS to define a mapping that allows
C     us to map a test case number to a set of input parameters.
C
C     Compute the number of test cases in the input cartesian product
C     space.
C
      NCART = PRODAI ( DIMS, NDIMS )
 
 
      DO CASE = 1, NCART
C
C        Find the multi-dimensional coordinates of the current
C        case in the cartesian product of the test input sets.
C
         CALL MULTIX ( 1, NDIMS, DIMS, CASE, COORDS )
         CALL CHCKXC ( .FALSE., ' ', OK )
 
C
C        Set frame definition parameters defined by COORDS.
C        Start out with the coefficient sets.
C
         DO I = 1, 3
            ANGCOF(I) = COFSTR( I, COORDS(COFIDX) )
         END DO
 
C
C        Set the axis sequence.
C
         DO I = 1, 3
            AXES(I)  = AXESEQ ( I, COORDS(SEQIDX) )
         END DO
 
C
C        Set the angular units.
C
         UNTSTR = UNITS  ( COORDS(UNTIDX) )
 
C
C        Set the epoch.
C
         EPOCH  = EPCSTR ( COORDS(EPCIDX) )
 
C
C        Set the base frame.
C
         BFRAME = BASEFR ( COORDS(BFRIDX) )
 
C
C        Set the frame center.
C
         CENTER = CTRCDE ( COORDS(CTRIDX) )
 
C
C        Set the rotation state.
C
         RSTSTR = RSTATE ( COORDS(RSTIDX) )
 
C
C        Decide whether the current test case is the special case
C        for Mars.
C
         ISMARS =         ( CENTER         .EQ. 499                  )
     .             .AND.  ( BFRAME         .EQ. 'J2000'              )
     .             .AND.  ( EPOCH          .EQ. '@2000-JAN-1/12:00'  )
     .             .AND.  ( UNTSTR         .EQ. '''DEGREES'''        )
     .             .AND.  ( COORDS(SEQIDX) .EQ. MSQIDX               )
     .             .AND.  ( COORDS(COFIDX) .EQ. MCOIDX               )
 
C
C        For this test family, there are no cases that must be
C        rejected, so the "GO" flag is always .TRUE.  We keep
C        this currently unused flag to simplify maintenance of
C        this code.
C
         GO = .TRUE.
 
         IF ( GO ) THEN
C
C           We're ready to create a frame definition.
C
C
C           Create a text buffer containing a frame definition that
C           uses the current frame definition
C
C
C           First comes the frame name-to-frame code assignment.
C
            FRNAME     = 'EULER_DYN_FRAME'
            FRCODE     = -2000000000
 
            DEFTXT(1)  = 'FRAME_#           = #'
            CALL REPMC  ( DEFTXT(1), '#', FRNAME, DEFTXT(1) )
            CALL CHCKXC ( .FALSE., ' ', OK )
            CALL REPMI  ( DEFTXT(1), '#', FRCODE, DEFTXT(1) )
            CALL CHCKXC ( .FALSE., ' ', OK )
 
C
C           We'll use the kernel variable name "stem"
C
C              FRAME_<ID code>_
C
C           repeatedly, so instead of making thousands of REPMI
C           calls, we just create this stem here.
C
            CALL REPMI ( 'FRAME_#_', '#', FRCODE, FRSTEM )
            CALL CHCKXC ( .FALSE., ' ', OK )
 
 
            DEFTXT(2)  =  FRSTEM // 'NAME             = ''#'''
            CALL REPMC  ( DEFTXT(2), '#', FRNAME, DEFTXT(2) )
            CALL CHCKXC ( .FALSE., ' ', OK )
 
C
C           Frame class and class ID come next.
C
            DEFTXT(3)   =  FRSTEM // 'CLASS            = 5'
 
            DEFTXT(4)   =  FRSTEM // 'CLASS_ID         = #'
            CALL CHCKXC ( .FALSE., ' ', OK )
            CALL REPMI   ( DEFTXT(4), '#', FRCODE, DEFTXT(4) )
            CALL CHCKXC ( .FALSE., ' ', OK )
 
C
C           The frame center is specified via a code.
C
            DEFTXT(5)   =  FRSTEM // 'CENTER           = #'
 
            CALL REPMI  ( DEFTXT(5), '#',  CENTER, DEFTXT(5) )
            CALL CHCKXC ( .FALSE., ' ', OK )
 
C
C           Set the base frame.
C
            DEFTXT(6)   =  FRSTEM // 'RELATIVE         = ''#'''
            CALL REPMC  ( DEFTXT(6), '#', BFRAME, DEFTXT(6) )
            CALL CHCKXC ( .FALSE., ' ', OK )
 
C
C           Set the frame definition style and family.
C
            DEFTXT(7)  =
     .         FRSTEM // KWSTYL //'    = ''' // KVPARM // ''''
            CALL REPMI  ( DEFTXT(7), '#', FRCODE,    DEFTXT(7) )
            CALL CHCKXC ( .FALSE., ' ', OK )
 
            DEFTXT(8)  =  FRSTEM // KWFFAM //'           = '''//
     .                    KVEULR //''''
 
            CALL CHCKXC ( .FALSE., ' ', OK )
            CALL REPMI  ( DEFTXT(8), '#', FRCODE,    DEFTXT(8) )
            CALL CHCKXC ( .FALSE., ' ', OK )
 
C
C           Set the epoch for the polynomials.
C
            DEFTXT(9)  =  FRSTEM // KWEPOC //
     .                    '            = ' // EPOCH
 
C
C           Set the units for the polynomial coefficients.
C
            DEFTXT(10) =  FRSTEM // KWUNIT //
     .                    '            = ' // UNTSTR
 
C
C           Set the axis sequence.
C
            AXESTR = '( # # # )'
 
            DO I = 1, 3
               CALL REPMI  ( AXESTR, '#',  AXES(I), AXESTR )
               CALL CHCKXC ( .FALSE., ' ', OK )
            END DO
 
            DEFTXT(11) =  FRSTEM // KWEUAX //
     .                    '             = ' // AXESTR
 
C
C           Set the coefficients.
C
            DEFTXT(12) =  FRSTEM // KWEAC1 // '   = ' // ANGCOF(1)
            DEFTXT(13) =  FRSTEM // KWEAC2 // '   = ' // ANGCOF(2)
            DEFTXT(14) =  FRSTEM // KWEAC3 // '   = ' // ANGCOF(3)
 
C
C           Set the frame's rotation state or freeze epoch.
C
            IF ( RSTSTR .EQ. 'FROZEN' ) THEN
 
               DEFTXT(15)  =  FRSTEM // KWFREZ //'     = '
     .                               // FRZTOK
            ELSE
 
               DEFTXT(15)  =  FRSTEM // KWRSTA // '     = '''
     .                               // RSTSTR(:RTRIM(RSTSTR)) // ''''
            END IF
 
            DX = 15
 
C
C           Enter the frame definition into the kernel pool.
C
            CALL LMPOOL ( DEFTXT, DX )
            CALL CHCKXC ( .FALSE., ' ', OK )
 
C
C           Here's where the testing begins.
C
C
C--- Case: ------------------------------------------------------
C
            CALL REPMI  ( 'SXFORM/PXFORM test number #A.  Test ' //
     .                    'results against those from PXFORM.',
     .                    '#', CASE, CASMSG )
            CALL CHCKXC ( .FALSE., ' ', OK )
            CALL TCASE  ( CASMSG )
 
C
C           Pick a time value.
C
            ET = 1.D3 * ( CASE - 100 )
 
            CALL SXFORM ( FRNAME, BFRAME, ET, XFORM )
            CALL CHCKXC ( .FALSE., ' ', OK )
 
 
            CALL PXFORM ( FRNAME, BFRAME, ET, R )
            CALL CHCKXC ( .FALSE., ' ', OK )
 
C
C           Extract the derivative block from XFORM into DRVBLK.
C           We'll use this block later; we're extracting it here
C           to keep this code close to the associated SXFORM call.
C
            DO I = 1, 3
               DO J = 1, 3
                  DRVBLK(J,I) = XFORM(J+3,I)
               END DO
            END DO
 
            DMAG = VNORMG( DRVBLK, 9 )
 
C
C           Extract the rotation block from XFORM into R2.
C
            DO I = 1, 3
               DO J = 1, 3
                  R2(J,I) = XFORM(J,I)
               END DO
            END DO
 
            CALL CHCKAD ( 'SXFORM vs PXFORM', R2, '~', R, 9, DM12, OK )
 
 
 
C
C--- Case: ------------------------------------------------------
C
            CALL REPMI  ( 'SXFORM test number #B.  Test derivative ' //
     .                    'block.',
     .                    '#', CASE, CASMSG )
            CALL CHCKXC ( .FALSE., ' ', OK )
            CALL TCASE  ( CASMSG )
 
C
C           Pick a time value.
C
            ET = 1.D3 * ( CASE - 100 )
 
            CALL SXFORM ( FRNAME, BFRAME, ET, XFORM )
            CALL CHCKXC ( .FALSE., ' ', OK )
 
C
C           Perform "sanity checks" on the returned matrix.
C           Make sure the diagonal blocks are identical rotations,
C           compute a discrete derivative and compare to the
C           lower left block, and make sure the upper right
C           block is a zero matrix.
C
C           The value of DELTA (1/16) was determined by experimentation.
C
            DELTA = 0.0625D0
 
            CALL PXFORM ( FRNAME, BFRAME, ET-DELTA, RMINUS )
            CALL CHCKXC ( .FALSE., ' ', OK )
            CALL PXFORM ( FRNAME, BFRAME, ET+DELTA, RPLUS  )
            CALL CHCKXC ( .FALSE., ' ', OK )
 
            CALL T_XFORM ( XFORM,  RMINUS, RPLUS,  DELTA,
     .                     NRMERR, DETERR, DRVERR, DRLERR, DRDIFF )
            CALL CHCKXC ( .FALSE., ' ', OK )
 
 
C           Check the error measurements we've made.  First
C           up is the determinant error.
C
            TOL = DM14
            CALL CHCKSD ( 'DETERR', DETERR, '~', 0.D0, TOL, OK )
 
C
C           Check norms of rows and columns in the rotation
C           blocks.
C
            CALL CHCKSD ( 'NRMERR', NRMERR, '~', 0.D0, TOL, OK )
 
C
C           If the frame is considered to be rotating, compare the
C           discrete derivative to the lower left block.
C
            IF (  EQSTR( RSTSTR, KVROTG )  ) THEN
C
C              Check the derivative absolute error.
C
               TOL = DM6
               CALL CHCKSD ( 'DRVERR', DRVERR, '~', 0.D0, TOL, OK )
 
C
C              Check the derivative relative error.
C
               TOL = DM4
               CALL CHCKSD ( 'DRLERR', DRLERR, '~', 0.D0, TOL, OK )
 
 
            ELSE IF (  EQSTR( RSTSTR, KVINRT )  ) THEN
C
C              The frame is supposed to be inertial, so the
C              state transformation mapping from this frame to
C              J2000 ideally should have a zero derivative block.
C              See whether the derivative block of the latter
C              transformation is close to zero.
C
               CALL SXFORM ( BFRAME, 'J2000', ET,      XFB2J )
               CALL CHCKXC ( .FALSE., ' ', OK )
 
               CALL MXMG   ( XFB2J,  XFORM,   6, 6, 6, XFF2J )
 
               DO I = 1, 3
                  DO J = 1, 3
                     DRV2(J,I) = XFF2J(J+3,I)
                  END DO
               END DO
 
               CALL CHCKAD ( 'Derivative block',
     .                        DRV2, '~', ZR, 9, DM19, OK )
 
            ELSE
C
C              The frozen case is quite simple:  the derivative
C              with respect to the base frame is zero.
C
               CALL CHCKAD ( 'Derivative block',
     .                        DRVBLK, '=', ZR, 9, 0.D0, OK )
 
            END IF
 
 
C
C--- Case: ------------------------------------------------------
C
            CALL REPMI  ( 'SXFORM test number #C. Construct '      //
     .                    'state transformation locally; compare ' //
     .                    'to XFORM output.',
     .                    '#', CASE, CASMSG )
            CALL CHCKXC ( .FALSE., ' ', OK )
            CALL TCASE  ( CASMSG )
 
 
 
C
C           Parse the reference epoch; compute the evaluation epoch. If
C           the rotation state is frozen, use the "freeze" value.
C
            CALL CMPRSS ( '@', 0, EPOCH, TIMSTR )
            CALL SUFFIX ( 'TDB',  1,     TIMSTR )
 
            CALL STR2ET ( TIMSTR, REFEPC )
 
            IF (  EQSTR ( RSTSTR, 'FROZEN' )  ) THEN
               T = T0 - REFEPC
            ELSE
               T = ET - REFEPC
            END IF
 
C
C           Compute the Euler angles and their derivatives;
C           scale to radians and radians/sec.
C
            DO I = 1, 3
C
C              Parse the string '( <coeff 0> <coeff 1> ... <coeff n> )'
C
               CALL LPARSM ( ANGCOF(I), ' ()', MAXTOK, N, TOKENS )
 
C
C              The first token will be the null token preceeding
C              the first parenthesis.  The next 1-3 tokens are
C              numbers.  The last token is also null.
C
               DO J = 2, N-1
                  CALL PRSDP ( TOKENS(J), COEFFS(J-2) )
               END DO
 
C               WRITE (*,*) '-------------------------------------'
C               WRITE (*,*) ( COEFFS(J), J = 0, N-3 )
C               WRITE (*,*) '-------------------------------------'
 
C
C              Evaluate the polynomial P representing the angle
C              and its time derivative DP at T.
C
               P  = 0.D0
 
               DO J = N-3, 0, -1
 
                  P = COEFFS(J) + T * P
 
               END DO
 
               DP = 0.D0
 
               DO J = N-3, 1, -1
 
                  DP =  J * ( COEFFS(J) + T * DP )
 
               END DO
 
C
C              Convert angular units to radians.
C
               CALL CMPRSS ( '''', 0, UNTSTR, INUNIT )
 
               CALL CONVRT ( P,  INUNIT, KVRADN, EULANG(I)   )
               CALL CONVRT ( DP, INUNIT, KVRADN, EULANG(I+3) )
 
            END DO
 
C
C           Compute the state transformation matrix.
C
            CALL EUL2XF ( EULANG, AXES(1), AXES(2), AXES(3), XF2 )
 
C
C           Now compare XF2 to XFORM.
C
C           Extract the derivative block from XF2.
C
            DO I = 1, 3
               DO J = 1, 3
                  DRV2(J,I) = XF2(J+3,I)
               END DO
            END DO
 
C
C           Extract the rotation block and compare to
C           that for our dynamic frame.
C
            DO I = 1, 3
               DO J = 1, 3
                  R2(J,I) = XF2(J,I)
               END DO
            END DO
 
C
C           Compare to the rotation from the Euler frame to
C           the base frame returned by PXFORM.
C
            CALL PXFORM ( FRNAME, BFRAME, ET, R )
 
            TOL = DM14
            CALL CHCKAD ( 'R2', R2, '~', R, 9, TOL, OK )
 
 
C
C           Check the derivative block of the state transformation
C           matrix.
C
            CALL SXFORM ( FRNAME, BFRAME, ET, XFORM )
 
            DO I = 1, 3
               DO J = 1, 3
                  DRVBLK(J,I) = XFORM(J+3,I)
               END DO
            END DO
 
C
C           Our locally derivative is relevant only if the frame's
C           rotation state is KVROTG.
C
            IF ( RSTSTR .EQ. KVROTG ) THEN
 
               TOL = DM14
               CALL CHCKAD ( 'd(R2)/dt',
     .                       DRV2, '~', DRVBLK, 9, TOL, OK )
            END IF
 
C
C--- Case: ------------------------------------------------------
C
C
C           We've finished the generic tests.  If our frame is
C           the pseudo IAU_MARS frame, compare the frame to that
C           defined by our PCK.
C
            IF ( ISMARS .AND. ( RSTSTR .EQ. KVROTG )  ) THEN
 
               CALL TCASE  ( 'IAU_MARS test case' )
 
C
C              Get the PCK-defined state transformation matrix.
C              Set the epoch to ~2030; this will allow velocity
C              errors to accumulate.
C
               ET = JYEAR() * 30
               CALL SXFORM ( 'J2000', 'IAU_MARS', ET, TSIPM )
 
C
C              Extract the derivative block.
C
               DO I = 1, 3
                  DO J = 1, 3
                     DRV2(J,I) = TSIPM(J+3,I)
                  END DO
               END DO
 
C
C              Extract the rotation block and compare to
C              that for our dynamic frame.
C
               DO I = 1, 3
                  DO J = 1, 3
                     TIPM(J,I) = TSIPM(J,I)
                  END DO
               END DO
 
               CALL PXFORM ( 'J2000', FRNAME, ET, R )
 
               TOL = DM11
               CALL CHCKAD ( 'Mars TIPM', R, '~', TIPM, 9, TOL, OK )
 
C
C              Check the derivative block of the state transformation
C              matrix.
C
               CALL SXFORM ( 'J2000', FRNAME, ET, XFORM )
 
               DO I = 1, 3
                  DO J = 1, 3
                     DRVBLK(J,I) = XFORM(J+3,I)
                  END DO
               END DO
 
               CALL CHCKAD ( 'Mars d(TIPM)/dt',
     .                        DRVBLK, '~', DRV2, 9, TOL, OK )
 
            END IF
 
         END IF
C
C        Expunge the optional keywords from the kernel pool.
C
         IF ( RSTSTR .EQ. 'FROZEN' ) THEN
            KEYWRD  =  FRSTEM // KWFREZ
         ELSE
            KEYWRD  =  FRSTEM // KWRSTA
         END IF
 
         CALL DVPOOL ( KEYWRD )
         CALL CHCKXC ( .FALSE., ' ', OK )
 
      END DO
 
C
C     Clean up the SPK file.
C
      CALL SPKUEF ( HANDLE )
      CALL CHCKXC ( .FALSE., ' ', OK )
 
      CALL DELFIL ( SPK    )
      CALL CHCKXC ( .FALSE., ' ', OK )
 
C
C     Close out the test family.
C
      CALL T_SUCCESS ( OK )
      RETURN
 
      END
