 
C$Procedure SPKAPP_O ( S/P Kernel, apparent state )
 
      SUBROUTINE SPKAPP_O ( TARG, ET, REF, SOBS, ABCORR, STARG, LT )
 
C$ Abstract
C
C     Return the apparent state (position and velocity) of a target
C     body relative to an observer, optionally corrected for planetary
C     (light time) and stellar aberrations.
C
C$ Required_Reading
C
C     SPK
C
C$ Keywords
C
C     EPHEMERIS
C
C$ Declarations
 
      INTEGER               TARG
      DOUBLE PRECISION      ET
      CHARACTER*(*)         REF
      DOUBLE PRECISION      SOBS     ( 6 )
      CHARACTER*(*)         ABCORR
      DOUBLE PRECISION      STARG    ( 6 )
      DOUBLE PRECISION      LT
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     TARG       I   Target body.
C     ET         I   Target epoch.
C     REF        I   Target inertial reference frame.
C     SOBS       I   State of observer.
C     ABCORR     I   Aberration correction.
C     STARG      O   State of target.
C     LT         O   Light time.
C
C$ Detailed_Input
C
C     TARG        is the standard NAIF ID code for a target body.
C
C     ET          is the epoch (ephemeris time) at which the state
C                 of the target body is to be computed.
C
C     REF         is the name of the inertial reference frame to
C                 which the vectors returned by the routine should
C                 be rotated. This may be any frame supported by
C                 the SPICELIB subroutine CHGIRF.
C
C     SOBS        is the geometric state (position and velocity) of
C                 an arbitrary observer, relative to the solar system
C                 barycenter, at epoch ET, in the specified inertial
C                 reference frame. Units are always km and km/sec.
C
C     ABCORR      indicates the aberration corrections to be applied
C                 in order to determine the state of the target body.
C                 ABCORR may be any of the following.
C
C                    'NONE'     Apply no correction. Return the true
C                               state of the target body relative to
C                               the observer.
C
C                    'LT'       Correct for planetary (light time)
C                               aberration.
C
C                    'LT+S'     Correct for planetary (light time)
C                               and stellar aberrations.
C
C$ Detailed_Output
C
C     STARG       contains the position and velocity of the target
C                 body, relative to the specified observer, corrected
C                 for the specified aberrations, at epoch ET. These
C                 vectors are rotated into the specified inertial
C                 reference frame. Units are always km and km/sec.
C
C     LT          is the one-way light time from the observer to
C                 the geometric position of the target body at the
C                 specified epoch. The value of LT is not affected
C                 by the value of ABCORR.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If the value of ABCORR is not recognized, the error
C        'SPICE(SPKINVALIDOPTION)' is signalled.
C
C$ Files
C
C     See: C$ Restrictions.
C
C$ Particulars
C
C     SPKAPP_O begins by computing the geometric state, T(t), of the
C     target body relative to the solar system barycenter. Subtracting
C     the geometric state of the observer, O(t), gives the geometric
C     state of the target body relative to the observer. The one-way
C     light time, tau, is given by
C
C               | T(t) - O(t) |
C        tau = -----------------
C                      c
C
C     If no aberration correction is requested, this state is returned.
C
C           SSB ----- O(t)
C            |      /
C            |     /
C            |    /                          | T(t) - O(t) |
C            |   /  T(t) - O(t),      tau = -----------------
C            |  /                                   c
C           T(t)
C
C
C     If correction for planetary (light time) aberration is requested,
C     SPKAPP_O computes the state of the target body, T(t-tau), at epoch
C     ET - LT. Subtracting the geometric state of the observer, O(t),
C     gives the astrographic state of the target body relative to the
C     observer.
C
C           SSB ---- O(t)
C            | \    |
C            |  \   |
C            |   \  | T(t-tau) - O(t)
C            |    \ |
C            |     \|
C           T(t)    T(t-tau)
C
C
C     If correction for stellar aberration is requested, this
C     state is rotated toward the velocity vector of the observer.
C     The magnitude of the rotation depends on the magnitude of
C     the velocity, and the angle between the velocity and the
C     astrographic state.
C
C     Ephemeris data from more than one segment may be required
C     to determine the state of the target body relative to the
C     barycenter. SPKAPP_O gets states from SPKSSB, which reads as
C     many segments as necessary, from as many files as necessary,
C     using files that have been loaded by previous calls to SPKLEF
C     (load ephemeris file).
C
C$ Examples
C
C     In the following code fragment, SPKSSB and SPKAPP_O are used
C     to display the astrographic position of Io (body 501) as seen
C     from the Voyager 2 spacecraft (Body -32) at a series of epochs.
C
C        IO    = 501
C        VGR2  = -32
C
C        DO WHILE ( EPOCH .LE. END )
C           CALL SPKSSB (  VGR2, EPOCH, 'J2000', STVGR2  )
C           CALL SPKAPP_O (  IO,   EPOCH, 'J2000', STVGR2,
C       .                 'LT',  STIO,   LT              )
C
C           CALL RECRAD (  STIO, RANGE,  RA,     DEC     )
C           WRITE (*,*)  RA * DPR(),  DEC * DPR()
C
C           EPOCH = EPOCH + DELTA
C        END DO
C
C$ Restrictions
C
C     1) The ephemeris files to be used by SPKAPP_O must be loaded
C        by SPKLEF before SPKSSB is called.
C
C$ Literature_References
C
C     NAIF Document 168.0, "S- and P- Kernel (SPK) Specification and
C     User's Guide"
C
C$ Author_and_Institution
C
C     H.A. Neilan     (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.1.2, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.1.1, 06-MAR-1991 (JML)
C
C        In the example program, the calling sequence of SPKAPP_O
C        was corrected.
C
C-    SPICELIB Version 1.1.0, 25-MAY-1990 (HAN)
C
C        The local variable CORR was added to eliminate a
C        run-time error that occurred when SPKAPP_O was determining
C        what corrections to apply to the state.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&
 
C$ Index_Entries
C
C     apparent state from spk file
C     get apparent state
C
C-&
 
 
 
C$ Revisions
C
C-    SPICELIB Version 1.1.1, 06-MAR-1991 (JML)
C
C        In the example program, the calling sequence of SPKAPP_O
C        was corrected.
C
C-    SPICELIB Version 1.1.0, 25-MAY-1990 (HAN)
C
C        The local varaible CORR was added to eliminate a run-time
C        error that occurred when SPKAPP_O was determining what
C        corrections to apply to the state. If the literal string
C        'LT' was assigned to ABCORR, SPKAPP_O attempted to look at
C        ABCORR(3:4). Because ABCORR is a passed length argument, its
C        length is not guaranteed, and those positions may not exist.
C        Searching beyond the bounds of a string resulted in a
C        run-time error at NAIF because NAIF compiles SPICELIB using the
C        CHECK=BOUNDS option for the DEC VAX/VMX DCL FORTRAN command.
C        Also, without the local variable CORR, SPKAPP_O would have to
C        modify the value of a passed argument, ABCORR. That's a no no.
C
C-&
 
C
C     SPICELIB functions
C
      DOUBLE PRECISION      CLIGHT
      INTEGER               ISRCHC
      LOGICAL               RETURN
      DOUBLE PRECISION      VNORM
 
C
C     Local variables
C
      CHARACTER*(5)         FLAGS    ( 3 )
      CHARACTER*(5)         CORR
 
C
C     Saved variables
C
      SAVE                  FLAGS
 
C
C     Initial values
C
      DATA                  FLAGS   / 'NONE', 'LT', 'LT+S' /
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SPKAPP_O' )
      END IF
 
 
      CORR = ABCORR
      CALL LJUST  ( CORR, CORR )
      CALL CMPRSS ( ' ',  0,   CORR, CORR )
      CALL UCASE  ( CORR, CORR )
 
 
      IF ( ISRCHC ( CORR, 3, FLAGS ) .EQ. 0 ) THEN
         CALL SETMSG ( 'Requested aberration correction was #.' )
         CALL ERRCH  ( '#', ABCORR                              )
         CALL SIGERR ( 'SPICE(SPKINVALIDOPTION)'                )
         CALL CHKOUT ( 'SPKAPP_O' )
         RETURN
      END IF
 
C
C     Find the true state of the target body with respect to the
C     solar system barycenter. Subtract the state of the observer
C     to get the relative state. Use this to compute the one-way
C     light time.
C
      CALL SPKSSB ( TARG,  ET,   REF, STARG )
      CALL VSUBG  ( STARG, SOBS, 6,   STARG )
      LT = VNORM  ( STARG ) / CLIGHT()
 
C
C     To correct for light time, find the state of the target body
C     at the current epoch minus the one-way light time. Note that
C     the observer remains where he is.
C
      IF ( CORR(1:2) .EQ. 'LT' ) THEN
         CALL SPKSSB ( TARG,  ET-LT, REF, STARG )
         CALL VSUBG  ( STARG, SOBS,  6,   STARG )
 
C
C        To correct for stellar aberration (possible only if we have
C        already corrected for light time), use the velocity of the
C        observer relative to the solar system barycenter.
C
         IF ( CORR(3:4) .EQ. '+S' ) THEN
            CALL STELAB ( STARG, SOBS(4), STARG )
         END IF
      END IF
 
      CALL CHKOUT ( 'SPKAPP_O' )
 
      RETURN
      END
