 
C$Procedure SPKGEO ( S/P Kernel, geometric state )
 
      SUBROUTINE SPKGEO_O ( TARG, ET, REF, OBS, STATE, LT )
 
C$ Abstract
C
C     Compute the geometric state (position and velocity) of a target
C     body relative to an observing body.
C
C$ Required_Reading
C
C     SPK
C
C$ Keywords
C
C     EPHEMERIS
C
C$ Declarations
 
      INTEGER               TARG
      DOUBLE PRECISION      ET
      CHARACTER*(*)         REF
      INTEGER               OBS
      DOUBLE PRECISION      STATE    ( 6 )
      DOUBLE PRECISION      LT
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     TARG       I   Target body.
C     ET         I   Target epoch.
C     REF        I   Target inertial reference frame.
C     OBS        I   Observing body.
C     STATE      O   State of target.
C     LT         O   Light time.
C
C$ Detailed_Input
C
C     TARG        is the standard NAIF ID code for a target body.
C
C     ET          is the epoch (ephemeris time) at which the state
C                 of the target body is to be computed.
C
C     REF         is the name of the inertial reference frame to
C                 which the vectors returned by the routine should
C                 be rotated. This may be any frame supported by
C                 the SPICELIB subroutine CHGIRF.
C
C     OBS         is the standard NAIF ID code for an observing body.
C
C$ Detailed_Output
C
C     STATE       contains the position and velocity of the target
C                 body, relative to the observing body, corrected
C                 for the specified aberrations, at epoch ET.  STATE
C                 has six elements:  the first three contain the
C                 target's position; the last three contain the target's
C                 velocity.  These vectors are rotated into the
C                 specified inertial reference frame. Units are always
C                 km and km/sec.
C
C     LT          is the one-way light time from the observing body
C                 to the geometric position of the target body at the
C                 specified epoch.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If insufficient ephemeris data has been loaded to compute
C        the necessary states, the error SPICE(SPKINSUFFDATA) is
C        signalled.
C
C$ Files
C
C     See: $Restrictions.
C
C$ Particulars
C
C     SPKGEO computes the geometric state, T(t), of the target
C     body and the geometric state, O(t), of the observing body
C     relative to the first common center of motion.  Subtracting
C     O(t) from T(t) gives the geometric state of the target
C     body relative to the observer.
C
C
C        CENTER ----- O(t)
C            |      /
C            |     /
C            |    /
C            |   /  T(t) - O(t)
C            |  /
C           T(t)
C
C
C     The one-way light time, tau, is given by
C
C
C               | T(t) - O(t) |
C        tau = -----------------
C                      c
C
C
C     For example, if the observing body is -94, the Mars Observer
C     spacecraft, and the target body is 401, Phobos, then the
C     first common center is probably 4, the Mars Barycenter.
C     O(t) is the state of -94 relative to 4 and T(t) is the
C     state of 401 relative to 4.
C
C     The center could also be the Solar System Barycenter, body 0.
C     For example, if the observer is 399, Earth, and the target
C     is 299, Venus, then O(t) would be the state of 399 relative
C     to 0 and T(t) would be the state of 299 relative to 0.
C
C     Ephemeris data from more than one segment may be required
C     to determine the states of the target body and observer
C     relative to a common center.  SPKGEO reads as many segments
C     as necessary, from as many files as necessary, using files
C     that have been loaded by previous calls to SPKLEF (load
C     ephemeris file).
C
C     SPKGEO is similar to SPKEZ but returns geometric states
C     only, with no option to make planetary (light-time) nor
C     stellar aberration corrections.  The geometric states
C     returned by SPKEZ and SPKGEO are the same.
C
C$ Examples
C
C     The following code example computes the geometric
C     state of the moon with respect to the earth and
C     then prints the distance of the moon from the
C     the earth at a number of epochs.
C
C     Assume the SPK file SAMPLE.BSP contains ephemeris data
C     for the moon relative to earth over the time interval
C     from BEGIN to END.
C
C            INTEGER               EARTH
C            PARAMETER           ( EARTH = 399 )
C
C            INTEGER               MOON
C            PARAMETER           ( MOON  = 301 )
C
C            INTEGER               N
C            PARAMETER           ( N     = 100 )
C
C            INTEGER               HANDLE
C            CHARACTER*(20)        UTC
C            DOUBLE PRECISION      BEGIN
C            DOUBLE PRECISION      DELTA
C            DOUBLE PRECISION      END
C            DOUBLE PRECISION      ET
C            DOUBLE PRECISION      STATE ( 6 )
C
C     C
C     C      Load the binary SPK ephemeris file.
C     C
C            CALL SPKLEF ( 'SAMPLE.BSP', HANDLE )
C
C            .
C            .
C            .
C
C     C
C     C      Divide the interval of coverage [BEGIN,END] into
C     C      N steps.  At each step, compute the state, and
C     C      print out the epoch in UTC time and position norm.
C     C
C            DELTA = ( END - BEGIN ) / N
C
C            DO I = 0, N
C
C               ET = BEGIN + I*DELTA
C
C               CALL SPKGEO ( MOON, ET, 'J2000', EARTH, STATE, LT )
C
C               CALL ET2UTC ( ET, 'C', 0, UTC )
C
C               WRITE (*,*) UTC, VNORM ( STATE )
C
C            END DO
C
C$ Restrictions
C
C     1) The ephemeris files to be used by SPKGEO must be loaded
C        by SPKLEF before SPKGEO is called.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     J.E. McLean (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 10-MAR-1992 (WLT)
C
C        Comment section for permuted index source lines was added
C        following the header.
C
C-    SPICELIB Version 1.0.0, 18-JUL-1991 (JEM)
C
C-&
 
C$ Index_Entries
C
C     geometric state of one body relative to another
C
C-&
 
 
 
C
C     An algorithm similar to the one in SPKSSB is employed in
C     SPKGEO.  This is the idea:
C
C     Every body moves with respect to some center. The center
C     is itself a body, which in turn moves about some other
C     center.  If we begin at the target body (T), follow
C     the chain,
C
C                                   T
C                                     \
C           SSB                        \
C               \                     C[1]
C                \                     /
C                 \                   /
C                  \                 /
C                   \               /
C                  C[3]-----------C[2]
C
C     and avoid circular definitions (A moves about B, and B moves
C     about A), eventually we get the state relative to the solar
C     system barycenter (which, for our purposes, doesn't move).
C     Thus,
C
C        T    = T     + C[1]     + C[2]     + ... + C[n]
C         SSB    C[1]       C[2]       [C3]             SSB
C
C     where
C
C        X
C         Y
C
C     is the state of body X relative to body Y.
C
C     The above is what SPKSSB does.  It computes the state of the
C     target body relative to the SSB.  In SPKGEO, we want the
C     state of the target body relative to an observing body.
C     We could do this by finding the state of the target relative
C     to SSB and then the state of the observer relative to SSB
C     and subtracting the two states.
C
C     However, we don't want to follow each chain back to the SSB
C     if it isn't necessary.  Instead we will just follow the chain
C     of the target body and follow the chain of the observing body
C     until we find a common node in the tree.
C
C     In the example below, C is the first common node.  We compute
C     the state of TARG relative to C and the state of OBS relative
C     to C, then subtract the two states.
C
C                                   TARG
C                                     \
C           SSB                        \
C               \                       A
C                \                     /            OBS
C                 \                   /              |
C                  \                 /               |
C                   \               /                |
C                    B-------------C-----------------D
C
C
 
C
C
C     SPICELIB functions
C
      LOGICAL               FAILED
      LOGICAL               RETURN
 
      DOUBLE PRECISION      CLIGHT
      DOUBLE PRECISION      VNORM
 
      INTEGER               ISRCHI
 
C
C     Local parameters
C
C     CHLEN is the maximum length of a chain.  That is,
C     it is the maximum number of bodies in the chain from
C     the target or observer to the SSB.
C
      INTEGER               CHLEN
      PARAMETER           ( CHLEN = 10 )
 
      INTEGER               SSB
      PARAMETER           ( SSB   =  0 )
 
C
C     Local variables
C
      CHARACTER*(40)        IDENT
 
      DOUBLE PRECISION      DESCR  ( 5        )
      DOUBLE PRECISION      SOBS   ( 6        )
      DOUBLE PRECISION      STARG  ( 6, CHLEN )
      DOUBLE PRECISION      STEMP  ( 6        )
 
      INTEGER               COBS
      INTEGER               CTARG  (    CHLEN )
      INTEGER               CTPOS
      INTEGER               HANDLE
      INTEGER               I
      INTEGER               NCT
 
      LOGICAL               FOUND
 
C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SPKGEO_O' )
      END IF
 
C
C     CTARG contains the integer codes of the bodies in the
C     target body chain, beginning with TARG itself and then
C     the successive centers of motion.
C
C     STARG(1,I) is the state of the target body relative
C     to CTARG(I).
C
C     COBS and SOBS will contain the centers and states of the
C     observing body.  (They are single elements instead of arrays
C     because we only need the current center and state of the
C     observer relative to it.)
C
C     First, we construct CTARG and STARG.  CTARG(1) is
C     just the target itself, and STARG(1,1) is just a zero
C     vector, that is, the state of the target relative
C     to itself.
C
C     Then we follow the chain, filling up CTARG and STARG
C     as we go.  We use SPKSFS to search through loaded
C     files to find the first segment applicable to CTARG(1)
C     and time ET.  Then we use SPKPV to compute the state
C     of the body CTARG(1) at ET in the segment that was found
C     and get its center of motion which is CTARG(2).  We
C     repeat the process for CTARG(2) and so on, until
C     there is no data found for some CTARG(I) or until we
C     reach the SSB.
C
C     Next, we find centers and states in a similar manner
C     for the observer.  It's a similar construction as
C     described above, but I is always 1.  COBS and SOBS
C     are overwritten with each new center and state,
C     beginning at OBS.  However, we stop when we encounter
C     a common center of motion, that is when COBS is equal
C     to CTARG(I) for some I.
C
C     Finally, we compute the desired state of the target
C     relative to the observer by subtracting the state of
C     the observing body relative to the common node from
C     the state of the target body relative to the common
C     node.
C
C     CTPOS is the position in CTARG of the common node.
C
 
 
C
C     Construct CTARG and STARG.  Begin by assigning the
C     first elements:  TARG and the state of TARG relative
C     to itself.
C
      I = 1
      CTARG(I) = TARG
      CALL CLEARD ( 6, STARG(1,I) )
 
C
C     Fill in CTARG and STARG until no more data is found
C     or until we reach the SSB.  If the chain gets too
C     long to fit in CTARG, that is if I equals CHLEN,
C     then overwrite the last elements of CTARG and STARG.
C
C     Note the check for FAILED in the loop.  If SPKSFS
C     or SPKPV happens to fail during execution, and the
C     current error handling action is to NOT abort, then
C     FOUND may be stuck at TRUE, CTARG(I) will never
C     become zero, and the loop will execute indefinitely.
C
      FOUND = .TRUE.
 
      DO WHILE (           FOUND
     .            .AND.  ( I        .LT. CHLEN )
     .            .AND.  ( CTARG(I) .NE. SSB   )  )
 
C
C        Find a file and segment that has state
C        data for CTARG(I).
C
         CALL SPKSFS ( CTARG(I), ET, HANDLE, DESCR, IDENT, FOUND )
 
         IF ( FOUND ) THEN
C
C           Get the state of CTARG(I) relative to some
C           center of motion.  This new center goes in
C           CTARG(I+1) and the state is called STEMP.
C
            CALL SPKPV ( HANDLE, DESCR, ET, REF, STEMP, CTARG(I+1) )
 
C
C           Add STEMP to the state of TARG relative to
C           CTARG(I) to get the state of TARG relative
C           to CTARG(I+1).
C
            CALL VADDG ( STARG(1,I), STEMP, 6, STARG(1,I+1) )
 
C
C           If one of the routines above failed during
C           execution, we just give up and check out.
C
            IF ( FAILED() ) THEN
               CALL CHKOUT ( 'SPKGEO_O' )
               RETURN
            END IF
 
C
C           If no failure occurred, increment I to
C           repeat the loop for the new center of
C           motion returned by SPKPV.
C
            I = I + 1
 
         END IF
 
      END DO
 
C
C     If the loop above ended because we ran out of
C     room in the arrays CTARG and STARG, then we
C     continue finding states but we overwrite the
C     last elements of CTARG and STARG.
C
C     If, as a result, the first common node is
C     overwritten, we'll just have to settle for
C     the last common node.  This will cause a small
C     loss of precision, but it's better than other
C     alternatives.
C
      IF ( I .EQ. CHLEN ) THEN
 
         DO WHILE (  FOUND .AND. ( CTARG(CHLEN) .NE. SSB )  )
 
C
C           Find a file and segment that has state
C           data for CTARG(CHLEN).
C
            CALL SPKSFS ( CTARG(CHLEN), ET, HANDLE, DESCR, IDENT, FOUND)
 
            IF ( FOUND ) THEN
C
C              Get the state of CTARG(CHLEN) relative to
C              some center of motion.  The new center
C              overwrites the old.  The state is called
C              STEMP.
C
               CALL SPKPV ( HANDLE, DESCR, ET, REF, STEMP, CTARG(CHLEN))
 
C
C              Add STEMP to the state of TARG relative to
C              the old center to get the state of TARG
C              relative to the new center.  Overwrite
C              the last element of STARG.
C
               CALL VADDG ( STARG(1,CHLEN), STEMP, 6, STARG(1,CHLEN) )
 
C
C              If one of the routines above failed during
C              execution, we just give up and check out.
C
               IF ( FAILED() ) THEN
                  CALL CHKOUT ( 'SPKGEO_O' )
                  RETURN
               END IF
 
            END IF
 
         END DO
 
      END IF
 
C
C     NCT is the number of elements in CTARG,
C     the chain length.
C
      NCT = I
 
C
C     Now follow the observer's chain.  Assign
C     the first values for COBS and SOBS.
C
      COBS = OBS
      CALL CLEARD ( 6, SOBS )
 
C
C     Perhaps we have a common node already.
C     Check by searching for COBS in CTARG.
C     CTPOS will be the position of the common
C     node in CTARG if one is found.  It will
C     be zero if COBS is not found in CTARG.
C
      CTPOS = ISRCHI ( COBS, NCT, CTARG )
 
C
C     Repeat the same loop as above, but each time
C     we encounter a new center of motion, check to
C     see if it is a common node.  (When CTPOS is
C     not zero, CTARG(CTPOS) is the first common node.)
C
C     Note that we don't need a centers array nor a
C     states array, just a single center and state
C     is sufficient --- we just keep overwriting them.
C     When the common node is found, we have everything
C     we need in that one center (COBS) and state
C     (SOBS-state of the target relative to COBS).
C
      FOUND = .TRUE.
 
      DO WHILE (           FOUND
     .            .AND.  ( COBS  .NE. SSB )
     .            .AND.  ( CTPOS .EQ. 0   )  )
 
C
C        Find a file and segment that has state
C        data for COBS.
C
         CALL SPKSFS ( COBS, ET, HANDLE, DESCR, IDENT, FOUND )
 
         IF ( FOUND ) THEN
C
C           Get the state of COBS; call it STEMP.
C           The center of motion of COBS becomes the
C           new COBS.
C
            CALL SPKPV ( HANDLE, DESCR, ET, REF, STEMP, COBS )
 
C
C           Add STEMP to the state of OBS relative to
C           the old COBS to get the state of OBS
C           relative to the new COBS.
C
            CALL VADDG ( SOBS, STEMP, 6, SOBS )
 
C
C           Check failed.  We don't want to loop
C           indefinitely.
C
            IF ( FAILED() ) THEN
               CALL CHKOUT ( 'SPKGEO_O' )
               RETURN
            END IF
 
C
C           See if the new center is a common node.
C           If not, repeat the loop.
C
            CTPOS = ISRCHI ( COBS, NCT, CTARG )
 
         END IF
 
      END DO
 
 
C
C     If CTPOS is zero at this point, it means we
C     have not found a common node though we have
C     searched through all the available data.
C
      IF ( CTPOS .EQ. 0 ) THEN
 
         CALL SETMSG ( 'Insufficient ephemeris data has been loaded ' //
     .                 'to compute the state of TARG relative to OBS.' )
         CALL ERRINT ( 'TARG', TARG                                    )
         CALL ERRINT ( 'OBS',  OBS                                     )
         CALL SIGERR ( 'SPICE(SPKINSUFFDATA)'                          )
         CALL CHKOUT ( 'SPKGEO_O'                                      )
         RETURN
 
      END IF
 
C
C     If CTPOS is not zero, then we have reached a
C     common node, specifically,
C
C        CTARG(CTPOS) = COBS = CENTER
C
C     (in diagram below).  The STATE of the target
C     (TARG) relative to the observer (OBS) is just
C
C        STARG(1,CTPOS) - SOBS.
C
C
C
C                     SOBS
C         CENTER ---------------->OBS
C            |                  .
C            |                .
C         S  |              .   E
C         T  |            .   T
C         A  |          .   A
C         R  |        .   T
C         G  |      .   S
C            |    .
C            |  .
C            V L
C           TARG
C
C
C     And the light-time between them is just
C
C               | STATE |
C          LT = ---------
C                   c
C
C
      CALL VSUBG ( STARG(1,CTPOS), SOBS, 6, STATE )
 
      LT = VNORM  ( STATE ) / CLIGHT()
 
      CALL CHKOUT ( 'SPKGEO_O' )
      RETURN
      END
