C$Procedure T_BINGO ( BINGO: Process binary kernels to alternate BFFs )
 
      SUBROUTINE T_BINGO ( INAME, ONAME, OBFF )
 
C$ Abstract
C
C     Convert SPICE binary kernels from one supported binary file
C     format to another.
C
C$ Copyright
C
C     Copyright (2001), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     TEST ROUTINE
C
C$ Declarations
 
      IMPLICIT NONE
 
      INCLUDE              'zzddhman.inc'
 
      CHARACTER*(*)         INAME
      CHARACTER*(*)         ONAME
      INTEGER               OBFF
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     INAME      I   Name of source binary kernel to convert.
C     ONAME      I   Name of output binary kernel to create.
C     OBFF       I   Integer code for binary file format of ONAME.
C
C$ Detailed_Input
C
C     INAME      is the name of the binary kernel to convert to
C                an alternate binary file format.
C
C     ONAME      is the name of the converted binary kernel to create.
C                The file named by ONAME will be destroyed and
C                replaced with the converted file.
C
C     OBFF       is an integer code that indicates the binary file
C                format targeted for ONAME.  Acceptable values are
C                the parameters:
C
C                   BIGI3E
C                   LTLI3E
C                   VAXGFL
C                   VAXDFL
C
C                as defined in the include file 'zzddhman.inc'.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     This routine opens the file named by INAME for read access, and
C     creates (destroying any pre-existing file if necessary) the
C     file named by ONAME.
C
C$ Exceptions
C
C     1)
C
C$ Particulars
C
C     This test routine allows existing test software that creates
C     DAFs using the high level writers to create native format
C     files and convert them to supported non-native formats formats
C     for testing purposes.
C
C     As new binary file formats for existing kernels are added to
C     the list of those officially supported, this routine and
C     the routines it calls may require significant updates.
C
C$ Examples
C
C     See some TBD test family routine.
C
C$ Restrictions
C
C     This code is cobbled together in a rather non-robust way.
C     Do not attempt to convert binary kernels that are not 100%
C     correct in their structure and composition.  Using this
C     routine on files with questionable construction will yield
C     unpredictable results at best.
C
C$ Author_and_Institution
C
C     F.S. Turner     (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 28-NOV-2001 (FST)
C
C
C-&
 
C
C     SPICELIB Functions
C
      INTEGER               RTRIM
 
      LOGICAL               EQSTR
      LOGICAL               RETURN
 
C
C     Local Variables
C
      CHARACTER*(8)         ARCH
      CHARACTER*(FILEN)     FNAME
      CHARACTER*(8)         IDWORD
      CHARACTER*(8)         TYPE
 
 
      INTEGER               HANDLE
      INTEGER               IAMH
      INTEGER               IARC
      INTEGER               IBFF
      INTEGER               IOSTAT
      INTEGER               UNIT
 
      LOGICAL               FOUND
 
C
C     Standard SPICE error handling
C
      IF ( RETURN() ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'T_BINGO' )
      END IF
 
C
C     First things first, check to see if the handle manager has
C     an entry for ONAME.
C
      CALL ZZDDHFNH ( ONAME, HANDLE, FOUND )
 
      IF ( FOUND ) THEN
 
         CALL SETMSG ( 'Can not create file, ''#''.  It is currently '
     .   //            'loaded and associated with handle, #.  This '
     .   //            'routine may only create files not currently '
     .   //            'loaded.'                                       )
         CALL ERRCH  ( '#', ONAME                                      )
         CALL ERRINT ( '#', HANDLE                                     )
         CALL SIGERR ( 'SPICE(CANTCREATEFILE)'                         )
         CALL CHKOUT ( 'T_BINGO'                                       )
         RETURN
 
      END IF
 
C
C     Now, see if IFNAME is loaded in the handle manager.
C
      CALL ZZDDHFNH ( INAME, HANDLE, FOUND )
 
C
C     If it is loaded, then use the value of IARC to indicate the
C     file architecture.
C
      IF ( FOUND ) THEN
 
         CALL ZZDDHNFO ( HANDLE, FNAME, IARC, IBFF, IAMH, FOUND )
 
C
C     Otherwise, open the file and extract the 8 character IDWORD.
C
      ELSE
 
         CALL GETLUN ( UNIT )
 
         OPEN ( UNIT   = UNIT,
     .          FILE   = INAME( 1 : RTRIM(INAME) ),
     .          ACCESS = 'DIRECT',
     .          RECL   = RECL,
     .          STATUS = 'OLD',
     .          IOSTAT = IOSTAT                     )
 
         IF ( IOSTAT .NE. 0 ) THEN
            CALL SETMSG ( 'Unable to open file, ''#''. IOSTAT = #.' )
            CALL ERRCH  ( '#', INAME                                )
            CALL ERRINT ( '#', IOSTAT                               )
            CALL SIGERR ( 'SPICE(FILEOPENFAILED)'                   )
            CALL CHKOUT ( 'T_BINGO'                                 )
            RETURN
         END IF
 
         READ ( UNIT   = UNIT,
     .          REC    = 1,
     .          IOSTAT = IOSTAT ) IDWORD
 
         IF ( IOSTAT .NE. 0 ) THEN
            CALL SETMSG ( 'Unable to read ID word from file, '
     .      //            '''#''.  IOSTAT = #.'                )
            CALL ERRCH  ( '#', INAME                           )
            CALL ERRINT ( '#', IOSTAT                          )
            CALL SIGERR ( 'SPICE(FILEREADFAILED)'              )
            CALL CHKOUT ( 'T_BINGO'                            )
            RETURN
         END IF
 
         CALL IDW2AT ( IDWORD, ARCH, TYPE )
 
         IF ( EQSTR( ARCH, 'DAF' ) ) THEN
            IARC = DAF
         ELSE IF ( EQSTR( ARCH, 'DAS' ) ) THEN
            IARC = DAS
         ELSE
            IARC = 0
         END IF
 
         CLOSE ( UNIT = UNIT )
 
      END IF
 
C
C     IARC is set properly, branch appropriately.
C
      IF ( IARC .EQ. DAF ) THEN
 
         CALL T_BGODAF ( INAME, ONAME, OBFF )
 
      ELSE IF ( IARC .EQ. DAS ) THEN
 
         CALL SETMSG ( 'DAS file conversion not supported at this '
     .   //            'time.'                                      )
         CALL SIGERR ( 'SPICE(UNSUPPORTEDARCH)'                     )
         CALL CHKOUT ( 'T_BINGO'                                    )
         RETURN
 
      ELSE
 
         CALL SETMSG ( 'The architecture of file, ''#'', is '
     .   //            'unknown.  No conversion possible.'    )
         CALL SIGERR ( 'SPICE(UNKNOWNARCH)'                   )
         CALL CHKOUT ( 'T_BINGO'                              )
         RETURN
 
      END IF
 
      CALL CHKOUT ( 'T_BINGO' )
      RETURN
 
      END
